/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileCore;

[CCode (cheader_filename = "oslib/adfs.h")]
namespace OSLib.ADFS
{
  [CCode (cname = "adfs_controller")]
  public enum Controller
  {
    [CCode (cname = "adfs_CONTROLLER_DISC_NOT_PRESENT")]
    DiscNotPresent,
    [CCode (cname = "adfs_CONTROLLER1772")]
    Controller1772,
    [CCode (cname = "adfs_CONTROLLER71X")]
    Controller71X,
    [CCode (cname = "adfs_CONTROLLER_ST506")]
    ST506,
    [CCode (cname = "adfs_CONTROLLER_IDE")]
    IDE
  }

  [CCode (cname="error_ADFS_DRIVER_IN_USE")]
  public const bits Error_ADFSDriverInUse;

  [CCode (cname = "xadfsdiscop_verify")]
  public static OS.Error* XADFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "adfsdiscop_verify")]
  public static int ADFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr);

  [CCode (cname = "xadfsdiscop_read_sectors")]
  public static OS.Error* XADFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "adfsdiscop_read_sectors")]
  public static int ADFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfsdiscop_write_sectors")]
  public static OS.Error* XADFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "adfsdiscop_write_sectors")]
  public static int ADFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfsdiscop_read_track")]
  public static OS.Error* XADFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop_read_track")]
  public static void ADFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop_read_id")]
  public static OS.Error* XADFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop_read_id")]
  public static void ADFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop_write_track")]
  public static OS.Error* XADFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop_write_track")]
  public static void ADFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop_format_track")]
  public static OS.Error* XADFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "adfsdiscop_format_track")]
  public static void ADFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "xadfsdiscop_seek")]
  public static OS.Error* XADFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "adfsdiscop_seek")]
  public static void ADFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xadfsdiscop_restore")]
  public static OS.Error* XADFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "adfsdiscop_restore")]
  public static void ADFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xadfsdiscop_step_in")]
  public static OS.Error* XADFSDiscOp_StepIn
    (bits flags);

  [CCode (cname = "adfsdiscop_step_in")]
  public static void ADFSDiscOp_StepIn
    (bits flags);

  [CCode (cname = "xadfsdiscop_step_out")]
  public static OS.Error* XADFSDiscOp_StepOut
    (bits flags);

  [CCode (cname = "adfsdiscop_step_out")]
  public static void ADFSDiscOp_StepOut
    (bits flags);

  [CCode (cname = "xadfsdiscop_read_sectors_via_cache")]
  public static OS.Error* XADFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "adfsdiscop_read_sectors_via_cache")]
  public static int ADFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xadfsdiscop_specify")]
  public static OS.Error* XADFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "adfsdiscop_specify")]
  public static void ADFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xadfs_hdc")]
  public static OS.Error* XHDC
    (char* controller,
     int* poll_location,
     bits poll_bits,
     char* enable_location,
     bits enable_bits);

  [CCode (cname = "adfs_hdc")]
  public static void HDC
    (char* controller,
     int* poll_location,
     bits poll_bits,
     char* enable_location,
     bits enable_bits);

  [CCode (cname = "xadfs_drives")]
  public static OS.Error* XDrives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "adfs_drives")]
  public static void Drives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "xadfs_free_space")]
  public static OS.Error* XFreeSpace
    (string disc_name,
     out int free,
     out int max);

  [CCode (cname = "adfs_free_space")]
  public static int FreeSpace
    (string disc_name,
     out int max);

  [CCode (cname = "xadfs_retries")]
  public static OS.Error* XRetries
    (bits mask,
     bits value,
     out bits old_value,
     out bits new_value);

  [CCode (cname = "adfs_retries")]
  public static bits Retries
    (bits mask,
     bits value,
     out bits new_value);

  [CCode (cname = "xadfs_describe_disc")]
  public static OS.Error* XDescribeDisc
    (string disc_name,
     FileCore.Disc disc);

  [CCode (cname = "adfs_describe_disc")]
  public static void DescribeDisc
    (string disc_name,
     FileCore.Disc disc);

  [CCode (cname = "xadfs_vet_format")]
  public static OS.Error* XVetFormat
    (FileCore.Format format,
     int drive_no);

  [CCode (cname = "adfs_vet_format")]
  public static void VetFormat
    (FileCore.Format format,
     int drive_no);

  [CCode (cname = "xadfs_controller_type")]
  public static OS.Error* XControllerType
    (int drive_no,
     out Controller controller);

  [CCode (cname = "adfs_controller_type")]
  public static Controller ControllerType
    (int drive_no);

  [CCode (cname = "xadfspowercontrol_read_spin")]
  public static OS.Error* XADFSPowerControl_ReadSpin
    (int drive_no,
     out bool spinning);

  [CCode (cname = "adfspowercontrol_read_spin")]
  public static bool ADFSPowerControl_ReadSpin
    (int drive_no);

  [CCode (cname = "xadfspowercontrol_set_autospindown")]
  public static OS.Error* XADFSPowerControl_SetAutospindown
    (int drive_no,
     int spindown_time,
     out int old_spindown_time);

  [CCode (cname = "adfspowercontrol_set_autospindown")]
  public static int ADFSPowerControl_SetAutospindown
    (int drive_no,
     int spindown_time);

  [CCode (cname = "xadfspowercontrol_set_spin")]
  public static OS.Error* XADFSPowerControl_SetSpin
    (int drive_no,
     bool spinning);

  [CCode (cname = "adfspowercontrol_set_spin")]
  public static void ADFSPowerControl_SetSpin
    (int drive_no,
     bool spinning);

  [CCode (cname = "xadfs_set_ide_controller")]
  public static OS.Error* XSetIDEController
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     AsmRoutine read_code,
     AsmRoutine write_code);

  [CCode (cname = "adfs_set_ide_controller")]
  public static void SetIDEController
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     AsmRoutine read_code,
     AsmRoutine write_code);

  [CCode (cname = "xadfs_ide_user_op")]
  public static OS.Error* XIDEUserOp
    (bits op,
     char* commands,
     char* buffer,
     int size,
     int timeout,
     out int status,
     out int untransferred);

  [CCode (cname = "adfs_ide_user_op")]
  public static int IDEUserOp
    (bits op,
     char* commands,
     char* buffer,
     int size,
     int timeout,
     out int status);

  [CCode (cname = "xadfsmiscop_mount")]
  public static OS.Error* XADFSMiscOp_Mount
    (int drive_no,
     FileCore.DiscAddress disc_addr,
     char* buffer,
     int size,
     FileCore.Disc disc);

  [CCode (cname = "adfsmiscop_mount")]
  public static void ADFSMiscOp_Mount
    (int drive_no,
     FileCore.DiscAddress disc_addr,
     char* buffer,
     int size,
     FileCore.Disc disc);

  [CCode (cname = "xadfsmiscop_poll_changed")]
  public static OS.Error* XADFSMiscOp_PollChanged
    (int drive_no,
     int sequence_no,
     out int sequence_no_out,
     out bits result);

  [CCode (cname = "adfsmiscop_poll_changed")]
  public static void ADFSMiscOp_PollChanged
    (int drive_no,
     int sequence_no,
     out int sequence_no_out,
     out bits result);

  [CCode (cname = "xadfsmiscop_lock_drive")]
  public static OS.Error* XADFSMiscOp_LockDrive
    (int drive_no);

  [CCode (cname = "adfsmiscop_lock_drive")]
  public static void ADFSMiscOp_LockDrive
    (int drive_no);

  [CCode (cname = "xadfsmiscop_unlock_drive")]
  public static OS.Error* XADFSMiscOp_UnlockDrive
    (int drive_no);

  [CCode (cname = "adfsmiscop_unlock_drive")]
  public static void ADFSMiscOp_UnlockDrive
    (int drive_no);

  [CCode (cname = "xadfsmiscop_poll_period")]
  public static OS.Error* XADFSMiscOp_PollPeriod
    (string disc_name,
     out int poll_period,
     out string media_type);

  [CCode (cname = "adfsmiscop_poll_period")]
  public static void ADFSMiscOp_PollPeriod
    (string disc_name,
     out int poll_period,
     out string media_type);

  [CCode (cname = "xadfsmiscop_eject_disc")]
  public static OS.Error* XADFSMiscOp_EjectDisc
    (int drive_no);

  [CCode (cname = "adfsmiscop_eject_disc")]
  public static void ADFSMiscOp_EjectDisc
    (int drive_no);

  [CCode (cname = "xadfsmiscop_read_info")]
  public static OS.Error* XADFSMiscOp_ReadInfo
    (out FileCore.Info info);

  [CCode (cname = "adfsmiscop_read_info")]
  public static void ADFSMiscOp_ReadInfo
    (out FileCore.Info info);

  [CCode (cname = "xadfssectordiscop_verify")]
  public static OS.Error* XADFSSectorDiscOp_Verify
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "adfssectordiscop_verify")]
  public static int ADFSSectorDiscOp_Verify
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr);

  [CCode (cname = "xadfssectordiscop_read_sectors")]
  public static OS.Error* XADFSSectorDiscOp_ReadSectors
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "adfssectordiscop_read_sectors")]
  public static int ADFSSectorDiscOp_ReadSectors
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfssectordiscop_write_sectors")]
  public static OS.Error* XADFSSectorDiscOp_WriteSectors
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "adfssectordiscop_write_sectors")]
  public static int ADFSSectorDiscOp_WriteSectors
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfssectordiscop_read_track")]
  public static OS.Error* XADFSSectorDiscOp_ReadTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfssectordiscop_read_track")]
  public static void ADFSSectorDiscOp_ReadTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfssectordiscop_read_id")]
  public static OS.Error* XADFSSectorDiscOp_ReadId
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfssectordiscop_read_id")]
  public static void ADFSSectorDiscOp_ReadId
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfssectordiscop_write_track")]
  public static OS.Error* XADFSSectorDiscOp_WriteTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "adfssectordiscop_write_track")]
  public static void ADFSSectorDiscOp_WriteTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data);

  [CCode (cname = "xadfssectordiscop_format_track")]
  public static OS.Error* XADFSSectorDiscOp_FormatTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "adfssectordiscop_format_track")]
  public static void ADFSSectorDiscOp_FormatTrack
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "xadfssectordiscop_seek")]
  public static OS.Error* XADFSSectorDiscOp_Seek
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "adfssectordiscop_seek")]
  public static void ADFSSectorDiscOp_Seek
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "xadfssectordiscop_restore")]
  public static OS.Error* XADFSSectorDiscOp_Restore
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "adfssectordiscop_restore")]
  public static void ADFSSectorDiscOp_Restore
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "xadfssectordiscop_step_in")]
  public static OS.Error* XADFSSectorDiscOp_StepIn
    (bits flags);

  [CCode (cname = "adfssectordiscop_step_in")]
  public static void ADFSSectorDiscOp_StepIn
    (bits flags);

  [CCode (cname = "xadfssectordiscop_step_out")]
  public static OS.Error* XADFSSectorDiscOp_StepOut
    (bits flags);

  [CCode (cname = "adfssectordiscop_step_out")]
  public static void ADFSSectorDiscOp_StepOut
    (bits flags);

  [CCode (cname = "xadfssectordiscop_read_sectors_via_cache")]
  public static OS.Error* XADFSSectorDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "adfssectordiscop_read_sectors_via_cache")]
  public static int ADFSSectorDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.SectorDiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xadfssectordiscop_specify")]
  public static OS.Error* XADFSSectorDiscOp_Specify
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "adfssectordiscop_specify")]
  public static void ADFSSectorDiscOp_Specify
    (bits flags,
     FileCore.SectorDiscAddress disc_addr);

  [CCode (cname = "xadfs_lock_ide")]
  public static OS.Error* XLockIDE
    (bool lock);

  [CCode (cname = "adfs_lock_ide")]
  public static void LockIDE
    (bool lock);

  [CCode (cname = "xadfs_free_space64")]
  public static OS.Error* XFreeSpace64
    (string disc_name,
     out bits free_lo,
     out int free_hi,
     out int max);

  [CCode (cname = "adfs_free_space64")]
  public static void FreeSpace64
    (string disc_name,
     out bits free_lo,
     out int free_hi,
     out int max);

  [CCode (cname = "xadfsdiscop64_verify")]
  public static OS.Error* XADFSDiscOp64_Verify
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     int size,
     out FileCore.DiscAddress64 next_disc_addr,
     out int unverified);

  [CCode (cname = "adfsdiscop64_verify")]
  public static int ADFSDiscOp64_Verify
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     int size,
     out FileCore.DiscAddress64 next_disc_addr);

  [CCode (cname = "xadfsdiscop64_read_sectors")]
  public static OS.Error* XADFSDiscOp64_ReadSectors
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "adfsdiscop64_read_sectors")]
  public static int ADFSDiscOp64_ReadSectors
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfsdiscop64_write_sectors")]
  public static OS.Error* XADFSDiscOp64_WriteSectors
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "adfsdiscop64_write_sectors")]
  public static int ADFSDiscOp64_WriteSectors
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data);

  [CCode (cname = "xadfsdiscop64_read_track")]
  public static OS.Error* XADFSDiscOp64_ReadTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop64_read_track")]
  public static void ADFSDiscOp64_ReadTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop64_read_id")]
  public static OS.Error* XADFSDiscOp64_ReadId
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop64_read_id")]
  public static void ADFSDiscOp64_ReadId
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop64_write_track")]
  public static OS.Error* XADFSDiscOp64_WriteTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "adfsdiscop64_write_track")]
  public static void ADFSDiscOp64_WriteTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data);

  [CCode (cname = "xadfsdiscop64_format_track")]
  public static OS.Error* XADFSDiscOp64_FormatTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "adfsdiscop64_format_track")]
  public static void ADFSDiscOp64_FormatTrack
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "xadfsdiscop64_seek")]
  public static OS.Error* XADFSDiscOp64_Seek
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "adfsdiscop64_seek")]
  public static void ADFSDiscOp64_Seek
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "xadfsdiscop64_restore")]
  public static OS.Error* XADFSDiscOp64_Restore
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "adfsdiscop64_restore")]
  public static void ADFSDiscOp64_Restore
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "xadfsdiscop64_step_in")]
  public static OS.Error* XADFSDiscOp64_StepIn
    (bits flags);

  [CCode (cname = "adfsdiscop64_step_in")]
  public static void ADFSDiscOp64_StepIn
    (bits flags);

  [CCode (cname = "xadfsdiscop64_step_out")]
  public static OS.Error* XADFSDiscOp64_StepOut
    (bits flags);

  [CCode (cname = "adfsdiscop64_step_out")]
  public static void ADFSDiscOp64_StepOut
    (bits flags);

  [CCode (cname = "xadfsdiscop64_read_sectors_via_cache")]
  public static OS.Error* XADFSDiscOp64_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "adfsdiscop64_read_sectors_via_cache")]
  public static int ADFSDiscOp64_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress64 disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress64 next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xadfsdiscop64_specify")]
  public static OS.Error* XADFSDiscOp64_Specify
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "adfsdiscop64_specify")]
  public static void ADFSDiscOp64_Specify
    (bits flags,
     FileCore.DiscAddress64 disc_addr);

  [CCode (cname = "xservice_identify_format")]
  public static OS.Error* XService_IdentifyFormat
    (string format,
     out string command,
     out bool unclaimed,
     out int disc_format_swi,
     out int disc_format_handle,
     out int layout_structure_swi,
     out int layout_structure_handle);

  [CCode (cname = "service_identify_format")]
  public static bool Service_IdentifyFormat
    (string format,
     out string command,
     out int disc_format_swi,
     out int disc_format_handle,
     out int layout_structure_swi,
     out int layout_structure_handle);

  [CCode (cname = "xservice_display_format_help")]
  public static OS.Error* XService_DisplayFormatHelp
    (out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_display_format_help")]
  public static bool Service_DisplayFormatHelp
    (out OS.Error error);

}
