/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/arcweb.h")]
namespace OSLib.Arcweb
{
  [CCode (cname = "arcweb_h", has_type_id = false)]
  public struct H : bits
  {
  }

  [CCode (cname = "arcweb_url")]
  public struct URL
  {
    [CCode (cname = "url_file_w")]
    public unowned OS.FW UrlFileW;
    [CCode (cname = "url")]
    public unowned char Url[200];
  }

  [CCode (cname = "arcweb_expiry")]
  public struct Expiry
  {
    [CCode (cname = "use_given_expiry")]
    public unowned int UseGivenExpiry;
    [CCode (cname = "use_default_expiry")]
    public unowned int UseDefaultExpiry;
    [CCode (cname = "date")]
    public unowned OS.DateAndTime Date;
    [CCode (cname = "reserved")]
    public unowned char Reserved[3];
    [CCode (cname = "response_code")]
    public unowned int ResponseCode;
    [CCode (cname = "mime_type")]
    public unowned char MimeType[208];
  }

  [CCode (cname = "arcweb_message_fetch_request")]
  public struct MessageFetchRequest
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_message_fetch_done")]
  public struct MessageFetchDone
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.expiry")]
    public unowned Expiry Result_Expiry;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_message_render_request")]
  public struct MessageRenderRequest
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "temporary_file")]
    public unowned int TemporaryFile;
    [CCode (cname = "diagram_file")]
    public unowned int DiagramFile;
    [CCode (cname = "link_file")]
    public unowned int LinkFile;
    [CCode (cname = "file_header_size")]
    public unowned int FileHeaderSize;
    [CCode (cname = "file_header")]
    public unowned char FileHeader[208];
  }

  [CCode (cname = "arcweb_message_render_done")]
  public struct MessageRenderDone
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.size")]
    public unowned OS.Coord Result_Size;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_message_post_request")]
  public struct MessagePostRequest
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "form_file")]
    public unowned int FormFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_message_post_done")]
  public struct MessagePostDone
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.expiry")]
    public unowned Expiry Result_Expiry;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_message_email_request")]
  public struct MessageEmailRequest
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "form_file")]
    public unowned int FormFile;
  }

  [CCode (cname = "arcweb_message_email_done")]
  public struct MessageEmailDone
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_message_image_size")]
  public struct MessageImageSize
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "renderer_handle")]
    public unowned int RendererHandle;
    [CCode (cname = "size")]
    public unowned OS.Coord Size;
  }

  [CCode (cname = "arcweb_message_image_complete")]
  public struct MessageImageComplete
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "renderer_handle")]
    public unowned int RendererHandle;
  }

  [CCode (cname = "arcweb_message_xfer_control")]
  public struct MessageXferControl
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_message_xfer_control_ack")]
  public struct MessageXferControlAck
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_message_expire")]
  public struct MessageExpire
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_message_abort_request")]
  public struct MessageAbortRequest
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "reason")]
    public unowned char Reason[228];
  }

  [CCode (cname = "arcweb_message_transfer_status")]
  public struct MessageTransferStatus
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "transmit_total")]
    public unowned int TransmitTotal;
    [CCode (cname = "transmit_current")]
    public unowned int TransmitCurrent;
    [CCode (cname = "receive_total")]
    public unowned int ReceiveTotal;
    [CCode (cname = "receive_current")]
    public unowned int ReceiveCurrent;
    [CCode (cname = "message")]
    public unowned char Message[208];
  }

  [CCode (cname = "arcweb_message_configure")]
  public struct MessageConfigure
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "application")]
    public unowned char Application[228];
  }

  [CCode (cname = "arcweb_message_external_launch")]
  public struct MessageExternalLaunch
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_message_launch_done")]
  public struct MessageLaunchDone
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_message_web_browser")]
  public struct MessageWebBrowser
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "browser")]
    public unowned char Browser[228];
  }

  [CCode (cname = "arcweb_message_colour_selection")]
  public struct MessageColourSelection
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
    [CCode (cname = "normal")]
    public unowned OS.Colour Normal;
    [CCode (cname = "link")]
    public unowned OS.Colour Link;
    [CCode (cname = "vlink")]
    public unowned OS.Colour Vlink;
    [CCode (cname = "alink")]
    public unowned OS.Colour Alink;
    [CCode (cname = "no_images")]
    public unowned OS.Colour NoImages;
  }

  [CCode (cname = "arcweb_message_permanent_move")]
  public struct MessagePermanentMove
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "from")]
    public unowned string From;
    [CCode (cname = "from_size")]
    public unowned int FromSize;
    [CCode (cname = "to")]
    public unowned string To;
    [CCode (cname = "to_size")]
    public unowned int ToSize;
  }

  [CCode (cname = "arcweb_full_message_fetch_request")]
  public struct FullMessageFetchRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_full_message_fetch_done")]
  public struct FullMessageFetchDone : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.expiry")]
    public unowned Expiry Result_Expiry;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_full_message_render_request")]
  public struct FullMessageRenderRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "temporary_file")]
    public unowned int TemporaryFile;
    [CCode (cname = "diagram_file")]
    public unowned int DiagramFile;
    [CCode (cname = "link_file")]
    public unowned int LinkFile;
    [CCode (cname = "file_header_size")]
    public unowned int FileHeaderSize;
    [CCode (cname = "file_header")]
    public unowned char FileHeader[208];
  }

  [CCode (cname = "arcweb_full_message_render_done")]
  public struct FullMessageRenderDone : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.size")]
    public unowned OS.Coord Result_Size;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_full_message_post_request")]
  public struct FullMessagePostRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "form_file")]
    public unowned int FormFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_full_message_post_done")]
  public struct FullMessagePostDone : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "result.expiry")]
    public unowned Expiry Result_Expiry;
    [CCode (cname = "result.error")]
    public unowned char Result_Error[228];
  }

  [CCode (cname = "arcweb_full_message_email_request")]
  public struct FullMessageEmailRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "form_file")]
    public unowned int FormFile;
  }

  [CCode (cname = "arcweb_full_message_email_done")]
  public struct FullMessageEmailDone : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_full_message_image_size")]
  public struct FullMessageImageSize : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "renderer_handle")]
    public unowned int RendererHandle;
    [CCode (cname = "image_coords")]
    public unowned OS.Coord ImageCoords;
  }

  [CCode (cname = "arcweb_full_message_image_complete")]
  public struct FullMessageImageComplete : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "renderer_handle")]
    public unowned int RendererHandle;
  }

  [CCode (cname = "arcweb_full_message_xfer_control")]
  public struct FullMessageXferControl : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "source_file")]
    public unowned int SourceFile;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_full_message_xfer_control_ack")]
  public struct FullMessageXferControlAck : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_full_message_expire")]
  public struct FullMessageExpire : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_full_message_abort_request")]
  public struct FullMessageAbortRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "reason")]
    public unowned char Reason[228];
  }

  [CCode (cname = "arcweb_full_message_transfer_status")]
  public struct FullMessageTransferStatus : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "transmit_total")]
    public unowned int TransmitTotal;
    [CCode (cname = "transmit_current")]
    public unowned int TransmitCurrent;
    [CCode (cname = "receive_total")]
    public unowned int ReceiveTotal;
    [CCode (cname = "receive_current")]
    public unowned int ReceiveCurrent;
    [CCode (cname = "message")]
    public unowned char Message[208];
  }

  [CCode (cname = "arcweb_full_message_configure")]
  public struct FullMessageConfigure : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "application")]
    public unowned char Application[228];
  }

  [CCode (cname = "arcweb_full_message_external_launch")]
  public struct FullMessageExternalLaunch : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "url")]
    public unowned URL Url;
  }

  [CCode (cname = "arcweb_full_message_launch_done")]
  public struct FullMessageLaunchDone : Wimp.MessageHeaderBase
  {
    [CCode (cname = "handle")]
    public unowned H Handle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "error")]
    public unowned char Error[228];
  }

  [CCode (cname = "arcweb_full_message_web_browser")]
  public struct FullMessageWebBrowser : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "browser")]
    public unowned char Browser[228];
  }

  [CCode (cname = "arcweb_full_message_colour_selection")]
  public struct FullMessageColourSelection : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
    [CCode (cname = "normal")]
    public unowned OS.Colour Normal;
    [CCode (cname = "link")]
    public unowned OS.Colour Link;
    [CCode (cname = "vlink")]
    public unowned OS.Colour Vlink;
    [CCode (cname = "alink")]
    public unowned OS.Colour Alink;
    [CCode (cname = "no_images")]
    public unowned OS.Colour NoImages;
  }

  [CCode (cname = "arcweb_full_message_permanent_move")]
  public struct FullMessagePermanentMove : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "from")]
    public unowned string From;
    [CCode (cname = "from_size")]
    public unowned int FromSize;
    [CCode (cname = "to")]
    public unowned string To;
    [CCode (cname = "to_size")]
    public unowned int ToSize;
  }

  [CCode (cname="message_ARCWEB_FETCH_REQUEST")]
  public const bits Message_ArcwebFetchRequest;
  [CCode (cname="message_ARCWEB_FETCH_DONE")]
  public const bits Message_ArcwebFetchDone;
  [CCode (cname="message_ARCWEB_RENDER_REQUEST")]
  public const bits Message_ArcwebRenderRequest;
  [CCode (cname="message_ARCWEB_RENDER_DONE")]
  public const bits Message_ArcwebRenderDone;
  [CCode (cname="message_ARCWEB_POST_REQUEST")]
  public const bits Message_ArcwebPostRequest;
  [CCode (cname="message_ARCWEB_POST_DONE")]
  public const bits Message_ArcwebPostDone;
  [CCode (cname="message_ARCWEB_EMAIL_REQUEST")]
  public const bits Message_ArcwebEmailRequest;
  [CCode (cname="message_ARCWEB_EMAIL_DONE")]
  public const bits Message_ArcwebEmailDone;
  [CCode (cname="message_ARCWEB_IMAGE_SIZE")]
  public const bits Message_ArcwebImageSize;
  [CCode (cname="message_ARCWEB_IMAGE_COMPLETE")]
  public const bits Message_ArcwebImageComplete;
  [CCode (cname="message_ARCWEB_XFER_CONTROL")]
  public const bits Message_ArcwebXferControl;
  [CCode (cname="message_ARCWEB_XFER_CONTROL_ACK")]
  public const bits Message_ArcwebXferControlAck;
  [CCode (cname="message_ARCWEB_QUIT")]
  public const bits Message_ArcwebQuit;
  [CCode (cname="message_ARCWEB_EXPIRE")]
  public const bits Message_ArcwebExpire;
  [CCode (cname="message_ARCWEB_ABORT_REQUEST")]
  public const bits Message_ArcwebAbortRequest;
  [CCode (cname="message_ARCWEB_TRANSFER_STATUS")]
  public const bits Message_ArcwebTransferStatus;
  [CCode (cname="message_ARCWEB_CONFIGURE")]
  public const bits Message_ArcwebConfigure;
  [CCode (cname="message_ARCWEB_EXTERNAL_LAUNCH")]
  public const bits Message_ArcwebExternalLaunch;
  [CCode (cname="message_ARCWEB_LAUNCH_DONE")]
  public const bits Message_ArcwebLaunchDone;
  [CCode (cname="message_ARCWEB_WEB_BROWSER")]
  public const bits Message_ArcwebWebBrowser;
  [CCode (cname="message_ARCWEB_COLOUR_SELECTED")]
  public const bits Message_ArcwebColourSelected;
  [CCode (cname="message_ARCWEB_PERMANENT_MOVE")]
  public const bits Message_ArcwebPermanentMove;
  [CCode (cname="arcweb_DOCUMENT_FETCH")]
  public const bits DocumentFetch;
  [CCode (cname="arcweb_LOCAL_LAUNCH")]
  public const bits LocalLaunch;
  [CCode (cname="arcweb_FAST_IMAGES")]
  public const bits FastImages;
  [CCode (cname="arcweb_POST_WAS_PUT")]
  public const bits PostWasPut;
  [CCode (cname="arcweb_IMAGES_DISABLED")]
  public const bits ImagesDisabled;
  [CCode (cname="arcweb_XFER_CONTROL")]
  public const bits XferControl;
  [CCode (cname="arcweb_REFERER_AVAILABLE")]
  public const bits RefererAvailable;
  [CCode (cname="arcweb_FORCING_IMAGES")]
  public const bits ForcingImages;
  [CCode (cname="arcweb_PUT")]
  public const bits Put;
  [CCode (cname="arcweb_MAKE_DIRECTORY")]
  public const bits MakeDirectory;
  [CCode (cname="arcweb_REMOVE_DIRECTORY")]
  public const bits RemoveDirectory;
  [CCode (cname="arcweb_REMOVE")]
  public const bits Remove;
  [CCode (cname="arcweb_RENAME")]
  public const bits Rename;
  [CCode (cname="arcweb_EXTENDED_TYPE")]
  public const bits ExtendedType;
  [CCode (cname="arcweb_TYPE_MASK")]
  public const bits TypeMask;
  [CCode (cname="arcweb_LOAD_TO_DISC")]
  public const bits LoadToDisc;
  [CCode (cname="arcweb_RELOAD")]
  public const bits Reload;
  [CCode (cname="arcweb_EMAIL_COMPLETE")]
  public const bits EmailComplete;
  [CCode (cname="arcweb_EXTENDED_URL")]
  public const bits ExtendedURL;
  [CCode (cname="arcweb_INHIBIT_DISPLAY")]
  public const bits InhibitDisplay;
  [CCode (cname="arcweb_ERROR_HANDLED")]
  public const bits ErrorHandled;
  [CCode (cname="arcweb_IMAGE_FETCH")]
  public const bits ImageFetch;
  [CCode (cname="arcweb_CLOSE_PREVIOUS")]
  public const bits ClosePrevious;
  [CCode (cname="arcweb_URL_CHANGED")]
  public const bits URLChanged;
  [CCode (cname="arcweb_WANT_SYMLINK")]
  public const bits WantSymlink;
  [CCode (cname="arcweb_INHIBIT_RENDER")]
  public const bits InhibitRender;
  [CCode (cname="arcweb_TYPE_KNOWN")]
  public const bits TypeKnown;
  [CCode (cname="arcweb_ERROR")]
  public const bits Error;
  [CCode (cname="arcweb_PLAIN_TEXT")]
  public const bits PlainText;
  [CCode (cname="arcweb_HTML")]
  public const bits HTML;
  [CCode (cname="arcweb_IMAGE")]
  public const bits Image;
  [CCode (cname="arcweb_AUDIO")]
  public const bits Audio;
  [CCode (cname="arcweb_VIDEO")]
  public const bits Video;
  [CCode (cname="arcweb_BINARY")]
  public const bits Binary;
  [CCode (cname="arcweb_IMAGE_JPEG")]
  public const bits ImageJPEG;
  [CCode (cname="arcweb_IMAGE_GIF")]
  public const bits ImageGIF;
  [CCode (cname="arcweb_IMAGE_JFIF")]
  public const bits ImageJFIF;
  [CCode (cname="arcweb_IMAGE_BMP")]
  public const bits ImageBMP;
  [CCode (cname="arcweb_IMAGE_XBM")]
  public const bits ImageXBM;
  [CCode (cname="arcweb_IMAGE_CLEAR")]
  public const bits ImageClear;
  [CCode (cname="arcweb_IMAGE_TIFF")]
  public const bits ImageTIFF;
  [CCode (cname="arcweb_IMAGE_PNG")]
  public const bits ImagePNG;
  [CCode (cname="arcweb_VIDEO_MPEG")]
  public const bits VideoMPEG;
  [CCode (cname="arcweb_VIDEO_ARMOVIE")]
  public const bits VideoArmovie;
  [CCode (cname="arcweb_AUDIO_MISC")]
  public const bits AudioMisc;
  [CCode (cname="arcweb_AUDIO_SIMPLE")]
  public const bits AudioSimple;
  [CCode (cname="arcweb_AUDIO_AIFF")]
  public const bits AudioAIFF;
  [CCode (cname="arcweb_UNKNOWN")]
  public const bits Unknown;
  [CCode (cname="arcweb_XMIT_RUNNING")]
  public const bits XmitRunning;
  [CCode (cname="arcweb_RECV_RUNNING")]
  public const bits RecvRunning;
  [CCode (cname="arcweb_XMIT_COMPLETE")]
  public const bits XmitComplete;
  [CCode (cname="arcweb_RECV_COMPLETE")]
  public const bits RecvComplete;
  [CCode (cname="arcweb_MISC_RUNNING")]
  public const bits MiscRunning;

}
