/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/blendtable.h")]
namespace OSLib.BlendTable
{
  [CCode (cname = "blendtable_generate_flags")]
  public enum GenerateFlags
  {
    [CCode (cname = "blendtable_SOURCE_IS_POINTER")]
    SourceIsPointer,
    [CCode (cname = "blendtable_DESTINAION_IS_POINTER")]
    DestinaionIsPointer,
    [CCode (cname = "blendtable_LOCK_TABLE")]
    LockTable
  }


  [CCode (cname = "xblendtable_generate_table_mode_to_mode")]
  public static OS.Error* XGenerateTableModeToMode
    (GenerateFlags flags,
     OS.Mode source_mode,
     OS.Mode destination_mode,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "blendtable_generate_table_mode_to_mode")]
  public static void GenerateTableModeToMode
    (GenerateFlags flags,
     OS.Mode source_mode,
     OS.Mode destination_mode,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "xblendtable_generate_table_mode_to_sprite")]
  public static OS.Error* XGenerateTableModeToSprite
    (GenerateFlags flags,
     OS.Mode source_mode,
     OSSpriteOp.Area destination_area,
     OSSpriteOp.Id destination_id,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "blendtable_generate_table_mode_to_sprite")]
  public static void GenerateTableModeToSprite
    (GenerateFlags flags,
     OS.Mode source_mode,
     OSSpriteOp.Area destination_area,
     OSSpriteOp.Id destination_id,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "xblendtable_generate_table_sprite_to_mode")]
  public static OS.Error* XGenerateTableSpriteToMode
    (GenerateFlags flags,
     OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode destination_mode,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "blendtable_generate_table_sprite_to_mode")]
  public static void GenerateTableSpriteToMode
    (GenerateFlags flags,
     OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode destination_mode,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "xblendtable_generate_table_sprite_to_sprite")]
  public static OS.Error* XGenerateTableSpriteToSprite
    (GenerateFlags flags,
     OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OSSpriteOp.Area destination_area,
     OSSpriteOp.Id destination_id,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "blendtable_generate_table_sprite_to_sprite")]
  public static void GenerateTableSpriteToSprite
    (GenerateFlags flags,
     OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OSSpriteOp.Area destination_area,
     OSSpriteOp.Id destination_id,
     int translucency,
     char* buffer_in,
     out int source_colours,
     out int destination_colours,
     out int size,
     out char* buffer_out);

  [CCode (cname = "xblendtable_unlock_table")]
  public static OS.Error* XUnlockTable
    (char* buffer);

  [CCode (cname = "blendtable_unlock_table")]
  public static void UnlockTable
    (char* buffer);

}
