/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileCore;

[CCode (cheader_filename = "oslib/cdfs.h")]
namespace OSLib.CDFS
{
  [CCode (cname = "cdfs_truncation_type")]
  public enum TruncationType
  {
    [CCode (cname = "cdfs_TRUNCATION_FROM_RIGHT")]
    TruncationFromRight,
    [CCode (cname = "cdfs_TRUNCATION_FROM_LEFT")]
    TruncationFromLeft,
    [CCode (cname = "cdfs_TRUNCATION_NONE")]
    TruncationNone,
    [CCode (cname = "cdfs_TRUNCATION_DEFAULT")]
    TruncationDefault
  }


  [CCode (cname = "xcdfs_convert_drive_to_device")]
  public static OS.Error* XConvertDriveToDevice
    (int log_no,
     out int phys_no);

  [CCode (cname = "cdfs_convert_drive_to_device")]
  public static int ConvertDriveToDevice
    (int log_no);

  [CCode (cname = "xcdfs_set_buffer_size")]
  public static OS.Error* XSetBufferSize
    (int buffer_size_exp);

  [CCode (cname = "cdfs_set_buffer_size")]
  public static void SetBufferSize
    (int buffer_size_exp);

  [CCode (cname = "xcdfs_get_buffer_size")]
  public static OS.Error* XGetBufferSize
    (out int buffer_size_exp);

  [CCode (cname = "cdfs_get_buffer_size")]
  public static int GetBufferSize ();

  [CCode (cname = "xcdfs_set_number_of_drives")]
  public static OS.Error* XSetNumberOfDrives
    (int drive_count);

  [CCode (cname = "cdfs_set_number_of_drives")]
  public static void SetNumberOfDrives
    (int drive_count);

  [CCode (cname = "xcdfs_get_number_of_drives")]
  public static OS.Error* XGetNumberOfDrives
    (out int drive_count);

  [CCode (cname = "cdfs_get_number_of_drives")]
  public static int GetNumberOfDrives ();

  [CCode (cname = "xcdfs_give_file_type")]
  public static OS.Error* XGiveFileType
    (string file_name,
     out bits file_type);

  [CCode (cname = "cdfs_give_file_type")]
  public static bits GiveFileType
    (string file_name);

  [CCode (cname = "xcdfs_describe_disc")]
  public static OS.Error* XDescribeDisc
    (int log_no,
     FileCore.Disc disc);

  [CCode (cname = "cdfs_describe_disc")]
  public static void DescribeDisc
    (int log_no,
     FileCore.Disc disc);

  [CCode (cname = "xcdfs_where_is_file")]
  public static OS.Error* XWhereIsFile
    (string file_name,
     out int block_no,
     out int size);

  [CCode (cname = "cdfs_where_is_file")]
  public static void WhereIsFile
    (string file_name,
     out int block_no,
     out int size);

  [CCode (cname = "xcdfstruncation_read")]
  public static OS.Error* XCDFSTruncation_Read
    (out TruncationType type);

  [CCode (cname = "cdfstruncation_read")]
  public static TruncationType CDFSTruncation_Read ();

  [CCode (cname = "xcdfstruncation_write")]
  public static OS.Error* XCDFSTruncation_Write
    (TruncationType type);

  [CCode (cname = "cdfstruncation_write")]
  public static void CDFSTruncation_Write
    (TruncationType type);

}
