/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Menu;

[CCode (cheader_filename = "oslib/colourmenu.h")]
namespace OSLib.ColourMenu
{
  [CCode (cname = "colourmenu_flags")]
  public enum Flags
  {
    [CCode (cname = "colourmenu_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "colourmenu_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "colourmenu_GENERATE_HAS_BEEN_HIDDEN")]
    GenerateHasBeenHidden,
    [CCode (cname = "colourmenu_NONE_ENTRY")]
    NoneEntry
  }

  [CCode (cname = "colourmenu_colour")]
  public enum Colour
  {
    [CCode (cname = "colourmenu_TRANSPARENT")]
    Transparent,
    [CCode (cname = "colourmenu_NONE")]
    None
  }

  [CCode (cname = "colourmenu_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "colour")]
    public unowned Colour Colour;
  }

  [CCode (cname = "colourmenu_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "colour")]
    public unowned Colour Colour;
  }

  [CCode (cname = "colourmenu_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "colourmenu_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "colourmenu_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "colourmenu_action_has_been_hidden_block")]
  public enum ActionHasBeenHiddenBlock
  {
    Default = 0
  }

  [CCode (cname = "colourmenu_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "colourmenu_action_selection")]
  public struct ActionSelection
  {
    [CCode (cname = "colour")]
    public unowned Colour Colour;
  }

  [CCode (cname = "colourmenu_action_selection_block")]
  public struct ActionSelectionBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "colour")]
    public unowned Colour Colour;
  }

  [CCode (cname="class_COLOUR_MENU")]
  public const Toolbox.Class Class_ColourMenu;
  [CCode (cname="action_COLOUR_MENU_ABOUT_TO_BE_SHOWN")]
  public const bits Action_ColourMenuAboutToBeShown;
  [CCode (cname="action_COLOUR_MENU_DIALOGUE_COMPLETED")]
  public const bits Action_ColourMenuDialogueCompleted;
  [CCode (cname="action_COLOUR_MENU_HAS_BEEN_HIDDEN")]
  public const bits Action_ColourMenuHasBeenHidden;
  [CCode (cname="action_COLOUR_MENU_SELECTION")]
  public const bits Action_ColourMenuSelection;
  [CCode (cname="error_COLOUR_MENU_TASKS_ACTIVE")]
  public const bits Error_ColourMenuTasksActive;
  [CCode (cname="error_COLOUR_MENU_ALLOC_FAILED")]
  public const bits Error_ColourMenuAllocFailed;
  [CCode (cname="error_COLOUR_MENU_SHORT_BUFFER")]
  public const bits Error_ColourMenuShortBuffer;
  [CCode (cname="error_COLOUR_MENU_NO_SUCH_TASK")]
  public const bits Error_ColourMenuNoSuchTask;
  [CCode (cname="error_COLOUR_MENU_NO_SUCH_METHOD")]
  public const bits Error_ColourMenuNoSuchMethod;
  [CCode (cname="error_COLOUR_MENU_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_ColourMenuNoSuchMiscOpMethod;

  [CCode (cname = "xcolourmenu_set_colour")]
  public static OS.Error* XSetColour
    (bits flags,
     Toolbox.O colour_menu,
     Colour colour);

  [CCode (cname = "colourmenu_set_colour")]
  public static void SetColour
    (bits flags,
     Toolbox.O colour_menu,
     Colour colour);

  [CCode (cname = "xcolourmenu_get_colour")]
  public static OS.Error* XGetColour
    (bits flags,
     Toolbox.O colour_menu,
     out Colour colour);

  [CCode (cname = "colourmenu_get_colour")]
  public static Colour GetColour
    (bits flags,
     Toolbox.O colour_menu);

  [CCode (cname = "xcolourmenu_set_none_available")]
  public static OS.Error* XSetNoneAvailable
    (bits flags,
     Toolbox.O colour_menu,
     bool none_available);

  [CCode (cname = "colourmenu_set_none_available")]
  public static void SetNoneAvailable
    (bits flags,
     Toolbox.O colour_menu,
     bool none_available);

  [CCode (cname = "xcolourmenu_get_none_available")]
  public static OS.Error* XGetNoneAvailable
    (bits flags,
     Toolbox.O colour_menu,
     out bool none_available);

  [CCode (cname = "colourmenu_get_none_available")]
  public static bool GetNoneAvailable
    (bits flags,
     Toolbox.O colour_menu);

  [CCode (cname = "xcolourmenu_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O colour_menu,
     string title);

  [CCode (cname = "colourmenu_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O colour_menu,
     string title);

  [CCode (cname = "xcolourmenu_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O colour_menu,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "colourmenu_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O colour_menu,
     string buffer,
     int size);

}
