/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.JPEG;

[CCode (cheader_filename = "oslib/compressjpeg.h")]
namespace OSLib.CompressJPEG
{
  [CCode (cname = "compressjpeg_tag", has_type_id = false)]
  public struct Tag : bits
  {
  }

  [CCode (cname = "compressjpeg_parameters")]
  public struct Parameters
  {
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "quality")]
    public unowned int Quality;
    [CCode (cname = "bytes_per_pixel")]
    public unowned int BytesPerPixel;
    [CCode (cname = "xdpi")]
    public unowned int Xdpi;
    [CCode (cname = "ydpi")]
    public unowned int Ydpi;
  }

  [CCode (cname = "compressjpeg_scan_description_base")]
  public struct ScanDescriptionBase
  {
    [CCode (cname = "scan_count")]
    public unowned int ScanCount;
  }

  [Compact]
  [CCode (cname = "compressjpeg_scan_description", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ScanDescription
  {
    public ScanDescription(int numEntries);
    [CCode (cname = "scan_count")]
    public unowned int ScanCount;
    [CCode (cname = "scan_data.components")]
    public unowned int ScanData_Components;
    [CCode (cname = "scan_data.r")]
    public unowned int ScanData_R;
    [CCode (cname = "scan_data.g")]
    public unowned int ScanData_G;
    [CCode (cname = "scan_data.b")]
    public unowned int ScanData_B;
    [CCode (cname = "scan_data.ss")]
    public unowned int ScanData_Ss;
    [CCode (cname = "scan_data.se")]
    public unowned int ScanData_Se;
    [CCode (cname = "scan_data.ah")]
    public unowned int ScanData_Ah;
    [CCode (cname = "scan_data.al")]
    public unowned int ScanData_Al;
  }

  [CCode (cname = "compressjpeg_comment_flags")]
  public enum CommentFlags
  {
    [CCode (cname = "compressjpeg_CONTROL_TERMINATED")]
    ControlTerminated
  }

  [CCode (cname = "compressjpeg_write_line_flags")]
  public enum WriteLineFlags
  {
    [CCode (cname = "compressjpeg_SOURCE_TYPE")]
    SourceType,
    [CCode (cname = "compressjpeg_SOURCE_TYPE1_BPP")]
    SourceType1BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE2_BPP")]
    SourceType2BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE4_BPP")]
    SourceType4BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE8_BPP")]
    SourceType8BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE16BPP")]
    SourceType16BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE32BPP")]
    SourceType32BPP,
    [CCode (cname = "compressjpeg_SOURCE_TYPE_RGB_TRIPLES")]
    SourceTypeRGBTriples,
    [CCode (cname = "compressjpeg_HAS_DOUBLE_ENTRIES")]
    HasDoubleEntries,
    [CCode (cname = "compressjpeg_IGNORE_HIGH_BYTE")]
    IgnoreHighByte,
    [CCode (cname = "compressjpeg_IS_RGB")]
    IsRGB
  }

  [CCode (cname = "compressjpeg_transcode_flags")]
  public enum TranscodeFlags
  {
    [CCode (cname = "compressjpeg_GENERATE_PROGRESSIVE")]
    GenerateProgressive,
    [CCode (cname = "compressjpeg_OPERATION_ON_UNKNOWNS")]
    OperationOnUnknowns,
    [CCode (cname = "compressjpeg_TRANSFORMATION_TYPE")]
    TransformationType,
    [CCode (cname = "compressjpeg_TRIM_EDGES")]
    TrimEdges,
    [CCode (cname = "compressjpeg_GENERATE_GREYSCALE")]
    GenerateGreyscale,
    [CCode (cname = "compressjpeg_OPTIMISE_FOR_SMALLER")]
    OptimiseForSmaller,
    [CCode (cname = "compressjpeg_COPY_NO_ADDITIONAL")]
    CopyNoAdditional,
    [CCode (cname = "compressjpeg_COPY_COMMENTS")]
    CopyComments,
    [CCode (cname = "compressjpeg_COPY_ALL_ELEMENTS")]
    CopyAllElements,
    [CCode (cname = "compressjpeg_NO_TRANSFORMATION")]
    NoTransformation,
    [CCode (cname = "compressjpeg_FLIP_HORIZONTALLY")]
    FlipHorizontally,
    [CCode (cname = "compressjpeg_FLIP_VERTICALLY")]
    FlipVertically,
    [CCode (cname = "compressjpeg_ROTATE90_CLOCKWISE")]
    Rotate90Clockwise,
    [CCode (cname = "compressjpeg_ROTATE180")]
    Rotate180,
    [CCode (cname = "compressjpeg_ROTATE90_ANTI_CLOCKWISE")]
    Rotate90AntiClockwise,
    [CCode (cname = "compressjpeg_TRANSPOSE")]
    Transpose,
    [CCode (cname = "compressjpeg_TRANSVERSE_TRANSPOSE")]
    TransverseTranspose
  }

  [CCode (cname="compressjpeg_OPERATION_ON_UNKNOWNS_SHIFT")]
  public const int OperationOnUnknownsShift;
  [CCode (cname="compressjpeg_TRANSFORMATION_TYPE_SHIFT")]
  public const int TransformationTypeShift;
  [CCode (cname="error_COMPRESS_JPEG_BAD_BPP")]
  public const bits Error_CompressJPEGBadBPP;
  [CCode (cname="error_COMPRESS_JPEG_BAD_LINE_COUNT")]
  public const bits Error_CompressJPEGBadLineCount;
  [CCode (cname="error_COMPRESS_JPEG_BAD_BUFFER")]
  public const bits Error_CompressJPEGBadBuffer;
  [CCode (cname="error_COMPRESS_JPEG_BAD_SIZE")]
  public const bits Error_CompressJPEGBadSize;
  [CCode (cname="error_COMPRESS_JPEG_ARITH_NOT_IMPL")]
  public const bits Error_CompressJPEGArithNotImpl;
  [CCode (cname="error_COMPRESS_JPEG_BAD_ALIGN_TYPE")]
  public const bits Error_CompressJPEGBadAlignType;
  [CCode (cname="error_COMPRESS_JPEG_BAD_ALLOC_CHUNK")]
  public const bits Error_CompressJPEGBadAllocChunk;
  [CCode (cname="error_COMPRESS_JPEG_BAD_BUFFER_MODE")]
  public const bits Error_CompressJPEGBadBufferMode;
  [CCode (cname="error_COMPRESS_JPEG_BAD_COMPONENT_ID")]
  public const bits Error_CompressJPEGBadComponentID;
  [CCode (cname="error_COMPRESS_JPEG_BAD_DCT_SIZE")]
  public const bits Error_CompressJPEGBadDctSize;
  [CCode (cname="error_COMPRESS_JPEG_BAD_IN_COLOUR_SPACE")]
  public const bits Error_CompressJPEGBadInColourSpace;
  [CCode (cname="error_COMPRESS_JPEG_BAD_KCOLOUR_SPACE")]
  public const bits Error_CompressJPEGBadKColourSpace;
  [CCode (cname="error_COMPRESS_JPEG_BAD_LENGTH")]
  public const bits Error_CompressJPEGBadLength;
  [CCode (cname="error_COMPRESS_JPEG_BAD_MCU_SIZE")]
  public const bits Error_CompressJPEGBadMCUSize;
  [CCode (cname="error_COMPRESS_JPEG_BAD_POOL_ID")]
  public const bits Error_CompressJPEGBadPoolID;
  [CCode (cname="error_COMPRESS_JPEG_BAD_PRECISION")]
  public const bits Error_CompressJPEGBadPrecision;
  [CCode (cname="error_COMPRESS_JPEG_BAD_SAMPLING")]
  public const bits Error_CompressJPEGBadSampling;
  [CCode (cname="error_COMPRESS_JPEG_BAD_STATE")]
  public const bits Error_CompressJPEGBadState;
  [CCode (cname="error_COMPRESS_JPEG_BAD_VIRTUAL_ACCESS")]
  public const bits Error_CompressJPEGBadVirtualAccess;
  [CCode (cname="error_COMPRESS_JPEG_BUFFER_SIZE")]
  public const bits Error_CompressJPEGBufferSize;
  [CCode (cname="error_COMPRESS_JPEG_CANT_SUSPEND")]
  public const bits Error_CompressJPEGCantSuspend;
  [CCode (cname="error_COMPRESS_JPEGCCIR601_NOT_IMPL")]
  public const bits Error_CompressJPEGCCIR601NotImpl;
  [CCode (cname="error_COMPRESS_JPEG_COMPONENT_COUNT")]
  public const bits Error_CompressJPEGComponentCount;
  [CCode (cname="error_COMPRESS_JPEG_CONVERSION_NOT_IMPL")]
  public const bits Error_CompressJPEGConversionNotImpl;
  [CCode (cname="error_COMPRESS_JPEGDAC_INDEX")]
  public const bits Error_CompressJPEGDACIndex;
  [CCode (cname="error_COMPRESS_JPEGDAC_VALUE")]
  public const bits Error_CompressJPEGDACValue;
  [CCode (cname="error_COMPRESS_JPEGDHT_INDEX")]
  public const bits Error_CompressJPEGDHTIndex;
  [CCode (cname="error_COMPRESS_JPEGDQT_INDEX")]
  public const bits Error_CompressJPEGDQTIndex;
  [CCode (cname="error_COMPRESS_JPEG_EMPTY_IMAGE")]
  public const bits Error_CompressJPEGEmptyImage;
  [CCode (cname="error_COMPRESS_JPEGEOI_EXPECTED")]
  public const bits Error_CompressJPEGEOIExpected;
  [CCode (cname="error_COMPRESS_JPEG_FILE_READ")]
  public const bits Error_CompressJPEGFileRead;
  [CCode (cname="error_COMPRESS_JPEG_FILE_WRITE")]
  public const bits Error_CompressJPEGFileWrite;
  [CCode (cname="error_COMPRESS_JPEG_FRACT_SAMPLE_NOT_IMPL")]
  public const bits Error_CompressJPEGFractSampleNotImpl;
  [CCode (cname="error_COMPRESS_JPEG_HUFF_CLEN_OVERFLOW")]
  public const bits Error_CompressJPEGHuffCLenOverflow;
  [CCode (cname="error_COMPRESS_JPEG_HUFF_MISSING_CODE")]
  public const bits Error_CompressJPEGHuffMissingCode;
  [CCode (cname="error_COMPRESS_JPEG_IMAGE_TOO_BIG")]
  public const bits Error_CompressJPEGImageTooBig;
  [CCode (cname="error_COMPRESS_JPEG_INPUT_EMPTY")]
  public const bits Error_CompressJPEGInputEmpty;
  [CCode (cname="error_COMPRESS_JPEG_INPUT_EOF")]
  public const bits Error_CompressJPEGInputEOF;
  [CCode (cname="error_COMPRESS_JPEG_NOT_IMPL")]
  public const bits Error_CompressJPEGNotImpl;
  [CCode (cname="error_COMPRESS_JPEG_NOT_COMPILED")]
  public const bits Error_CompressJPEGNotCompiled;
  [CCode (cname="error_COMPRESS_JPEG_NO_BACKING_STORE")]
  public const bits Error_CompressJPEGNoBackingStore;
  [CCode (cname="error_COMPRESS_JPEG_NO_HUFF_TABLE")]
  public const bits Error_CompressJPEGNoHuffTable;
  [CCode (cname="error_COMPRESS_JPEG_NO_IMAGE")]
  public const bits Error_CompressJPEGNoImage;
  [CCode (cname="error_COMPRESS_JPEG_NO_QUANT_TABLE")]
  public const bits Error_CompressJPEGNoQuantTable;
  [CCode (cname="error_COMPRESS_JPEG_NO_SOI")]
  public const bits Error_CompressJPEGNoSOI;
  [CCode (cname="error_COMPRESS_JPEG_OUT_OF_MEMORY")]
  public const bits Error_CompressJPEGOutOfMemory;
  [CCode (cname="error_COMPRESS_JPEG_QUANT_COMPONENTS")]
  public const bits Error_CompressJPEGQuantComponents;
  [CCode (cname="error_COMPRESS_JPEG_QUANT_FEW_COLOURS")]
  public const bits Error_CompressJPEGQuantFewColours;
  [CCode (cname="error_COMPRESS_JPEG_QUANT_MANY_COLOURS")]
  public const bits Error_CompressJPEGQuantManyColours;
  [CCode (cname="error_COMPRESS_JPEGSOF_DUPLICATE")]
  public const bits Error_CompressJPEGSOFDuplicate;
  [CCode (cname="error_COMPRESS_JPEGSOF_NO_SOS")]
  public const bits Error_CompressJPEGSOFNoSOS;
  [CCode (cname="error_COMPRESS_JPEGSOF_UNSUPPORTED")]
  public const bits Error_CompressJPEGSOFUnsupported;
  [CCode (cname="error_COMPRESS_JPEGSOI_DUPLICATE")]
  public const bits Error_CompressJPEGSOIDuplicate;
  [CCode (cname="error_COMPRESS_JPEGSOS_NO_SOF")]
  public const bits Error_CompressJPEGSOSNoSOF;
  [CCode (cname="error_COMPRESS_JPEG_TOO_LITTLE_DATA")]
  public const bits Error_CompressJPEGTooLittleData;
  [CCode (cname="error_COMPRESS_JPEG_UNKNOWN_MARKER")]
  public const bits Error_CompressJPEGUnknownMarker;
  [CCode (cname="error_COMPRESS_JPEG_VIRTUAL_BUG")]
  public const bits Error_CompressJPEGVirtualBug;
  [CCode (cname="error_COMPRESS_JPEG_WIDTH_OVERFLOW")]
  public const bits Error_CompressJPEGWidthOverflow;
  [CCode (cname="error_COMPRESS_JPEG_BAD_DCT_COEF")]
  public const bits Error_CompressJPEGBadDCTCoef;
  [CCode (cname="error_COMPRESS_JPEG_BAD_HUFF_TABLE")]
  public const bits Error_CompressJPEGBadHuffTable;
  [CCode (cname="error_COMPRESS_JPEG_BAD_PROGRESSION")]
  public const bits Error_CompressJPEGBadProgression;
  [CCode (cname="error_COMPRESS_JPEG_BAD_PROG_SCRIPT")]
  public const bits Error_CompressJPEGBadProgScript;
  [CCode (cname="error_COMPRESS_JPEG_BAD_SCAN_SCRIPT")]
  public const bits Error_CompressJPEGBadScanScript;
  [CCode (cname="error_COMPRESS_JPEG_MISMATCHED_QUANT_TABLE")]
  public const bits Error_CompressJPEGMismatchedQuantTable;
  [CCode (cname="error_COMPRESS_JPEG_MISSING_DATA")]
  public const bits Error_CompressJPEGMissingData;
  [CCode (cname="error_COMPRESS_JPEG_MODE_CHANGE")]
  public const bits Error_CompressJPEGModeChange;
  [CCode (cname="error_COMPRESS_JPEGW_BUFFER_SIZE")]
  public const bits Error_CompressJPEGWBufferSize;

  [CCode (cname = "xcompressjpeg_start")]
  public static OS.Error* XStart
    (OSLib.JPEG.Image image,
     int image_size,
     Parameters parameters,
     char* workspace,
     int workspace_size,
     out Tag tag);

  [CCode (cname = "compressjpeg_start")]
  public static Tag Start
    (OSLib.JPEG.Image image,
     int image_size,
     Parameters parameters,
     char* workspace,
     int workspace_size);

  [CCode (cname = "xcompressjpeg_write_line")]
  public static OS.Error* XWriteLine
    (Tag tag,
     char* pixels);

  [CCode (cname = "compressjpeg_write_line")]
  public static void WriteLine
    (Tag tag,
     char* pixels);

  [CCode (cname = "xcompressjpeg_finish")]
  public static OS.Error* XFinish
    (Tag tag,
     out int image_size);

  [CCode (cname = "compressjpeg_finish")]
  public static int Finish
    (Tag tag);

  [CCode (cname = "xcompressjpeg_comment")]
  public static OS.Error* XComment
    (Tag tag,
     CommentFlags flags,
     char* comment,
     int comment_size);

  [CCode (cname = "compressjpeg_comment")]
  public static void Comment
    (Tag tag,
     CommentFlags flags,
     char* comment,
     int comment_size);

  [CCode (cname = "xcompressjpeg_write_line_extended")]
  public static OS.Error* XWriteLineExtended
    (Tag tag,
     WriteLineFlags flags,
     char* image_data,
     char* palette_data,
     int lines,
     int bytes_between_lines,
     out char* image_data_end,
     out int lines_written);

  [CCode (cname = "compressjpeg_write_line_extended")]
  public static char* WriteLineExtended
    (Tag tag,
     WriteLineFlags flags,
     char* image_data,
     char* palette_data,
     int lines,
     int bytes_between_lines,
     out int lines_written);

  [CCode (cname = "xcompressjpeg_transcode")]
  public static OS.Error* XTranscode
    (TranscodeFlags flags,
     char* source_data,
     int source_size,
     char* destination_data,
     int destination_size,
     char* workspace,
     int workspace_size,
     ScanDescription scans,
     out int out_destination_size,
     out int out_workspace_size);

  [CCode (cname = "compressjpeg_transcode")]
  public static void Transcode
    (TranscodeFlags flags,
     char* source_data,
     int source_size,
     char* destination_data,
     int destination_size,
     char* workspace,
     int workspace_size,
     ScanDescription scans,
     out int out_destination_size,
     out int out_workspace_size);

}
