/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/dcs.h")]
namespace OSLib.DCS
{
  [CCode (cname = "dcs_flags")]
  public enum Flags
  {
    [CCode (cname = "dcs_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "dcs_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted
  }

  [CCode (cname = "dcs_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "message")]
    public unowned Toolbox.MsgReference Message;
    [CCode (cname = "message_limit")]
    public unowned int MessageLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "dcs_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "message")]
    public unowned Toolbox.MsgReference Message;
    [CCode (cname = "message_limit")]
    public unowned int MessageLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "dcs_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "dcs_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "dcs_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "dcs_action_discard_block")]
  public enum ActionDiscardBlock
  {
    Default = 0
  }

  [CCode (cname = "dcs_action_save_block")]
  public enum ActionSaveBlock
  {
    Default = 0
  }

  [CCode (cname = "dcs_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "dcs_action_cancel_block")]
  public enum ActionCancelBlock
  {
    Default = 0
  }

  [CCode (cname="class_DCS")]
  public const Toolbox.Class Class_DCS;
  [CCode (cname="action_DCS_ABOUT_TO_BE_SHOWN")]
  public const bits Action_DCSAboutToBeShown;
  [CCode (cname="action_DCS_DISCARD")]
  public const bits Action_DCSDiscard;
  [CCode (cname="action_DCS_SAVE")]
  public const bits Action_DCSSave;
  [CCode (cname="action_DCS_DIALOGUE_COMPLETED")]
  public const bits Action_DCSDialogueCompleted;
  [CCode (cname="action_DCS_CANCEL")]
  public const bits Action_DCSCancel;
  [CCode (cname="error_DCS_ALLOC_FAILED")]
  public const bits Error_DCSAllocFailed;
  [CCode (cname="error_DCS_TASKS_ACTIVE")]
  public const bits Error_DCSTasksActive;

  [CCode (cname = "xdcs_get_window_id")]
  public static OS.Error* XGetWindowId
    (bits flags,
     Toolbox.O dcs,
     out Toolbox.O window);

  [CCode (cname = "dcs_get_window_id")]
  public static Toolbox.O GetWindowId
    (bits flags,
     Toolbox.O dcs);

  [CCode (cname = "xdcs_set_message")]
  public static OS.Error* XSetMessage
    (bits flags,
     Toolbox.O dcs,
     string message);

  [CCode (cname = "dcs_set_message")]
  public static void SetMessage
    (bits flags,
     Toolbox.O dcs,
     string message);

  [CCode (cname = "xdcs_get_message")]
  public static OS.Error* XGetMessage
    (bits flags,
     Toolbox.O dcs,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "dcs_get_message")]
  public static int GetMessage
    (bits flags,
     Toolbox.O dcs,
     string buffer,
     int size);

  [CCode (cname = "xdcs_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O dcs,
     string title);

  [CCode (cname = "dcs_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O dcs,
     string title);

  [CCode (cname = "xdcs_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O dcs,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "dcs_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O dcs,
     string buffer,
     int size);

}
