/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.Gadget;
using OSLib.Window;

[CCode (cheader_filename = "oslib/draggable.h")]
namespace OSLib.Draggable
{
  [CCode (cname = "draggable_gadget")]
  public struct Gadget
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "sprite_limit")]
    public unowned int SpriteLimit;
  }

  [CCode (cname = "draggable_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "sprite_limit")]
    public unowned int SpriteLimit;
  }

  [CCode (cname = "draggable_action_drag_ended")]
  public struct ActionDragEnded
  {
    [CCode (cname = "ids.wimp.w")]
    public unowned Wimp.W Ids_Wimp_W;
    [CCode (cname = "ids.wimp.i")]
    public unowned Wimp.I Ids_Wimp_I;
    [CCode (cname = "ids.toolbox.o")]
    public unowned Toolbox.O Ids_Toolbox_O;
    [CCode (cname = "ids.toolbox.c")]
    public unowned Toolbox.C Ids_Toolbox_C;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
  }

  [CCode (cname = "draggable_action_drag_started_block")]
  public enum ActionDragStartedBlock
  {
    Default = 0
  }

  [CCode (cname = "draggable_action_drag_ended_block")]
  public struct ActionDragEndedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "ids.wimp.w")]
    public unowned Wimp.W Ids_Wimp_W;
    [CCode (cname = "ids.wimp.i")]
    public unowned Wimp.I Ids_Wimp_I;
    [CCode (cname = "ids.toolbox.o")]
    public unowned Toolbox.O Ids_Toolbox_O;
    [CCode (cname = "ids.toolbox.c")]
    public unowned Toolbox.C Ids_Toolbox_C;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
  }

  [CCode (cname="class_DRAGGABLE")]
  public const Toolbox.Class Class_Draggable;
  [CCode (cname="draggable_GENERATE_DRAG_STARTED")]
  public const OSLib.Gadget.Flags GenerateDragStarted;
  [CCode (cname="draggable_SPRITE")]
  public const OSLib.Gadget.Flags Sprite;
  [CCode (cname="draggable_TEXT")]
  public const OSLib.Gadget.Flags Text;
  [CCode (cname="draggable_TYPE")]
  public const OSLib.Gadget.Flags Type;
  [CCode (cname="draggable_TYPE_SHIFT")]
  public const int TypeShift;
  [CCode (cname="draggable_TYPE_DRAG")]
  public const bits TypeDrag;
  [CCode (cname="draggable_TYPE_CLICK_DRAG")]
  public const bits TypeClickDrag;
  [CCode (cname="draggable_TYPE_CLICK_DOUBLE_DRAG")]
  public const bits TypeClickDoubleDrag;
  [CCode (cname="draggable_USE_TOOLBOX_IDS")]
  public const OSLib.Gadget.Flags UseToolboxIds;
  [CCode (cname="draggable_DROP_SHADOW")]
  public const OSLib.Gadget.Flags DropShadow;
  [CCode (cname="draggable_NO_DITHER")]
  public const OSLib.Gadget.Flags NoDither;
  [CCode (cname="action_DRAGGABLE_DRAG_STARTED")]
  public const bits Action_DraggableDragStarted;
  [CCode (cname="action_DRAGGABLE_DRAG_ENDED")]
  public const bits Action_DraggableDragEnded;
  [CCode (cname="draggable_DRAG_STARTED_ADJUST")]
  public const bits DragStartedAdjust;
  [CCode (cname="draggable_DRAG_STARTED_SELECT")]
  public const bits DragStartedSelect;
  [CCode (cname="draggable_DRAG_STARTED_SHIFT")]
  public const bits DragStartedShift;
  [CCode (cname="draggable_DRAG_STARTED_CTRL")]
  public const bits DragStartedCtrl;
  [CCode (cname="draggable_DRAG_ENDED_TOOLBOX_IDS")]
  public const bits DragEndedToolboxIds;

  [CCode (cname = "xdraggable_set_sprite")]
  public static OS.Error* XSetSprite
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string sprite_name);

  [CCode (cname = "draggable_set_sprite")]
  public static void SetSprite
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string sprite_name);

  [CCode (cname = "xdraggable_get_sprite")]
  public static OS.Error* XGetSprite
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "draggable_get_sprite")]
  public static int GetSprite
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string buffer,
     int size);

  [CCode (cname = "xdraggable_set_text")]
  public static OS.Error* XSetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string text);

  [CCode (cname = "draggable_set_text")]
  public static void SetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string text);

  [CCode (cname = "xdraggable_get_text")]
  public static OS.Error* XGetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "draggable_get_text")]
  public static int GetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     string buffer,
     int size);

  [CCode (cname = "xdraggable_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     int state);

  [CCode (cname = "draggable_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     int state);

  [CCode (cname = "xdraggable_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable,
     out int state);

  [CCode (cname = "draggable_get_state")]
  public static int GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C draggable);

}
