/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.Draw;
using OSLib.Font;
using OSLib.Wimp;
using OSLib.JPEG;

[CCode (cheader_filename = "oslib/drawfile.h")]
namespace OSLib.DrawFile
{
  [CCode (cname = "drawfile_type")]
  public enum Type
  {
    [CCode (cname = "drawfile_TYPE_FONT_TABLE")]
    FontTable,
    [CCode (cname = "drawfile_TYPE_TEXT")]
    Text,
    [CCode (cname = "drawfile_TYPE_PATH")]
    Path,
    [CCode (cname = "drawfile_TYPE_SPRITE")]
    Sprite,
    [CCode (cname = "drawfile_TYPE_GROUP")]
    Group,
    [CCode (cname = "drawfile_TYPE_TAGGED")]
    Tagged,
    [CCode (cname = "drawfile_TYPE_TEXT_AREA")]
    TextArea,
    [CCode (cname = "drawfile_TYPE_TEXT_COLUMN")]
    TextColumn,
    [CCode (cname = "drawfile_TYPE_OPTIONS")]
    Options,
    [CCode (cname = "drawfile_TYPE_TRFM_TEXT")]
    TrfmText,
    [CCode (cname = "drawfile_TYPE_TRFM_SPRITE")]
    TrfmSprite,
    [CCode (cname = "drawfile_TYPE_JPEG")]
    JPEG
  }

  [CCode (cname = "drawfile_path_style_flags")]
  public enum PathStyleFlags
  {
    [CCode (cname = "drawfile_PATH_MITRED")]
    PathMitred,
    [CCode (cname = "drawfile_PATH_ROUND")]
    PathRound,
    [CCode (cname = "drawfile_PATH_BEVELLED")]
    PathBevelled,
    [CCode (cname = "drawfile_PATH_BUTT")]
    PathButt,
    [CCode (cname = "drawfile_PATH_SQUARE")]
    PathSquare,
    [CCode (cname = "drawfile_PATH_TRIANGLE")]
    PathTriangle,
    [CCode (cname = "drawfile_PATH_JOIN")]
    PathJoin,
    [CCode (cname = "drawfile_PATH_END")]
    PathEnd,
    [CCode (cname = "drawfile_PATH_START")]
    PathStart,
    [CCode (cname = "drawfile_PATH_WINDING_EVEN_ODD")]
    PathWindingEvenOdd,
    [CCode (cname = "drawfile_PATH_DASHED")]
    PathDashed,
    [CCode (cname = "drawfile_PATH_CAP_WIDTH")]
    PathCapWidth,
    [CCode (cname = "drawfile_PATH_CAP_LENGTH")]
    PathCapLength
  }

  [CCode (cname = "drawfile_text_flags")]
  public enum TextFlags
  {
    [CCode (cname = "drawfile_TEXT_KERN")]
    TextKern,
    [CCode (cname = "drawfile_TEXT_RIGHT_TO_LEFT")]
    TextRightToLeft,
    [CCode (cname = "drawfile_TEXT_UNDERLINE")]
    TextUnderline
  }

  [CCode (cname = "drawfile_render_flags")]
  public enum RenderFlags
  {
    [CCode (cname = "drawfile_RENDER_BBOXES")]
    RenderBBoxes,
    [CCode (cname = "drawfile_RENDER_SUPPRESS")]
    RenderSuppress,
    [CCode (cname = "drawfile_RENDER_GIVEN_FLATNESS")]
    RenderGivenFlatness,
    [CCode (cname = "drawfile_RENDER_GIVEN_COLOUR_MAPPING")]
    RenderGivenColourMapping
  }

  [CCode (cname = "drawfile_declare_fonts_flags")]
  public enum DeclareFontsFlags
  {
    [CCode (cname = "drawfile_NO_DOWNLOAD")]
    NoDownload
  }

  [CCode (cname = "drawfile_paper_options")]
  public enum PaperOptions
  {
    [CCode (cname = "drawfile_PAPER_SHOW")]
    PaperShow,
    [CCode (cname = "drawfile_PAPER_LANDSCAPE")]
    PaperLandscape,
    [CCode (cname = "drawfile_PAPER_DEFAULT")]
    PaperDefault
  }

  [CCode (cname = "drawfile_entry_mode")]
  public enum EntryMode
  {
    [CCode (cname = "drawfile_ENTRY_MODE_LINE")]
    Line,
    [CCode (cname = "drawfile_ENTRY_MODE_CLOSED_LINE")]
    ClosedLine,
    [CCode (cname = "drawfile_ENTRY_MODE_CURVE")]
    Curve,
    [CCode (cname = "drawfile_ENTRY_MODE_CLOSED_CURVE")]
    ClosedCurve,
    [CCode (cname = "drawfile_ENTRY_MODE_RECTANGLE")]
    Rectangle,
    [CCode (cname = "drawfile_ENTRY_MODE_ELLIPSE")]
    Ellipse,
    [CCode (cname = "drawfile_ENTRY_MODE_TEXT_LINE")]
    TextLine,
    [CCode (cname = "drawfile_ENTRY_MODE_SELECT")]
    Select
  }

  [CCode (cname = "drawfile_font_def_base")]
  public struct FontDefBase
  {
    [CCode (cname = "font_index")]
    public unowned byte FontIndex;
  }

  [Compact]
  [CCode (cname = "drawfile_font_def", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class FontDef
  {
    public FontDef(int numEntries);
    [CCode (cname = "font_index")]
    public unowned byte FontIndex;
    [CCode (cname = "font_name", array_length = false)]
    public unowned char[] FontName;
  }

  [CCode (cname = "drawfile_text_style")]
  public struct TextStyle
  {
    [CCode (cname = "font_index")]
    public unowned byte FontIndex;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
  }

  [CCode (cname = "drawfile_path_style")]
  public struct PathStyle
  {
    [CCode (cname = "flags")]
    public unowned PathStyleFlags Flags;
    [CCode (cname = "reserved")]
    public unowned byte Reserved;
    [CCode (cname = "cap_width")]
    public unowned byte CapWidth;
    [CCode (cname = "cap_length")]
    public unowned byte CapLength;
  }

  [CCode (cname = "drawfile_font_table")]
  public struct FontTable
  {
    [CCode (cname = "font_def", array_length = false)]
    public unowned FontDef[] FontDef;
  }

  [CCode (cname = "drawfile_text_base")]
  public struct TextBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
    [CCode (cname = "style")]
    public unowned TextStyle Style;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "base")]
    public unowned OS.Coord Base;
  }

  [Compact]
  [CCode (cname = "drawfile_text", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Text
  {
    public Text(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
    [CCode (cname = "style")]
    public unowned TextStyle Style;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "base")]
    public unowned OS.Coord Base;
    [CCode (cname = "text", array_length = false)]
    public unowned char[] Text;
  }

  [CCode (cname = "drawfile_path")]
  public struct Path
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "outline")]
    public unowned OS.Colour Outline;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "style")]
    public unowned PathStyle Style;
    [CCode (cname = "path")]
    public unowned Draw.Path Path;
  }

  [CCode (cname = "drawfile_path_with_pattern")]
  public struct PathWithPattern
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "outline")]
    public unowned OS.Colour Outline;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "style")]
    public unowned PathStyle Style;
    [CCode (cname = "pattern")]
    public unowned Draw.DashPattern Pattern;
    [CCode (cname = "path")]
    public unowned Draw.Path Path;
  }

  [CCode (cname = "drawfile_sprite_base")]
  public struct SpriteBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "header")]
    public unowned OSSpriteOp.Header Header;
  }

  [Compact]
  [CCode (cname = "drawfile_sprite", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Sprite
  {
    public Sprite(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "header")]
    public unowned OSSpriteOp.Header Header;
    [CCode (cname = "data", array_length = false)]
    public unowned byte[] Data;
  }

  [CCode (cname = "drawfile_group_base")]
  public struct GroupBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "name")]
    public unowned char Name[12];
  }

  [Compact]
  [CCode (cname = "drawfile_group", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Group
  {
    public Group(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "name")]
    public unowned char Name[12];
    [CCode (cname = "objects", array_length = false)]
    public unowned int[] Objects;
  }

  [CCode (cname = "drawfile_tagged_base")]
  public struct TaggedBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "tag")]
    public unowned Type Tag;
  }

  [Compact]
  [CCode (cname = "drawfile_tagged", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Tagged
  {
    public Tagged(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "tag")]
    public unowned Type Tag;
    [CCode (cname = "object", array_length = false)]
    public unowned int[] Object;
  }

  [CCode (cname = "drawfile_text_column")]
  public struct TextColumn
  {
    [CCode (cname = "box")]
    public unowned OS.Box Box;
  }

  [CCode (cname = "drawfile_text_column_list")]
  public struct TextColumnList
  {
    [CCode (cname = "columns.type")]
    public unowned Type Columns_Type;
    [CCode (cname = "columns.size")]
    public unowned int Columns_Size;
    [CCode (cname = "columns.data")]
    public unowned TextColumn Columns_Data;
  }

  [CCode (cname = "drawfile_area_text_base")]
  public struct AreaTextBase
  {
    [CCode (cname = "type")]
    public unowned Type Type;
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
  }

  [Compact]
  [CCode (cname = "drawfile_area_text", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class AreaText
  {
    public AreaText(int numEntries);
    [CCode (cname = "type")]
    public unowned Type Type;
    [CCode (cname = "reserved")]
    public unowned int Reserved[2];
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
    [CCode (cname = "text", array_length = false)]
    public unowned char[] Text;
  }

  [CCode (cname = "drawfile_text_area")]
  public struct TextArea
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "header")]
    public unowned TextColumnList Header;
    [CCode (cname = "area_text")]
    public unowned AreaText AreaText;
  }

  [CCode (cname = "drawfile_options")]
  public struct Options
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "paper_size")]
    public unowned int PaperSize;
    [CCode (cname = "paper_options")]
    public unowned PaperOptions PaperOptions;
    [CCode (cname = "grid_spacing")]
    public unowned double GridSpacing;
    [CCode (cname = "grid_division")]
    public unowned int GridDivision;
    [CCode (cname = "isometric")]
    public unowned bool Isometric;
    [CCode (cname = "auto_adjust")]
    public unowned bool AutoAdjust;
    [CCode (cname = "show")]
    public unowned bool Show;
    [CCode (cname = "lock")]
    public unowned bool Lock;
    [CCode (cname = "cm")]
    public unowned bool Cm;
    [CCode (cname = "zoom_mul")]
    public unowned int ZoomMul;
    [CCode (cname = "zoom_div")]
    public unowned int ZoomDiv;
    [CCode (cname = "zoom_lock")]
    public unowned bool ZoomLock;
    [CCode (cname = "toolbox")]
    public unowned bool Toolbox;
    [CCode (cname = "entry_mode")]
    public unowned EntryMode EntryMode;
    [CCode (cname = "undo_size")]
    public unowned int UndoSize;
  }

  [CCode (cname = "drawfile_trfm_text_base")]
  public struct TrfmTextBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "flags")]
    public unowned TextFlags Flags;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
    [CCode (cname = "style")]
    public unowned TextStyle Style;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "base")]
    public unowned OS.Coord Base;
  }

  [Compact]
  [CCode (cname = "drawfile_trfm_text", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class TrfmText
  {
    public TrfmText(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "flags")]
    public unowned TextFlags Flags;
    [CCode (cname = "fill")]
    public unowned OS.Colour Fill;
    [CCode (cname = "bg_hint")]
    public unowned OS.Colour BgHint;
    [CCode (cname = "style")]
    public unowned TextStyle Style;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "base")]
    public unowned OS.Coord Base;
    [CCode (cname = "text", array_length = false)]
    public unowned char[] Text;
  }

  [CCode (cname = "drawfile_trfm_sprite_base")]
  public struct TrfmSpriteBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "header")]
    public unowned OSSpriteOp.Header Header;
  }

  [Compact]
  [CCode (cname = "drawfile_trfm_sprite", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class TrfmSprite
  {
    public TrfmSprite(int numEntries);
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "header")]
    public unowned OSSpriteOp.Header Header;
    [CCode (cname = "data", array_length = false)]
    public unowned byte[] Data;
  }

  [CCode (cname = "drawfile_jpeg")]
  public struct JPEG
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "xdpi")]
    public unowned int Xdpi;
    [CCode (cname = "ydpi")]
    public unowned int Ydpi;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "len")]
    public unowned int Len;
    [CCode (cname = "image")]
    public unowned OSLib.JPEG.Image Image;
  }

  [CCode (cname = "drawfile_object")]
  public struct Object
  {
    [CCode (cname = "type")]
    public unowned Type Type;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "data.font_table")]
    public unowned FontTable Data_FontTable;
    [CCode (cname = "data.text")]
    public unowned Text Data_Text;
    [CCode (cname = "data.path")]
    public unowned Path Data_Path;
    [CCode (cname = "data.path_with_pattern")]
    public unowned PathWithPattern Data_PathWithPattern;
    [CCode (cname = "data.sprite")]
    public unowned Sprite Data_Sprite;
    [CCode (cname = "data.group")]
    public unowned Group Data_Group;
    [CCode (cname = "data.tagged")]
    public unowned Tagged Data_Tagged;
    [CCode (cname = "data.text_column")]
    public unowned TextColumn Data_TextColumn;
    [CCode (cname = "data.text_area")]
    public unowned TextArea Data_TextArea;
    [CCode (cname = "data.options")]
    public unowned Options Data_Options;
    [CCode (cname = "data.trfm_text")]
    public unowned TrfmText Data_TrfmText;
    [CCode (cname = "data.trfm_sprite")]
    public unowned TrfmSprite Data_TrfmSprite;
    [CCode (cname = "data.jpeg")]
    public unowned JPEG Data_Jpeg;
  }

  [CCode (cname = "drawfile_diagram_base")]
  public struct DiagramBase
  {
    [CCode (cname = "tag")]
    public unowned char Tag[4];
    [CCode (cname = "major_version")]
    public unowned int MajorVersion;
    [CCode (cname = "minor_version")]
    public unowned int MinorVersion;
    [CCode (cname = "source")]
    public unowned char Source[12];
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
  }

  [Compact]
  [CCode (cname = "drawfile_diagram", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Diagram
  {
    public Diagram(int numEntries);
    [CCode (cname = "tag")]
    public unowned char Tag[4];
    [CCode (cname = "major_version")]
    public unowned int MajorVersion;
    [CCode (cname = "minor_version")]
    public unowned int MinorVersion;
    [CCode (cname = "source")]
    public unowned char Source[12];
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "objects", array_length = false)]
    public unowned Object[] Objects;
  }

  [CCode (cname = "drawfile_bbox_flags")]
  public enum BBoxFlags
  {
    Default = 0
  }

  [CCode (cname = "drawfile_render_state")]
  public struct RenderState
  {
    [CCode (cname = "object")]
    public unowned Object* Object;
    [CCode (cname = "diagram")]
    public unowned Diagram* Diagram;
    [CCode (cname = "font_table")]
    public unowned Object* FontTable;
    [CCode (cname = "flags")]
    public unowned RenderFlags Flags;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm* Trfm;
    [CCode (cname = "clip")]
    public unowned OS.Box* Clip;
    [CCode (cname = "flatness")]
    public unowned int Flatness;
    [CCode (cname = "error")]
    public unowned OS.Error* Error;
    [CCode (cname = "mapping")]
    public unowned OSSpriteOp.ColourMapping* Mapping;
  }

  [CCode (cname = "drawfile_declare_fonts_state")]
  public struct DeclareFontsState
  {
    [CCode (cname = "object")]
    public unowned Object* Object;
    [CCode (cname = "diagram")]
    public unowned Diagram* Diagram;
    [CCode (cname = "font_table")]
    public unowned Object* FontTable;
    [CCode (cname = "flags")]
    public unowned DeclareFontsFlags Flags;
    [CCode (cname = "error")]
    public unowned OS.Error* Error;
  }

  [CCode (cname="error_DRAW_FILE_NOT_DRAW")]
  public const bits Error_DrawFileNotDraw;
  [CCode (cname="error_DRAW_FILE_VERSION")]
  public const bits Error_DrawFileVersion;
  [CCode (cname="error_DRAW_FILE_FONT_TAB")]
  public const bits Error_DrawFileFontTab;
  [CCode (cname="error_DRAW_FILE_BAD_FONT_NO")]
  public const bits Error_DrawFileBadFontNo;
  [CCode (cname="error_DRAW_FILE_BAD_MODE")]
  public const bits Error_DrawFileBadMode;
  [CCode (cname="error_DRAW_FILE_BAD_FILE")]
  public const bits Error_DrawFileBadFile;
  [CCode (cname="error_DRAW_FILE_BAD_GROUP")]
  public const bits Error_DrawFileBadGroup;
  [CCode (cname="error_DRAW_FILE_BAD_TAG")]
  public const bits Error_DrawFileBadTag;
  [CCode (cname="error_DRAW_FILE_SYNTAX")]
  public const bits Error_DrawFileSyntax;
  [CCode (cname="error_DRAW_FILE_FONT_NO")]
  public const bits Error_DrawFileFontNo;
  [CCode (cname="error_DRAW_FILE_AREA_VER")]
  public const bits Error_DrawFileAreaVer;
  [CCode (cname="error_DRAW_FILE_NO_AREA_VER")]
  public const bits Error_DrawFileNoAreaVer;
  [CCode (cname="drawfile_PATH_JOIN_SHIFT")]
  public const int PathJoinShift;
  [CCode (cname="drawfile_PATH_END_SHIFT")]
  public const int PathEndShift;
  [CCode (cname="drawfile_PATH_START_SHIFT")]
  public const int PathStartShift;
  [CCode (cname="drawfile_PATH_CAP_WIDTH_SHIFT")]
  public const int PathCapWidthShift;
  [CCode (cname="drawfile_PATH_CAP_LENGTH_SHIFT")]
  public const int PathCapLengthShift;

  [CCode (cname = "xdrawfile_render")]
  public static OS.Error* XRender
    (RenderFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box clip,
     int flatness);

  [CCode (cname = "drawfile_render")]
  public static void Render
    (RenderFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box clip,
     int flatness);

  [CCode (cname = "xdrawfile_render_mapped")]
  public static OS.Error* XRenderMapped
    (RenderFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box clip,
     int flatness,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "drawfile_render_mapped")]
  public static void RenderMapped
    (RenderFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box clip,
     int flatness,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xdrawfile_bbox")]
  public static OS.Error* XBBox
    (BBoxFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box bbox);

  [CCode (cname = "drawfile_bbox")]
  public static void BBox
    (BBoxFlags flags,
     Diagram diagram,
     int size,
     OS.Trfm trfm,
     OS.Box bbox);

  [CCode (cname = "xdrawfile_declare_fonts")]
  public static OS.Error* XDeclareFonts
    (DeclareFontsFlags flags,
     Diagram diagram,
     int size);

  [CCode (cname = "drawfile_declare_fonts")]
  public static void DeclareFonts
    (DeclareFontsFlags flags,
     Diagram diagram,
     int size);

  [CCode (cname = "xservice_draw_object_render")]
  public static OS.Error* XService_DrawObjectRender
    (Type type,
     RenderState state,
     out bool unclaimed);

  [CCode (cname = "service_draw_object_render")]
  public static bool Service_DrawObjectRender
    (Type type,
     RenderState state);

  [CCode (cname = "xservice_draw_object_declare_fonts")]
  public static OS.Error* XService_DrawObjectDeclareFonts
    (Type type,
     DeclareFontsState state,
     out bool unclaimed);

  [CCode (cname = "service_draw_object_declare_fonts")]
  public static bool Service_DrawObjectDeclareFonts
    (Type type,
     DeclareFontsState state);

}
