/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/econet.h")]
namespace OSLib.Econet
{
  [CCode (cname = "econet_status")]
  public enum Status
  {
    [CCode (cname = "econet_STATUS_TRANSMITTED")]
    Transmitted,
    [CCode (cname = "econet_STATUS_LINE_JAMMED")]
    LineJammed,
    [CCode (cname = "econet_STATUS_NET_ERROR")]
    NetError,
    [CCode (cname = "econet_STATUS_NOT_LISTENING")]
    NotListening,
    [CCode (cname = "econet_STATUS_NO_CLOCK")]
    NoClock,
    [CCode (cname = "econet_STATUS_TX_READY")]
    TxReady,
    [CCode (cname = "econet_STATUS_TRANSMITTING")]
    Transmitting,
    [CCode (cname = "econet_STATUS_RX_READY")]
    RxReady,
    [CCode (cname = "econet_STATUS_RECEIVING")]
    Receiving,
    [CCode (cname = "econet_STATUS_RECEIVED")]
    Received,
    [CCode (cname = "econet_STATUS_NO_REPLY")]
    NoReply,
    [CCode (cname = "econet_STATUS_ESCAPE")]
    Escape,
    [CCode (cname = "econet_STATUS_NOT_PRESENT")]
    NotPresent
  }

  [CCode (cname = "econet_immediate_op")]
  public enum ImmediateOp
  {
    [CCode (cname = "econet_IMMEDIATE_PEEK")]
    ImmediatePeek,
    [CCode (cname = "econet_IMMEDIATE_POKE")]
    ImmediatePoke,
    [CCode (cname = "econet_IMMEDIATE_JSR")]
    ImmediateJSR,
    [CCode (cname = "econet_IMMEDIATE_USER_PROCEDURE_CALL")]
    ImmediateUserProcedureCall,
    [CCode (cname = "econet_IMMEDIATE_OS_PROCEDURE_CALL")]
    ImmediateOSProcedureCall,
    [CCode (cname = "econet_IMMEDIATE_HALT")]
    ImmediateHalt,
    [CCode (cname = "econet_IMMEDIATE_CONTINUE")]
    ImmediateContinue,
    [CCode (cname = "econet_IMMEDIATE_MACHINE_PEEK")]
    ImmediateMachinePeek,
    [CCode (cname = "econet_IMMEDIATE_GET_REGISTERS")]
    ImmediateGetRegisters
  }

  [CCode (cname = "econet_immediate_os_procedure")]
  public enum ImmediateOSProcedure
  {
    [CCode (cname = "econet_OS_CHAR_FROM_NOTIFY")]
    OSCharFromNotify,
    [CCode (cname = "econet_OS_INITIALISE_REMOTE")]
    OSInitialiseRemote,
    [CCode (cname = "econet_OS_GET_VIEW_PARAMETERS")]
    OSGetViewParameters,
    [CCode (cname = "econet_OS_CAUSE_FATAL_ERROR")]
    OSCauseFatalError,
    [CCode (cname = "econet_OS_CHAR_FROM_REMOTE")]
    OSCharFromRemote
  }

  [CCode (cname = "econet_protection")]
  public enum Protection
  {
    [CCode (cname = "econet_PROTECTION_PEEK")]
    Peek,
    [CCode (cname = "econet_PROTECTION_POKE")]
    Poke,
    [CCode (cname = "econet_PROTECTION_JSR")]
    JSR,
    [CCode (cname = "econet_PROTECTION_USER_PROCEDURE_CALL")]
    UserProcedureCall,
    [CCode (cname = "econet_PROTECTION_OS_PROCEDURE_CALL")]
    OSProcedureCall,
    [CCode (cname = "econet_PROTECTION_HALT")]
    Halt,
    [CCode (cname = "econet_PROTECTION_CONTINUE")]
    Continue,
    [CCode (cname = "econet_PROTECTION_MACHINE_PEEK")]
    MachinePeek,
    [CCode (cname = "econet_PROTECTION_GET_REGISTERS")]
    GetRegisters,
    [CCode (cname = "econet_PROTECTION_WRITE_CMOS")]
    WriteCMOS
  }

  [CCode (cname = "econet_transport_type")]
  public enum TransportType
  {
    [CCode (cname = "econet_TRANSPORT_NOT_KNOWN")]
    TransportNotKnown,
    [CCode (cname = "econet_TRANSPORT_INTERNET")]
    TransportInternet,
    [CCode (cname = "econet_TRANSPORT_ECONET")]
    TransportEconet,
    [CCode (cname = "econet_TRANSPORT_NEXUS")]
    TransportNexus
  }

  [CCode (cname = "econet_state")]
  public enum State
  {
    [CCode (cname = "econet_STATE_OK")]
    OK,
    [CCode (cname = "econet_STATE_NO_CLOCK")]
    NoClock
  }

  [CCode (cname = "econet_rx_cb", has_type_id = false)]
  public struct RxCB : bits
  {
  }

  [CCode (cname = "econet_tx_cb", has_type_id = false)]
  public struct TxCB : bits
  {
  }

  [CCode (cname = "econet_action")]
  public enum Action
  {
    [CCode (cname = "econet_NET_FS_START_LOAD")]
    NetFSStartLoad,
    [CCode (cname = "econet_NET_FS_PART_LOAD")]
    NetFSPartLoad,
    [CCode (cname = "econet_NET_FS_FINISH_LOAD")]
    NetFSFinishLoad,
    [CCode (cname = "econet_NET_FS_START_SAVE")]
    NetFSStartSave,
    [CCode (cname = "econet_NET_FS_PART_SAVE")]
    NetFSPartSave,
    [CCode (cname = "econet_NET_FS_FINISH_SAVE")]
    NetFSFinishSave,
    [CCode (cname = "econet_NET_FS_START_CREATE")]
    NetFSStartCreate,
    [CCode (cname = "econet_NET_FS_PART_CREATE")]
    NetFSPartCreate,
    [CCode (cname = "econet_NET_FS_FINISH_CREATE")]
    NetFSFinishCreate,
    [CCode (cname = "econet_NET_FS_START_GET_BYTES")]
    NetFSStartGetBytes,
    [CCode (cname = "econet_NET_FS_PART_GET_BYTES")]
    NetFSPartGetBytes,
    [CCode (cname = "econet_NET_FS_FINISH_GET_BYTES")]
    NetFSFinishGetBytes,
    [CCode (cname = "econet_NET_FS_START_PUT_BYTES")]
    NetFSStartPutBytes,
    [CCode (cname = "econet_NET_FS_PART_PUT_BYTES")]
    NetFSPartPutBytes,
    [CCode (cname = "econet_NET_FS_FINISH_PUT_BYTES")]
    NetFSFinishPutBytes,
    [CCode (cname = "econet_NET_FS_START_WAIT")]
    NetFSStartWait,
    [CCode (cname = "econet_NET_FS_FINISH_WAIT")]
    NetFSFinishWait,
    [CCode (cname = "econet_NET_FS_START_BROADCAST_LOAD")]
    NetFSStartBroadcastLoad,
    [CCode (cname = "econet_NET_FS_PART_BROADCAST_LOAD")]
    NetFSPartBroadcastLoad,
    [CCode (cname = "econet_NET_FS_FINISH_BROADCAST_LOAD")]
    NetFSFinishBroadcastLoad,
    [CCode (cname = "econet_NET_FS_START_BROADCAST_SAVE")]
    NetFSStartBroadcastSave,
    [CCode (cname = "econet_NET_FS_PART_BROADCAST_SAVE")]
    NetFSPartBroadcastSave,
    [CCode (cname = "econet_NET_FS_FINISH_BROADCAST_SAVE")]
    NetFSFinishBroadcastSave,
    [CCode (cname = "econet_ECONET_START_TRANSMISSION")]
    EconetStartTransmission,
    [CCode (cname = "econet_ECONET_FINISH_TRANSMISSION")]
    EconetFinishTransmission,
    [CCode (cname = "econet_ECONET_START_RECEPTION")]
    EconetStartReception,
    [CCode (cname = "econet_ECONET_FINISH_RECEPTION")]
    EconetFinishReception
  }

  [CCode (cname = "econet_map_flags")]
  public enum MapFlags
  {
    Default = 0
  }

  [CCode (cname="error_ECONET_TX_READY")]
  public const bits Error_EconetTxReady;
  [CCode (cname="error_ECONET_TRANSMITTING")]
  public const bits Error_EconetTransmitting;
  [CCode (cname="error_ECONET_RX_READY")]
  public const bits Error_EconetRxReady;
  [CCode (cname="error_ECONET_RECEIVING")]
  public const bits Error_EconetReceiving;
  [CCode (cname="error_ECONET_RECEIVED")]
  public const bits Error_EconetReceived;
  [CCode (cname="error_ECONET_TRANSMITTED")]
  public const bits Error_EconetTransmitted;
  [CCode (cname="error_ECONET_BAD_STATION")]
  public const bits Error_EconetBadStation;
  [CCode (cname="error_ECONET_BAD_NETWORK")]
  public const bits Error_EconetBadNetwork;
  [CCode (cname="error_ECONET_UNABLE_TO_DEFAULT")]
  public const bits Error_EconetUnableToDefault;
  [CCode (cname="error_ECONET_BAD_PORT")]
  public const bits Error_EconetBadPort;
  [CCode (cname="error_ECONET_BAD_CONTROL")]
  public const bits Error_EconetBadControl;
  [CCode (cname="error_ECONET_BAD_BUFFER")]
  public const bits Error_EconetBadBuffer;
  [CCode (cname="error_ECONET_BAD_SIZE")]
  public const bits Error_EconetBadSize;
  [CCode (cname="error_ECONET_BAD_MASK")]
  public const bits Error_EconetBadMask;
  [CCode (cname="error_ECONET_BAD_COUNT")]
  public const bits Error_EconetBadCount;
  [CCode (cname="error_ECONET_BAD_DELAY")]
  public const bits Error_EconetBadDelay;
  [CCode (cname="error_ECONET_BAD_STATUS")]
  public const bits Error_EconetBadStatus;
  [CCode (cname="error_ECONET_NO_HARDWARE")]
  public const bits Error_EconetNoHardware;
  [CCode (cname="error_ECONET_NO_ECONET")]
  public const bits Error_EconetNoEconet;
  [CCode (cname="error_ECONET_NO_MORE_DOMAINS")]
  public const bits Error_EconetNoMoreDomains;
  [CCode (cname="error_ECONET_BAD_DOMAIN")]
  public const bits Error_EconetBadDomain;
  [CCode (cname="error_ECONET_UN_REGISTERED_DOMAIN")]
  public const bits Error_EconetUnRegisteredDomain;
  [CCode (cname="error_ECONET_PORT_NOT_ALLOCATED")]
  public const bits Error_EconetPortNotAllocated;
  [CCode (cname="error_ECONET_PORT_ALLOCATED")]
  public const bits Error_EconetPortAllocated;
  [CCode (cname="error_ECONET_NO_MORE_PORTS")]
  public const bits Error_EconetNoMorePorts;
  [CCode (cname="econet_SECOND")]
  public const int Second;
  [CCode (cname="econet_KHZ")]
  public const int kHz;

  [CCode (cname = "xeconet_create_receive")]
  public static OS.Error* XCreateReceive
    (byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     out RxCB rx_cb,
     out bool nonlocal);

  [CCode (cname = "econet_create_receive")]
  public static RxCB CreateReceive
    (byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     out bool nonlocal);

  [CCode (cname = "xeconet_examine_receive")]
  public static OS.Error* XExamineReceive
    (RxCB rx_cb,
     out Status status);

  [CCode (cname = "econet_examine_receive")]
  public static Status ExamineReceive
    (RxCB rx_cb);

  [CCode (cname = "xeconet_read_receive")]
  public static OS.Error* XReadReceive
    (RxCB rx_cb,
     out Status status,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "econet_read_receive")]
  public static Status ReadReceive
    (RxCB rx_cb,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "xeconet_abandon_receive")]
  public static OS.Error* XAbandonReceive
    (RxCB rx_cb,
     out Status status);

  [CCode (cname = "econet_abandon_receive")]
  public static Status AbandonReceive
    (RxCB rx_cb);

  [CCode (cname = "xeconet_wait_for_reception")]
  public static OS.Error* XWaitForReception
    (RxCB rx_cb,
     int delay,
     bool abort_on_escape,
     out Status status,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "econet_wait_for_reception")]
  public static Status WaitForReception
    (RxCB rx_cb,
     int delay,
     bool abort_on_escape,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "xeconet_enumerate_receive")]
  public static OS.Error* XEnumerateReceive
    (int rx_cb_no,
     out RxCB rx_cb);

  [CCode (cname = "econet_enumerate_receive")]
  public static RxCB EnumerateReceive
    (int rx_cb_no);

  [CCode (cname = "xeconet_start_transmit")]
  public static OS.Error* XStartTransmit
    (byte flags,
     byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out TxCB tx_cb,
     out bool nonlocal);

  [CCode (cname = "econet_start_transmit")]
  public static TxCB StartTransmit
    (byte flags,
     byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out bool nonlocal);

  [CCode (cname = "xeconet_poll_transmit")]
  public static OS.Error* XPollTransmit
    (TxCB tx_cb,
     out Status status);

  [CCode (cname = "econet_poll_transmit")]
  public static Status PollTransmit
    (TxCB tx_cb);

  [CCode (cname = "xeconet_abandon_transmit")]
  public static OS.Error* XAbandonTransmit
    (TxCB tx_cb,
     out Status status);

  [CCode (cname = "econet_abandon_transmit")]
  public static Status AbandonTransmit
    (TxCB tx_cb);

  [CCode (cname = "xeconet_do_transmit")]
  public static OS.Error* XDoTransmit
    (byte flags,
     byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out Status status,
     out bool nonlocal);

  [CCode (cname = "econet_do_transmit")]
  public static Status DoTransmit
    (byte flags,
     byte port,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out bool nonlocal);

  [CCode (cname = "xeconet_read_local_station_and_net")]
  public static OS.Error* XReadLocalStationAndNet
    (out byte station,
     out byte net);

  [CCode (cname = "econet_read_local_station_and_net")]
  public static void ReadLocalStationAndNet
    (out byte station,
     out byte net);

  [CCode (cname = "xeconet_convert_status_to_string")]
  public static OS.Error* XConvertStatusToString
    (Status status,
     string buffer,
     int size,
     byte station,
     byte net,
     out char* end);

  [CCode (cname = "econet_convert_status_to_string")]
  public static char ConvertStatusToString
    (Status status,
     string buffer,
     int size,
     byte station,
     byte net);

  [CCode (cname = "xeconet_convert_status_to_error")]
  public static OS.Error* XConvertStatusToError
    (Status status,
     OS.Error buffer,
     int size,
     byte station,
     byte net);

  [CCode (cname = "econet_convert_status_to_error")]
  public static void ConvertStatusToError
    (Status status,
     OS.Error buffer,
     int size,
     byte station,
     byte net);

  [CCode (cname = "xeconet_read_protection")]
  public static OS.Error* XReadProtection
    (out Protection protection);

  [CCode (cname = "econet_read_protection")]
  public static Protection ReadProtection ();

  [CCode (cname = "xeconet_set_protection")]
  public static OS.Error* XSetProtection
    (Protection eor_mask,
     Protection and_mask,
     out Protection old_protection);

  [CCode (cname = "econet_set_protection")]
  public static Protection SetProtection
    (Protection eor_mask,
     Protection and_mask);

  [CCode (cname = "xeconet_read_station_number")]
  public static OS.Error* XReadStationNumber
    (string s,
     out char* end,
     out int station,
     out int net);

  [CCode (cname = "econet_read_station_number")]
  public static char ReadStationNumber
    (string s,
     out int station,
     out int net);

  [CCode (cname = "xeconet_print_banner")]
  public static OS.Error* XPrintBanner ();

  [CCode (cname = "econet_print_banner")]
  public static void PrintBanner ();

  [CCode (cname = "xeconet_read_transport_type")]
  public static OS.Error* XReadTransportType
    (byte station,
     byte net,
     out TransportType transport_type);

  [CCode (cname = "econet_read_transport_type")]
  public static TransportType ReadTransportType
    (byte station,
     byte net);

  [CCode (cname = "xeconet_release_port")]
  public static OS.Error* XReleasePort
    (byte port);

  [CCode (cname = "econet_release_port")]
  public static void ReleasePort
    (byte port);

  [CCode (cname = "xeconet_allocate_port")]
  public static OS.Error* XAllocatePort
    (out byte port);

  [CCode (cname = "econet_allocate_port")]
  public static byte AllocatePort ();

  [CCode (cname = "xeconet_de_allocate_port")]
  public static OS.Error* XDeAllocatePort
    (byte port);

  [CCode (cname = "econet_de_allocate_port")]
  public static void DeAllocatePort
    (byte port);

  [CCode (cname = "xeconet_claim_port")]
  public static OS.Error* XClaimPort
    (byte port);

  [CCode (cname = "econet_claim_port")]
  public static void ClaimPort
    (byte port);

  [CCode (cname = "xeconet_start_immediate")]
  public static OS.Error* XStartImmediate
    (ImmediateOp op_type,
     ImmediateOSProcedure procedure,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out TxCB tx_cb,
     out bool nonlocal);

  [CCode (cname = "econet_start_immediate")]
  public static TxCB StartImmediate
    (ImmediateOp op_type,
     ImmediateOSProcedure procedure,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out bool nonlocal);

  [CCode (cname = "xeconet_do_immediate")]
  public static OS.Error* XDoImmediate
    (ImmediateOp op_type,
     ImmediateOSProcedure procedure,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out Status status,
     out bool nonlocal);

  [CCode (cname = "econet_do_immediate")]
  public static Status DoImmediate
    (ImmediateOp op_type,
     ImmediateOSProcedure procedure,
     byte station,
     byte net,
     char* buffer,
     int size,
     int count,
     int delay,
     out bool nonlocal);

  [CCode (cname = "xeconet_abandon_and_read_receive")]
  public static OS.Error* XAbandonAndReadReceive
    (RxCB rx_cb,
     out Status status,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "econet_abandon_and_read_receive")]
  public static Status AbandonAndReadReceive
    (RxCB rx_cb,
     out byte flag,
     out byte port,
     out byte station,
     out byte net,
     out char* buffer,
     out int size);

  [CCode (cname = "xeconet_version")]
  public static OS.Error* XVersion
    (byte station,
     byte net,
     out int version);

  [CCode (cname = "econet_version")]
  public static int Version
    (byte station,
     byte net);

  [CCode (cname = "xeconet_network_state")]
  public static OS.Error* XNetworkState
    (byte station,
     byte net,
     out State state);

  [CCode (cname = "econet_network_state")]
  public static State NetworkState
    (byte station,
     byte net);

  [CCode (cname = "xeconet_packet_size")]
  public static OS.Error* XPacketSize
    (byte station,
     byte net,
     out int packet_limit);

  [CCode (cname = "econet_packet_size")]
  public static int PacketSize
    (byte station,
     byte net);

  [CCode (cname = "xeconet_read_transport_name")]
  public static OS.Error* XReadTransportName
    (byte station,
     byte net,
     out string transport_name);

  [CCode (cname = "econet_read_transport_name")]
  public static string ReadTransportName
    (byte station,
     byte net);

  [CCode (cname = "xeconet_enumerate_map")]
  public static OS.Error* XEnumerateMap
    (MapFlags flags,
     int context,
     out byte net,
     out string net_name,
     out int ip_subnetwork_address,
     out int context_out);

  [CCode (cname = "econet_enumerate_map")]
  public static int EnumerateMap
    (MapFlags flags,
     int context,
     out byte net,
     out string net_name,
     out int ip_subnetwork_address);

  [CCode (cname = "xeconet_enumerate_transmit")]
  public static OS.Error* XEnumerateTransmit
    (int tx_cb_no,
     out TxCB tx_cb);

  [CCode (cname = "econet_enumerate_transmit")]
  public static TxCB EnumerateTransmit
    (int tx_cb_no);

  [CCode (cname = "xeconet_hardware_addresses")]
  public static OS.Error* XHardwareAddresses
    (out char* hardware_address,
     out char* register_address,
     out bits register_mask);

  [CCode (cname = "econet_hardware_addresses")]
  public static void HardwareAddresses
    (out char* hardware_address,
     out char* register_address,
     out bits register_mask);

  [CCode (cname = "xeconet_network_parameters")]
  public static OS.Error* XNetworkParameters
    (out int period,
     out int frequency);

  [CCode (cname = "econet_network_parameters")]
  public static void NetworkParameters
    (out int period,
     out int frequency);

  [CCode (cname = "xeconetv")]
  public static OS.Error* XEconetV
    (Action action,
     int count);

  [CCode (cname = "econetv")]
  public static void EconetV
    (Action action,
     int count);

  [CCode (cname = "xservice_reallocate_ports")]
  public static OS.Error* XService_ReallocatePorts ();

  [CCode (cname = "service_reallocate_ports")]
  public static void Service_ReallocatePorts ();

  [CCode (cname = "xservice_econet_dying")]
  public static OS.Error* XService_EconetDying ();

  [CCode (cname = "service_econet_dying")]
  public static void Service_EconetDying ();

  [CCode (cname = "xservice_protocol_dying")]
  public static OS.Error* XService_ProtocolDying ();

  [CCode (cname = "service_protocol_dying")]
  public static void Service_ProtocolDying ();

  [CCode (cname = "xservice_find_network_driver")]
  public static OS.Error* XService_FindNetworkDriver ();

  [CCode (cname = "service_find_network_driver")]
  public static void Service_FindNetworkDriver ();

  [CCode (cname = "xservice_network_driver_status")]
  public static OS.Error* XService_NetworkDriverStatus ();

  [CCode (cname = "service_network_driver_status")]
  public static void Service_NetworkDriverStatus ();

  [CCode (cname = "xevent_econet_user_rpc")]
  public static OS.Error* XEvent_EconetUserRPC
    (char* args,
     ImmediateOSProcedure procedure,
     byte station,
     byte net);

  [CCode (cname = "event_econet_user_rpc")]
  public static void Event_EconetUserRPC
    (char* args,
     ImmediateOSProcedure procedure,
     byte station,
     byte net);

  [CCode (cname = "xevent_econet_rx")]
  public static OS.Error* XEvent_EconetRx
    (RxCB rx_cb,
     Status status);

  [CCode (cname = "event_econet_rx")]
  public static void Event_EconetRx
    (RxCB rx_cb,
     Status status);

  [CCode (cname = "xevent_econet_tx")]
  public static OS.Error* XEvent_EconetTx
    (TxCB tx_cb,
     Status status);

  [CCode (cname = "event_econet_tx")]
  public static void Event_EconetTx
    (TxCB tx_cb,
     Status status);

  [CCode (cname = "xevent_econet_os_proc")]
  public static OS.Error* XEvent_EconetOSProc
    (char* args,
     ImmediateOSProcedure procedure,
     byte station,
     byte net);

  [CCode (cname = "event_econet_os_proc")]
  public static void Event_EconetOSProc
    (char* args,
     ImmediateOSProcedure procedure,
     byte station,
     byte net);

}
