/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/fpemulator.h")]
namespace OSLib.FPEmulator
{
  [CCode (cname = "fpemulator_context", has_type_id = false)]
  public struct Context : bits
  {
  }

  [CCode (cname = "fpemulator_fpsr")]
  public enum FPSR
  {
    [CCode (cname = "fpemulator_FPSR_CUMULATIVE_IVO")]
    CumulativeIVO,
    [CCode (cname = "fpemulator_FPSR_CUMULATIVE_DVZ")]
    CumulativeDVZ,
    [CCode (cname = "fpemulator_FPSR_CUMULATIVE_UFL")]
    CumulativeUFL,
    [CCode (cname = "fpemulator_FPSR_CUMULATIVE_INX")]
    CumulativeINX,
    [CCode (cname = "fpemulator_FPSR_CONTROL_ND")]
    ControlND,
    [CCode (cname = "fpemulator_FPSR_CONTROL_NE")]
    ControlNE,
    [CCode (cname = "fpemulator_FPSR_CONTROL_SO")]
    ControlSO,
    [CCode (cname = "fpemulator_FPSR_CONTROL_EP")]
    ControlEP,
    [CCode (cname = "fpemulator_FPSR_CONTROL_AC")]
    ControlAC,
    [CCode (cname = "fpemulator_FPSR_ENABLE_IVO")]
    EnableIVO,
    [CCode (cname = "fpemulator_FPSR_ENABLE_DVZ")]
    EnableDVZ,
    [CCode (cname = "fpemulator_FPSR_ENABLE_OFL")]
    EnableOFL,
    [CCode (cname = "fpemulator_FPSR_ENABLE_UFL")]
    EnableUFL,
    [CCode (cname = "fpemulator_FPSR_ENABLE_INX")]
    EnableINX,
    [CCode (cname = "fpemulator_FPSR_SYSTEM_ID")]
    SystemID,
    [CCode (cname = "fpemulator_FPSR_SYSTEM_CLASS")]
    SystemClass
  }

  [CCode (cname = "fpemulator_register_dump")]
  public struct RegisterDump
  {
    [CCode (cname = "fpsr")]
    public unowned FPSR Fpsr;
    [CCode (cname = "f0")]
    public unowned byte F0[12];
    [CCode (cname = "f1")]
    public unowned byte F1[12];
    [CCode (cname = "f2")]
    public unowned byte F2[12];
    [CCode (cname = "f3")]
    public unowned byte F3[12];
    [CCode (cname = "f4")]
    public unowned byte F4[12];
    [CCode (cname = "f5")]
    public unowned byte F5[12];
    [CCode (cname = "f6")]
    public unowned byte F6[12];
    [CCode (cname = "f7")]
    public unowned byte F7[12];
  }

  [CCode (cname="fpemulator_FPSR_SYSTEM_ID_SHIFT")]
  public const int FPSRSystemIDShift;
  [CCode (cname="fpemulator_FPSR_SYSTEM_CLASS_SHIFT")]
  public const int FPSRSystemClassShift;
  [CCode (cname="fpemulator_SYSTEM_OLD_FPE")]
  public const int SystemOldFPE;
  [CCode (cname="fpemulator_SYSTEM_FPE400")]
  public const int SystemFPE400;
  [CCode (cname="fpemulator_SYSTEM_FPPC")]
  public const int SystemFPPC;
  [CCode (cname="fpemulator_SYSTEM_FPA")]
  public const int SystemFPA;
  [CCode (cname="fpemulator_CLASS_SOFTWARE")]
  public const int ClassSoftware;
  [CCode (cname="fpemulator_CLASS_HARDWARE")]
  public const int ClassHardware;

  [CCode (cname = "xfpemulator_version")]
  public static OS.Error* XVersion
    (out int version);

  [CCode (cname = "fpemulator_version")]
  public static int Version ();

  [CCode (cname = "xfpemulator_deactivate_context")]
  public static OS.Error* XDeactivateContext
    (out Context old_context);

  [CCode (cname = "fpemulator_deactivate_context")]
  public static Context DeactivateContext ();

  [CCode (cname = "xfpemulator_activate_context")]
  public static OS.Error* XActivateContext
    (Context new_context);

  [CCode (cname = "fpemulator_activate_context")]
  public static void ActivateContext
    (Context new_context);

  [CCode (cname = "xfpemulator_changed_context")]
  public static OS.Error* XChangedContext
    (Context new_context,
     out Context old_context);

  [CCode (cname = "fpemulator_changed_context")]
  public static Context ChangedContext
    (Context new_context);

  [CCode (cname = "xfpemulator_context_length")]
  public static OS.Error* XContextLength
    (out int context_length);

  [CCode (cname = "fpemulator_context_length")]
  public static int ContextLength ();

  [CCode (cname = "xfpemulator_init_context")]
  public static OS.Error* XInitContext
    (Context context);

  [CCode (cname = "fpemulator_init_context")]
  public static void InitContext
    (Context context);

  [CCode (cname = "xfpemulator_exception_dump")]
  public static OS.Error* XExceptionDump
    (out RegisterDump register_dump);

  [CCode (cname = "fpemulator_exception_dump")]
  public static RegisterDump ExceptionDump ();

  [CCode (cname = "xfpemulator_abort")]
  public static OS.Error* XAbort
    (Context context,
     int r12_value,
     int pc_value,
     out char* frame);

  [CCode (cname = "fpemulator_abort")]
  public static char* Abort
    (Context context,
     int r12_value,
     int pc_value);

  [CCode (cname = "xfpemulator_load_context")]
  public static OS.Error* XLoadContext
    (Context context,
     RegisterDump register_dump);

  [CCode (cname = "fpemulator_load_context")]
  public static void LoadContext
    (Context context,
     RegisterDump register_dump);

  [CCode (cname = "xfpemulator_save_context")]
  public static OS.Error* XSaveContext
    (Context context,
     RegisterDump register_dump,
     int pc_value,
     out bool has_exception);

  [CCode (cname = "fpemulator_save_context")]
  public static bool SaveContext
    (Context context,
     RegisterDump register_dump,
     int pc_value);

}
