/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/fileswitch.h")]
namespace OSLib.FileSwitch
{
  [CCode (cname = "fileswitch_fs_no")]
  public enum FSNo
  {
    [CCode (cname = "fileswitch_FS_NUMBER_NONE")]
    FSNumberNone,
    [CCode (cname = "fileswitch_FS_NUMBER_ROMFS")]
    FSNumberROMFS,
    [CCode (cname = "fileswitch_FS_NUMBER_NETFS")]
    FSNumberNETFS,
    [CCode (cname = "fileswitch_FS_NUMBER_ADFS")]
    FSNumberADFS,
    [CCode (cname = "fileswitch_FS_NUMBER_NETPRINT")]
    FSNumberNETPRINT,
    [CCode (cname = "fileswitch_FS_NUMBER_NULL")]
    FSNumberNull,
    [CCode (cname = "fileswitch_FS_NUMBER_PRINTER")]
    FSNumberPrinter,
    [CCode (cname = "fileswitch_FS_NUMBER_SERIAL")]
    FSNumberSerial,
    [CCode (cname = "fileswitch_FS_NUMBER_VDU")]
    FSNumberVDU,
    [CCode (cname = "fileswitch_FS_NUMBER_RAWVDU")]
    FSNumberRAWVDU,
    [CCode (cname = "fileswitch_FS_NUMBER_KBD")]
    FSNumberKBD,
    [CCode (cname = "fileswitch_FS_NUMBER_RAWKBD")]
    FSNumberRAWKBD,
    [CCode (cname = "fileswitch_FS_NUMBER_DESKFS")]
    FSNumberDESKFS,
    [CCode (cname = "fileswitch_FS_NUMBER_RAMFS")]
    FSNumberRAMFS,
    [CCode (cname = "fileswitch_FS_NUMBER_RISCIXFS")]
    FSNumberRISCIXFS,
    [CCode (cname = "fileswitch_FS_NUMBER_STREAMER")]
    FSNumberStreamer,
    [CCode (cname = "fileswitch_FS_NUMBER_SCSIFS")]
    FSNumberSCSIFS,
    [CCode (cname = "fileswitch_FS_NUMBER_DIGITISER")]
    FSNumberDigitiser,
    [CCode (cname = "fileswitch_FS_NUMBER_SCANNER")]
    FSNumberScanner,
    [CCode (cname = "fileswitch_FS_NUMBER_MULTIFS")]
    FSNumberMULTIFS,
    [CCode (cname = "fileswitch_FS_NUMBER_NFS")]
    FSNumberNFS,
    [CCode (cname = "fileswitch_FS_NUMBER_CDFS")]
    FSNumberCDFS,
    [CCode (cname = "fileswitch_FS_NUMBER_DOSFS")]
    FSNumberDOSFS,
    [CCode (cname = "fileswitch_FS_NUMBER_RESOURCEFS")]
    FSNumberRESOURCEFS,
    [CCode (cname = "fileswitch_FS_NUMBER_PIPEFS")]
    FSNumberPIPEFS,
    [CCode (cname = "fileswitch_FS_NUMBER_DEVICEFS")]
    FSNumberDEVICEFS,
    [CCode (cname = "fileswitch_FS_NUMBER_PARALLEL")]
    FSNumberParallel,
    [CCode (cname = "fileswitch_FS_NUMBER_SPARKFS")]
    FSNumberSPARKFS,
    [CCode (cname = "fileswitch_FS_NUMBER_PCCARDFS")]
    FSNumberPCCARDFS,
    [CCode (cname = "fileswitch_FS_NUMBER_MEMFS")]
    FSNumberMEMFS,
    [CCode (cname = "fileswitch_FS_NUMBER_SHAREFS")]
    FSNumberSHAREFS,
    [CCode (cname = "fileswitch_FS_NUMBER_LANMAN")]
    FSNumberLANMAN,
    [CCode (cname = "fileswitch_FS_NUMBER_OMNIPRINT")]
    FSNumberOMNIPRINT,
    [CCode (cname = "fileswitch_FS_NUMBER_RSDFS")]
    FSNumberRSDFS
  }

  [CCode (cname = "fileswitch_fs_info")]
  public enum FSInfo
  {
    [CCode (cname = "fileswitch_FS_NUMBER")]
    FSNumber,
    [CCode (cname = "fileswitch_FS_OPEN_FILE_LIMIT")]
    FSOpenFileLimit,
    [CCode (cname = "fileswitch_FS_READ_ONLY")]
    FSReadOnly,
    [CCode (cname = "fileswitch_HAS_EXTRA_FLAGS")]
    HasExtraFlags,
    [CCode (cname = "fileswitch_NO_FILE_ENTRIES")]
    NoFileEntries,
    [CCode (cname = "fileswitch_NO_SAVE_ENTRY")]
    NoSaveEntry,
    [CCode (cname = "fileswitch_NO_LOAD_ENTRY")]
    NoLoadEntry,
    [CCode (cname = "fileswitch_IMPLICIT_DIRECTORIES")]
    ImplicitDirectories,
    [CCode (cname = "fileswitch_NEEDS_URD_AND_LIB")]
    NeedsURDAndLib,
    [CCode (cname = "fileswitch_SUPPORTS_IMAGE")]
    SupportsImage,
    [CCode (cname = "fileswitch_RESERVED")]
    Reserved,
    [CCode (cname = "fileswitch_SUPPORTS_FILE_INFO")]
    SupportsFileInfo,
    [CCode (cname = "fileswitch_SUPPORTS_STAMP_NAMED")]
    SupportsStampNamed,
    [CCode (cname = "fileswitch_NEEDS_FLUSH")]
    NeedsFlush,
    [CCode (cname = "fileswitch_NEEDS_CREATE")]
    NeedsCreate,
    [CCode (cname = "fileswitch_SUPPORTS_EMPTY_NAMES")]
    SupportsEmptyNames,
    [CCode (cname = "fileswitch_INTERACTIVE")]
    Interactive,
    [CCode (cname = "fileswitch_SUPPORTS_SPECIAL")]
    SupportsSpecial
  }

  [CCode (cname = "fileswitch_fs_extra_info")]
  public enum FSExtraInfo
  {
    [CCode (cname = "fileswitch_SUPPORTS_DIR_CHANGE")]
    SupportsDirChange,
    [CCode (cname = "fileswitch_NEEDS_CAT")]
    NeedsCat,
    [CCode (cname = "fileswitch_NEEDS_EX")]
    NeedsEx,
    [CCode (cname = "fileswitch_SUPPORTS_IO_CTL")]
    SupportsIOCtl
  }

  [CCode (cname = "fileswitch_object_type")]
  public enum ObjectType
  {
    [CCode (cname = "fileswitch_NOT_FOUND")]
    NotFound,
    [CCode (cname = "fileswitch_IS_FILE")]
    IsFile,
    [CCode (cname = "fileswitch_IS_DIR")]
    IsDir,
    [CCode (cname = "fileswitch_IS_IMAGE")]
    IsImage
  }

  [CCode (cname = "fileswitch_attr")]
  public enum Attr
  {
    [CCode (cname = "fileswitch_ATTR_OWNER_READ")]
    OwnerRead,
    [CCode (cname = "fileswitch_ATTR_OWNER_WRITE")]
    OwnerWrite,
    [CCode (cname = "fileswitch_ATTR_OWNER_SPECIAL")]
    OwnerSpecial,
    [CCode (cname = "fileswitch_ATTR_OWNER_LOCKED")]
    OwnerLocked,
    [CCode (cname = "fileswitch_ATTR_WORLD_READ")]
    WorldRead,
    [CCode (cname = "fileswitch_ATTR_WORLD_WRITE")]
    WorldWrite,
    [CCode (cname = "fileswitch_ATTR_WORLD_SPECIAL")]
    WorldSpecial,
    [CCode (cname = "fileswitch_ATTR_WORLD_LOCKED")]
    WorldLocked
  }

  [CCode (cname = "fileswitch_info_words")]
  public struct InfoWords
  {
    [CCode (cname = "date_and_time")]
    public unowned OS.DateAndTime DateAndTime;
    [CCode (cname = "addrs.exec_addr")]
    public unowned bits Addrs_ExecAddr;
    [CCode (cname = "addrs.load_addr")]
    public unowned bits Addrs_LoadAddr;
  }

  [CCode (cname="error_FILE_SWITCH_NO_CLAIM")]
  public const bits Error_FileSwitchNoClaim;
  [CCode (cname="error_BAD_FS_CONTROL_REASON")]
  public const bits Error_BadFSControlReason;
  [CCode (cname="error_BAD_OS_FILE_REASON")]
  public const bits Error_BadOSFileReason;
  [CCode (cname="error_BAD_OS_ARGS_REASON")]
  public const bits Error_BadOSArgsReason;
  [CCode (cname="error_BAD_OSGBPB_REASON")]
  public const bits Error_BadOSGBPBReason;
  [CCode (cname="error_BAD_MODE_FOR_OS_FIND")]
  public const bits Error_BadModeForOSFind;
  [CCode (cname="error_NO_ROOM_FOR_TRANSIENT")]
  public const bits Error_NoRoomForTransient;
  [CCode (cname="error_EXEC_ADDR_NOT_IN_CODE")]
  public const bits Error_ExecAddrNotInCode;
  [CCode (cname="error_EXEC_ADDR_TOO_LOW")]
  public const bits Error_ExecAddrTooLow;
  [CCode (cname="error_UNKNOWN_ACTION_TYPE")]
  public const bits Error_UnknownActionType;
  [CCode (cname="error_TOO_MANY_LEVELS")]
  public const bits Error_TooManyLevels;
  [CCode (cname="error_NO_SELECTED_FILING_SYSTEM")]
  public const bits Error_NoSelectedFilingSystem;
  [CCode (cname="error_CANT_REMOVE_FS_BY_NUMBER")]
  public const bits Error_CantRemoveFSByNumber;
  [CCode (cname="error_UNALIGNED_FS_ENTRY")]
  public const bits Error_UnalignedFSEntry;
  [CCode (cname="error_UNSUPPORTED_FS_ENTRY")]
  public const bits Error_UnsupportedFSEntry;
  [CCode (cname="error_FS_NOT_SPECIAL")]
  public const bits Error_FSNotSpecial;
  [CCode (cname="error_CORE_NOT_READABLE")]
  public const bits Error_CoreNotReadable;
  [CCode (cname="error_CORE_NOT_WRITEABLE")]
  public const bits Error_CoreNotWriteable;
  [CCode (cname="error_BAD_BUFFER_SIZE_FOR_STREAM")]
  public const bits Error_BadBufferSizeForStream;
  [CCode (cname="error_NOT_OPEN_FOR_READING")]
  public const bits Error_NotOpenForReading;
  [CCode (cname="error_NOT_ENOUGH_STACK_FOR_FS_ENTRY")]
  public const bits Error_NotEnoughStackForFSEntry;
  [CCode (cname="error_NOTHING_TO_COPY")]
  public const bits Error_NothingToCopy;
  [CCode (cname="error_NOTHING_TO_DELETE")]
  public const bits Error_NothingToDelete;
  [CCode (cname="error_FILE_SWITCH_CANT_BE_KILLED_WHILST_THREADED")]
  public const bits Error_FileSwitchCantBeKilledWhilstThreaded;
  [CCode (cname="error_INVALID_ERROR_BLOCK")]
  public const bits Error_InvalidErrorBlock;
  [CCode (cname="error_FS_FILE_TOO_BIG")]
  public const bits Error_FSFileTooBig;
  [CCode (cname="error_CANT_RM_FASTER_FILE_SWITCH")]
  public const bits Error_CantRMFasterFileSwitch;
  [CCode (cname="error_INCONSISTENT_HANDLE_SET")]
  public const bits Error_InconsistentHandleSet;
  [CCode (cname="error_IS_AFILE")]
  public const bits Error_IsAFile;
  [CCode (cname="error_BAD_FILE_TYPE")]
  public const bits Error_BadFileType;
  [CCode (cname="error_LIBRARY_SOMEWHERE_ELSE")]
  public const bits Error_LibrarySomewhereElse;
  [CCode (cname="error_PATH_IS_SELF_CONTRADICTORY")]
  public const bits Error_PathIsSelfContradictory;
  [CCode (cname="error_WASNT_DOLLAR_AFTER_DISC")]
  public const bits Error_WasntDollarAfterDisc;
  [CCode (cname="error_NOT_ENOUGH_MEMORY_FOR_WILDCARD_RESOLUTION")]
  public const bits Error_NotEnoughMemoryForWildcardResolution;
  [CCode (cname="error_NOT_ENOUGH_STACK_FOR_WILDCARD_RESOLUTION")]
  public const bits Error_NotEnoughStackForWildcardResolution;
  [CCode (cname="error_DIR_WANTED_FILE_FOUND")]
  public const bits Error_DirWantedFileFound;
  [CCode (cname="error_NOT_FOUND")]
  public const bits Error_NotFound;
  [CCode (cname="error_MULTIPART_PATH_USED")]
  public const bits Error_MultipartPathUsed;
  [CCode (cname="error_RECURSIVE_PATH")]
  public const bits Error_RecursivePath;
  [CCode (cname="error_MULTI_FS_DOES_NOT_SUPPORT_GBPB11")]
  public const bits Error_MultiFSDoesNotSupportGBPB11;
  [CCode (cname="error_FILE_SWITCH_DATA_LOST")]
  public const bits Error_FileSwitchDataLost;
  [CCode (cname="error_TOO_MANY_ERROR_LOOKUPS")]
  public const bits Error_TooManyErrorLookups;
  [CCode (cname="error_MESSAGE_FILE_BUSY")]
  public const bits Error_MessageFileBusy;
  [CCode (cname="error_PARTITION_BUSY")]
  public const bits Error_PartitionBusy;
  [CCode (cname="error_NOT_SUPPORTED")]
  public const bits Error_NotSupported;
  [CCode (cname="error_FS_WRITE_ONLY")]
  public const bits Error_FSWriteOnly;
  [CCode (cname="error_FS_READ_ONLY")]
  public const bits Error_FSReadOnly;

  [CCode (cname = "xbgetv")]
  public static OS.Error* XBGetV
    (OS.F file,
     out char c,
     out uint psr);

  [CCode (cname = "bgetv")]
  public static bits BGetV
    (OS.F file,
     out char c);

  [CCode (cname = "xbgetvw")]
  public static OS.Error* XBGetVW
    (OS.FW file,
     out char c,
     out uint psr);

  [CCode (cname = "bgetvw")]
  public static bits BGetVW
    (OS.FW file,
     out char c);

  [CCode (cname = "xbputv")]
  public static OS.Error* XBPutV
    (char c,
     OS.F file);

  [CCode (cname = "bputv")]
  public static void BPutV
    (char c,
     OS.F file);

  [CCode (cname = "xbputvw")]
  public static OS.Error* XBPutVW
    (char c,
     OS.FW file);

  [CCode (cname = "bputvw")]
  public static void BPutVW
    (char c,
     OS.FW file);

  [CCode (cname = "xos_bget")]
  public static OS.Error* XBGet
    (OS.F file,
     out char c,
     out uint psr);

  [CCode (cname = "os_bget")]
  public static bits BGet
    (OS.F file,
     out char c);

  [CCode (cname = "xos_bgetw")]
  public static OS.Error* XBGetW
    (OS.FW file,
     out char c,
     out uint psr);

  [CCode (cname = "os_bgetw")]
  public static bits BGetW
    (OS.FW file,
     out char c);

  [CCode (cname = "xos_bput")]
  public static OS.Error* XBPut
    (char c,
     OS.F file);

  [CCode (cname = "os_bput")]
  public static void BPut
    (char c,
     OS.F file);

  [CCode (cname = "xos_bputw")]
  public static OS.Error* XBPutW
    (char c,
     OS.FW file);

  [CCode (cname = "os_bputw")]
  public static void BPutW
    (char c,
     OS.FW file);

  [CCode (cname = "xservice_start_up_fs")]
  public static OS.Error* XService_StartUpFS
    (FSNo fs_no);

  [CCode (cname = "service_start_up_fs")]
  public static void Service_StartUpFS
    (FSNo fs_no);

  [CCode (cname = "xservice_fs_redeclare")]
  public static OS.Error* XService_FSRedeclare ();

  [CCode (cname = "service_fs_redeclare")]
  public static void Service_FSRedeclare ();

  [CCode (cname = "xservice_close_file")]
  public static OS.Error* XService_CloseFile
    (string file_name,
     int count,
     out int count_out);

  [CCode (cname = "service_close_file")]
  public static void Service_CloseFile
    (string file_name,
     int count,
     out int count_out);

  [CCode (cname = "xservice_uk_compression_unsqueeze")]
  public static OS.Error* XService_UKCompressionUnsqueeze
    (bits load_addr,
     int size,
     bits exec_addr,
     string filename,
     out bool unclaimed,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "service_uk_compression_unsqueeze")]
  public static bool Service_UKCompressionUnsqueeze
    (bits load_addr,
     int size,
     bits exec_addr,
     string filename,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "xservice_uk_compression_patch")]
  public static OS.Error* XService_UKCompressionPatch
    (bits load_addr,
     int size,
     bits exec_addr,
     string filename,
     out bool unclaimed,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "service_uk_compression_patch")]
  public static bool Service_UKCompressionPatch
    (bits load_addr,
     int size,
     bits exec_addr,
     string filename,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "xservice_uk_compression_non_aif")]
  public static OS.Error* XService_UKCompressionNonAIF
    (int size,
     bits exec_addr,
     string filename,
     char* header,
     string args,
     out bool unclaimed,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "service_uk_compression_non_aif")]
  public static bool Service_UKCompressionNonAIF
    (int size,
     bits exec_addr,
     string filename,
     char* header,
     string args,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "xservice_uk_compression26_bit_on32_bit")]
  public static OS.Error* XService_UKCompression26BitOn32Bit
    (int size,
     bits exec_addr,
     string filename,
     char* header,
     string args,
     out bool unclaimed,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "service_uk_compression26_bit_on32_bit")]
  public static bool Service_UKCompression26BitOn32Bit
    (int size,
     bits exec_addr,
     string filename,
     char* header,
     string args,
     out int size_out,
     out bits new_exec_addr);

  [CCode (cname = "xservice_uk_compression_debugger")]
  public static OS.Error* XService_UKCompressionDebugger
    (int size,
     bits exec_addr,
     string filename,
     out bool unclaimed);

  [CCode (cname = "service_uk_compression_debugger")]
  public static bool Service_UKCompressionDebugger
    (int size,
     bits exec_addr,
     string filename);

}
