/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/fileraction.h")]
namespace OSLib.FilerAction
{
  [CCode (cname = "fileraction_operation")]
  public enum Operation
  {
    [CCode (cname = "fileraction_COPY")]
    Copy,
    [CCode (cname = "fileraction_MOVE_LOCAL")]
    MoveLocal,
    [CCode (cname = "fileraction_DELETE")]
    Delete,
    [CCode (cname = "fileraction_ACCESS")]
    Access,
    [CCode (cname = "fileraction_SET_TYPE")]
    SetType,
    [CCode (cname = "fileraction_COUNT")]
    Count,
    [CCode (cname = "fileraction_MOVE")]
    Move,
    [CCode (cname = "fileraction_COPY_LOCAL")]
    CopyLocal,
    [CCode (cname = "fileraction_STAMP")]
    Stamp,
    [CCode (cname = "fileraction_FIND")]
    Find
  }

  [CCode (cname = "fileraction_flags")]
  public enum Flags
  {
    [CCode (cname = "fileraction_VERBOSE")]
    Verbose,
    [CCode (cname = "fileraction_CONFIRM")]
    Confirm,
    [CCode (cname = "fileraction_FORCE")]
    Force,
    [CCode (cname = "fileraction_NEWER")]
    Newer,
    [CCode (cname = "fileraction_RECURSE")]
    Recurse,
    [CCode (cname = "fileraction_CONFIRM_DELETES_ONLY")]
    ConfirmDeletesOnly,
    [CCode (cname = "fileraction_FASTER")]
    Faster,
    [CCode (cname = "fileraction_NEVER_PROXY")]
    NeverProxy
  }


  [CCode (cname = "xfileraction_send_selected_directory")]
  public static OS.Error* XSendSelectedDirectory
    (Wimp.T task,
     string name);

  [CCode (cname = "fileraction_send_selected_directory")]
  public static void SendSelectedDirectory
    (Wimp.T task,
     string name);

  [CCode (cname = "xfileraction_send_selected_file")]
  public static OS.Error* XSendSelectedFile
    (Wimp.T task,
     string leaf_name);

  [CCode (cname = "fileraction_send_selected_file")]
  public static void SendSelectedFile
    (Wimp.T task,
     string leaf_name);

  [CCode (cname = "xfileractionsendstartoperation_copy")]
  public static OS.Error* XFilerActionSendStartOperation_Copy
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "fileractionsendstartoperation_copy")]
  public static void FilerActionSendStartOperation_Copy
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "xfileractionsendstartoperation_move_local")]
  public static OS.Error* XFilerActionSendStartOperation_MoveLocal
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "fileractionsendstartoperation_move_local")]
  public static void FilerActionSendStartOperation_MoveLocal
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "xfileractionsendstartoperation_delete")]
  public static OS.Error* XFilerActionSendStartOperation_Delete
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "fileractionsendstartoperation_delete")]
  public static void FilerActionSendStartOperation_Delete
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "xfileractionsendstartoperation_access")]
  public static OS.Error* XFilerActionSendStartOperation_Access
    (Wimp.T task,
     Flags flags,
     int* access);

  [CCode (cname = "fileractionsendstartoperation_access")]
  public static void FilerActionSendStartOperation_Access
    (Wimp.T task,
     Flags flags,
     int* access);

  [CCode (cname = "xfileractionsendstartoperation_set_type")]
  public static OS.Error* XFilerActionSendStartOperation_SetType
    (Wimp.T task,
     Flags flags,
     bits* file_type);

  [CCode (cname = "fileractionsendstartoperation_set_type")]
  public static void FilerActionSendStartOperation_SetType
    (Wimp.T task,
     Flags flags,
     bits* file_type);

  [CCode (cname = "xfileractionsendstartoperation_count")]
  public static OS.Error* XFilerActionSendStartOperation_Count
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "fileractionsendstartoperation_count")]
  public static void FilerActionSendStartOperation_Count
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "xfileractionsendstartoperation_move")]
  public static OS.Error* XFilerActionSendStartOperation_Move
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "fileractionsendstartoperation_move")]
  public static void FilerActionSendStartOperation_Move
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "xfileractionsendstartoperation_copy_local")]
  public static OS.Error* XFilerActionSendStartOperation_CopyLocal
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "fileractionsendstartoperation_copy_local")]
  public static void FilerActionSendStartOperation_CopyLocal
    (Wimp.T task,
     Flags flags,
     string destination,
     int size);

  [CCode (cname = "xfileractionsendstartoperation_stamp")]
  public static OS.Error* XFilerActionSendStartOperation_Stamp
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "fileractionsendstartoperation_stamp")]
  public static void FilerActionSendStartOperation_Stamp
    (Wimp.T task,
     Flags flags);

  [CCode (cname = "xfileractionsendstartoperation_find")]
  public static OS.Error* XFilerActionSendStartOperation_Find
    (Wimp.T task,
     Flags flags,
     string file_name,
     int size);

  [CCode (cname = "fileractionsendstartoperation_find")]
  public static void FilerActionSendStartOperation_Find
    (Wimp.T task,
     Flags flags,
     string file_name,
     int size);

}
