/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/imagefileconvert.h")]
namespace OSLib.ImageFileConvert
{
  [CCode (cname = "imagefileconvertconvert_flags")]
  public enum ImageFileConvertConvert_Flags
  {
    Default = 0
  }

  [CCode (cname = "imagefileconvertconvert_type")]
  public enum ImageFileConvertConvert_Type
  {
    [CCode (cname = "imagefileconvertconverttype_START_ENUMERATION")]
    StartEnumeration
  }

  [CCode (cname = "imagefileconvert_op_type")]
  public enum OpType
  {
    Default = 0
  }

  [CCode (cname = "imagefileconvertreadinfo_flags")]
  public enum ImageFileConvertReadInfo_Flags
  {
    [CCode (cname = "imagefileconvertreadinfoflags_PALETTE")]
    Palette,
    [CCode (cname = "imagefileconvertreadinfoflags_COLOUR")]
    Colour,
    [CCode (cname = "imagefileconvertreadinfoflags_ALPHA_OR_MASK")]
    AlphaOrMask,
    [CCode (cname = "imagefileconvertreadinfoflagsinterlace_NO")]
    No,
    [CCode (cname = "imagefileconvertreadinfoflagsinterlace_ADAM7")]
    Adam7,
    [CCode (cname = "imagefileconvertreadinfoflagsinterlace_VERTICAL")]
    Vertical
  }

  [CCode (cname = "imagefileconvertregister_flags")]
  public enum ImageFileConvertRegister_Flags
  {
    Default = 0
  }

  [CCode (cname = "imagefileconvert_definition")]
  public struct Definition
  {
    [CCode (cname = "api_version")]
    public unowned int ApiVersion;
    [CCode (cname = "flags")]
    public unowned int Flags;
    [CCode (cname = "type")]
    public unowned ImageFileConvertConvert_Type Type;
    [CCode (cname = "name")]
    public unowned string Name;
    [CCode (cname = "ws_value")]
    public unowned int WsValue;
    [CCode (cname = "do_convert")]
    public unowned AsmRoutine DoConvert;
    [CCode (cname = "do_miscop")]
    public unowned AsmRoutine DoMiscop;
  }

  [CCode (cname = "imagefileconvertderegister_flags")]
  public enum ImageFileConvertDeregister_Flags
  {
    Default = 0
  }

  [CCode (cname = "imagefileconvertenumerateconverters_flags")]
  public enum ImageFileConvertEnumerateConverters_Flags
  {
    Default = 0
  }

  [CCode (cname = "imagefileconvertconverterinfo_flags")]
  public enum ImageFileConvertConverterInfo_Flags
  {
    Default = 0
  }

  [CCode (cname="imagefileconvertconvertflags_OUT_OPTION_SHIFT")]
  public const int ImageFileConvertConvertFlags_OutOptionShift;
  [CCode (cname="imagefileconvertconvertflags_IN_OPTION_SHIFT")]
  public const int ImageFileConvertConvertFlags_InOptionShift;
  [CCode (cname="imagefileconvertconvertflags_IN_OUT_OPTION_SHIFT")]
  public const int ImageFileConvertConvertFlags_InOutOptionShift;
  [CCode (cname="imagefileconvertconvertflagsoutjpeg_QUALITY_SHIFT")]
  public const int ImageFileConvertConvertFlagsOutJPEG_QualityShift;
  [CCode (cname="imagefileconvertconverttype_DEST_FILE_TYPE_SHIFT")]
  public const int ImageFileConvertConvertType_DestFileTypeShift;
  [CCode (cname="imagefileconvertconverttype_SRC_FILE_TYPE_SHIFT")]
  public const int ImageFileConvertConvertType_SrcFileTypeShift;
  [CCode (cname="imagefileconvertreadinfoflags_INTERLACE_SHIFT")]
  public const int ImageFileConvertReadInfoFlags_InterlaceShift;
  [CCode (cname="imagefileconvertreadinfoflags_INTERLACE_MASK")]
  public const int ImageFileConvertReadInfoFlags_InterlaceMask;

  [CCode (cname = "ximagefileconvert_convert")]
  public static OS.Error* XConvert
    (ImageFileConvertConvert_Flags flags,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     char* img_out,
     int img_out_size_in,
     OS.Colour background,
     int param0,
     int param1,
     int param2,
     out int img_out_size_out);

  [CCode (cname = "imagefileconvert_convert")]
  public static void Convert
    (ImageFileConvertConvert_Flags flags,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     char* img_out,
     int img_out_size_in,
     OS.Colour background,
     int param0,
     int param1,
     int param2,
     out int img_out_size_out);

  [CCode (cname = "ximagefileconvert_misc_op")]
  public static OS.Error* XMiscOp
    (ImageFileConvertConvert_Flags flags,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     OpType operation,
     int param0_in,
     int param1_in,
     int param2_in,
     int param3_in,
     int param4_in,
     out int param0_out,
     out int param1_out,
     out int param2_out,
     out int param3_out,
     out int param4_out,
     out int param5_out,
     out int param6_out,
     out int param7_out,
     out int param8_out,
     out int param9_out);

  [CCode (cname = "imagefileconvert_misc_op")]
  public static void MiscOp
    (ImageFileConvertConvert_Flags flags,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     OpType operation,
     int param0_in,
     int param1_in,
     int param2_in,
     int param3_in,
     int param4_in,
     out int param0_out,
     out int param1_out,
     out int param2_out,
     out int param3_out,
     out int param4_out,
     out int param5_out,
     out int param6_out,
     out int param7_out,
     out int param8_out,
     out int param9_out);

  [CCode (cname = "ximagefileconvert_read_info")]
  public static OS.Error* XReadInfo
    (ImageFileConvertConvert_Flags flags_in,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     out int width,
     out int height,
     out int bps,
     out ImageFileConvertReadInfo_Flags flags_out,
     out int xdpi,
     out int ydpi);

  [CCode (cname = "imagefileconvert_read_info")]
  public static void ReadInfo
    (ImageFileConvertConvert_Flags flags_in,
     ImageFileConvertConvert_Type conversion_type,
     char* img_in,
     int img_in_size,
     out int width,
     out int height,
     out int bps,
     out ImageFileConvertReadInfo_Flags flags_out,
     out int xdpi,
     out int ydpi);

  [CCode (cname = "ximagefileconvert_register")]
  public static OS.Error* XRegister
    (ImageFileConvertRegister_Flags flags,
     Definition def,
     out int api_version);

  [CCode (cname = "imagefileconvert_register")]
  public static void Register
    (ImageFileConvertRegister_Flags flags,
     Definition def,
     out int api_version);

  [CCode (cname = "ximagefileconvert_deregister")]
  public static OS.Error* XDeregister
    (ImageFileConvertDeregister_Flags flags,
     ImageFileConvertConvert_Type type,
     string name);

  [CCode (cname = "imagefileconvert_deregister")]
  public static void Deregister
    (ImageFileConvertDeregister_Flags flags,
     ImageFileConvertConvert_Type type,
     string name);

  [CCode (cname = "ximagefileconvert_enumerate_converters")]
  public static OS.Error* XEnumerateConverters
    (ImageFileConvertEnumerateConverters_Flags flags,
     ImageFileConvertConvert_Type pair_in,
     out Definition def,
     out ImageFileConvertConvert_Type pair_out);

  [CCode (cname = "imagefileconvert_enumerate_converters")]
  public static void EnumerateConverters
    (ImageFileConvertEnumerateConverters_Flags flags,
     ImageFileConvertConvert_Type pair_in,
     out Definition def,
     out ImageFileConvertConvert_Type pair_out);

  [CCode (cname = "ximagefileconvert_converter_info")]
  public static OS.Error* XConverterInfo
    (ImageFileConvertConverterInfo_Flags flags,
     ImageFileConvertConvert_Type pair,
     out Definition def,
     out string name);

  [CCode (cname = "imagefileconvert_converter_info")]
  public static void ConverterInfo
    (ImageFileConvertConverterInfo_Flags flags,
     ImageFileConvertConvert_Type pair,
     out Definition def,
     out string name);

}
