/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/jpeg.h")]
namespace OSLib.JPEG
{
  [CCode (cname = "jpeg_info_flags")]
  public enum InfoFlags
  {
    [CCode (cname = "jpeg_INFO_MONOCHROME")]
    InfoMonochrome,
    [CCode (cname = "jpeg_INFO_NO_TRFM")]
    InfoNoTrfm,
    [CCode (cname = "jpeg_INFO_DPI_UNKNOWN")]
    InfoDPIUnknown
  }

  [CCode (cname = "jpeg_scale_flags")]
  public enum ScaleFlags
  {
    [CCode (cname = "jpeg_SCALE_DITHERED")]
    ScaleDithered,
    [CCode (cname = "jpeg_SCALE_ERROR_DIFFUSED")]
    ScaleErrorDiffused,
    [CCode (cname = "jpeg_SCALE_GIVEN_COLOUR_MAPPING")]
    ScaleGivenColourMapping,
    [CCode (cname = "jpeg_SCALE_TRANSLUCENCY")]
    ScaleTranslucency
  }

  [CCode (cname = "jpeg_transform_flags")]
  public enum TransformFlags
  {
    [CCode (cname = "jpeg_DESTINATION_COORDS")]
    DestinationCoords,
    [CCode (cname = "jpeg_TRANSFORM_DITHERED")]
    TransformDithered,
    [CCode (cname = "jpeg_TRANSFORM_ERROR_DIFFUSED")]
    TransformErrorDiffused,
    [CCode (cname = "jpeg_TRANSFORM_GIVEN_COLOUR_MAPPING")]
    TransformGivenColourMapping,
    [CCode (cname = "jpeg_TRANSFORM_TRANSLUCENCY")]
    TransformTranslucency
  }

  [CCode (cname = "jpeg_image")]
  public struct Image
  {
    [CCode (cname = "i", array_length = false)]
    public unowned byte[] I;
  }

  [CCode (cname = "jpeg_print_flags")]
  public enum PrintFlags
  {
    [CCode (cname = "jpeg_PRINT_PLOTTING")]
    PrintPlotting,
    [CCode (cname = "jpeg_PRINT_USING_TRANS_TAB")]
    PrintUsingTransTab
  }

  [CCode (cname="jpeg_SCALE_TRANSLUCENCY_SHIFT")]
  public const int ScaleTranslucencyShift;
  [CCode (cname="jpeg_TRANSFORM_TRANSLUCENCY_SHIFT")]
  public const int TransformTranslucencyShift;
  [CCode (cname="jpeg_FILE_TYPE")]
  public const bits FileType;

  [CCode (cname = "xjpeginfo_dimensions")]
  public static OS.Error* XJPEGInfo_Dimensions
    (OSLib.JPEG.Image image,
     int size,
     out InfoFlags info_flags,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "jpeginfo_dimensions")]
  public static InfoFlags JPEGInfo_Dimensions
    (OSLib.JPEG.Image image,
     int size,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "xjpegfileinfo_dimensions")]
  public static OS.Error* XJPEGFileInfo_Dimensions
    (string file_name,
     out InfoFlags info_flags,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "jpegfileinfo_dimensions")]
  public static InfoFlags JPEGFileInfo_Dimensions
    (string file_name,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "xjpeg_plot_scaled")]
  public static OS.Error* XPlotScaled
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     ScaleFlags flags);

  [CCode (cname = "jpeg_plot_scaled")]
  public static void PlotScaled
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     ScaleFlags flags);

  [CCode (cname = "xjpeg_plot_scaled_mapped")]
  public static OS.Error* XPlotScaledMapped
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     ScaleFlags flags,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "jpeg_plot_scaled_mapped")]
  public static void PlotScaledMapped
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     ScaleFlags flags,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xjpeg_plot_file_scaled")]
  public static OS.Error* XPlotFileScaled
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     ScaleFlags flags);

  [CCode (cname = "jpeg_plot_file_scaled")]
  public static void PlotFileScaled
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     ScaleFlags flags);

  [CCode (cname = "xjpeg_plot_file_scaled_mapped")]
  public static OS.Error* XPlotFileScaledMapped
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     ScaleFlags flags,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "jpeg_plot_file_scaled_mapped")]
  public static void PlotFileScaledMapped
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     ScaleFlags flags,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xjpeg_plot_transformed")]
  public static OS.Error* XPlotTransformed
    (OSLib.JPEG.Image image,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size);

  [CCode (cname = "jpeg_plot_transformed")]
  public static void PlotTransformed
    (OSLib.JPEG.Image image,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size);

  [CCode (cname = "xjpeg_plot_transformed_mapped")]
  public static OS.Error* XPlotTransformedMapped
    (OSLib.JPEG.Image image,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "jpeg_plot_transformed_mapped")]
  public static void PlotTransformedMapped
    (OSLib.JPEG.Image image,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xjpeg_plot_file_transformed")]
  public static OS.Error* XPlotFileTransformed
    (string file_name,
     TransformFlags flags,
     OS.Trfm trfm_or_rect);

  [CCode (cname = "jpeg_plot_file_transformed")]
  public static void PlotFileTransformed
    (string file_name,
     TransformFlags flags,
     OS.Trfm trfm_or_rect);

  [CCode (cname = "xjpeg_plot_file_transformed_mapped")]
  public static OS.Error* XPlotFileTransformedMapped
    (string file_name,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "jpeg_plot_file_transformed_mapped")]
  public static void PlotFileTransformedMapped
    (string file_name,
     TransformFlags flags,
     OS.Trfm trfm_or_rect,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xjpeg_pdriver_intercept")]
  public static OS.Error* XPDriverIntercept
    (PrintFlags flags,
     out PrintFlags flags_out);

  [CCode (cname = "jpeg_pdriver_intercept")]
  public static PrintFlags PDriverIntercept
    (PrintFlags flags);

}
