/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/netprint.h")]
namespace OSLib.NetPrint
{
  [CCode (cname = "netprint_status")]
  public enum Status
  {
    [CCode (cname = "netprint_STATUS_READY")]
    Ready,
    [CCode (cname = "netprint_STATUS_BUSY")]
    Busy,
    [CCode (cname = "netprint_STATUS_JAMMED")]
    Jammed,
    [CCode (cname = "netprint_STATUS_OFFLINE")]
    Offline,
    [CCode (cname = "netprint_STATUS_OPEN")]
    Open
  }

  [CCode (cname = "netprint_brief_entry")]
  public struct BriefEntry
  {
    [CCode (cname = "station_no")]
    public unowned byte StationNo;
    [CCode (cname = "net_no")]
    public unowned byte NetNo;
    [CCode (cname = "ps_name")]
    public unowned char PsName[256];
  }

  [CCode (cname = "netprint_name_entry")]
  public struct NameEntry
  {
    [CCode (cname = "ps_name")]
    public unowned char PsName[256];
  }

  [CCode (cname = "netprint_full_entry")]
  public struct FullEntry
  {
    [CCode (cname = "station_no")]
    public unowned byte StationNo;
    [CCode (cname = "net_no")]
    public unowned byte NetNo;
    [CCode (cname = "status")]
    public unowned Status Status;
    [CCode (cname = "status_station_no")]
    public unowned byte StatusStationNo;
    [CCode (cname = "status_net_no")]
    public unowned byte StatusNetNo;
    [CCode (cname = "ps_name")]
    public unowned char PsName[256];
  }

  [CCode (cname="netprint_PS_NAME_LIMIT")]
  public const int PSNameLimit;
  [CCode (cname="error_NET_PRINT_NAME_TOO_LONG")]
  public const bits Error_NetPrintNameTooLong;
  [CCode (cname="error_NET_PRINT_SINGLE_STREAM")]
  public const bits Error_NetPrintSingleStream;
  [CCode (cname="error_NET_PRINT_ALL_PRINTERS_BUSY")]
  public const bits Error_NetPrintAllPrintersBusy;
  [CCode (cname="error_NET_PRINT_OFF_LINE")]
  public const bits Error_NetPrintOffLine;
  [CCode (cname="error_NET_PRINT_NOT_FOUND")]
  public const bits Error_NetPrintNotFound;
  [CCode (cname="error_NET_PRINT_INTERNAL_ERROR")]
  public const bits Error_NetPrintInternalError;

  [CCode (cname = "xnetprint_read_ps_number")]
  public static OS.Error* XReadPSNumber
    (out byte station_no,
     out byte net_no);

  [CCode (cname = "netprint_read_ps_number")]
  public static void ReadPSNumber
    (out byte station_no,
     out byte net_no);

  [CCode (cname = "xnetprint_set_ps_number")]
  public static OS.Error* XSetPSNumber
    (byte station_no,
     byte net_no);

  [CCode (cname = "netprint_set_ps_number")]
  public static void SetPSNumber
    (byte station_no,
     byte net_no);

  [CCode (cname = "xnetprint_read_ps_name")]
  public static OS.Error* XReadPSName
    (string buffer,
     int size,
     out char* end);

  [CCode (cname = "netprint_read_ps_name")]
  public static char ReadPSName
    (string buffer,
     int size);

  [CCode (cname = "xnetprint_set_ps_name")]
  public static OS.Error* XSetPSName
    (string ps_name);

  [CCode (cname = "netprint_set_ps_name")]
  public static void SetPSName
    (string ps_name);

  [CCode (cname = "xnetprint_read_ps_timeouts")]
  public static OS.Error* XReadPSTimeouts
    (out int transmit_count,
     out int transmit_delay,
     out int peek_count,
     out int peek_delay,
     out int receive_delay,
     out int broadcast_delay);

  [CCode (cname = "netprint_read_ps_timeouts")]
  public static void ReadPSTimeouts
    (out int transmit_count,
     out int transmit_delay,
     out int peek_count,
     out int peek_delay,
     out int receive_delay,
     out int broadcast_delay);

  [CCode (cname = "xnetprint_set_ps_timeouts")]
  public static OS.Error* XSetPSTimeouts
    (int transmit_count,
     int transmit_delay,
     int peek_count,
     int peek_delay,
     int receive_delay,
     int broadcast_delay);

  [CCode (cname = "netprint_set_ps_timeouts")]
  public static void SetPSTimeouts
    (int transmit_count,
     int transmit_delay,
     int peek_count,
     int peek_delay,
     int receive_delay,
     int broadcast_delay);

  [CCode (cname = "xnetprint_bind_ps_name")]
  public static OS.Error* XBindPSName
    (string ps_name,
     out byte station_no,
     out byte net_no);

  [CCode (cname = "netprint_bind_ps_name")]
  public static void BindPSName
    (string ps_name,
     out byte station_no,
     out byte net_no);

  [CCode (cname = "xnetprintlistservers_brief")]
  public static OS.Error* XNetPrintListServers_Brief
    (BriefEntry entries,
     int size,
     int timeout,
     out int entry_count,
     out bool no_more);

  [CCode (cname = "netprintlistservers_brief")]
  public static bool NetPrintListServers_Brief
    (BriefEntry entries,
     int size,
     int timeout,
     out int entry_count);

  [CCode (cname = "xnetprintlistservers_name")]
  public static OS.Error* XNetPrintListServers_Name
    (NameEntry entries,
     int size,
     int timeout,
     out int entry_count,
     out bool no_more);

  [CCode (cname = "netprintlistservers_name")]
  public static bool NetPrintListServers_Name
    (NameEntry entries,
     int size,
     int timeout,
     out int entry_count);

  [CCode (cname = "xnetprintlistservers_full")]
  public static OS.Error* XNetPrintListServers_Full
    (FullEntry entries,
     int size,
     int timeout,
     out int entry_count,
     out bool no_more);

  [CCode (cname = "netprintlistservers_full")]
  public static bool NetPrintListServers_Full
    (FullEntry entries,
     int size,
     int timeout,
     out int entry_count);

  [CCode (cname = "xnetprint_convert_status_to_string")]
  public static OS.Error* XConvertStatusToString
    (bits* status_station_net,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "netprint_convert_status_to_string")]
  public static char ConvertStatusToString
    (bits* status_station_net,
     string buffer,
     int size);

  [CCode (cname = "xservice_net_print_check_d1")]
  public static OS.Error* XService_NetPrintCheckD1
    (out bool unclaimed);

  [CCode (cname = "service_net_print_check_d1")]
  public static void Service_NetPrintCheckD1
    (out bool unclaimed);

}
