/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/numberrange.h")]
namespace OSLib.NumberRange
{
  [CCode (cname = "numberrange_gadget")]
  public struct Gadget
  {
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "initial_value")]
    public unowned int InitialValue;
    [CCode (cname = "precision")]
    public unowned int Precision;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
    [CCode (cname = "display_length")]
    public unowned int DisplayLength;
  }

  [CCode (cname = "numberrange_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "initial_value")]
    public unowned int InitialValue;
    [CCode (cname = "precision")]
    public unowned int Precision;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
    [CCode (cname = "display_length")]
    public unowned int DisplayLength;
  }

  [CCode (cname = "numberrange_bounds_flags")]
  public enum BoundsFlags
  {
    [CCode (cname = "numberrange_BOUND_LOWER")]
    BoundLower,
    [CCode (cname = "numberrange_BOUND_UPPER")]
    BoundUpper,
    [CCode (cname = "numberrange_BOUND_STEP")]
    BoundStep,
    [CCode (cname = "numberrange_BOUND_PRECISION")]
    BoundPrecision
  }

  [CCode (cname = "numberrange_action_value_changed")]
  public struct ActionValueChanged
  {
    [CCode (cname = "new_value")]
    public unowned int NewValue;
  }

  [CCode (cname = "numberrange_action_value_changed_block")]
  public struct ActionValueChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "new_value")]
    public unowned int NewValue;
  }

  [CCode (cname="class_NUMBER_RANGE")]
  public const Toolbox.Class Class_NumberRange;
  [CCode (cname="numberrange_GENERATE_VALUE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateValueChanged;
  [CCode (cname="numberrange_WRITABLE")]
  public const OSLib.Gadget.Flags Writable;
  [CCode (cname="numberrange_HAS_NUMERICAL_DISPLAY")]
  public const OSLib.Gadget.Flags HasNumericalDisplay;
  [CCode (cname="numberrange_ADJUSTERS")]
  public const OSLib.Gadget.Flags Adjusters;
  [CCode (cname="numberrange_SLIDER_TYPE")]
  public const OSLib.Gadget.Flags SliderType;
  [CCode (cname="numberrange_SLIDER_TYPE_SHIFT")]
  public const int SliderTypeShift;
  [CCode (cname="numberrange_SLIDER_TYPE_NONE")]
  public const int SliderTypeNone;
  [CCode (cname="numberrange_SLIDER_TYPE_RIGHT")]
  public const int SliderTypeRight;
  [CCode (cname="numberrange_SLIDER_TYPE_LEFT")]
  public const int SliderTypeLeft;
  [CCode (cname="numberrange_RJUSTIFIED")]
  public const OSLib.Gadget.Flags RJustified;
  [CCode (cname="numberrange_HCENTRED")]
  public const OSLib.Gadget.Flags HCentred;
  [CCode (cname="numberrange_KNOB_COLOUR")]
  public const OSLib.Gadget.Flags KnobColour;
  [CCode (cname="numberrange_KNOB_COLOUR_SHIFT")]
  public const int KnobColourShift;
  [CCode (cname="numberrange_WELL_COLOUR")]
  public const OSLib.Gadget.Flags WellColour;
  [CCode (cname="numberrange_WELL_COLOUR_SHIFT")]
  public const int WellColourShift;
  [CCode (cname="numberrange_GET_NUMERICAL_FIELD")]
  public const bits GetNumericalField;
  [CCode (cname="numberrange_GET_LEFT_ADJUSTER")]
  public const bits GetLeftAdjuster;
  [CCode (cname="numberrange_GET_RIGHT_ADJUSTER")]
  public const bits GetRightAdjuster;
  [CCode (cname="numberrange_GET_SLIDER")]
  public const bits GetSlider;
  [CCode (cname="action_NUMBER_RANGE_VALUE_CHANGED")]
  public const bits Action_NumberRangeValueChanged;

  [CCode (cname = "xnumberrange_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range,
     int value);

  [CCode (cname = "numberrange_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range,
     int value);

  [CCode (cname = "xnumberrange_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range,
     out int value);

  [CCode (cname = "numberrange_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range);

  [CCode (cname = "xnumberrange_set_bounds")]
  public static OS.Error* XSetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C number_range,
     int lower_bound,
     int upper_bound,
     int step_size,
     int precision);

  [CCode (cname = "numberrange_set_bounds")]
  public static void SetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C number_range,
     int lower_bound,
     int upper_bound,
     int step_size,
     int precision);

  [CCode (cname = "xnumberrange_get_bounds")]
  public static OS.Error* XGetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C number_range,
     out int lower_bound,
     out int upper_bound,
     out int step_size,
     out int precision);

  [CCode (cname = "numberrange_get_bounds")]
  public static void GetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C number_range,
     out int lower_bound,
     out int upper_bound,
     out int step_size,
     out int precision);

  [CCode (cname = "xnumberrange_get_components")]
  public static OS.Error* XGetComponents
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range,
     out Toolbox.C numeric_field,
     out Toolbox.C down,
     out Toolbox.C up,
     out Toolbox.C slider);

  [CCode (cname = "numberrange_get_components")]
  public static void GetComponents
    (bits flags,
     Toolbox.O window,
     Toolbox.C number_range,
     out Toolbox.C numeric_field,
     out Toolbox.C down,
     out Toolbox.C up,
     out Toolbox.C slider);

}
