/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/ole.h")]
namespace OSLib.OLE
{
  [CCode (cname = "ole_s", has_type_id = false)]
  public struct S : bits
  {
  }

  [CCode (cname = "ole_file_changed_tag", has_type_id = false)]
  public struct FileChangedTag : bits
  {
  }

  [CCode (cname = "ole_close_session_tag", has_type_id = false)]
  public struct CloseSessionTag : bits
  {
  }

  [CCode (cname = "ole_message_file_changed")]
  public struct MessageFileChanged
  {
    [CCode (cname = "tag")]
    public unowned FileChangedTag Tag;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
    [CCode (cname = "file_name")]
    public unowned char FileName[228];
  }

  [CCode (cname = "ole_message_open_session")]
  public struct MessageOpenSession
  {
    [CCode (cname = "name")]
    public unowned char Name[16];
    [CCode (cname = "w")]
    public unowned Wimp.W W;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "format")]
    public unowned int Format;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[196];
  }

  [CCode (cname = "ole_message_close_session")]
  public struct MessageCloseSession
  {
    [CCode (cname = "tag")]
    public unowned CloseSessionTag Tag;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
  }

  [CCode (cname = "ole_full_message_file_changed")]
  public struct FullMessageFileChanged : Wimp.MessageHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned FileChangedTag Tag;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
    [CCode (cname = "file_name")]
    public unowned char FileName[228];
  }

  [CCode (cname = "ole_full_message_open_session")]
  public struct FullMessageOpenSession : Wimp.MessageHeaderBase
  {
    [CCode (cname = "name")]
    public unowned char Name[16];
    [CCode (cname = "w")]
    public unowned Wimp.W W;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "format")]
    public unowned int Format;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[196];
  }

  [CCode (cname = "ole_full_message_close_session")]
  public struct FullMessageCloseSession : Wimp.MessageHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned CloseSessionTag Tag;
    [CCode (cname = "session_handle")]
    public unowned S SessionHandle;
  }

  [CCode (cname="message_OLE_FILE_CHANGED")]
  public const bits Message_OLEFileChanged;
  [CCode (cname="message_OLE_OPEN_SESSION")]
  public const bits Message_OLEOpenSession;
  [CCode (cname="message_OLE_OPEN_SESSION_ACK")]
  public const bits Message_OLEOpenSessionAck;
  [CCode (cname="message_OLE_CLOSE_SESSION")]
  public const bits Message_OLECloseSession;

}
