/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;

[CCode (cheader_filename = "oslib/os.h")]
namespace OSLib.OS
{
  [CCode (cname = "os_t")]
  public enum T
  {
    Default = 0
  }

  [CCode (cname = "os_f")]
  public enum F
  {
    Default = 0
  }

  [CCode (cname = "os_fw")]
  public enum FW
  {
    Default = 0
  }

  [CCode (cname = "os_mode", has_type_id = false)]
  public struct Mode : bits
  {
  }

  [CCode (cname = "os_gcol")]
  public enum GCOL
  {
    [CCode (cname = "os_GCOL_SET_FG")]
    SetFG,
    [CCode (cname = "os_GCOL_SET_BG")]
    SetBG
  }

  [CCode (cname = "os_tint")]
  public enum Tint
  {
    Default = 0
  }

  [CCode (cname = "os_action")]
  public enum Action
  {
    [CCode (cname = "os_ACTION_OVERWRITE")]
    Overwrite,
    [CCode (cname = "os_ACTION_DISJOIN")]
    Disjoin,
    [CCode (cname = "os_ACTION_CONJOIN")]
    Conjoin,
    [CCode (cname = "os_ACTION_EXCLUSIVE_DISJOIN")]
    ExclusiveDisjoin,
    [CCode (cname = "os_ACTION_INVERT")]
    Invert,
    [CCode (cname = "os_ACTION_IDENTITY")]
    Identity,
    [CCode (cname = "os_ACTION_CONJOIN_NEGATION")]
    ConjoinNegation,
    [CCode (cname = "os_ACTION_DISJOIN_NEGATION")]
    DisjoinNegation,
    [CCode (cname = "os_ACTION_USE_MASK")]
    UseMask,
    [CCode (cname = "os_ACTION_USE_ECF1")]
    UseECF1,
    [CCode (cname = "os_ACTION_USE_ECF2")]
    UseECF2,
    [CCode (cname = "os_ACTION_USE_ECF3")]
    UseECF3,
    [CCode (cname = "os_ACTION_USE_ECF4")]
    UseECF4,
    [CCode (cname = "os_ACTION_USE_GIANT_ECF")]
    UseGiantECF
  }

  [CCode (cname = "os_colour")]
  public enum Colour
  {
    [CCode (cname = "os_COLOUR_TRANSPARENT")]
    Transparent,
    [CCode (cname = "os_COLOUR_RED")]
    Red,
    [CCode (cname = "os_COLOUR_GREEN")]
    Green,
    [CCode (cname = "os_COLOUR_BLUE")]
    Blue,
    [CCode (cname = "os_COLOUR_CYAN")]
    Cyan,
    [CCode (cname = "os_COLOUR_MAGENTA")]
    Magenta,
    [CCode (cname = "os_COLOUR_YELLOW")]
    Yellow,
    [CCode (cname = "os_COLOUR_WHITE")]
    White,
    [CCode (cname = "os_COLOUR_VERY_LIGHT_GREY")]
    VeryLightGrey,
    [CCode (cname = "os_COLOUR_LIGHT_GREY")]
    LightGrey,
    [CCode (cname = "os_COLOUR_MID_LIGHT_GREY")]
    MidLightGrey,
    [CCode (cname = "os_COLOUR_MID_DARK_GREY")]
    MidDarkGrey,
    [CCode (cname = "os_COLOUR_DARK_GREY")]
    DarkGrey,
    [CCode (cname = "os_COLOUR_VERY_DARK_GREY")]
    VeryDarkGrey,
    [CCode (cname = "os_COLOUR_BLACK")]
    Black,
    [CCode (cname = "os_COLOUR_DARK_BLUE")]
    DarkBlue,
    [CCode (cname = "os_COLOUR_LIGHT_YELLOW")]
    LightYellow,
    [CCode (cname = "os_COLOUR_LIGHT_GREEN")]
    LightGreen,
    [CCode (cname = "os_COLOUR_LIGHT_RED")]
    LightRed,
    [CCode (cname = "os_COLOUR_CREAM")]
    Cream,
    [CCode (cname = "os_COLOUR_DARK_GREEN")]
    DarkGreen,
    [CCode (cname = "os_COLOUR_ORANGE")]
    Orange,
    [CCode (cname = "os_COLOUR_LIGHT_BLUE")]
    LightBlue
  }

  [CCode (cname = "os_colour_number")]
  public enum ColourNumber
  {
    Default = 0
  }

  [CCode (cname = "os_colour_pair")]
  public struct ColourPair
  {
    [CCode (cname = "on")]
    public unowned Colour On;
    [CCode (cname = "off")]
    public unowned Colour Off;
  }

  [CCode (cname = "os_palette")]
  public struct Palette
  {
    [CCode (cname = "entries", array_length = false)]
    public unowned Colour[] Entries;
  }

  [CCode (cname = "os_sprite_palette")]
  public struct SpritePalette
  {
    [CCode (cname = "entries", array_length = false)]
    public unowned ColourPair[] Entries;
  }

  [CCode (cname = "os_ecf")]
  public struct ECF
  {
    [CCode (cname = "ecf")]
    public unowned bits Ecf[8];
  }

  [CCode (cname = "os_correction_table")]
  public struct CorrectionTable
  {
    [CCode (cname = "gamma")]
    public unowned byte Gamma[256];
  }

  [CCode (cname = "os_bcd_date_and_time")]
  public struct BCDDateAndTime
  {
    [CCode (cname = "year")]
    public unowned byte Year;
    [CCode (cname = "month")]
    public unowned byte Month;
    [CCode (cname = "date")]
    public unowned byte Date;
    [CCode (cname = "weekday")]
    public unowned byte Weekday;
    [CCode (cname = "hour")]
    public unowned byte Hour;
    [CCode (cname = "minute")]
    public unowned byte Minute;
    [CCode (cname = "second")]
    public unowned byte Second;
  }

  [CCode (cname = "os_station_number")]
  public struct StationNumber
  {
    [CCode (cname = "station")]
    public unowned int Station;
    [CCode (cname = "net")]
    public unowned int Net;
  }

  [CCode (cname = "os_box")]
  public struct Box
  {
    [CCode (cname = "x0")]
    public unowned int X0;
    [CCode (cname = "y0")]
    public unowned int Y0;
    [CCode (cname = "x1")]
    public unowned int X1;
    [CCode (cname = "y1")]
    public unowned int Y1;
  }

  [CCode (cname = "os_factors")]
  public struct Factors
  {
    [CCode (cname = "xmul")]
    public unowned int Xmul;
    [CCode (cname = "ymul")]
    public unowned int Ymul;
    [CCode (cname = "xdiv")]
    public unowned int Xdiv;
    [CCode (cname = "ydiv")]
    public unowned int Ydiv;
  }

  [CCode (cname = "os_change_box")]
  public struct ChangeBox
  {
    [CCode (cname = "state")]
    public unowned int State;
    [CCode (cname = "box")]
    public unowned Box Box;
  }

  [CCode (cname = "os_error")]
  public struct Error
  {
    [CCode (cname = "errnum")]
    public unowned bits Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[252];
  }

  [CCode (cname = "os_date_and_time")]
  public enum DateAndTime
  {
    Default = 0
  }

  [CCode (cname = "os_coord")]
  public struct Coord
  {
    [CCode (cname = "x")]
    public unowned int X;
    [CCode (cname = "y")]
    public unowned int Y;
  }

  [CCode (cname = "os_trfm")]
  public struct Trfm
  {
    [CCode (cname = "entries")]
    public unowned int[,] Entries;
  }

  [CCode (cname = "os_hom_trfm")]
  public struct HomTrfm
  {
    [CCode (cname = "entries")]
    public unowned int[,] Entries;
  }

  [CCode (cname = "os_pgm")]
  public struct Pgm
  {
    [CCode (cname = "coords")]
    public unowned Coord Coords[4];
  }

  [CCode (cname = "os_error_handler_buffer")]
  public struct ErrorHandlerBuffer
  {
    [CCode (cname = "pc")]
    public unowned int Pc;
    [CCode (cname = "errnum")]
    public unowned bits Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[248];
  }

  [CCode (cname = "os_string_value")]
  public struct StringValue
  {
    [CCode (cname = "offset")]
    public unowned int Offset;
    [CCode (cname = "pointer")]
    public unowned string Pointer;
  }

  [CCode (cname = "os_mode_var")]
  public enum ModeVar
  {
    [CCode (cname = "os_MODEVAR_MODE_FLAGS")]
    ModevarModeFlags,
    [CCode (cname = "os_MODEVAR_RSCREEN")]
    ModevarRScreen,
    [CCode (cname = "os_MODEVAR_BSCREEN")]
    ModevarBScreen,
    [CCode (cname = "os_MODEVAR_NCOLOUR")]
    ModevarNColour,
    [CCode (cname = "os_MODEVAR_XEIG_FACTOR")]
    ModevarXEigFactor,
    [CCode (cname = "os_MODEVAR_YEIG_FACTOR")]
    ModevarYEigFactor,
    [CCode (cname = "os_MODEVAR_LINE_LENGTH")]
    ModevarLineLength,
    [CCode (cname = "os_MODEVAR_SCREEN_SIZE")]
    ModevarScreenSize,
    [CCode (cname = "os_MODEVAR_YSHIFT_FACTOR")]
    ModevarYShiftFactor,
    [CCode (cname = "os_MODEVAR_LOG2_BPP")]
    ModevarLog2BPP,
    [CCode (cname = "os_MODEVAR_LOG2_BPC")]
    ModevarLog2BPC,
    [CCode (cname = "os_MODEVAR_XWIND_LIMIT")]
    ModevarXWindLimit,
    [CCode (cname = "os_MODEVAR_YWIND_LIMIT")]
    ModevarYWindLimit
  }

  [CCode (cname = "os_plot_code")]
  public enum PlotCode
  {
    [CCode (cname = "os_PLOT_SOLID")]
    PlotSolid,
    [CCode (cname = "os_PLOT_SOLID_EX_END")]
    PlotSolidExEnd,
    [CCode (cname = "os_PLOT_DOTTED")]
    PlotDotted,
    [CCode (cname = "os_PLOT_DOTTED_EX_END")]
    PlotDottedExEnd,
    [CCode (cname = "os_PLOT_SOLID_EX_START")]
    PlotSolidExStart,
    [CCode (cname = "os_PLOT_SOLID_EX_BOTH")]
    PlotSolidExBoth,
    [CCode (cname = "os_PLOT_DOTTED_EX_START")]
    PlotDottedExStart,
    [CCode (cname = "os_PLOT_DOTTED_EX_BOTH")]
    PlotDottedExBoth,
    [CCode (cname = "os_PLOT_POINT")]
    PlotPoint,
    [CCode (cname = "os_PLOT_HORIZONTAL_TO_NON_BG")]
    PlotHorizontalToNonBG,
    [CCode (cname = "os_PLOT_TRIANGLE")]
    PlotTriangle,
    [CCode (cname = "os_PLOT_RIGHT_TO_BG")]
    PlotRightToBG,
    [CCode (cname = "os_PLOT_RECTANGLE")]
    PlotRectangle,
    [CCode (cname = "os_PLOT_HORIZONTAL_TO_FG")]
    PlotHorizontalToFG,
    [CCode (cname = "os_PLOT_PARALLELOGRAM")]
    PlotParallelogram,
    [CCode (cname = "os_PLOT_RIGHT_TO_NON_FG")]
    PlotRightToNonFG,
    [CCode (cname = "os_PLOT_FLOOD_TO_NON_BG")]
    PlotFloodToNonBG,
    [CCode (cname = "os_PLOT_FLOOD_TO_FG")]
    PlotFloodToFG,
    [CCode (cname = "os_PLOT_CIRCLE_OUTLINE")]
    PlotCircleOutline,
    [CCode (cname = "os_PLOT_CIRCLE")]
    PlotCircle,
    [CCode (cname = "os_PLOT_ARC")]
    PlotArc,
    [CCode (cname = "os_PLOT_SEGMENT")]
    PlotSegment,
    [CCode (cname = "os_PLOT_SECTOR")]
    PlotSector,
    [CCode (cname = "os_PLOT_BLOCK")]
    PlotBlock,
    [CCode (cname = "os_PLOT_ELLIPSE_OUTLINE")]
    PlotEllipseOutline,
    [CCode (cname = "os_PLOT_ELLIPSE")]
    PlotEllipse,
    [CCode (cname = "os_MOVE_BY")]
    MoveBy,
    [CCode (cname = "os_PLOT_BY")]
    PlotBy,
    [CCode (cname = "os_PLOT_INVERSE_BY")]
    PlotInverseBy,
    [CCode (cname = "os_PLOT_BG_BY")]
    PlotBGBy,
    [CCode (cname = "os_MOVE_TO")]
    MoveTo,
    [CCode (cname = "os_PLOT_TO")]
    PlotTo,
    [CCode (cname = "os_PLOT_INVERSE_TO")]
    PlotInverseTo,
    [CCode (cname = "os_PLOT_BG_TO")]
    PlotBGTo
  }

  [CCode (cname = "os_var_type")]
  public enum VarType
  {
    [CCode (cname = "os_VARTYPE_STRING")]
    VartypeString,
    [CCode (cname = "os_VARTYPE_NUMBER")]
    VartypeNumber,
    [CCode (cname = "os_VARTYPE_MACRO")]
    VartypeMacro,
    [CCode (cname = "os_VARTYPE_EXPANDED")]
    VartypeExpanded,
    [CCode (cname = "os_VARTYPE_LITERAL_STRING")]
    VartypeLiteralString,
    [CCode (cname = "os_VARTYPE_CODE")]
    VartypeCode
  }

  [CCode (cname = "os_mode_flags")]
  public enum ModeFlags
  {
    [CCode (cname = "os_MODE_FLAG_NON_GRAPHICS")]
    ModeFlagNonGraphics,
    [CCode (cname = "os_MODE_FLAG_TELETEXT")]
    ModeFlagTeletext,
    [CCode (cname = "os_MODE_FLAG_GAP")]
    ModeFlagGap,
    [CCode (cname = "os_MODE_FLAG_BBC_GAP")]
    ModeFlagBBCGap,
    [CCode (cname = "os_MODE_FLAG_HI_RES_MONO")]
    ModeFlagHiResMono,
    [CCode (cname = "os_MODE_FLAG_DOUBLE_HEIGHT")]
    ModeFlagDoubleHeight,
    [CCode (cname = "os_MODE_FLAG_HARDWARE_SCROLL")]
    ModeFlagHardwareScroll,
    [CCode (cname = "os_MODE_FLAG_FULL_PALETTE")]
    ModeFlagFullPalette,
    [CCode (cname = "os_MODE_FLAG_INTERLACED")]
    ModeFlagInterlaced,
    [CCode (cname = "os_MODE_FLAG_GREYSCALE")]
    ModeFlagGreyscale,
    [CCode (cname = "os_MODE_FLAG_DATA_FORMAT")]
    ModeFlagDataFormat,
    [CCode (cname = "os_MODE_FLAG_DATA_FORMAT_RGB")]
    ModeFlagDataFormatRGB,
    [CCode (cname = "os_MODE_FLAG_DATA_FORMAT_CMYK")]
    ModeFlagDataFormatCMYK
  }

  [CCode (cname = "os_vdu_var")]
  public enum VDUVar
  {
    [CCode (cname = "os_VDUVAR_MODE_FLAGS")]
    VduvarModeFlags,
    [CCode (cname = "os_VDUVAR_RSCREEN")]
    VduvarRScreen,
    [CCode (cname = "os_VDUVAR_BSCREEN")]
    VduvarBScreen,
    [CCode (cname = "os_VDUVAR_NCOLOUR")]
    VduvarNColour,
    [CCode (cname = "os_VDUVAR_XEIG_FACTOR")]
    VduvarXEigFactor,
    [CCode (cname = "os_VDUVAR_YEIG_FACTOR")]
    VduvarYEigFactor,
    [CCode (cname = "os_VDUVAR_LINE_LENGTH")]
    VduvarLineLength,
    [CCode (cname = "os_VDUVAR_SCREEN_SIZE")]
    VduvarScreenSize,
    [CCode (cname = "os_VDUVAR_YSHIFT_FACTOR")]
    VduvarYShiftFactor,
    [CCode (cname = "os_VDUVAR_LOG2_BPP")]
    VduvarLog2BPP,
    [CCode (cname = "os_VDUVAR_LOG2_BPC")]
    VduvarLog2BPC,
    [CCode (cname = "os_VDUVAR_XWIND_LIMIT")]
    VduvarXWindLimit,
    [CCode (cname = "os_VDUVAR_YWIND_LIMIT")]
    VduvarYWindLimit,
    [CCode (cname = "os_VDUVAR_GWL_COL")]
    VduvarGWLCol,
    [CCode (cname = "os_VDUVAR_GWB_ROW")]
    VduvarGWBRow,
    [CCode (cname = "os_VDUVAR_GWR_COL")]
    VduvarGWRCol,
    [CCode (cname = "os_VDUVAR_GWT_ROW")]
    VduvarGWTRow,
    [CCode (cname = "os_VDUVAR_TWL_COL")]
    VduvarTWLCol,
    [CCode (cname = "os_VDUVAR_TWB_ROW")]
    VduvarTWBRow,
    [CCode (cname = "os_VDUVAR_TWR_COL")]
    VduvarTWRCol,
    [CCode (cname = "os_VDUVAR_TWT_ROW")]
    VduvarTWTRow,
    [CCode (cname = "os_VDUVAR_ORGX")]
    VduvarOrgX,
    [CCode (cname = "os_VDUVAR_ORGY")]
    VduvarOrgY,
    [CCode (cname = "os_VDUVAR_GCSX")]
    VduvarGCsX,
    [CCode (cname = "os_VDUVAR_GCSY")]
    VduvarGCsY,
    [CCode (cname = "os_VDUVAR_OLDER_CSX")]
    VduvarOlderCsX,
    [CCode (cname = "os_VDUVAR_OLDER_CSY")]
    VduvarOlderCsY,
    [CCode (cname = "os_VDUVAR_OLD_CSX")]
    VduvarOldCsX,
    [CCode (cname = "os_VDUVAR_OLD_CSY")]
    VduvarOldCsY,
    [CCode (cname = "os_VDUVAR_GCS_IX")]
    VduvarGCsIX,
    [CCode (cname = "os_VDUVAR_GCS_IY")]
    VduvarGCsIY,
    [CCode (cname = "os_VDUVAR_NEW_PTX")]
    VduvarNewPtX,
    [CCode (cname = "os_VDUVAR_NEW_PTY")]
    VduvarNewPtY,
    [CCode (cname = "os_VDUVAR_SCREEN_START")]
    VduvarScreenStart,
    [CCode (cname = "os_VDUVAR_DISPLAY_START")]
    VduvarDisplayStart,
    [CCode (cname = "os_VDUVAR_TOTAL_SCREEN_SIZE")]
    VduvarTotalScreenSize,
    [CCode (cname = "os_VDUVAR_GPLFMD")]
    VduvarGPLFMD,
    [CCode (cname = "os_VDUVAR_GPLBMD")]
    VduvarGPLBMD,
    [CCode (cname = "os_VDUVAR_GFCOL")]
    VduvarGFCOL,
    [CCode (cname = "os_VDUVAR_GBCOL")]
    VduvarGBCOL,
    [CCode (cname = "os_VDUVAR_TFORE_COL")]
    VduvarTForeCol,
    [CCode (cname = "os_VDUVAR_TBACK_COL")]
    VduvarTBackCol,
    [CCode (cname = "os_VDUVAR_GF_TINT")]
    VduvarGFTint,
    [CCode (cname = "os_VDUVAR_GB_TINT")]
    VduvarGBTint,
    [CCode (cname = "os_VDUVAR_TF_TINT")]
    VduvarTFTint,
    [CCode (cname = "os_VDUVAR_TB_TINT")]
    VduvarTBTint,
    [CCode (cname = "os_VDUVAR_MAX_MODE")]
    VduvarMaxMode,
    [CCode (cname = "os_VDUVAR_GCHAR_SIZEX")]
    VduvarGCharSizeX,
    [CCode (cname = "os_VDUVAR_GCHAR_SIZEY")]
    VduvarGCharSizeY,
    [CCode (cname = "os_VDUVAR_GCHAR_SPACEX")]
    VduvarGCharSpaceX,
    [CCode (cname = "os_VDUVAR_GCHAR_SPACEY")]
    VduvarGCharSpaceY,
    [CCode (cname = "os_VDUVAR_HLINE_ADDR")]
    VduvarHLineAddr,
    [CCode (cname = "os_VDUVAR_TCHAR_SIZEX")]
    VduvarTCharSizeX,
    [CCode (cname = "os_VDUVAR_TCHAR_SIZEY")]
    VduvarTCharSizeY,
    [CCode (cname = "os_VDUVAR_TCHAR_SPACEX")]
    VduvarTCharSpaceX,
    [CCode (cname = "os_VDUVAR_TCHAR_SPACEY")]
    VduvarTCharSpaceY,
    [CCode (cname = "os_VDUVAR_GCOL_ORA_EOR_ADDR")]
    VduvarGCOLOraEorAddr,
    [CCode (cname = "os_VDUVAR_VIDC_CLOCK_SPEED")]
    VduvarVIDCClockSpeed,
    [CCode (cname = "os_VDUVAR_DISPLAY_NUMBER")]
    VduvarDisplayNumber,
    [CCode (cname = "os_VDUVAR_POLY_HLINE_ADDR")]
    VduvarPolyHLineAddr,
    [CCode (cname = "os_VDUVAR_WINDOW_WIDTH")]
    VduvarWindowWidth,
    [CCode (cname = "os_VDUVAR_WINDOW_HEIGHT")]
    VduvarWindowHeight,
    [CCode (cname = "os_VDUVAR_END_LIST")]
    VduvarEndList
  }

  [CCode (cname = "os_device_type")]
  public enum DeviceType
  {
    [CCode (cname = "osdevicetype_CORE_IRQ_HANDLER")]
    CoreIRQHandler,
    [CCode (cname = "os_DEVICE_PRINTER_BUSY")]
    DevicePrinterBusy,
    [CCode (cname = "os_DEVICE_PRINTER")]
    DevicePrinter,
    [CCode (cname = "os_DEVICE_SERIAL_RINGING")]
    DeviceSerialRinging,
    [CCode (cname = "os_DEVICE_BATTERY_WARNING")]
    DeviceBatteryWarning,
    [CCode (cname = "os_DEVICE_PRINTER_ACKNOWLEDGE")]
    DevicePrinterAcknowledge,
    [CCode (cname = "os_DEVICE_FLOPPY_DISC_INDEX")]
    DeviceFloppyDiscIndex,
    [CCode (cname = "os_DEVICE_VSYNC")]
    DeviceVSync,
    [CCode (cname = "os_DEVICE_POWER_ON")]
    DevicePowerOn,
    [CCode (cname = "os_DEVICE_IOC_TIMER0")]
    DeviceIOCTimer0,
    [CCode (cname = "os_DEVICE_IOC_TIMER1")]
    DeviceIOCTimer1,
    [CCode (cname = "os_DEVICE_FIQ_DOWNGRADE")]
    DeviceFIQDowngrade,
    [CCode (cname = "os_DEVICE_EXPANSION_FIQ_DOWNGRADE")]
    DeviceExpansionFIQDowngrade,
    [CCode (cname = "os_DEVICE_SOUND_BUFFER")]
    DeviceSoundBuffer,
    [CCode (cname = "os_DEVICE_IDE_DISCA")]
    DeviceIDEDiscA,
    [CCode (cname = "os_DEVICE_SERIAL_CONTROLLER")]
    DeviceSerialController,
    [CCode (cname = "os_DEVICE_SERIAL_PORT")]
    DeviceSerialPort,
    [CCode (cname = "os_DEVICE_HARD_DISC_CONTROLLER")]
    DeviceHardDiscController,
    [CCode (cname = "os_DEVICE_FLOPPY_DISC")]
    DeviceFloppyDisc,
    [CCode (cname = "os_DEVICE_NETWORK")]
    DeviceNetwork,
    [CCode (cname = "os_DEVICE_FLOPPY_DISC_CHANGED")]
    DeviceFloppyDiscChanged,
    [CCode (cname = "os_DEVICE_IDE_DISC")]
    DeviceIDEDisc,
    [CCode (cname = "os_DEVICE_FLOPPY_DISCA")]
    DeviceFloppyDiscA,
    [CCode (cname = "os_DEVICE_EXPANSION_CARD")]
    DeviceExpansionCard,
    [CCode (cname = "os_DEVICE_KEYBOARD_TX_EMPTY")]
    DeviceKeyboardTxEmpty,
    [CCode (cname = "os_DEVICE_KEYBOARD_RX_FULL")]
    DeviceKeyboardRxFull,
    [CCode (cname = "os_DEVICE_DMA_CHANNEL0")]
    DeviceDMAChannel0,
    [CCode (cname = "os_DEVICE_DMA_CHANNEL1")]
    DeviceDMAChannel1,
    [CCode (cname = "os_DEVICE_DMA_CHANNEL2")]
    DeviceDMAChannel2,
    [CCode (cname = "os_DEVICE_DMA_CHANNEL3")]
    DeviceDMAChannel3,
    [CCode (cname = "os_DEVICE_DMA_SOUND_CHANNEL0")]
    DeviceDMASoundChannel0,
    [CCode (cname = "os_DEVICE_DMA_SOUND_CHANNEL1")]
    DeviceDMASoundChannel1,
    [CCode (cname = "os_DEVICE_MOUSE_RX_FULL")]
    DeviceMouseRxFull,
    [CCode (cname = "os_DEVICE_MOUSE_TX_EMPTY")]
    DeviceMouseTxEmpty,
    [CCode (cname = "os_DEVICE_JOYSTICK_CONVERSION")]
    DeviceJoystickConversion,
    [CCode (cname = "os_DEVICE_EVENT0")]
    DeviceEvent0,
    [CCode (cname = "os_DEVICE_EVENT1")]
    DeviceEvent1,
    [CCode (cname = "osdevicetype_GET_IRQ_HANDLER")]
    GetIRQHandler,
    [CCode (cname = "osdevicetype_REGISTER_SUB_CHAIN")]
    RegisterSubChain,
    [CCode (cname = "osdevicetype_PASS_ON")]
    PassOn
  }

  [CCode (cname = "osreadline_flags")]
  public enum OSReadLine_Flags
  {
    [CCode (cname = "osreadline_GIVEN_ECHO_CHARACTER")]
    GivenEchoCharacter,
    [CCode (cname = "osreadline_SUPPRESS_INVALID")]
    SuppressInvalid,
    [CCode (cname = "osreadline_ECHO_CHARACTER")]
    EchoCharacter
  }

  [CCode (cname = "os_register_block")]
  public struct RegisterBlock
  {
    [CCode (cname = "registers")]
    public unowned int Registers[16];
    [CCode (cname = "cpsr")]
    public unowned int Cpsr;
  }

  [CCode (cname = "os_read_unsigned_flags")]
  public enum ReadUnsignedFlags
  {
    [CCode (cname = "os_READ_CONTROL_TERMINATED")]
    ReadControlTerminated,
    [CCode (cname = "os_READ_BYTE")]
    ReadByte,
    [CCode (cname = "os_READ_LIMITED")]
    ReadLimited
  }

  [CCode (cname = "os_gs_flags")]
  public enum GSFlags
  {
    [CCode (cname = "os_GS_TERMINATE_ON_SPACE")]
    GSTerminateOnSpace,
    [CCode (cname = "os_GS_RETAIN_STICK")]
    GSRetainStick,
    [CCode (cname = "os_GS_RETAIN_DOUBLE_QUOTE")]
    GSRetainDoubleQuote
  }

  [CCode (cname = "os_dynamic_area_no")]
  public enum DynamicAreaNo
  {
    [CCode (cname = "os_DYNAMIC_AREA_APPLICATION_SPACE")]
    DynamicAreaApplicationSpace,
    [CCode (cname = "os_DYNAMIC_AREA_SYSTEM_HEAP")]
    DynamicAreaSystemHeap,
    [CCode (cname = "os_DYNAMIC_AREA_RMA")]
    DynamicAreaRMA,
    [CCode (cname = "os_DYNAMIC_AREA_SCREEN_MEMORY")]
    DynamicAreaScreenMemory,
    [CCode (cname = "os_DYNAMIC_AREA_SYSTEM_SPRITES")]
    DynamicAreaSystemSprites,
    [CCode (cname = "os_DYNAMIC_AREA_FONT_CACHE")]
    DynamicAreaFontCache,
    [CCode (cname = "os_DYNAMIC_AREA_RAM_DISC")]
    DynamicAreaRAMDisc,
    [CCode (cname = "os_DYNAMIC_AREA_FREE_POOL")]
    DynamicAreaFreePool,
    [CCode (cname = "os_DYNAMIC_VRAM_RESCUE")]
    DynamicVRAMRescue,
    [CCode (cname = "os_DYNAMIC_KERNEL_BUFFERS")]
    DynamicKernelBuffers,
    [CCode (cname = "os_DYNAMIC_ABORT_SVC_AND_IRQ_STACK_COPY")]
    DynamicAbortSVCAndIRQStackCopy,
    [CCode (cname = "os_DYNAMIC_OPERATING_SYSTEM_ROM")]
    DynamicOperatingSystemROM,
    [CCode (cname = "os_DYNAMIC_SWI_DISPATCH_IRQ_WORKSPACE")]
    DynamicSWIDispatchIRQWorkspace,
    [CCode (cname = "os_DYNAMIC_SVC_STACK")]
    DynamicSVCStack,
    [CCode (cname = "os_DYNAMIC_IRQ_STACK")]
    DynamicIRQStack,
    [CCode (cname = "os_DYNAMIC_UND_STACK")]
    DynamicUNDStack,
    [CCode (cname = "os_DYNAMIC_LEVEL2_PAGE_TABLES")]
    DynamicLevel2PageTables,
    [CCode (cname = "os_DYNAMIC_SOFT_CAM")]
    DynamicSoftCAM
  }

  [CCode (cname = "os_vdu_var_list")]
  public struct VDUVarList
  {
    [CCode (cname = "var", array_length = false)]
    public unowned VDUVar[] Var;
  }

  [CCode (cname = "os_key_handler")]
  public struct KeyHandler
  {
    [CCode (cname = "k", array_length = false)]
    public unowned byte[] K;
  }

  [CCode (cname = "os_handler_type")]
  public enum HandlerType
  {
    [CCode (cname = "os_HANDLER_MEMORY_LIMIT")]
    HandlerMemoryLimit,
    [CCode (cname = "os_HANDLER_UNDEFINED_INSTRUCTION")]
    HandlerUndefinedInstruction,
    [CCode (cname = "os_HANDLER_PREFETCH_ABORT")]
    HandlerPrefetchAbort,
    [CCode (cname = "os_HANDLER_DATA_ABORT")]
    HandlerDataAbort,
    [CCode (cname = "os_HANDLER_ADDRESS_EXCEPTION")]
    HandlerAddressException,
    [CCode (cname = "os_HANDLER_OTHER_EXCEPTIONS")]
    HandlerOtherExceptions,
    [CCode (cname = "os_HANDLER_ERROR")]
    HandlerError,
    [CCode (cname = "os_HANDLER_CALL_BACK")]
    HandlerCallBack,
    [CCode (cname = "os_HANDLER_BREAK_PT")]
    HandlerBreakPt,
    [CCode (cname = "os_HANDLER_ESCAPE")]
    HandlerEscape,
    [CCode (cname = "os_HANDLER_EVENT")]
    HandlerEvent,
    [CCode (cname = "os_HANDLER_EXIT")]
    HandlerExit,
    [CCode (cname = "os_HANDLER_UNUSED_SWI")]
    HandlerUnusedSWI,
    [CCode (cname = "os_HANDLER_EXCEPTION_REGISTERS")]
    HandlerExceptionRegisters,
    [CCode (cname = "os_HANDLER_APPLICATION_SPACE")]
    HandlerApplicationSpace,
    [CCode (cname = "os_HANDLER_CAO")]
    HandlerCAO,
    [CCode (cname = "os_HANDLER_UP_CALL")]
    HandlerUpCall,
    [CCode (cname = "os_HANDLER_BACKWARD_COMPATIBILITY")]
    HandlerBackwardCompatibility
  }

  [CCode (cname = "os_substitute_args_flags")]
  public enum SubstituteArgsFlags
  {
    [CCode (cname = "os_SUBSTITUTE_ARGS_IGNORE_EXTRAS")]
    SubstituteArgsIgnoreExtras
  }

  [CCode (cname = "os_gi")]
  public struct GI
  {
    [CCode (cname = "type")]
    public unowned byte Type;
    [CCode (cname = "i")]
    public unowned byte I[4];
  }

  [CCode (cname = "os_gs_base")]
  public struct GSBase
  {
    [CCode (cname = "size")]
    public unowned short Size;
  }

  [Compact]
  [CCode (cname = "os_gs", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class GS
  {
    public GS(int numEntries);
    [CCode (cname = "size")]
    public unowned short Size;
    [CCode (cname = "s", array_length = false)]
    public unowned char[] S;
  }

  [CCode (cname = "os_sort_type")]
  public enum SortType
  {
    [CCode (cname = "os_SORT_CARDINAL")]
    SortCardinal,
    [CCode (cname = "os_SORT_INTEGER")]
    SortInteger,
    [CCode (cname = "os_SORT_CARDINAL_POINTER")]
    SortCardinalPointer,
    [CCode (cname = "os_SORT_INTEGER_POINTER")]
    SortIntegerPointer,
    [CCode (cname = "os_SORT_CI_STRING")]
    SortCIString,
    [CCode (cname = "os_SORT_STRING")]
    SortString
  }

  [CCode (cname = "os_heap_sort_flags", has_type_id = false)]
  public struct HeapSortFlags : bits
  {
  }

  [CCode (cname = "os_area_flags")]
  public enum AreaFlags
  {
    [CCode (cname = "os_AREA_ACCESS_READ_WRITE")]
    AreaAccessReadWrite,
    [CCode (cname = "os_AREA_ACCESS_READ_ONLY")]
    AreaAccessReadOnly,
    [CCode (cname = "os_AREA_ACCESS_NONE")]
    AreaAccessNone,
    [CCode (cname = "os_AREA_NOT_BUFFERABLE")]
    AreaNotBufferable,
    [CCode (cname = "os_AREA_NOT_CACHEABLE")]
    AreaNotCacheable,
    [CCode (cname = "os_AREA_DOUBLY_MAPPED")]
    AreaDoublyMapped,
    [CCode (cname = "os_AREA_NO_USER_DRAG")]
    AreaNoUserDrag,
    [CCode (cname = "os_AREA_NEEDS_GIVEN_PAGES")]
    AreaNeedsGivenPages,
    [CCode (cname = "os_AREA_SHRINKABLE")]
    AreaShrinkable,
    [CCode (cname = "os_AREA_SPARSE")]
    AreaSparse,
    [CCode (cname = "os_AREA_PRIVATE")]
    AreaPrivate,
    [CCode (cname = "os_AREA_DMA")]
    AreaDMA,
    [CCode (cname = "os_AREA_LOCKED")]
    AreaLocked,
    [CCode (cname = "os_AREA_IS_HEAP")]
    AreaIsHeap,
    [CCode (cname = "os_AREA_PHYSICAL")]
    AreaPhysical,
    [CCode (cname = "os_AREA_ABORTABLE")]
    AreaAbortable,
    [CCode (cname = "os_AREA_DOMAIN")]
    AreaDomain,
    [CCode (cname = "os_AREA_INVISIBLE")]
    AreaInvisible,
    [CCode (cname = "os_AREA_FIXED_PAGE")]
    AreaFixedPage
  }

  [CCode (cname = "os_mem_map_request")]
  public struct MemMapRequest
  {
    [CCode (cname = "page_no")]
    public unowned int PageNo;
    [CCode (cname = "map")]
    public unowned char* Map;
    [CCode (cname = "access")]
    public unowned AreaFlags Access;
  }

  [CCode (cname = "os_mem_map_request_list")]
  public struct MemMapRequestList
  {
    [CCode (cname = "requests", array_length = false)]
    public unowned MemMapRequest[] Requests;
  }

  [CCode (cname = "os_page_block")]
  public struct PageBlock
  {
    [CCode (cname = "page_no")]
    public unowned int PageNo;
    [CCode (cname = "log_addr")]
    public unowned char* LogAddr;
    [CCode (cname = "phys_addr")]
    public unowned char* PhysAddr;
  }

  [CCode (cname = "os_changed_box_state")]
  public enum ChangedBoxState
  {
    [CCode (cname = "os_DISABLE_CHANGED_BOX")]
    DisableChangedBox,
    [CCode (cname = "os_ENABLE_CHANGED_BOX")]
    EnableChangedBox,
    [CCode (cname = "os_RESET_CHANGED_BOX")]
    ResetChangedBox,
    [CCode (cname = "os_READ_CHANGED_BOX")]
    ReadChangedBox
  }

  [CCode (cname = "os_colour_flags")]
  public enum ColourFlags
  {
    [CCode (cname = "os_COLOUR_SET_BG")]
    ColourSetBG,
    [CCode (cname = "os_COLOUR_GIVEN_PATTERN")]
    ColourGivenPattern,
    [CCode (cname = "os_COLOUR_SET_TEXT")]
    ColourSetText,
    [CCode (cname = "os_COLOUR_READ")]
    ColourRead
  }

  [CCode (cname = "os_pointer_type")]
  public enum PointerType
  {
    [CCode (cname = "pointerv_TYPE_QUADRATURE")]
    TypeQuadrature,
    [CCode (cname = "pointerv_TYPE_MICROSOFT")]
    TypeMicrosoft,
    [CCode (cname = "pointerv_TYPE_MOUSE_SYSTEMS")]
    TypeMouseSystems,
    [CCode (cname = "pointerv_TYPE_PS2")]
    TypePS2
  }

  [CCode (cname = "osscreenmodeconfigacceleration_flags")]
  public enum OSScreenModeConfigAcceleration_Flags
  {
    [CCode (cname = "osscreenmodeconfigaccelerationflags_SUSPEND_CACHEING")]
    SuspendCacheing,
    [CCode (cname = "osscreenmodeconfigaccelerationflags_SUSPEND_CLEANING")]
    SuspendCleaning,
    [CCode (cname = "osscreenmodeconfigaccelerationflags_DISABLE_HW_ACCELERATION")]
    DisableHWAcceleration
  }

  [CCode (cname = "osscreenmodeconfigacceleration_cleaner_laziness")]
  public enum OSScreenModeConfigAcceleration_CleanerLaziness
  {
    [CCode (cname = "osscreenmode_READ_LAZINESS")]
    ReadLaziness,
    [CCode (cname = "osscreenmode_SECOND_VSYNC")]
    SecondVSync,
    [CCode (cname = "osscreenmode_THIRD_VSYNC")]
    ThirdVSync
  }

  [CCode (cname = "osdisplaydevice_tag_tag_length")]
  public enum OSDisplayDevice_TagTagLength
  {
    [CCode (cname = "osdisplaydevicetag_END_LIST")]
    EndList,
    [CCode (cname = "osdisplaydevicetag_DEVICE_NAME")]
    DeviceName,
    [CCode (cname = "osdisplaydevicetag_FEATURES")]
    Features,
    [CCode (cname = "osdisplaydevicetag_CAPABILITIES")]
    Capabilities,
    [CCode (cname = "osdisplaydevicetag_BANDWIDTH")]
    Bandwidth,
    [CCode (cname = "osdisplaydevicetag_MAX_PHYS_MEM")]
    MaxPhysMem,
    [CCode (cname = "osdisplaydevicetag_ALIGNMENT")]
    Alignment,
    [CCode (cname = "osdisplaydevicetag_MODEL")]
    Model,
    [CCode (cname = "osdisplaydevicetag_DEVICE_SPECIFIC")]
    DeviceSpecific
  }

  [CCode (cname = "osdisplaydevicedescriptor")]
  public struct OSDisplayDeviceDescriptor
  {
    [CCode (cname = "entry.tag_taglength")]
    public unowned OSDisplayDevice_TagTagLength Entry_TagTaglength;
    [CCode (cname = "entry.value")]
    public unowned int Entry_Value;
  }

  [CCode (cname = "osscreenmoderegisterdisplaydriver_action")]
  public enum OSScreenModeRegisterDisplayDriver_Action
  {
    [CCode (cname = "osscreenmoderegisterdisplaydriveraction_NONE")]
    None,
    [CCode (cname = "osscreenmoderegisterdisplaydriveraction_SELECT_DISPLAY")]
    SelectDisplay
  }

  [CCode (cname = "osdynamicareadomain_state")]
  public enum OSDynamicAreaDomain_State
  {
    [CCode (cname = "osdynamicareadomainstate_READ")]
    Read,
    [CCode (cname = "osdynamicareadomainstate_ABORT")]
    Abort,
    [CCode (cname = "osdynamicareadomainstate_NORMAL")]
    Normal,
    [CCode (cname = "osdynamicareadomainstate_MANAGER")]
    Manager
  }

  [CCode (cname = "osaborttrap_flags")]
  public enum OSAbortTrap_Flags
  {
    [CCode (cname = "osaborttrap_REGISTER")]
    Register,
    [CCode (cname = "osaborttrap_DEREGISTER")]
    Deregister
  }

  [CCode (cname = "osmemory_flags")]
  public enum OSMemory_Flags
  {
    [CCode (cname = "osmemory_GIVEN_PAGE_NO")]
    GivenPageNo,
    [CCode (cname = "osmemory_GIVEN_LOG_ADDR")]
    GivenLogAddr,
    [CCode (cname = "osmemory_GIVEN_PHYS_ADDR")]
    GivenPhysAddr,
    [CCode (cname = "osmemory_RETURN_PAGE_NO")]
    ReturnPageNo,
    [CCode (cname = "osmemory_RETURN_LOG_ADDR")]
    ReturnLogAddr,
    [CCode (cname = "osmemory_RETURN_PHYS_ADDR")]
    ReturnPhysAddr,
    [CCode (cname = "osmemory_CACHE")]
    Cache,
    [CCode (cname = "osmemory_CACHE_DISABLE")]
    CacheDisable,
    [CCode (cname = "osmemory_CACHE_ENABLE")]
    CacheEnable
  }

  [CCode (cname = "osmemory_type_flags")]
  public enum OSMemory_TypeFlags
  {
    [CCode (cname = "osmemory_TYPE")]
    Type,
    [CCode (cname = "osmemory_TYPE_ABSENT")]
    TypeAbsent,
    [CCode (cname = "osmemory_TYPE_DRAM")]
    TypeDRAM,
    [CCode (cname = "osmemory_TYPE_VRAM")]
    TypeVRAM,
    [CCode (cname = "osmemory_TYPE_ROM")]
    TypeROM,
    [CCode (cname = "osmemory_TYPE_IO")]
    TypeIO,
    [CCode (cname = "osmemory_TYPE_SOFT_ROM")]
    TypeSoftROM
  }

  [CCode (cname = "osmemory_controller_flags")]
  public enum OSMemory_ControllerFlags
  {
    Default = 0
  }

  [CCode (cname = "osmemory_io_flags")]
  public enum OSMemory_IOFlags
  {
    [CCode (cname = "osmemory_IO_BUFFERABLE")]
    IOBufferable,
    [CCode (cname = "osmemory_IO_CACHEABLE")]
    IOCacheable,
    [CCode (cname = "osmemory_IO_DOUBLY_MAPPED")]
    IODoublyMapped,
    [CCode (cname = "osmemory_IO_RESTRICT_ACCESS")]
    IORestrictAccess,
    [CCode (cname = "osmemory_IO_ACCESS")]
    IOAccess
  }

  [CCode (cname = "osmemory_io_reference_key", has_type_id = false)]
  public struct OSMemory_IOReferenceKey : bits
  {
  }

  [CCode (cname = "osmemory_area_flags")]
  public enum OSMemory_AreaFlags
  {
    [CCode (cname = "osmemory_AREA")]
    Area,
    [CCode (cname = "osmemory_AREA_CURSOR_SYSTEM_SOUND")]
    AreaCursorSystemSound,
    [CCode (cname = "osmemory_AREA_IRQ_STACK")]
    AreaIRQStack,
    [CCode (cname = "osmemory_AREA_SVC_STACK")]
    AreaSVCStack,
    [CCode (cname = "osmemory_AREA_ABT_STACK")]
    AreaABTStack,
    [CCode (cname = "osmemory_AREA_UND_STACK")]
    AreaUNDStack,
    [CCode (cname = "osmemory_AREA_SOFT_CAM")]
    AreaSoftCAM,
    [CCode (cname = "osmemory_AREA_LEVEL_ONE_PAGE_TABLES")]
    AreaLevelOnePageTables,
    [CCode (cname = "osmemory_AREA_LEVEL_TWO_PAGE_TABLES")]
    AreaLevelTwoPageTables,
    [CCode (cname = "osmemory_AREA_HAL_WORKSPACE")]
    AreaHALWorkspace,
    [CCode (cname = "osmemory_AREA_KERNEL_BUFFERS")]
    AreaKernelBuffers,
    [CCode (cname = "osmemory_AREA_UNCACHEABLE_HAL_WORKSPACE")]
    AreaUncacheableHALWorkspace
  }

  [CCode (cname = "osmemoryrecommendpage_flags")]
  public enum OSMemoryRecommendPage_Flags
  {
    [CCode (cname = "osmemoryrecommendpageflags_DMA")]
    DMA
  }

  [CCode (cname = "osmemorycheckmemoryaccess_flags")]
  public enum OSMemoryCheckMemoryAccess_Flags
  {
    Default = 0
  }

  [CCode (cname = "osmemorycheckmemoryaccess_bits")]
  public enum OSMemoryCheckMemoryAccess_Bits
  {
    [CCode (cname = "osmemorycheckmemoryaccessbits_USER_RFULL")]
    UserRFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_USER_WFULL")]
    UserWFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_PRIV_RFULL")]
    PrivRFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_PRIV_WFULL")]
    PrivWFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_USER_RPART")]
    UserRPart,
    [CCode (cname = "osmemorycheckmemoryaccessbits_USER_WPART")]
    UserWPart,
    [CCode (cname = "osmemorycheckmemoryaccessbits_PRIV_RPART")]
    PrivRPart,
    [CCode (cname = "osmemorycheckmemoryaccessbits_PHYS_FULL")]
    PhysFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_ABORT_FULL")]
    AbortFull,
    [CCode (cname = "osmemorycheckmemoryaccessbits_PHYS_PART")]
    PhysPart,
    [CCode (cname = "osmemorycheckmemoryaccessbits_ABORT_PART")]
    AbortPart
  }

  [CCode (cname = "osmemoryreadcontrollerbaseaddress_flags")]
  public enum OSMemoryReadControllerBaseAddress_Flags
  {
    Default = 0
  }

  [CCode (cname = "os_mmu_control_flush_request_flags")]
  public enum MMUControlFlushRequestFlags
  {
    [CCode (cname = "os_MMU_CONTROL_FLUSH_REQUEST_FLAG_SINGLE_ENTRY")]
    MMUControlFlushRequestFlagSingleEntry,
    [CCode (cname = "os_MMU_CONTROL_FLUSH_REQUEST_FLAG_PROC_TLB")]
    MMUControlFlushRequestFlagProcTLB,
    [CCode (cname = "os_MMU_CONTROL_FLUSH_REQUEST_FLAG_PROC_CACHE")]
    MMUControlFlushRequestFlagProcCache
  }

  [CCode (cname = "os_platform_feature_flags")]
  public enum PlatformFeatureFlags
  {
    [CCode (cname = "os_PLATFORM_CODE_NEEDS_SYNCHRONISATION")]
    PlatformCodeNeedsSynchronisation,
    [CCode (cname = "os_PLATFORM_NEEDS_PREDISABLE_FUNCTION")]
    PlatformNeedsPredisableFunction,
    [CCode (cname = "os_PLATFORM_EXCEPTION_VECTOR_MODIFY_NEEDS32BIT")]
    PlatformExceptionVectorModifyNeeds32bit,
    [CCode (cname = "os_PLATFORM_PIPELINE_LENGTH2")]
    PlatformPipelineLength2,
    [CCode (cname = "os_PLATFORM_FULL_EARLY_DATA_ABORTS")]
    PlatformFullEarlyDataAborts,
    [CCode (cname = "os_PLATFORM32_BIT_OS")]
    Platform32BitOS,
    [CCode (cname = "os_PLATFORM26_BIT_MODE_NOT_AVAILABLE")]
    Platform26BitModeNotAvailable
  }

  [CCode (cname = "os_synchronise_code_flags")]
  public enum SynchroniseCodeFlags
  {
    [CCode (cname = "os_GIVEN_CODE_RANGE")]
    GivenCodeRange
  }

  [CCode (cname = "os_amb")]
  public enum AMB
  {
    Default = 0
  }

  [CCode (cname = "os_amb_control_flags")]
  public enum AMBControlFlags
  {
    [CCode (cname = "os_GIVEN_AMB_PAGE_RANGE")]
    GivenAMBPageRange
  }

  [CCode (cname = "os_lazy_swapping")]
  public enum LazySwapping
  {
    [CCode (cname = "os_LAZY_SWAPPING_DISABLE")]
    Disable,
    [CCode (cname = "os_LAZY_SWAPPING_ENABLE")]
    Enable,
    [CCode (cname = "os_LAZY_SWAPPING_READ")]
    Read
  }

  [CCode (cname = "os_amb_flags")]
  public enum AMBFlags
  {
    Default = 0
  }

  [CCode (cname = "os_amb_info")]
  public struct AMBInfo
  {
    [CCode (cname = "amb")]
    public unowned AMB Amb;
    [CCode (cname = "pages")]
    public unowned int Pages;
    [CCode (cname = "base_address")]
    public unowned char* BaseAddress;
    [CCode (cname = "flags")]
    public unowned AMBFlags Flags;
  }

  [CCode (cname = "os_amb_table_base")]
  public struct AMBTableBase
  {
    [CCode (cname = "count")]
    public unowned int Count;
    [CCode (cname = "current_amb")]
    public unowned AMB CurrentAmb;
  }

  [Compact]
  [CCode (cname = "os_amb_table", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class AMBTable
  {
    public AMBTable(int numEntries);
    [CCode (cname = "count")]
    public unowned int Count;
    [CCode (cname = "current_amb")]
    public unowned AMB CurrentAmb;
    [CCode (cname = "entries", array_length = false)]
    public unowned AMBInfo[] Entries;
  }

  [CCode (cname = "os_hardware_flags")]
  public enum HardwareFlags
  {
    Default = 0
  }

  [CCode (cname = "keyv_type")]
  public enum KeyV_Type
  {
    [CCode (cname = "keyv_TYPE_ARCHIMEDES")]
    TypeArchimedes,
    [CCode (cname = "keyv_TYPE_PCAT")]
    TypePCAT
  }

  [CCode (cname = "palettev_screen_state")]
  public enum PaletteV_ScreenState
  {
    [CCode (cname = "palettev_SCREEN_STATE_READ")]
    ScreenStateRead,
    [CCode (cname = "palettev_SCREEN_STATE_NORMAL")]
    ScreenStateNormal,
    [CCode (cname = "palettev_SCREEN_STATE_QUIET")]
    ScreenStateQuiet
  }

  [CCode (cname = "pointerv_record")]
  public struct PointerV_Record
  {
    [CCode (cname = "next")]
    public unowned PointerV_Record* Next;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "pointer_type")]
    public unowned PointerType PointerType;
    [CCode (cname = "name")]
    public unowned char Name[30];
  }

  [CCode (cname = "os_mode_block_base")]
  public struct ModeBlockBase
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "xres")]
    public unowned int Xres;
    [CCode (cname = "yres")]
    public unowned int Yres;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "frame_rate")]
    public unowned int FrameRate;
  }

  [Compact]
  [CCode (cname = "os_mode_block", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ModeBlock
  {
    public ModeBlock(int numEntries);
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "xres")]
    public unowned int Xres;
    [CCode (cname = "yres")]
    public unowned int Yres;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "frame_rate")]
    public unowned int FrameRate;
    [CCode (cname = "name", array_length = false)]
    public unowned char[] Name;
  }

  [CCode (cname = "os_mode_selector_base")]
  public struct ModeSelectorBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "xres")]
    public unowned int Xres;
    [CCode (cname = "yres")]
    public unowned int Yres;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "frame_rate")]
    public unowned int FrameRate;
  }

  [Compact]
  [CCode (cname = "os_mode_selector", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ModeSelector
  {
    public ModeSelector(int numEntries);
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "xres")]
    public unowned int Xres;
    [CCode (cname = "yres")]
    public unowned int Yres;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "frame_rate")]
    public unowned int FrameRate;
    [CCode (cname = "modevars.var")]
    public unowned ModeVar Modevars_Var;
    [CCode (cname = "modevars.val")]
    public unowned int Modevars_Val;
  }

  [CCode (cname = "servicedisplaychanged_state")]
  public enum ServiceDisplayChanged_State
  {
    [CCode (cname = "servicedisplaychangedstate_IN_PROCESS")]
    InProcess,
    [CCode (cname = "servicedisplaychangedstate_DONE")]
    Done
  }

  [CCode (cname = "servicedisplaystatus_reason")]
  public enum ServiceDisplayStatus_Reason
  {
    [CCode (cname = "servicedisplaystatusreason_DEREGISTERED")]
    Deregistered,
    [CCode (cname = "servicedisplaystatusreason_REGISTERED")]
    Registered
  }

  [CCode (cname="os_ERROR_LIMIT")]
  public const int ErrorLimit;
  [CCode (cname="os_FILE_NAME_LIMIT")]
  public const int FileNameLimit;
  [CCode (cname="os_CLI_LIMIT")]
  public const int CLILimit;
  [CCode (cname="os_CLI_LIMIT_RO4")]
  public const int CLILimitRO4;
  [CCode (cname="os_RSHIFT")]
  public const int RShift;
  [CCode (cname="os_GSHIFT")]
  public const int GShift;
  [CCode (cname="os_BSHIFT")]
  public const int BShift;
  [CCode (cname="os_R")]
  public const bits R;
  [CCode (cname="os_G")]
  public const bits G;
  [CCode (cname="os_B")]
  public const bits B;
  [CCode (cname="os_COLOUR_RANGE")]
  public const int ColourRange;
  [CCode (cname="os_INCH")]
  public const int Inch;
  [CCode (cname="os_VDU_NULL")]
  public const char VDUNull;
  [CCode (cname="os_VDU_CHAR_TO_PRINTER")]
  public const char VDUCharToPrinter;
  [CCode (cname="os_VDU_PRINTER_ON")]
  public const char VDUPrinterOn;
  [CCode (cname="os_VDU_PRINTER_OFF")]
  public const char VDUPrinterOff;
  [CCode (cname="os_VDU_GRAPH_TEXT_OFF")]
  public const char VDUGraphTextOff;
  [CCode (cname="os_VDU_SPLIT_CURSORS")]
  public const char VDUSplitCursors;
  [CCode (cname="os_VDU_GRAPH_TEXT_ON")]
  public const char VDUGraphTextOn;
  [CCode (cname="os_VDU_JOIN_CURSORS")]
  public const char VDUJoinCursors;
  [CCode (cname="os_VDU_SCREEN_ON")]
  public const char VDUScreenOn;
  [CCode (cname="os_VDU_VDU_ON")]
  public const char VDUVduOn;
  [CCode (cname="os_VDU_BELL")]
  public const char VDUBell;
  [CCode (cname="os_VDU_BACKSPACE")]
  public const char VDUBackspace;
  [CCode (cname="os_VDU_TAB")]
  public const char VDUTab;
  [CCode (cname="os_VDU_LINEFEED")]
  public const char VDULinefeed;
  [CCode (cname="os_VDU_LINE_FEED")]
  public const char VDULineFeed;
  [CCode (cname="os_VDU_VERTICAL_TAB")]
  public const char VDUVerticalTab;
  [CCode (cname="os_VDU_CLS")]
  public const char VDUClS;
  [CCode (cname="os_VDU_RETURN")]
  public const char VDUReturn;
  [CCode (cname="os_VDU_PAGE_MODE_ON")]
  public const char VDUPageModeOn;
  [CCode (cname="os_VDU_PAGE_MODE_OFF")]
  public const char VDUPageModeOff;
  [CCode (cname="os_VDU_CLG")]
  public const char VDUClG;
  [CCode (cname="os_VDU_SET_TEXT_COLOUR")]
  public const char VDUSetTextColour;
  [CCode (cname="os_VDU_SET_GCOL")]
  public const char VDUSetGCOL;
  [CCode (cname="os_VDU_SET_PALETTE")]
  public const char VDUSetPalette;
  [CCode (cname="os_VDU_RESET_COLOURS")]
  public const char VDUResetColours;
  [CCode (cname="os_VDU_SCREEN_OFF")]
  public const char VDUScreenOff;
  [CCode (cname="os_VDU_VDU_OFF")]
  public const char VDUVduOff;
  [CCode (cname="os_VDU_MODE")]
  public const char VDUMode;
  [CCode (cname="os_VDU_SET_MODE")]
  public const char VDUSetMode;
  [CCode (cname="os_VDU_MISC")]
  public const char VDUMisc;
  [CCode (cname="os_VDU_SET_GRAPHICS_WINDOW")]
  public const char VDUSetGraphicsWindow;
  [CCode (cname="os_VDU_PLOT")]
  public const char VDUPlot;
  [CCode (cname="os_VDU_PLOT_VDU")]
  public const char VDUPlotVdu;
  [CCode (cname="os_VDU_RESET_WINDOWS")]
  public const char VDUResetWindows;
  [CCode (cname="os_VDU_ESCAPE")]
  public const char VDUEscape;
  [CCode (cname="os_VDU_SET_TEXT_WINDOW")]
  public const char VDUSetTextWindow;
  [CCode (cname="os_VDU_SET_GRAPHICS_ORIGIN")]
  public const char VDUSetGraphicsOrigin;
  [CCode (cname="os_VDU_HOME_TEXT_CURSOR")]
  public const char VDUHomeTextCursor;
  [CCode (cname="os_VDU_SET_TEXT_CURSOR")]
  public const char VDUSetTextCursor;
  [CCode (cname="os_VDU_SPACE")]
  public const char VDUSpace;
  [CCode (cname="os_VDU_DELETE")]
  public const char VDUDelete;
  [CCode (cname="os_MISC_INTERLACE")]
  public const char MiscInterlace;
  [CCode (cname="os_MISC_CURSOR")]
  public const char MiscCursor;
  [CCode (cname="os_MISC_SET_ECF1")]
  public const char MiscSetECF1;
  [CCode (cname="os_MISC_SET_ECF2")]
  public const char MiscSetECF2;
  [CCode (cname="os_MISC_SET_ECF3")]
  public const char MiscSetECF3;
  [CCode (cname="os_MISC_SET_ECF4")]
  public const char MiscSetECF4;
  [CCode (cname="os_MISC_SET_DOT_STYLE")]
  public const char MiscSetDotStyle;
  [CCode (cname="os_MISC_SCROLL")]
  public const char MiscScroll;
  [CCode (cname="os_MISC_CLEAR_REGION")]
  public const char MiscClearRegion;
  [CCode (cname="os_MISC_SET_ON")]
  public const char MiscSetOn;
  [CCode (cname="os_MISC_SET_OFF")]
  public const char MiscSetOff;
  [CCode (cname="os_MISC_SET_BBCECF")]
  public const char MiscSetBBCECF;
  [CCode (cname="os_MISC_SET_UNPACKED_ECF1")]
  public const char MiscSetUnpackedECF1;
  [CCode (cname="os_MISC_SET_UNPACKED_ECF2")]
  public const char MiscSetUnpackedECF2;
  [CCode (cname="os_MISC_SET_UNPACKED_ECF3")]
  public const char MiscSetUnpackedECF3;
  [CCode (cname="os_MISC_SET_UNPACKED_ECF4")]
  public const char MiscSetUnpackedECF4;
  [CCode (cname="os_MISC_SET_CURSOR_MOVEMENT")]
  public const char MiscSetCursorMovement;
  [CCode (cname="os_MISC_MISC")]
  public const char MiscMisc;
  [CCode (cname="os_MISC_SET_TEXT_FG_TINT")]
  public const char MiscSetTextFGTint;
  [CCode (cname="os_MISC_SET_TEXT_BG_TINT")]
  public const char MiscSetTextBGTint;
  [CCode (cname="os_MISC_SET_GRAPHICS_FG_TINT")]
  public const char MiscSetGraphicsFGTint;
  [CCode (cname="os_MISC_SET_GRAPHICS_BG_TINT")]
  public const char MiscSetGraphicsBGTint;
  [CCode (cname="os_MISC_SET_ECF_INTERPRETATION")]
  public const char MiscSetECFInterpretation;
  [CCode (cname="os_MISC_INVERT_TEXT")]
  public const char MiscInvertText;
  [CCode (cname="os_MISC_SET_ECF_ORIGIN")]
  public const char MiscSetECFOrigin;
  [CCode (cname="os_MISC_SET_CHAR_SIZE")]
  public const char MiscSetCharSize;
  [CCode (cname="os_MODE_FLAG_DATA_FORMAT_SHIFT")]
  public const int ModeFlagDataFormatShift;
  [CCode (cname="os_FOREGROUND")]
  public const bits Foreground;
  [CCode (cname="os_BACKGROUND")]
  public const bits Background;
  [CCode (cname="osdevicetype_DEVICE_NUMBER_SHIFT")]
  public const int OSDeviceType_DeviceNumberShift;
  [CCode (cname="osdevicetype_DEVICE_NUMBER_MASK")]
  public const int OSDeviceType_DeviceNumberMask;
  [CCode (cname="os_ERROR_NUMBER_SHIFT")]
  public const int ErrorNumberShift;
  [CCode (cname="os_ERROR_NUMBER")]
  public const bits ErrorNumber;
  [CCode (cname="os_ERROR_SOURCE_SHIFT")]
  public const int ErrorSourceShift;
  [CCode (cname="os_ERROR_SOURCE")]
  public const bits ErrorSource;
  [CCode (cname="os_ERROR_CATEGORY_SHIFT")]
  public const int ErrorCategoryShift;
  [CCode (cname="os_ERROR_CATEGORY")]
  public const bits ErrorCategory;
  [CCode (cname="os_CATEGORY_UNCLASSIFIED")]
  public const int CategoryUnclassified;
  [CCode (cname="os_CATEGORY_INFORMATION")]
  public const int CategoryInformation;
  [CCode (cname="os_CATEGORY_WARNING")]
  public const int CategoryWarning;
  [CCode (cname="os_CATEGORY_SYSTEM")]
  public const int CategorySystem;
  [CCode (cname="os_GLOBAL_ESCAPE")]
  public const int GlobalEscape;
  [CCode (cname="os_GLOBAL_FATAL")]
  public const int GlobalFatal;
  [CCode (cname="os_GLOBAL_IS_AFIL")]
  public const int GlobalIsAFil;
  [CCode (cname="os_GLOBAL_ISNT_FIL")]
  public const int GlobalIsntFil;
  [CCode (cname="os_GLOBAL_IS_ADIR")]
  public const int GlobalIsADir;
  [CCode (cname="os_GLOBAL_ISNT_DIR")]
  public const int GlobalIsntDir;
  [CCode (cname="os_GLOBAL_NO_FILE")]
  public const int GlobalNoFile;
  [CCode (cname="os_GLOBAL_NO_DIR")]
  public const int GlobalNoDir;
  [CCode (cname="os_GLOBAL_NO_ANY")]
  public const int GlobalNoAny;
  [CCode (cname="os_GLOBAL_TYPS_BAD")]
  public const int GlobalTypsBad;
  [CCode (cname="os_GLOBAL_BAD_REN")]
  public const int GlobalBadRen;
  [CCode (cname="os_GLOBAL_BAD_ACC")]
  public const int GlobalBadAcc;
  [CCode (cname="os_GLOBAL_OPN_FILS")]
  public const int GlobalOpnFils;
  [CCode (cname="os_GLOBAL_BAD_NAME")]
  public const int GlobalBadName;
  [CCode (cname="os_GLOBAL_LONG_NAM")]
  public const int GlobalLongNam;
  [CCode (cname="os_GLOBAL_CHANNEL")]
  public const int GlobalChannel;
  [CCode (cname="os_GLOBAL_BAD_SWI")]
  public const int GlobalBadSWI;
  [CCode (cname="os_GLOBAL_MOD_IN_US")]
  public const int GlobalModInUs;
  [CCode (cname="os_GLOBAL_STR_OFLO")]
  public const int GlobalStrOFlo;
  [CCode (cname="os_GLOBAL_NUM_OFLO")]
  public const int GlobalNumOFlo;
  [CCode (cname="os_GLOBAL_BUF_OFLO")]
  public const int GlobalBufOFlo;
  [CCode (cname="os_GLOBAL_BAD_STN")]
  public const int GlobalBadStn;
  [CCode (cname="os_GLOBAL_BAD_NET")]
  public const int GlobalBadNet;
  [CCode (cname="os_GLOBAL_FULL_NET")]
  public const int GlobalFullNet;
  [CCode (cname="os_GLOBAL_NO_ECO")]
  public const int GlobalNoEco;
  [CCode (cname="os_GLOBAL_BAD_READ")]
  public const int GlobalBadRead;
  [CCode (cname="os_GLOBAL_BAD_WRT")]
  public const int GlobalBadWrt;
  [CCode (cname="os_GLOBAL_DATA_LST")]
  public const int GlobalDataLst;
  [CCode (cname="os_GLOBAL_BAD_FS_OP")]
  public const int GlobalBadFSOp;
  [CCode (cname="os_GLOBAL_FS_NO_RD")]
  public const int GlobalFSNoRd;
  [CCode (cname="os_GLOBAL_FS_NO_WRT")]
  public const int GlobalFSNoWrt;
  [CCode (cname="os_GLOBAL_FS_IN_USE")]
  public const int GlobalFSInUse;
  [CCode (cname="os_GLOBAL_BAD_ADDRESS")]
  public const int GlobalBadAddress;
  [CCode (cname="os_GLOBAL_NO_STORE")]
  public const int GlobalNoStore;
  [CCode (cname="os_GLOBAL_RAM_LOADC")]
  public const int GlobalRAMLoadC;
  [CCode (cname="os_GLOBAL_DDS0")]
  public const int GlobalDDS0;
  [CCode (cname="os_GLOBAL_DDS1")]
  public const int GlobalDDS1;
  [CCode (cname="os_GLOBAL_DDS2")]
  public const int GlobalDDS2;
  [CCode (cname="os_GLOBAL_DDS3")]
  public const int GlobalDDS3;
  [CCode (cname="os_GLOBAL_CSDS")]
  public const int GlobalCSDS;
  [CCode (cname="os_GLOBAL_LDS")]
  public const int GlobalLDS;
  [CCode (cname="os_GLOBAL_URDS")]
  public const int GlobalURDS;
  [CCode (cname="os_GLOBAL_DR")]
  public const int GlobalDr;
  [CCode (cname="os_GLOBAL_APP")]
  public const int GlobalApp;
  [CCode (cname="os_GLOBAL_EX_DT")]
  public const int GlobalExDt;
  [CCode (cname="os_GLOBAL_EX_LD_EX")]
  public const int GlobalExLdEx;
  [CCode (cname="os_GLOBAL_USE_DESK")]
  public const int GlobalUseDesk;
  [CCode (cname="os_GLOBAL_APP_QUIT")]
  public const int GlobalAppQuit;
  [CCode (cname="os_GLOBAL_NO_MEM")]
  public const int GlobalNoMem;
  [CCode (cname="os_GLOBAL_BAD_HARD")]
  public const int GlobalBadHard;
  [CCode (cname="os_GLOBAL_BAD_PARM")]
  public const int GlobalBadParm;
  [CCode (cname="os_GLOBAL_NO_WIMP")]
  public const int GlobalNoWimp;
  [CCode (cname="error_ESCAPE")]
  public const bits Error_Escape;
  [CCode (cname="error_BAD_MODE")]
  public const bits Error_BadMode;
  [CCode (cname="error_IS_ADIR")]
  public const bits Error_IsADir;
  [CCode (cname="error_TYPES_DONT_MATCH")]
  public const bits Error_TypesDontMatch;
  [CCode (cname="error_BAD_RENAME")]
  public const bits Error_BadRename;
  [CCode (cname="error_BAD_COPY")]
  public const bits Error_BadCopy;
  [CCode (cname="error_OUTSIDE_FILE")]
  public const bits Error_OutsideFile;
  [CCode (cname="error_ACCESS_VIOLATION")]
  public const bits Error_AccessViolation;
  [CCode (cname="error_TOO_MANY_OPEN_FILES")]
  public const bits Error_TooManyOpenFiles;
  [CCode (cname="error_NOT_OPEN_FOR_UPDATE")]
  public const bits Error_NotOpenForUpdate;
  [CCode (cname="error_FILE_OPEN")]
  public const bits Error_FileOpen;
  [CCode (cname="error_OBJECT_LOCKED")]
  public const bits Error_ObjectLocked;
  [CCode (cname="error_ALREADY_EXISTS")]
  public const bits Error_AlreadyExists;
  [CCode (cname="error_BAD_FILE_NAME")]
  public const bits Error_BadFileName;
  [CCode (cname="error_FILE_NOT_FOUND")]
  public const bits Error_FileNotFound;
  [CCode (cname="error_SYNTAX")]
  public const bits Error_Syntax;
  [CCode (cname="error_BAD_NO_PARMS")]
  public const bits Error_BadNoParms;
  [CCode (cname="error_TOO_MANY_PARMS")]
  public const bits Error_TooManyParms;
  [CCode (cname="error_CHANNEL")]
  public const bits Error_Channel;
  [CCode (cname="error_END_OF_FILE")]
  public const bits Error_EndOfFile;
  [CCode (cname="error_BAD_FILING_SYSTEM_NAME")]
  public const bits Error_BadFilingSystemName;
  [CCode (cname="error_UNKNOWN_FILING_SYSTEM")]
  public const bits Error_UnknownFilingSystem;
  [CCode (cname="error_BAD_KEY")]
  public const bits Error_BadKey;
  [CCode (cname="error_BAD_ADDRESS")]
  public const bits Error_BadAddress;
  [CCode (cname="error_BAD_STRING")]
  public const bits Error_BadString;
  [CCode (cname="error_BAD_ALIAS_STRING")]
  public const bits Error_BadAliasString;
  [CCode (cname="error_BAD_PARM_STRING")]
  public const bits Error_BadParmString;
  [CCode (cname="error_WILD_CARDS")]
  public const bits Error_WildCards;
  [CCode (cname="error_BAD_COMMAND")]
  public const bits Error_BadCommand;
  [CCode (cname="error_BAD_MAC_VAL")]
  public const bits Error_BadMacVal;
  [CCode (cname="error_BAD_VAR_NAM")]
  public const bits Error_BadVarNam;
  [CCode (cname="error_BAD_VAR_TYPE")]
  public const bits Error_BadVarType;
  [CCode (cname="error_VAR_NO_ROOM")]
  public const bits Error_VarNoRoom;
  [CCode (cname="error_VAR_CANT_FIND")]
  public const bits Error_VarCantFind;
  [CCode (cname="error_VAR_TOO_LONG")]
  public const bits Error_VarTooLong;
  [CCode (cname="error_REDIRECT_FAIL")]
  public const bits Error_RedirectFail;
  [CCode (cname="error_STACK_FULL")]
  public const bits Error_StackFull;
  [CCode (cname="error_BAD_HEX")]
  public const bits Error_BadHex;
  [CCode (cname="error_BAD_EXPR")]
  public const bits Error_BadExpr;
  [CCode (cname="error_BAD_BRA")]
  public const bits Error_BadBra;
  [CCode (cname="error_STK_OFLO")]
  public const bits Error_StkOFlo;
  [CCode (cname="error_MISS_OPN")]
  public const bits Error_MissOpn;
  [CCode (cname="error_MISS_OPR")]
  public const bits Error_MissOpr;
  [CCode (cname="error_BAD_BITS")]
  public const bits Error_BadBits;
  [CCode (cname="error_STR_OFLO")]
  public const bits Error_StrOFlo;
  [CCode (cname="error_BAD_ITM")]
  public const bits Error_BadItm;
  [CCode (cname="error_DIV_ZERO")]
  public const bits Error_DivZero;
  [CCode (cname="error_BAD_BASE")]
  public const bits Error_BadBase;
  [CCode (cname="error_BAD_NUMB")]
  public const bits Error_BadNumb;
  [CCode (cname="error_NUMB_TOO_BIG")]
  public const bits Error_NumbTooBig;
  [CCode (cname="error_BAD_CLAIM_NUM")]
  public const bits Error_BadClaimNum;
  [CCode (cname="error_BAD_RELEASE")]
  public const bits Error_BadRelease;
  [CCode (cname="error_BAD_DEV_NO")]
  public const bits Error_BadDevNo;
  [CCode (cname="error_BAD_DEV_VEC_REL")]
  public const bits Error_BadDevVecRel;
  [CCode (cname="error_BAD_ENV_NUMBER")]
  public const bits Error_BadEnvNumber;
  [CCode (cname="error_CANT_CANCEL_QUIT")]
  public const bits Error_CantCancelQuit;
  [CCode (cname="error_CH_DYNAM_CAO")]
  public const bits Error_ChDynamCAO;
  [CCode (cname="error_CH_DYNAM_NOT_ALL_MOVED")]
  public const bits Error_ChDynamNotAllMoved;
  [CCode (cname="error_APL_WSPACE_IN_USE")]
  public const bits Error_AplWSpaceInUse;
  [CCode (cname="error_RAM_FS_UNCHANGEABLE")]
  public const bits Error_RAMFsUnchangeable;
  [CCode (cname="error_OSCLI_LONG_LINE")]
  public const bits Error_OSCLILongLine;
  [CCode (cname="error_OSCLI_TOO_HARD")]
  public const bits Error_OSCLITooHard;
  [CCode (cname="error_RC_EXC")]
  public const bits Error_RCExc;
  [CCode (cname="error_RC_NEGATIVE")]
  public const bits Error_RCNegative;
  [CCode (cname="error_SYS_HEAP_FULL")]
  public const bits Error_SysHeapFull;
  [CCode (cname="error_BUFF_OVERFLOW")]
  public const bits Error_BuffOverflow;
  [CCode (cname="error_BAD_TIME")]
  public const bits Error_BadTime;
  [CCode (cname="error_NO_SUCH_SWI")]
  public const bits Error_NoSuchSWI;
  [CCode (cname="error_UNIMPLEMENTED")]
  public const bits Error_Unimplemented;
  [CCode (cname="error_OUT_OF_RANGE")]
  public const bits Error_OutOfRange;
  [CCode (cname="error_NO_OSCLI_SPECIALS")]
  public const bits Error_NoOSCLISpecials;
  [CCode (cname="error_BAD_PARAMETERS")]
  public const bits Error_BadParameters;
  [CCode (cname="error_ARG_REPEATED")]
  public const bits Error_ArgRepeated;
  [CCode (cname="error_BAD_READ_SYS_INFO")]
  public const bits Error_BadReadSysInfo;
  [CCode (cname="error_CDAT_STACK_OVERFLOW")]
  public const bits Error_CDATStackOverflow;
  [CCode (cname="error_CDAT_BUFFER_OVERFLOW")]
  public const bits Error_CDATBufferOverflow;
  [CCode (cname="error_CDAT_BAD_FIELD")]
  public const bits Error_CDATBadField;
  [CCode (cname="error_CANT_START_APPLICATION")]
  public const bits Error_CantStartApplication;
  [CCode (cname="error_BAD_COMMAND_OPTION")]
  public const bits Error_BadCommandOption;
  [CCode (cname="error_UNKNOWN_SERIAL_OP")]
  public const bits Error_UnknownSerialOp;
  [CCode (cname="error_BAD_HARD")]
  public const bits Error_BadHard;
  [CCode (cname="osreadline_ECHO_CHARACTER_SHIFT")]
  public const int OSReadLine_EchoCharacterShift;
  [CCode (cname="os_DESTROY_VAR")]
  public const int DestroyVar;
  [CCode (cname="os_NONEXISTENT_MODE")]
  public const int NonexistentMode;
  [CCode (cname="os_NOT_ENOUGH_MEMORY_FOR_MODE")]
  public const int NotEnoughMemoryForMode;
  [CCode (cname="os_AREA_ACCESS_SHIFT")]
  public const int AreaAccessShift;
  [CCode (cname="osdisplaydevicetagtaglength_TAG_SHIFT")]
  public const int OSDisplayDeviceTagTagLength_TagShift;
  [CCode (cname="osdisplaydevicetagtaglength_TAG_MASK")]
  public const int OSDisplayDeviceTagTagLength_TagMask;
  [CCode (cname="osdisplaydevicetagtaglength_TAG_LENGTH")]
  public const int OSDisplayDeviceTagTagLength_TagLength;
  [CCode (cname="osdisplaydevicetagtaglength_TAG_LENGTH_MASK")]
  public const int OSDisplayDeviceTagTagLength_TagLengthMask;
  [CCode (cname="osdynamicarea_ALLOCATE_AREA")]
  public const int OSDynamicArea_AllocateArea;
  [CCode (cname="osdynamicarea_ALLOCATE_BASE")]
  public static char* OSDynamicArea_AllocateBase;
  [CCode (cname="osmemory_CACHE_SHIFT")]
  public const int OSMemory_CacheShift;
  [CCode (cname="osmemory_TYPE_SHIFT")]
  public const int OSMemory_TypeShift;
  [CCode (cname="osmemory_CONTROLLER_NO_SHIFT")]
  public const int OSMemory_ControllerNoShift;
  [CCode (cname="osmemory_CONTROLLER_TYPE_SHIFT")]
  public const int OSMemory_ControllerTypeShift;
  [CCode (cname="osmemory_IO_ACCESS_SHIFT")]
  public const int OSMemory_IOAccessShift;
  [CCode (cname="osmemory_AREA_SHIFT")]
  public const int OSMemory_AreaShift;
  [CCode (cname="os_HEX1_LIMIT")]
  public const int Hex1Limit;
  [CCode (cname="os_HEX2_LIMIT")]
  public const int Hex2Limit;
  [CCode (cname="os_HEX4_LIMIT")]
  public const int Hex4Limit;
  [CCode (cname="os_HEX6_LIMIT")]
  public const int Hex6Limit;
  [CCode (cname="os_HEX8_LIMIT")]
  public const int Hex8Limit;
  [CCode (cname="os_CARDINAL1_LIMIT")]
  public const int Cardinal1Limit;
  [CCode (cname="os_CARDINAL2_LIMIT")]
  public const int Cardinal2Limit;
  [CCode (cname="os_CARDINAL3_LIMIT")]
  public const int Cardinal3Limit;
  [CCode (cname="os_CARDINAL4_LIMIT")]
  public const int Cardinal4Limit;
  [CCode (cname="os_INTEGER1_LIMIT")]
  public const int Integer1Limit;
  [CCode (cname="os_INTEGER2_LIMIT")]
  public const int Integer2Limit;
  [CCode (cname="os_INTEGER3_LIMIT")]
  public const int Integer3Limit;
  [CCode (cname="os_INTEGER4_LIMIT")]
  public const int Integer4Limit;
  [CCode (cname="os_BINARY1_LIMIT")]
  public const int Binary1Limit;
  [CCode (cname="os_BINARY2_LIMIT")]
  public const int Binary2Limit;
  [CCode (cname="os_BINARY3_LIMIT")]
  public const int Binary3Limit;
  [CCode (cname="os_BINARY4_LIMIT")]
  public const int Binary4Limit;
  [CCode (cname="os_SPACED_CARDINAL1_LIMIT")]
  public const int SpacedCardinal1Limit;
  [CCode (cname="os_SPACED_CARDINAL2_LIMIT")]
  public const int SpacedCardinal2Limit;
  [CCode (cname="os_SPACED_CARDINAL3_LIMIT")]
  public const int SpacedCardinal3Limit;
  [CCode (cname="os_SPACED_CARDINAL4_LIMIT")]
  public const int SpacedCardinal4Limit;
  [CCode (cname="os_SPACED_INTEGER1_LIMIT")]
  public const int SpacedInteger1Limit;
  [CCode (cname="os_SPACED_INTEGER2_LIMIT")]
  public const int SpacedInteger2Limit;
  [CCode (cname="os_SPACED_INTEGER3_LIMIT")]
  public const int SpacedInteger3Limit;
  [CCode (cname="os_SPACED_INTEGER4_LIMIT")]
  public const int SpacedInteger4Limit;
  [CCode (cname="os_FIXED_NET_STATION_LIMIT")]
  public const int FixedNetStationLimit;
  [CCode (cname="os_NET_STATION_LIMIT")]
  public const int NetStationLimit;
  [CCode (cname="os_FIXED_FILE_SIZE_LIMIT")]
  public const int FixedFileSizeLimit;
  [CCode (cname="os_FILE_SIZE_LIMIT")]
  public const int FileSizeLimit;
  [CCode (cname="os_TRANSITION_MOUSE_RIGHT")]
  public const bits TransitionMouseRight;
  [CCode (cname="os_TRANSITION_MOUSE_MIDDLE")]
  public const bits TransitionMouseMiddle;
  [CCode (cname="os_TRANSITION_MOUSE_LEFT")]
  public const bits TransitionMouseLeft;
  [CCode (cname="os_TRANSITION_KEY_ESCAPE")]
  public const byte TransitionKeyEscape;
  [CCode (cname="os_TRANSITION_KEY_F1")]
  public const byte TransitionKeyF1;
  [CCode (cname="os_TRANSITION_KEY_F2")]
  public const byte TransitionKeyF2;
  [CCode (cname="os_TRANSITION_KEY_F3")]
  public const byte TransitionKeyF3;
  [CCode (cname="os_TRANSITION_KEY_F4")]
  public const byte TransitionKeyF4;
  [CCode (cname="os_TRANSITION_KEY_F5")]
  public const byte TransitionKeyF5;
  [CCode (cname="os_TRANSITION_KEY_F6")]
  public const byte TransitionKeyF6;
  [CCode (cname="os_TRANSITION_KEY_F7")]
  public const byte TransitionKeyF7;
  [CCode (cname="os_TRANSITION_KEY_F8")]
  public const byte TransitionKeyF8;
  [CCode (cname="os_TRANSITION_KEY_F9")]
  public const byte TransitionKeyF9;
  [CCode (cname="os_TRANSITION_KEY_F10")]
  public const byte TransitionKeyF10;
  [CCode (cname="os_TRANSITION_KEY_F11")]
  public const byte TransitionKeyF11;
  [CCode (cname="os_TRANSITION_KEY_F12")]
  public const byte TransitionKeyF12;
  [CCode (cname="os_TRANSITION_KEY_PRINT")]
  public const byte TransitionKeyPrint;
  [CCode (cname="os_TRANSITION_KEY_SCROLL_LOCK")]
  public const byte TransitionKeyScrollLock;
  [CCode (cname="os_TRANSITION_KEY_BREAK")]
  public const byte TransitionKeyBreak;
  [CCode (cname="os_TRANSITION_KEY_GRAVE")]
  public const byte TransitionKeyGrave;
  [CCode (cname="os_TRANSITION_KEY1")]
  public const byte TransitionKey1;
  [CCode (cname="os_TRANSITION_KEY2")]
  public const byte TransitionKey2;
  [CCode (cname="os_TRANSITION_KEY3")]
  public const byte TransitionKey3;
  [CCode (cname="os_TRANSITION_KEY4")]
  public const byte TransitionKey4;
  [CCode (cname="os_TRANSITION_KEY5")]
  public const byte TransitionKey5;
  [CCode (cname="os_TRANSITION_KEY6")]
  public const byte TransitionKey6;
  [CCode (cname="os_TRANSITION_KEY7")]
  public const byte TransitionKey7;
  [CCode (cname="os_TRANSITION_KEY8")]
  public const byte TransitionKey8;
  [CCode (cname="os_TRANSITION_KEY9")]
  public const byte TransitionKey9;
  [CCode (cname="os_TRANSITION_KEY0")]
  public const byte TransitionKey0;
  [CCode (cname="os_TRANSITION_KEY_HYPHEN")]
  public const byte TransitionKeyHyphen;
  [CCode (cname="os_TRANSITION_KEY_EQUALS")]
  public const byte TransitionKeyEquals;
  [CCode (cname="os_TRANSITION_KEY_POUND")]
  public const byte TransitionKeyPound;
  [CCode (cname="os_TRANSITION_KEY_BACKSPACE")]
  public const byte TransitionKeyBackspace;
  [CCode (cname="os_TRANSITION_KEY_INSERT")]
  public const byte TransitionKeyInsert;
  [CCode (cname="os_TRANSITION_KEY_HOME")]
  public const byte TransitionKeyHome;
  [CCode (cname="os_TRANSITION_KEY_PAGE_UP")]
  public const byte TransitionKeyPageUp;
  [CCode (cname="os_TRANSITION_KEY_NUM_LOCK")]
  public const byte TransitionKeyNumLock;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_SLASH")]
  public const byte TransitionKeyKeypadSlash;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_STAR")]
  public const byte TransitionKeyKeypadStar;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_HASH")]
  public const byte TransitionKeyKeypadHash;
  [CCode (cname="os_TRANSITION_KEY_TAB")]
  public const byte TransitionKeyTab;
  [CCode (cname="os_TRANSITION_KEYQ")]
  public const byte TransitionKeyQ;
  [CCode (cname="os_TRANSITION_KEYW")]
  public const byte TransitionKeyW;
  [CCode (cname="os_TRANSITION_KEYE")]
  public const byte TransitionKeyE;
  [CCode (cname="os_TRANSITION_KEYR")]
  public const byte TransitionKeyR;
  [CCode (cname="os_TRANSITION_KEYT")]
  public const byte TransitionKeyT;
  [CCode (cname="os_TRANSITION_KEYY")]
  public const byte TransitionKeyY;
  [CCode (cname="os_TRANSITION_KEYU")]
  public const byte TransitionKeyU;
  [CCode (cname="os_TRANSITION_KEYI")]
  public const byte TransitionKeyI;
  [CCode (cname="os_TRANSITION_KEYO")]
  public const byte TransitionKeyO;
  [CCode (cname="os_TRANSITION_KEYP")]
  public const byte TransitionKeyP;
  [CCode (cname="os_TRANSITION_KEY_SQUARE_BRA")]
  public const byte TransitionKeySquareBra;
  [CCode (cname="os_TRANSITION_KEY_SQUARE_KET")]
  public const byte TransitionKeySquareKet;
  [CCode (cname="os_TRANSITION_KEY_BACKSLASH")]
  public const byte TransitionKeyBackslash;
  [CCode (cname="os_TRANSITION_KEY_DELETE")]
  public const byte TransitionKeyDelete;
  [CCode (cname="os_TRANSITION_KEY_COPY")]
  public const byte TransitionKeyCopy;
  [CCode (cname="os_TRANSITION_KEY_PAGE_DOWN")]
  public const byte TransitionKeyPageDown;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD7")]
  public const byte TransitionKeyKeypad7;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD8")]
  public const byte TransitionKeyKeypad8;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD9")]
  public const byte TransitionKeyKeypad9;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_MINUS")]
  public const byte TransitionKeyKeypadMinus;
  [CCode (cname="os_TRANSITION_KEY_LEFT_CONTROL")]
  public const byte TransitionKeyLeftControl;
  [CCode (cname="os_TRANSITION_KEYA")]
  public const byte TransitionKeyA;
  [CCode (cname="os_TRANSITION_KEYS")]
  public const byte TransitionKeyS;
  [CCode (cname="os_TRANSITION_KEYD")]
  public const byte TransitionKeyD;
  [CCode (cname="os_TRANSITION_KEYF")]
  public const byte TransitionKeyF;
  [CCode (cname="os_TRANSITION_KEYG")]
  public const byte TransitionKeyG;
  [CCode (cname="os_TRANSITION_KEYH")]
  public const byte TransitionKeyH;
  [CCode (cname="os_TRANSITION_KEYJ")]
  public const byte TransitionKeyJ;
  [CCode (cname="os_TRANSITION_KEYK")]
  public const byte TransitionKeyK;
  [CCode (cname="os_TRANSITION_KEYL")]
  public const byte TransitionKeyL;
  [CCode (cname="os_TRANSITION_KEY_SEMICOLON")]
  public const byte TransitionKeySemicolon;
  [CCode (cname="os_TRANSITION_KEY_ACUTE")]
  public const byte TransitionKeyAcute;
  [CCode (cname="os_TRANSITION_KEY_RETURN")]
  public const byte TransitionKeyReturn;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD4")]
  public const byte TransitionKeyKeypad4;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD5")]
  public const byte TransitionKeyKeypad5;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD6")]
  public const byte TransitionKeyKeypad6;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_PLUS")]
  public const byte TransitionKeyKeypadPlus;
  [CCode (cname="os_TRANSITION_KEY_LEFT_SHIFT")]
  public const byte TransitionKeyLeftShift;
  [CCode (cname="os_TRANSITION_KEYZ")]
  public const byte TransitionKeyZ;
  [CCode (cname="os_TRANSITION_KEYX")]
  public const byte TransitionKeyX;
  [CCode (cname="os_TRANSITION_KEYC")]
  public const byte TransitionKeyC;
  [CCode (cname="os_TRANSITION_KEYV")]
  public const byte TransitionKeyV;
  [CCode (cname="os_TRANSITION_KEYB")]
  public const byte TransitionKeyB;
  [CCode (cname="os_TRANSITION_KEYN")]
  public const byte TransitionKeyN;
  [CCode (cname="os_TRANSITION_KEYM")]
  public const byte TransitionKeyM;
  [CCode (cname="os_TRANSITION_KEY_COMMA")]
  public const byte TransitionKeyComma;
  [CCode (cname="os_TRANSITION_KEY_POINT")]
  public const byte TransitionKeyPoint;
  [CCode (cname="os_TRANSITION_KEY_SLASH")]
  public const byte TransitionKeySlash;
  [CCode (cname="os_TRANSITION_KEY_RIGHT_SHIFT")]
  public const byte TransitionKeyRightShift;
  [CCode (cname="os_TRANSITION_KEY_UP")]
  public const byte TransitionKeyUp;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD1")]
  public const byte TransitionKeyKeypad1;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD2")]
  public const byte TransitionKeyKeypad2;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD3")]
  public const byte TransitionKeyKeypad3;
  [CCode (cname="os_TRANSITION_KEY_CAPS_LOCK")]
  public const byte TransitionKeyCapsLock;
  [CCode (cname="os_TRANSITION_KEY_LEFT_ALT")]
  public const byte TransitionKeyLeftAlt;
  [CCode (cname="os_TRANSITION_KEY_SPACE")]
  public const byte TransitionKeySpace;
  [CCode (cname="os_TRANSITION_KEY_RIGHT_ALT")]
  public const byte TransitionKeyRightAlt;
  [CCode (cname="os_TRANSITION_KEY_RIGHT_CONTROL")]
  public const byte TransitionKeyRightControl;
  [CCode (cname="os_TRANSITION_KEY_LEFT")]
  public const byte TransitionKeyLeft;
  [CCode (cname="os_TRANSITION_KEY_DOWN")]
  public const byte TransitionKeyDown;
  [CCode (cname="os_TRANSITION_KEY_RIGHT")]
  public const byte TransitionKeyRight;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD0")]
  public const byte TransitionKeyKeypad0;
  [CCode (cname="os_TRANSITION_KEY_KEYPAD_POINT")]
  public const byte TransitionKeyKeypadPoint;
  [CCode (cname="os_TRANSITION_KEY_ENTER")]
  public const byte TransitionKeyEnter;
  [CCode (cname="os_TRANSITION_UP")]
  public const int TransitionUp;
  [CCode (cname="os_TRANSITION_DOWN")]
  public const int TransitionDown;

  [CCode (cname = "xos_read_var_val_size")]
  public static OS.Error* XReadVarValSize
    (string var,
     int context,
     VarType var_type,
     out int used,
     out int context_out,
     out VarType var_type_out);

  [CCode (cname = "os_read_var_val_size")]
  public static int ReadVarValSize
    (string var,
     int context,
     VarType var_type,
     out int used,
     out VarType var_type_out);

  [CCode (cname = "xos_change_dynamic_area_no_fail")]
  public static OS.Error* XChangeDynamicAreaNoFail
    (DynamicAreaNo area_type,
     int change,
     out int change_out);

  [CCode (cname = "os_change_dynamic_area_no_fail")]
  public static int ChangeDynamicAreaNoFail
    (DynamicAreaNo area_type,
     int change);

  [CCode (cname = "xos_writec")]
  public static OS.Error* XWriteC
    (char c);

  [CCode (cname = "os_writec")]
  public static void WriteC
    (char c);

  [CCode (cname = "xos_write0")]
  public static OS.Error* XWrite0
    (string s);

  [CCode (cname = "os_write0")]
  public static void Write0
    (string s);

  [CCode (cname = "xos_new_line")]
  public static OS.Error* XNewLine ();

  [CCode (cname = "os_new_line")]
  public static void NewLine ();

  [CCode (cname = "xos_readc")]
  public static OS.Error* XReadC
    (out char c,
     out uint psr);

  [CCode (cname = "os_readc")]
  public static bits ReadC
    (out char c);

  [CCode (cname = "xos_cli")]
  public static OS.Error* XCLI
    (string command);

  [CCode (cname = "os_cli")]
  public static void CLI
    (string command);

  [CCode (cname = "xos_read_line")]
  public static OS.Error* XReadLine
    (string buffer,
     int size,
     char min_char,
     char max_char,
     out int used,
     out uint psr);

  [CCode (cname = "os_read_line")]
  public static bits ReadLine
    (string buffer,
     int size,
     char min_char,
     char max_char,
     out int used);

  [CCode (cname = "xos_read_line_given_echo")]
  public static OS.Error* XReadLineGivenEcho
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used,
     out uint psr);

  [CCode (cname = "os_read_line_given_echo")]
  public static bits ReadLineGivenEcho
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used);

  [CCode (cname = "xos_read_line_suppress_invalid")]
  public static OS.Error* XReadLineSuppressInvalid
    (string buffer,
     int size,
     char min_char,
     char max_char,
     out int used,
     out uint psr);

  [CCode (cname = "os_read_line_suppress_invalid")]
  public static bits ReadLineSuppressInvalid
    (string buffer,
     int size,
     char min_char,
     char max_char,
     out int used);

  [CCode (cname = "xos_read_line_given_echo_suppress_invalid")]
  public static OS.Error* XReadLineGivenEchoSuppressInvalid
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used,
     out uint psr);

  [CCode (cname = "os_read_line_given_echo_suppress_invalid")]
  public static bits ReadLineGivenEchoSuppressInvalid
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used);

  [CCode (cname = "xos_read_line32")]
  public static OS.Error* XReadLine32
    (string buffer,
     int size,
     char min_char,
     char max_char,
     OSReadLine_Flags flags,
     out int used,
     out uint psr);

  [CCode (cname = "os_read_line32")]
  public static bits ReadLine32
    (string buffer,
     int size,
     char min_char,
     char max_char,
     OSReadLine_Flags flags,
     out int used);

  [CCode (cname = "xos_control")]
  public static OS.Error* XControl
    (AsmRoutine error_handler,
     Error error_buffer,
     AsmRoutine escape_handler,
     AsmRoutine event_handler,
     out AsmRoutine old_error_handler,
     out Error old_error_buffer,
     out AsmRoutine old_escape_handler,
     out AsmRoutine old_event_handler);

  [CCode (cname = "os_control")]
  public static void Control
    (AsmRoutine error_handler,
     Error error_buffer,
     AsmRoutine escape_handler,
     AsmRoutine event_handler,
     out AsmRoutine old_error_handler,
     out Error old_error_buffer,
     out AsmRoutine old_escape_handler,
     out AsmRoutine old_event_handler);

  [CCode (cname = "xos_get_env")]
  public static OS.Error* XGetEnv
    (out string command,
     out char* ram_limit,
     out DateAndTime start);

  [CCode (cname = "os_get_env")]
  public static string GetEnv
    (out char* ram_limit,
     out DateAndTime start);

  [CCode (cname = "xos_exit")]
  public static OS.Error* XExit
    (Error error,
     int rc);

  [CCode (cname = "os_exit")]
  public static void Exit
    (Error error,
     int rc);

  [CCode (cname = "xos_set_env")]
  public static OS.Error* XSetEnv
    (AsmRoutine exit_handler,
     char* ram_limit,
     AsmRoutine undefined_instruction_handler,
     AsmRoutine prefetch_abort_handler,
     AsmRoutine data_abort_handler,
     AsmRoutine address_exception_handler,
     out AsmRoutine old_exit_handler,
     out char* old_ram_limit,
     out AsmRoutine old_undefined_instruction_handler,
     out AsmRoutine old_prefetch_abort_handler,
     out AsmRoutine old_data_abort_handler,
     out AsmRoutine old_address_exception_handler);

  [CCode (cname = "os_set_env")]
  public static void SetEnv
    (AsmRoutine exit_handler,
     char* ram_limit,
     AsmRoutine undefined_instruction_handler,
     AsmRoutine prefetch_abort_handler,
     AsmRoutine data_abort_handler,
     AsmRoutine address_exception_handler,
     out AsmRoutine old_exit_handler,
     out char* old_ram_limit,
     out AsmRoutine old_undefined_instruction_handler,
     out AsmRoutine old_prefetch_abort_handler,
     out AsmRoutine old_data_abort_handler,
     out AsmRoutine old_address_exception_handler);

  [CCode (cname = "xos_int_on")]
  public static OS.Error* XIntOn ();

  [CCode (cname = "os_int_on")]
  public static void IntOn ();

  [CCode (cname = "xos_int_off")]
  public static OS.Error* XIntOff ();

  [CCode (cname = "os_int_off")]
  public static void IntOff ();

  [CCode (cname = "xos_call_back")]
  public static OS.Error* XCallBack
    (RegisterBlock register_block,
     AsmRoutine call_back_handler,
     out RegisterBlock old_register_block,
     out AsmRoutine old_call_back_handler);

  [CCode (cname = "os_call_back")]
  public static void CallBack
    (RegisterBlock register_block,
     AsmRoutine call_back_handler,
     out RegisterBlock old_register_block,
     out AsmRoutine old_call_back_handler);

  [CCode (cname = "xos_break_pt")]
  public static OS.Error* XBreakPt ();

  [CCode (cname = "os_break_pt")]
  public static void BreakPt ();

  [CCode (cname = "xos_break_ctrl")]
  public static OS.Error* XBreakCtrl
    (RegisterBlock register_block,
     AsmRoutine break_pt_handler,
     out RegisterBlock old_register_block,
     out AsmRoutine old_break_pt_handler);

  [CCode (cname = "os_break_ctrl")]
  public static void BreakCtrl
    (RegisterBlock register_block,
     AsmRoutine break_pt_handler,
     out RegisterBlock old_register_block,
     out AsmRoutine old_break_pt_handler);

  [CCode (cname = "xos_unused_swi")]
  public static OS.Error* XUnusedSWI
    (AsmRoutine unused_swi_handler,
     out AsmRoutine old_unused_swi_handler);

  [CCode (cname = "os_unused_swi")]
  public static AsmRoutine UnusedSWI
    (AsmRoutine unused_swi_handler);

  [CCode (cname = "xos_update_memc")]
  public static OS.Error* XUpdateMEMC
    (bits value,
     bits mask,
     out bits old_value);

  [CCode (cname = "os_update_memc")]
  public static bits UpdateMEMC
    (bits value,
     bits mask);

  [CCode (cname = "xos_set_call_back")]
  public static OS.Error* XSetCallBack ();

  [CCode (cname = "os_set_call_back")]
  public static void SetCallBack ();

  [CCode (cname = "xos_mouse")]
  public static OS.Error* XMouse
    (out int x,
     out int y,
     out bits buttons,
     out T t);

  [CCode (cname = "os_mouse")]
  public static void Mouse
    (out int x,
     out int y,
     out bits buttons,
     out T t);

  [CCode (cname = "xos_claim")]
  public static OS.Error* XClaim
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "os_claim")]
  public static void Claim
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xos_release")]
  public static OS.Error* XRelease
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "os_release")]
  public static void Release
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xos_read_unsigned")]
  public static OS.Error* XReadUnsigned
    (ReadUnsignedFlags flags,
     string number,
     bits limit,
     out char* end,
     out bits value);

  [CCode (cname = "os_read_unsigned")]
  public static bits ReadUnsigned
    (ReadUnsignedFlags flags,
     string number,
     bits limit,
     out char* end);

  [CCode (cname = "xos_read_var_val")]
  public static OS.Error* XReadVarVal
    (string var,
     string value,
     int size,
     int context,
     VarType var_type,
     out int used,
     out int context_out,
     out VarType var_type_out);

  [CCode (cname = "os_read_var_val")]
  public static int ReadVarVal
    (string var,
     string value,
     int size,
     int context,
     VarType var_type,
     out int used,
     out VarType var_type_out);

  [CCode (cname = "xos_set_var_val")]
  public static OS.Error* XSetVarVal
    (string var,
     char* value,
     int size,
     int context,
     VarType var_type,
     out int context_out,
     out VarType var_type_out);

  [CCode (cname = "os_set_var_val")]
  public static int SetVarVal
    (string var,
     char* value,
     int size,
     int context,
     VarType var_type,
     out VarType var_type_out);

  [CCode (cname = "xossetvarval_code")]
  public static OS.Error* XOSSetVarVal_Code
    (string var,
     AsmRoutine read_write_code,
     int size,
     int context,
     out int context_out,
     out VarType var_type_out);

  [CCode (cname = "ossetvarval_code")]
  public static int OSSetVarVal_Code
    (string var,
     AsmRoutine read_write_code,
     int size,
     int context,
     out VarType var_type_out);

  [CCode (cname = "xos_gs_init")]
  public static OS.Error* XGSInit
    (string s,
     GSFlags flags,
     out int context1,
     out int context2);

  [CCode (cname = "os_gs_init")]
  public static void GSInit
    (string s,
     GSFlags flags,
     out int context1,
     out int context2);

  [CCode (cname = "xos_gs_read")]
  public static OS.Error* XGSRead
    (int context1,
     int context2,
     out int context1_out,
     out char c,
     out int context2_out,
     out uint psr);

  [CCode (cname = "os_gs_read")]
  public static bits GSRead
    (int context1,
     int context2,
     out int context1_out,
     out char c,
     out int context2_out);

  [CCode (cname = "xos_gs_trans")]
  public static OS.Error* XGSTrans
    (string s,
     string buffer,
     int size,
     out int used,
     out uint psr);

  [CCode (cname = "os_gs_trans")]
  public static bits GSTrans
    (string s,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "xos_binary_to_decimal")]
  public static OS.Error* XBinaryToDecimal
    (int i,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "os_binary_to_decimal")]
  public static int BinaryToDecimal
    (int i,
     string buffer,
     int size);

  [CCode (cname = "xos_change_dynamic_area")]
  public static OS.Error* XChangeDynamicArea
    (DynamicAreaNo area_type,
     int change,
     out int change_out);

  [CCode (cname = "os_change_dynamic_area")]
  public static int ChangeDynamicArea
    (DynamicAreaNo area_type,
     int change);

  [CCode (cname = "xos_generate_error")]
  public static OS.Error* XGenerateError
    (Error error);

  [CCode (cname = "os_generate_error")]
  public static void GenerateError
    (Error error);

  [CCode (cname = "xos_read_escape_state")]
  public static OS.Error* XReadEscapeState
    (out uint psr);

  [CCode (cname = "os_read_escape_state")]
  public static bits ReadEscapeState ();

  [CCode (cname = "xos_evaluate_expression")]
  public static OS.Error* XEvaluateExpression
    (string expr,
     string buffer,
     int size,
     out bool expr_is_str,
     out int int_value_or_str_used);

  [CCode (cname = "os_evaluate_expression")]
  public static bool EvaluateExpression
    (string expr,
     string buffer,
     int size,
     out int int_value_or_str_used);

  [CCode (cname = "xos_read_palette")]
  public static OS.Error* XReadPalette
    (ColourNumber entry,
     int colour_type,
     out Colour on,
     out Colour off);

  [CCode (cname = "os_read_palette")]
  public static void ReadPalette
    (ColourNumber entry,
     int colour_type,
     out Colour on,
     out Colour off);

  [CCode (cname = "xos_read_vdu_variables")]
  public static OS.Error* XReadVduVariables
    (VDUVarList var_list,
     int* value_list);

  [CCode (cname = "os_read_vdu_variables")]
  public static void ReadVduVariables
    (VDUVarList var_list,
     int* value_list);

  [CCode (cname = "xos_read_point")]
  public static OS.Error* XReadPoint
    (int x,
     int y,
     out GCOL gcol,
     out Tint tint,
     out bool off_screen);

  [CCode (cname = "os_read_point")]
  public static bool ReadPoint
    (int x,
     int y,
     out GCOL gcol,
     out Tint tint);

  [CCode (cname = "xos_read_mode_variable")]
  public static OS.Error* XReadModeVariable
    (Mode mode,
     ModeVar var,
     out int var_val,
     out uint psr);

  [CCode (cname = "os_read_mode_variable")]
  public static bits ReadModeVariable
    (Mode mode,
     ModeVar var,
     out int var_val);

  [CCode (cname = "xos_remove_cursors")]
  public static OS.Error* XRemoveCursors ();

  [CCode (cname = "os_remove_cursors")]
  public static void RemoveCursors ();

  [CCode (cname = "xos_restore_cursors")]
  public static OS.Error* XRestoreCursors ();

  [CCode (cname = "os_restore_cursors")]
  public static void RestoreCursors ();

  [CCode (cname = "xos_swi_number_to_string")]
  public static OS.Error* XSWINumberToString
    (int swi,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "os_swi_number_to_string")]
  public static int SWINumberToString
    (int swi,
     string buffer,
     int size);

  [CCode (cname = "xos_swi_number_from_string")]
  public static OS.Error* XSWINumberFromString
    (string swi_name,
     out int swi);

  [CCode (cname = "os_swi_number_from_string")]
  public static int SWINumberFromString
    (string swi_name);

  [CCode (cname = "xos_validate_address")]
  public static OS.Error* XValidateAddress
    (char* min,
     char* max,
     out uint psr);

  [CCode (cname = "os_validate_address")]
  public static bits ValidateAddress
    (char* min,
     char* max);

  [CCode (cname = "xos_call_after")]
  public static OS.Error* XCallAfter
    (int delay,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "os_call_after")]
  public static void CallAfter
    (int delay,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "xos_call_every")]
  public static OS.Error* XCallEvery
    (int delay,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "os_call_every")]
  public static void CallEvery
    (int delay,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "xos_remove_ticker_event")]
  public static OS.Error* XRemoveTickerEvent
    (AsmRoutine code,
     char* handle);

  [CCode (cname = "os_remove_ticker_event")]
  public static void RemoveTickerEvent
    (AsmRoutine code,
     char* handle);

  [CCode (cname = "xos_install_key_handler")]
  public static OS.Error* XInstallKeyHandler
    (KeyHandler key_handler,
     out KeyHandler old_key_handler);

  [CCode (cname = "os_install_key_handler")]
  public static KeyHandler InstallKeyHandler
    (KeyHandler key_handler);

  [CCode (cname = "xos_check_mode_valid")]
  public static OS.Error* XCheckModeValid
    (Mode mode,
     out int mode_status,
     out Mode alternative_mode,
     out uint psr);

  [CCode (cname = "os_check_mode_valid")]
  public static bits CheckModeValid
    (Mode mode,
     out int mode_status,
     out Mode alternative_mode);

  [CCode (cname = "xos_change_environment")]
  public static OS.Error* XChangeEnvironment
    (HandlerType handler_type,
     AsmRoutine handler,
     char* handle,
     char* buffer,
     out AsmRoutine old_handler,
     out char* old_handle,
     out char* old_buffer);

  [CCode (cname = "os_change_environment")]
  public static AsmRoutine ChangeEnvironment
    (HandlerType handler_type,
     AsmRoutine handler,
     char* handle,
     char* buffer,
     out char* old_handle,
     out char* old_buffer);

  [CCode (cname = "xosclaimscreenmemory_free")]
  public static OS.Error* XOSClaimScreenMemory_Free ();

  [CCode (cname = "osclaimscreenmemory_free")]
  public static void OSClaimScreenMemory_Free ();

  [CCode (cname = "xosclaimscreenmemory_alloc")]
  public static OS.Error* XOSClaimScreenMemory_Alloc
    (int size,
     out int size_out,
     out char* blk,
     out uint psr);

  [CCode (cname = "osclaimscreenmemory_alloc")]
  public static bits OSClaimScreenMemory_Alloc
    (int size,
     out int size_out,
     out char* blk);

  [CCode (cname = "xos_read_monotonic_time")]
  public static OS.Error* XReadMonotonicTime
    (out T t);

  [CCode (cname = "os_read_monotonic_time")]
  public static T ReadMonotonicTime ();

  [CCode (cname = "xos_substitute_args")]
  public static OS.Error* XSubstituteArgs
    (string args,
     string buffer,
     int size,
     string source,
     int source_size,
     out int used);

  [CCode (cname = "os_substitute_args")]
  public static int SubstituteArgs
    (string args,
     string buffer,
     int size,
     string source,
     int source_size);

  [CCode (cname = "xos_substitute_args32")]
  public static OS.Error* XSubstituteArgs32
    (string args,
     string buffer,
     int size,
     string source,
     int source_size,
     SubstituteArgsFlags flags,
     out int used);

  [CCode (cname = "os_substitute_args32")]
  public static int SubstituteArgs32
    (string args,
     string buffer,
     int size,
     string source,
     int source_size,
     SubstituteArgsFlags flags);

  [CCode (cname = "xos_pretty_print")]
  public static OS.Error* XPrettyPrint
    (string string,
     char* dictionary,
     string special);

  [CCode (cname = "os_pretty_print")]
  public static void PrettyPrint
    (string string,
     char* dictionary,
     string special);

  [CCode (cname = "xosprettyprint_get_os_dict")]
  public static OS.Error* XOSPrettyPrint_GetOSDict
    (out char* os_dict);

  [CCode (cname = "osprettyprint_get_os_dict")]
  public static char* OSPrettyPrint_GetOSDict ();

  [CCode (cname = "xos_plot")]
  public static OS.Error* XPlot
    (PlotCode plot_code,
     int x,
     int y);

  [CCode (cname = "os_plot")]
  public static void Plot
    (PlotCode plot_code,
     int x,
     int y);

  [CCode (cname = "xos_writen")]
  public static OS.Error* XWriteN
    (string s,
     int size);

  [CCode (cname = "os_writen")]
  public static void WriteN
    (string s,
     int size);

  [CCode (cname = "xos_add_to_vector")]
  public static OS.Error* XAddToVector
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "os_add_to_vector")]
  public static void AddToVector
    (int vector,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xos_write_env")]
  public static OS.Error* XWriteEnv
    (string command,
     DateAndTime start);

  [CCode (cname = "os_write_env")]
  public static void WriteEnv
    (string command,
     DateAndTime start);

  [CCode (cname = "xos_read_args")]
  public static OS.Error* XReadArgs
    (string keywords,
     string input,
     string buffer,
     int size,
     out int spare);

  [CCode (cname = "os_read_args")]
  public static int ReadArgs
    (string keywords,
     string input,
     string buffer,
     int size);

  [CCode (cname = "xos_read_ram_fs_limits")]
  public static OS.Error* XReadRAMFsLimits
    (out char* start,
     out char* end);

  [CCode (cname = "os_read_ram_fs_limits")]
  public static void ReadRAMFsLimits
    (out char* start,
     out char* end);

  [CCode (cname = "xos_claim_device_vector")]
  public static OS.Error* XClaimDeviceVector
    (DeviceType device,
     AsmRoutine driver,
     int handle,
     int* status,
     bits mask);

  [CCode (cname = "os_claim_device_vector")]
  public static void ClaimDeviceVector
    (DeviceType device,
     AsmRoutine driver,
     int handle,
     int* status,
     bits mask);

  [CCode (cname = "xos_claim_device_vector_and_get_handler")]
  public static OS.Error* XClaimDeviceVectorAndGetHandler
    (DeviceType device,
     AsmRoutine driver,
     int handle,
     int* status,
     bits mask,
     out AsmRoutine handler,
     out int ws_ptr);

  [CCode (cname = "os_claim_device_vector_and_get_handler")]
  public static void ClaimDeviceVectorAndGetHandler
    (DeviceType device,
     AsmRoutine driver,
     int handle,
     int* status,
     bits mask,
     out AsmRoutine handler,
     out int ws_ptr);

  [CCode (cname = "xos_claim_core_irq_handler")]
  public static OS.Error* XClaimCoreIRQHandler
    (AsmRoutine handler,
     int ws_ptr,
     out AsmRoutine handler_unclaimed,
     out int ws_ptr_unclaimed);

  [CCode (cname = "os_claim_core_irq_handler")]
  public static void ClaimCoreIRQHandler
    (AsmRoutine handler,
     int ws_ptr,
     out AsmRoutine handler_unclaimed,
     out int ws_ptr_unclaimed);

  [CCode (cname = "xos_release_device_vector")]
  public static OS.Error* XReleaseDeviceVector
    (DeviceType device,
     AsmRoutine driver,
     char* handle,
     int* status,
     bits mask);

  [CCode (cname = "os_release_device_vector")]
  public static void ReleaseDeviceVector
    (DeviceType device,
     AsmRoutine driver,
     char* handle,
     int* status,
     bits mask);

  [CCode (cname = "xos_release_core_irq_handler")]
  public static OS.Error* XReleaseCoreIRQHandler
    (AsmRoutine handler,
     int ws_ptr);

  [CCode (cname = "os_release_core_irq_handler")]
  public static void ReleaseCoreIRQHandler
    (AsmRoutine handler,
     int ws_ptr);

  [CCode (cname = "xos_delink_application")]
  public static OS.Error* XDelinkApplication
    (char* vector_details,
     int size,
     out int used);

  [CCode (cname = "os_delink_application")]
  public static int DelinkApplication
    (char* vector_details,
     int size);

  [CCode (cname = "xos_relink_application")]
  public static OS.Error* XRelinkApplication
    (char* vector_details);

  [CCode (cname = "os_relink_application")]
  public static void RelinkApplication
    (char* vector_details);

  [CCode (cname = "xos_heap_sort")]
  public static OS.Error* XHeapSort
    (int count,
     int* array,
     SortType compare_fn,
     char* handle,
     char* objects,
     int object_size,
     char* workspace);

  [CCode (cname = "os_heap_sort")]
  public static void HeapSort
    (int count,
     int* array,
     SortType compare_fn,
     char* handle,
     char* objects,
     int object_size,
     char* workspace);

  [CCode (cname = "xos_heap_sort32")]
  public static OS.Error* XHeapSort32
    (int count,
     int* array,
     SortType compare_fn,
     char* handle,
     char* objects,
     int object_size,
     char* workspace,
     HeapSortFlags flags);

  [CCode (cname = "os_heap_sort32")]
  public static void HeapSort32
    (int count,
     int* array,
     SortType compare_fn,
     char* handle,
     char* objects,
     int object_size,
     char* workspace,
     HeapSortFlags flags);

  [CCode (cname = "xos_exit_and_die")]
  public static OS.Error* XExitAndDie
    (Error error,
     int rc,
     string module_name);

  [CCode (cname = "os_exit_and_die")]
  public static void ExitAndDie
    (Error error,
     int rc,
     string module_name);

  [CCode (cname = "xos_read_mem_map_info")]
  public static OS.Error* XReadMemMapInfo
    (out int page_size,
     out int page_count);

  [CCode (cname = "os_read_mem_map_info")]
  public static void ReadMemMapInfo
    (out int page_size,
     out int page_count);

  [CCode (cname = "xos_read_mem_map_entries")]
  public static OS.Error* XReadMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "os_read_mem_map_entries")]
  public static void ReadMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "xos_set_mem_map_entries")]
  public static OS.Error* XSetMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "os_set_mem_map_entries")]
  public static void SetMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "xos_add_call_back")]
  public static OS.Error* XAddCallBack
    (AsmRoutine call_back,
     char* handle);

  [CCode (cname = "os_add_call_back")]
  public static void AddCallBack
    (AsmRoutine call_back,
     char* handle);

  [CCode (cname = "xos_read_default_handler")]
  public static OS.Error* XReadDefaultHandler
    (int handler_type,
     out AsmRoutine handler,
     out char* handle,
     out char* buffer);

  [CCode (cname = "os_read_default_handler")]
  public static AsmRoutine ReadDefaultHandler
    (int handler_type,
     out char* handle,
     out char* buffer);

  [CCode (cname = "xos_set_ecf_origin")]
  public static OS.Error* XSetECFOrigin
    (int x,
     int y);

  [CCode (cname = "os_set_ecf_origin")]
  public static void SetECFOrigin
    (int x,
     int y);

  [CCode (cname = "xos_confirm")]
  public static OS.Error* XConfirm
    (out char key,
     out uint psr);

  [CCode (cname = "os_confirm")]
  public static bits Confirm
    (out char key);

  [CCode (cname = "xos_changed_box")]
  public static OS.Error* XChangedBox
    (ChangedBoxState state,
     out bool enabled,
     out ChangeBox change_box);

  [CCode (cname = "os_changed_box")]
  public static bool ChangedBox
    (ChangedBoxState state,
     out ChangeBox change_box);

  [CCode (cname = "xos_crc")]
  public static OS.Error* XCRC
    (int crc_in,
     char* block,
     char* end,
     int stride,
     out int crc);

  [CCode (cname = "os_crc")]
  public static int CRC
    (int crc_in,
     char* block,
     char* end,
     int stride);

  [CCode (cname = "xos_read_dynamic_area")]
  public static OS.Error* XReadDynamicArea
    (DynamicAreaNo area,
     out char* area_start,
     out int size,
     out int size_limit);

  [CCode (cname = "os_read_dynamic_area")]
  public static char* ReadDynamicArea
    (DynamicAreaNo area,
     out int size,
     out int size_limit);

  [CCode (cname = "xos_print_char")]
  public static OS.Error* XPrintChar
    (char c);

  [CCode (cname = "os_print_char")]
  public static void PrintChar
    (char c);

  [CCode (cname = "xos_change_redirection")]
  public static OS.Error* XChangeRedirection
    (F input,
     F output,
     out F old_input,
     out F old_output);

  [CCode (cname = "os_change_redirection")]
  public static void ChangeRedirection
    (F input,
     F output,
     out F old_input,
     out F old_output);

  [CCode (cname = "xos_change_redirectionw")]
  public static OS.Error* XChangeRedirectionW
    (FW input,
     FW output,
     out FW old_input,
     out FW old_output);

  [CCode (cname = "os_change_redirectionw")]
  public static void ChangeRedirectionW
    (FW input,
     FW output,
     out FW old_input,
     out FW old_output);

  [CCode (cname = "xos_remove_call_back")]
  public static OS.Error* XRemoveCallBack
    (AsmRoutine call_back,
     char* handle);

  [CCode (cname = "os_remove_call_back")]
  public static void RemoveCallBack
    (AsmRoutine call_back,
     char* handle);

  [CCode (cname = "xos_find_mem_map_entries")]
  public static OS.Error* XFindMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "os_find_mem_map_entries")]
  public static void FindMemMapEntries
    (MemMapRequestList request_list);

  [CCode (cname = "xos_set_colour")]
  public static OS.Error* XSetColour
    (ColourFlags flags,
     ColourNumber colour_number);

  [CCode (cname = "os_set_colour")]
  public static void SetColour
    (ColourFlags flags,
     ColourNumber colour_number);

  [CCode (cname = "xos_read_colour")]
  public static OS.Error* XReadColour
    (ColourFlags flags,
     ECF pattern,
     out ColourFlags flags_out,
     out ColourNumber colour_number);

  [CCode (cname = "os_read_colour")]
  public static ColourNumber ReadColour
    (ColourFlags flags,
     ECF pattern,
     out ColourFlags flags_out);

  [CCode (cname = "xos_claim_swi")]
  public static OS.Error* XClaimSWI
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "os_claim_swi")]
  public static void ClaimSWI
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xos_release_swi")]
  public static OS.Error* XReleaseSWI
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "os_release_swi")]
  public static void ReleaseSWI
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xospointer_get")]
  public static OS.Error* XOSPointer_Get
    (out PointerType pointer_type);

  [CCode (cname = "ospointer_get")]
  public static PointerType OSPointer_Get ();

  [CCode (cname = "xospointer_set")]
  public static OS.Error* XOSPointer_Set
    (PointerType pointer_type);

  [CCode (cname = "ospointer_set")]
  public static void OSPointer_Set
    (PointerType pointer_type);

  [CCode (cname = "xospointer_read_alternate_position")]
  public static OS.Error* XOSPointer_ReadAlternatePosition
    (out int xposition,
     out int yposition);

  [CCode (cname = "ospointer_read_alternate_position")]
  public static void OSPointer_ReadAlternatePosition
    (out int xposition,
     out int yposition);

  [CCode (cname = "xosscreenmode_select")]
  public static OS.Error* XOSScreenMode_Select
    (Mode mode);

  [CCode (cname = "osscreenmode_select")]
  public static void OSScreenMode_Select
    (Mode mode);

  [CCode (cname = "xosscreenmode_current")]
  public static OS.Error* XOSScreenMode_Current
    (out Mode mode);

  [CCode (cname = "osscreenmode_current")]
  public static Mode OSScreenMode_Current ();

  [CCode (cname = "xosscreenmode_enumerate")]
  public static OS.Error* XOSScreenMode_Enumerate
    (int context,
     ModeBlock block,
     int block_size,
     out bool complete,
     out int context_out,
     out char* end,
     out int free);

  [CCode (cname = "osscreenmode_enumerate")]
  public static bool OSScreenMode_Enumerate
    (int context,
     ModeBlock block,
     int block_size,
     out int context_out,
     out char* end,
     out int free);

  [CCode (cname = "xosscreenmode_config_acceleration")]
  public static OS.Error* XOSScreenMode_ConfigAcceleration
    (OSScreenModeConfigAcceleration_Flags flags,
     OSScreenModeConfigAcceleration_CleanerLaziness laziness,
     out OSScreenModeConfigAcceleration_Flags current_flags,
     out OSScreenModeConfigAcceleration_CleanerLaziness current_laziness);

  [CCode (cname = "osscreenmode_config_acceleration")]
  public static OSScreenModeConfigAcceleration_Flags OSScreenMode_ConfigAcceleration
    (OSScreenModeConfigAcceleration_Flags flags,
     OSScreenModeConfigAcceleration_CleanerLaziness laziness,
     out OSScreenModeConfigAcceleration_CleanerLaziness current_laziness);

  [CCode (cname = "xosscreenmode_request_clean")]
  public static OS.Error* XOSScreenMode_RequestClean ();

  [CCode (cname = "osscreenmode_request_clean")]
  public static void OSScreenMode_RequestClean ();

  [CCode (cname = "xosscreenmode_force_clean")]
  public static OS.Error* XOSScreenMode_ForceClean ();

  [CCode (cname = "osscreenmode_force_clean")]
  public static void OSScreenMode_ForceClean ();

  [CCode (cname = "xosscreenmode_count_number_banks")]
  public static OS.Error* XOSScreenMode_CountNumberBanks
    (out int num_banks);

  [CCode (cname = "osscreenmode_count_number_banks")]
  public static int OSScreenMode_CountNumberBanks ();

  [CCode (cname = "xosscreenmode_select_display_bank")]
  public static OS.Error* XOSScreenMode_SelectDisplayBank
    (int bank_number,
     out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "osscreenmode_select_display_bank")]
  public static void OSScreenMode_SelectDisplayBank
    (int bank_number,
     out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "xosscreenmode_read_display_bank")]
  public static OS.Error* XOSScreenMode_ReadDisplayBank
    (out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "osscreenmode_read_display_bank")]
  public static void OSScreenMode_ReadDisplayBank
    (out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "xosscreenmode_select_driver_bank")]
  public static OS.Error* XOSScreenMode_SelectDriverBank
    (int bank_number,
     out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "osscreenmode_select_driver_bank")]
  public static void OSScreenMode_SelectDriverBank
    (int bank_number,
     out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "xosscreenmode_read_driver_bank")]
  public static OS.Error* XOSScreenMode_ReadDriverBank
    (out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "osscreenmode_read_driver_bank")]
  public static void OSScreenMode_ReadDriverBank
    (out int prev_bank,
     out int new_bank,
     out char* bank_addr);

  [CCode (cname = "xosscreenmode_copy_bank")]
  public static OS.Error* XOSScreenMode_CopyBank
    (int src_bank,
     int dst_bank);

  [CCode (cname = "osscreenmode_copy_bank")]
  public static void OSScreenMode_CopyBank
    (int src_bank,
     int dst_bank);

  [CCode (cname = "xosscreenmode_select_display_device")]
  public static OS.Error* XOSScreenMode_SelectDisplayDevice
    (int display_number,
     out int prev_display_number);

  [CCode (cname = "osscreenmode_select_display_device")]
  public static void OSScreenMode_SelectDisplayDevice
    (int display_number,
     out int prev_display_number);

  [CCode (cname = "xosscreenmode_read_display_device")]
  public static OS.Error* XOSScreenMode_ReadDisplayDevice
    (out int cur_display_number);

  [CCode (cname = "osscreenmode_read_display_device")]
  public static void OSScreenMode_ReadDisplayDevice
    (out int cur_display_number);

  [CCode (cname = "xosscreenmode_read_display_details")]
  public static OS.Error* XOSScreenMode_ReadDisplayDetails
    (int display_number,
     OSDisplayDevice_TagTagLength tag,
     int* buf,
     int buf_size,
     out int buf_size_out);

  [CCode (cname = "osscreenmode_read_display_details")]
  public static void OSScreenMode_ReadDisplayDetails
    (int display_number,
     OSDisplayDevice_TagTagLength tag,
     int* buf,
     int buf_size,
     out int buf_size_out);

  [CCode (cname = "xosscreenmode_mode_string_to_specifier")]
  public static OS.Error* XOSScreenMode_ModeStringToSpecifier
    (char* mode_string,
     Mode mode_specifier,
     int size_mode_specifier);

  [CCode (cname = "osscreenmode_mode_string_to_specifier")]
  public static void OSScreenMode_ModeStringToSpecifier
    (char* mode_string,
     Mode mode_specifier,
     int size_mode_specifier);

  [CCode (cname = "xosscreenmode_mode_specifier_to_string")]
  public static OS.Error* XOSScreenMode_ModeSpecifierToString
    (Mode mode_specifier,
     char* buf_mode_string,
     int buf_mode_string_size,
     out int buf_free);

  [CCode (cname = "osscreenmode_mode_specifier_to_string")]
  public static void OSScreenMode_ModeSpecifierToString
    (Mode mode_specifier,
     char* buf_mode_string,
     int buf_mode_string_size,
     out int buf_free);

  [CCode (cname = "xosscreenmode_select_mode_with_mode_string")]
  public static OS.Error* XOSScreenMode_SelectModeWithModeString
    (char* mode_string);

  [CCode (cname = "osscreenmode_select_mode_with_mode_string")]
  public static void OSScreenMode_SelectModeWithModeString
    (char* mode_string);

  [CCode (cname = "xosscreenmode_read_max_display_number")]
  public static OS.Error* XOSScreenMode_ReadMaxDisplayNumber
    (out int display_number);

  [CCode (cname = "osscreenmode_read_max_display_number")]
  public static int OSScreenMode_ReadMaxDisplayNumber ();

  [CCode (cname = "xosscreenmode_deregister_display_device")]
  public static OS.Error* XOSScreenMode_DeregisterDisplayDevice
    (int display_number);

  [CCode (cname = "osscreenmode_deregister_display_device")]
  public static void OSScreenMode_DeregisterDisplayDevice
    (int display_number);

  [CCode (cname = "xosscreenmode_register_display_device")]
  public static OS.Error* XOSScreenMode_RegisterDisplayDevice
    (OSDisplayDeviceDescriptor descriptor,
     out int display_number,
     out AsmRoutine vsync_dispatch,
     out int ws_ptr,
     out OSScreenModeRegisterDisplayDriver_Action action);

  [CCode (cname = "osscreenmode_register_display_device")]
  public static void OSScreenMode_RegisterDisplayDevice
    (OSDisplayDeviceDescriptor descriptor,
     out int display_number,
     out AsmRoutine vsync_dispatch,
     out int ws_ptr,
     out OSScreenModeRegisterDisplayDriver_Action action);

  [CCode (cname = "xosdynamicarea_create")]
  public static OS.Error* XOSDynamicArea_Create
    (DynamicAreaNo area,
     int size,
     char* base_address,
     AreaFlags flags,
     int size_limit,
     AsmRoutine handler,
     char* workspace,
     string description,
     out DynamicAreaNo area_out,
     out char* base_address_out,
     out int size_limit_out);

  [CCode (cname = "osdynamicarea_create")]
  public static DynamicAreaNo OSDynamicArea_Create
    (DynamicAreaNo area,
     int size,
     char* base_address,
     AreaFlags flags,
     int size_limit,
     AsmRoutine handler,
     char* workspace,
     string description,
     out char* base_address_out,
     out int size_limit_out);

  [CCode (cname = "xosdynamicarea_delete")]
  public static OS.Error* XOSDynamicArea_Delete
    (DynamicAreaNo area);

  [CCode (cname = "osdynamicarea_delete")]
  public static void OSDynamicArea_Delete
    (DynamicAreaNo area);

  [CCode (cname = "xosdynamicarea_read")]
  public static OS.Error* XOSDynamicArea_Read
    (DynamicAreaNo area,
     out int size,
     out char* base_address,
     out AreaFlags flags,
     out int size_limit,
     out AsmRoutine handler,
     out char* workspace,
     out string description);

  [CCode (cname = "osdynamicarea_read")]
  public static void OSDynamicArea_Read
    (DynamicAreaNo area,
     out int size,
     out char* base_address,
     out AreaFlags flags,
     out int size_limit,
     out AsmRoutine handler,
     out char* workspace,
     out string description);

  [CCode (cname = "xosdynamicarea_enumerate")]
  public static OS.Error* XOSDynamicArea_Enumerate
    (int context,
     out int context_out);

  [CCode (cname = "osdynamicarea_enumerate")]
  public static int OSDynamicArea_Enumerate
    (int context);

  [CCode (cname = "xosdynamicarea_renumber")]
  public static OS.Error* XOSDynamicArea_Renumber
    (DynamicAreaNo old_area,
     DynamicAreaNo new_area);

  [CCode (cname = "osdynamicarea_renumber")]
  public static void OSDynamicArea_Renumber
    (DynamicAreaNo old_area,
     DynamicAreaNo new_area);

  [CCode (cname = "xosdynamicarea_free_space")]
  public static OS.Error* XOSDynamicArea_FreeSpace
    (DynamicAreaNo exclude_area,
     out int free_space);

  [CCode (cname = "osdynamicarea_free_space")]
  public static int OSDynamicArea_FreeSpace
    (DynamicAreaNo exclude_area);

  [CCode (cname = "xosdynamicarea_set_clamps")]
  public static OS.Error* XOSDynamicArea_SetClamps
    (int unlimited_area_clamp,
     int limited_area_clamp,
     int sparse_area_clamp,
     out int old_unlimited_area_clamp,
     out int old_limited_area_clamp,
     out int old_sparse_area_clamp);

  [CCode (cname = "osdynamicarea_set_clamps")]
  public static void OSDynamicArea_SetClamps
    (int unlimited_area_clamp,
     int limited_area_clamp,
     int sparse_area_clamp,
     out int old_unlimited_area_clamp,
     out int old_limited_area_clamp,
     out int old_sparse_area_clamp);

  [CCode (cname = "xosdynamicarea_ensure_region")]
  public static OS.Error* XOSDynamicArea_EnsureRegion
    (DynamicAreaNo area,
     char* base_address,
     int size);

  [CCode (cname = "osdynamicarea_ensure_region")]
  public static void OSDynamicArea_EnsureRegion
    (DynamicAreaNo area,
     char* base_address,
     int size);

  [CCode (cname = "xosdynamicarea_release_region")]
  public static OS.Error* XOSDynamicArea_ReleaseRegion
    (DynamicAreaNo area,
     char* base_address,
     int size);

  [CCode (cname = "osdynamicarea_release_region")]
  public static void OSDynamicArea_ReleaseRegion
    (DynamicAreaNo area,
     char* base_address,
     int size);

  [CCode (cname = "xosdynamicarea_lock_area")]
  public static OS.Error* XOSDynamicArea_LockArea
    (DynamicAreaNo area,
     int lock_code);

  [CCode (cname = "osdynamicarea_lock_area")]
  public static void OSDynamicArea_LockArea
    (DynamicAreaNo area,
     int lock_code);

  [CCode (cname = "xosdynamicarea_unlock_area")]
  public static OS.Error* XOSDynamicArea_UnlockArea
    (DynamicAreaNo area,
     int lock_code);

  [CCode (cname = "osdynamicarea_unlock_area")]
  public static void OSDynamicArea_UnlockArea
    (DynamicAreaNo area,
     int lock_code);

  [CCode (cname = "xosdynamicarea_heap_describe")]
  public static OS.Error* XOSDynamicArea_HeapDescribe
    (DynamicAreaNo area,
     out int max,
     out int free);

  [CCode (cname = "osdynamicarea_heap_describe")]
  public static int OSDynamicArea_HeapDescribe
    (DynamicAreaNo area,
     out int max);

  [CCode (cname = "xosdynamicarea_heap_alloc")]
  public static OS.Error* XOSDynamicArea_HeapAlloc
    (DynamicAreaNo area,
     int size,
     out char* blk);

  [CCode (cname = "osdynamicarea_heap_alloc")]
  public static char* OSDynamicArea_HeapAlloc
    (DynamicAreaNo area,
     int size);

  [CCode (cname = "xosdynamicarea_heap_free")]
  public static OS.Error* XOSDynamicArea_HeapFree
    (DynamicAreaNo area,
     char* blk);

  [CCode (cname = "osdynamicarea_heap_free")]
  public static void OSDynamicArea_HeapFree
    (DynamicAreaNo area,
     char* blk);

  [CCode (cname = "xosdynamicarea_heap_realloc")]
  public static OS.Error* XOSDynamicArea_HeapRealloc
    (DynamicAreaNo area,
     char* blk,
     int size_increase,
     out char* blk_out);

  [CCode (cname = "osdynamicarea_heap_realloc")]
  public static char* OSDynamicArea_HeapRealloc
    (DynamicAreaNo area,
     char* blk,
     int size_increase);

  [CCode (cname = "xosdynamicarea_heap_read_size")]
  public static OS.Error* XOSDynamicArea_HeapReadSize
    (DynamicAreaNo area,
     char* blk,
     out int size);

  [CCode (cname = "osdynamicarea_heap_read_size")]
  public static int OSDynamicArea_HeapReadSize
    (DynamicAreaNo area,
     char* blk);

  [CCode (cname = "xosdynamicarea_change_domain_state")]
  public static OS.Error* XOSDynamicArea_ChangeDomainState
    (DynamicAreaNo area,
     OSDynamicAreaDomain_State state,
     out OSDynamicAreaDomain_State old_state);

  [CCode (cname = "osdynamicarea_change_domain_state")]
  public static OSDynamicAreaDomain_State OSDynamicArea_ChangeDomainState
    (DynamicAreaNo area,
     OSDynamicAreaDomain_State state);

  [CCode (cname = "xos_abort_trap")]
  public static OS.Error* XAbortTrap
    (OSAbortTrap_Flags operation,
     int low_address,
     int high_address,
     AsmRoutine abort_handler,
     int r12_abort_handler);

  [CCode (cname = "os_abort_trap")]
  public static void AbortTrap
    (OSAbortTrap_Flags operation,
     int low_address,
     int high_address,
     AsmRoutine abort_handler,
     int r12_abort_handler);

  [CCode (cname = "xosmemory_page_op")]
  public static OS.Error* XOSMemory_PageOp
    (OSMemory_Flags flags,
     PageBlock page_block,
     int page_count);

  [CCode (cname = "osmemory_page_op")]
  public static void OSMemory_PageOp
    (OSMemory_Flags flags,
     PageBlock page_block,
     int page_count);

  [CCode (cname = "xosmemory_read_arrangement_table_size")]
  public static OS.Error* XOSMemory_ReadArrangementTableSize
    (out int table_size,
     out int page_size);

  [CCode (cname = "osmemory_read_arrangement_table_size")]
  public static int OSMemory_ReadArrangementTableSize
    (out int page_size);

  [CCode (cname = "xosmemory_read_arrangement_table")]
  public static OS.Error* XOSMemory_ReadArrangementTable
    (char* arrangement_table);

  [CCode (cname = "osmemory_read_arrangement_table")]
  public static void OSMemory_ReadArrangementTable
    (char* arrangement_table);

  [CCode (cname = "xosmemory_read_size")]
  public static OS.Error* XOSMemory_ReadSize
    (OSMemory_TypeFlags type,
     out int page_count,
     out int page_size);

  [CCode (cname = "osmemory_read_size")]
  public static void OSMemory_ReadSize
    (OSMemory_TypeFlags type,
     out int page_count,
     out int page_size);

  [CCode (cname = "xosmemory_read_controller")]
  public static OS.Error* XOSMemory_ReadController
    (OSMemory_ControllerFlags controller,
     out AsmRoutine controller_addr);

  [CCode (cname = "osmemory_read_controller")]
  public static void OSMemory_ReadController
    (OSMemory_ControllerFlags controller,
     out AsmRoutine controller_addr);

  [CCode (cname = "xosmemory_recommend_page")]
  public static OS.Error* XOSMemory_RecommendPage
    (OSMemoryRecommendPage_Flags flags,
     int size,
     int alignment,
     out int page_number);

  [CCode (cname = "osmemory_recommend_page")]
  public static int OSMemory_RecommendPage
    (OSMemoryRecommendPage_Flags flags,
     int size,
     int alignment);

  [CCode (cname = "xosmemory_map_in_permanent_io")]
  public static OS.Error* XOSMemory_MapInPermanentIO
    (OSMemory_IOFlags flags,
     char* phys_addr,
     int size,
     out char* log_addr);

  [CCode (cname = "osmemory_map_in_permanent_io")]
  public static char* OSMemory_MapInPermanentIO
    (OSMemory_IOFlags flags,
     char* phys_addr,
     int size);

  [CCode (cname = "xosmemory_map_in_temporary_io")]
  public static OS.Error* XOSMemory_MapInTemporaryIO
    (OSMemory_IOFlags flags,
     char* phys_addr,
     out char* log_addr,
     out OSMemory_IOReferenceKey key);

  [CCode (cname = "osmemory_map_in_temporary_io")]
  public static char* OSMemory_MapInTemporaryIO
    (OSMemory_IOFlags flags,
     char* phys_addr,
     out OSMemory_IOReferenceKey key);

  [CCode (cname = "xosmemory_map_out_temporary_io")]
  public static OS.Error* XOSMemory_MapOutTemporaryIO
    (OSMemory_IOReferenceKey key);

  [CCode (cname = "osmemory_map_out_temporary_io")]
  public static void OSMemory_MapOutTemporaryIO
    (OSMemory_IOReferenceKey key);

  [CCode (cname = "xosmemory_get_area_info")]
  public static OS.Error* XOSMemory_GetAreaInfo
    (OSMemory_AreaFlags type,
     out char* base,
     out int allocated,
     out int used);

  [CCode (cname = "osmemory_get_area_info")]
  public static char* OSMemory_GetAreaInfo
    (OSMemory_AreaFlags type,
     out int allocated,
     out int used);

  [CCode (cname = "xosmemory_check_memory_access")]
  public static OS.Error* XOSMemory_CheckMemoryAccess
    (OSMemoryCheckMemoryAccess_Flags flags,
     char* low,
     char* high,
     out OSMemoryCheckMemoryAccess_Bits access_flags);

  [CCode (cname = "osmemory_check_memory_access")]
  public static OSMemoryCheckMemoryAccess_Bits OSMemory_CheckMemoryAccess
    (OSMemoryCheckMemoryAccess_Flags flags,
     char* low,
     char* high);

  [CCode (cname = "xosmemory_read_controller_base_address")]
  public static OS.Error* XOSMemory_ReadControllerBaseAddress
    (OSMemoryReadControllerBaseAddress_Flags flags,
     OSMemory_ControllerFlags controller,
     out char* address);

  [CCode (cname = "osmemory_read_controller_base_address")]
  public static char* OSMemory_ReadControllerBaseAddress
    (OSMemoryReadControllerBaseAddress_Flags flags,
     OSMemory_ControllerFlags controller);

  [CCode (cname = "xosclaimprocessorvector_alloc")]
  public static OS.Error* XOSClaimProcessorVector_Alloc
    (bits vector,
     AsmRoutine routine,
     out AsmRoutine old_routine);

  [CCode (cname = "osclaimprocessorvector_alloc")]
  public static AsmRoutine OSClaimProcessorVector_Alloc
    (bits vector,
     AsmRoutine routine);

  [CCode (cname = "xosclaimprocessorvector_free")]
  public static OS.Error* XOSClaimProcessorVector_Free
    (bits vector,
     AsmRoutine old_routine);

  [CCode (cname = "osclaimprocessorvector_free")]
  public static void OSClaimProcessorVector_Free
    (bits vector,
     AsmRoutine old_routine);

  [CCode (cname = "xos_reset")]
  public static OS.Error* XReset ();

  [CCode (cname = "os_reset")]
  public static void Reset ();

  [CCode (cname = "xos_mmu_control")]
  public static OS.Error* XMMUControl
    (bits eor_mask,
     bits and_mask,
     out bits old_value);

  [CCode (cname = "os_mmu_control")]
  public static bits MMUControl
    (bits eor_mask,
     bits and_mask);

  [CCode (cname = "xos_mmu_control_flush_request")]
  public static OS.Error* XMMUControlFlushRequest
    (MMUControlFlushRequestFlags request_type);

  [CCode (cname = "os_mmu_control_flush_request")]
  public static void MMUControlFlushRequest
    (MMUControlFlushRequestFlags request_type);

  [CCode (cname = "xos_mmu_control_flush_request_single_entry")]
  public static OS.Error* XMMUControlFlushRequestSingleEntry
    (char* entry_address);

  [CCode (cname = "os_mmu_control_flush_request_single_entry")]
  public static void MMUControlFlushRequestSingleEntry
    (char* entry_address);

  [CCode (cname = "xosplatformfeatures_get_features")]
  public static OS.Error* XOSPlatformFeatures_GetFeatures
    (out PlatformFeatureFlags flags,
     out AsmRoutine predisable_fn);

  [CCode (cname = "osplatformfeatures_get_features")]
  public static PlatformFeatureFlags OSPlatformFeatures_GetFeatures
    (out AsmRoutine predisable_fn);

  [CCode (cname = "xosplatformfeatures_get_mmu_features")]
  public static OS.Error* XOSPlatformFeatures_GetMMUFeatures
    (out int cache_enabled_state,
     out int cache_disabled_state,
     out int acceptable_flags_proc,
     out int required_flags_proc,
     out int acceptable_flags_os,
     out int required_flags_os);

  [CCode (cname = "osplatformfeatures_get_mmu_features")]
  public static void OSPlatformFeatures_GetMMUFeatures
    (out int cache_enabled_state,
     out int cache_disabled_state,
     out int acceptable_flags_proc,
     out int required_flags_proc,
     out int acceptable_flags_os,
     out int required_flags_os);

  [CCode (cname = "xos_synchronise_code_areas")]
  public static OS.Error* XSynchroniseCodeAreas
    (bits flags,
     AsmRoutine start,
     AsmRoutine end);

  [CCode (cname = "os_synchronise_code_areas")]
  public static void SynchroniseCodeAreas
    (bits flags,
     AsmRoutine start,
     AsmRoutine end);

  [CCode (cname = "xosambcontrol_create")]
  public static OS.Error* XOSAMBControl_Create
    (int pages,
     out int pages_out,
     out AMB amb);

  [CCode (cname = "osambcontrol_create")]
  public static void OSAMBControl_Create
    (int pages,
     out int pages_out,
     out AMB amb);

  [CCode (cname = "xosambcontrol_delete")]
  public static OS.Error* XOSAMBControl_Delete
    (AMB amb);

  [CCode (cname = "osambcontrol_delete")]
  public static void OSAMBControl_Delete
    (AMB amb);

  [CCode (cname = "xosambcontrol_resize")]
  public static OS.Error* XOSAMBControl_Resize
    (int pages,
     AMB amb,
     out int pages_out,
     out AMB amb_out,
     out int pages_before);

  [CCode (cname = "osambcontrol_resize")]
  public static void OSAMBControl_Resize
    (int pages,
     AMB amb,
     out int pages_out,
     out AMB amb_out,
     out int pages_before);

  [CCode (cname = "xosambcontrol_map")]
  public static OS.Error* XOSAMBControl_Map
    (AMBControlFlags flags,
     char* base_address,
     AMB amb,
     int start,
     int end);

  [CCode (cname = "osambcontrol_map")]
  public static void OSAMBControl_Map
    (AMBControlFlags flags,
     char* base_address,
     AMB amb,
     int start,
     int end);

  [CCode (cname = "xosambcontrol_info")]
  public static OS.Error* XOSAMBControl_Info
    (AMB amb,
     out char* base_address,
     out int pages);

  [CCode (cname = "osambcontrol_info")]
  public static void OSAMBControl_Info
    (AMB amb,
     out char* base_address,
     out int pages);

  [CCode (cname = "xosambcontrol_lazy_swapping")]
  public static OS.Error* XOSAMBControl_LazySwapping
    (LazySwapping action,
     out LazySwapping state);

  [CCode (cname = "osambcontrol_lazy_swapping")]
  public static void OSAMBControl_LazySwapping
    (LazySwapping action,
     out LazySwapping state);

  [CCode (cname = "xosambcontrol_enumerate")]
  public static OS.Error* XOSAMBControl_Enumerate
    (AMBTable table);

  [CCode (cname = "osambcontrol_enumerate")]
  public static void OSAMBControl_Enumerate
    (AMBTable table);

  [CCode (cname = "xos_enter_usr32")]
  public static OS.Error* XEnterUSR32 ();

  [CCode (cname = "os_enter_usr32")]
  public static void EnterUSR32 ();

  [CCode (cname = "xos_enter_usr26")]
  public static OS.Error* XEnterUSR26 ();

  [CCode (cname = "os_enter_usr26")]
  public static void EnterUSR26 ();

  [CCode (cname = "xosclaimosswi_claim")]
  public static OS.Error* XOSClaimOSSWI_Claim
    (int swi,
     AsmRoutine routine,
     char* handle,
     out AsmRoutine old_routine,
     out char* old_handle);

  [CCode (cname = "osclaimosswi_claim")]
  public static void OSClaimOSSWI_Claim
    (int swi,
     AsmRoutine routine,
     char* handle,
     out AsmRoutine old_routine,
     out char* old_handle);

  [CCode (cname = "xosclaimosswi_release")]
  public static OS.Error* XOSClaimOSSWI_Release
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "osclaimosswi_release")]
  public static void OSClaimOSSWI_Release
    (int swi,
     AsmRoutine routine,
     char* handle);

  [CCode (cname = "xostaskcontrol_read_addr_stack_reset_code")]
  public static OS.Error* XOSTaskControl_ReadAddrStackResetCode
    (out AsmRoutine routine);

  [CCode (cname = "ostaskcontrol_read_addr_stack_reset_code")]
  public static AsmRoutine OSTaskControl_ReadAddrStackResetCode ();

  [CCode (cname = "xos_hardware_call_hal")]
  public static OS.Error* XHardwareCallHAL
    (int arg1,
     int arg2,
     int arg3,
     int arg4,
     int arg5,
     int arg6,
     int arg7,
     int arg8,
     HardwareFlags flags,
     int call,
     out int result1,
     out int result2,
     out int result3,
     out int result4);

  [CCode (cname = "os_hardware_call_hal")]
  public static void HardwareCallHAL
    (int arg1,
     int arg2,
     int arg3,
     int arg4,
     int arg5,
     int arg6,
     int arg7,
     int arg8,
     HardwareFlags flags,
     int call,
     out int result1,
     out int result2,
     out int result3,
     out int result4);

  [CCode (cname = "xos_hardware_find_hal_routine")]
  public static OS.Error* XHardwareFindHALRoutine
    (HardwareFlags flags,
     int call,
     out AsmRoutine routine,
     out int static_base);

  [CCode (cname = "os_hardware_find_hal_routine")]
  public static AsmRoutine HardwareFindHALRoutine
    (HardwareFlags flags,
     int call,
     out int static_base);

  [CCode (cname = "xos_convert_standard_date_and_time")]
  public static OS.Error* XConvertStandardDateAndTime
    (DateAndTime date_and_time,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_standard_date_and_time")]
  public static char ConvertStandardDateAndTime
    (DateAndTime date_and_time,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_date_and_time")]
  public static OS.Error* XConvertDateAndTime
    (DateAndTime date_and_time,
     string buffer,
     int size,
     string format,
     out char* end);

  [CCode (cname = "os_convert_date_and_time")]
  public static char ConvertDateAndTime
    (DateAndTime date_and_time,
     string buffer,
     int size,
     string format);

  [CCode (cname = "xos_convert_hex1")]
  public static OS.Error* XConvertHex1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_hex1")]
  public static char ConvertHex1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_hex2")]
  public static OS.Error* XConvertHex2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_hex2")]
  public static char ConvertHex2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_hex4")]
  public static OS.Error* XConvertHex4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_hex4")]
  public static char ConvertHex4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_hex6")]
  public static OS.Error* XConvertHex6
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_hex6")]
  public static char ConvertHex6
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_hex8")]
  public static OS.Error* XConvertHex8
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_hex8")]
  public static char ConvertHex8
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_cardinal1")]
  public static OS.Error* XConvertCardinal1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_cardinal1")]
  public static char ConvertCardinal1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_cardinal2")]
  public static OS.Error* XConvertCardinal2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_cardinal2")]
  public static char ConvertCardinal2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_cardinal3")]
  public static OS.Error* XConvertCardinal3
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_cardinal3")]
  public static char ConvertCardinal3
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_cardinal4")]
  public static OS.Error* XConvertCardinal4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_cardinal4")]
  public static char ConvertCardinal4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_integer1")]
  public static OS.Error* XConvertInteger1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_integer1")]
  public static char ConvertInteger1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_integer2")]
  public static OS.Error* XConvertInteger2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_integer2")]
  public static char ConvertInteger2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_integer3")]
  public static OS.Error* XConvertInteger3
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_integer3")]
  public static char ConvertInteger3
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_integer4")]
  public static OS.Error* XConvertInteger4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_integer4")]
  public static char ConvertInteger4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_binary1")]
  public static OS.Error* XConvertBinary1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_binary1")]
  public static char ConvertBinary1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_binary2")]
  public static OS.Error* XConvertBinary2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_binary2")]
  public static char ConvertBinary2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_binary3")]
  public static OS.Error* XConvertBinary3
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_binary3")]
  public static char ConvertBinary3
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_binary4")]
  public static OS.Error* XConvertBinary4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_binary4")]
  public static char ConvertBinary4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_cardinal1")]
  public static OS.Error* XConvertSpacedCardinal1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_cardinal1")]
  public static char ConvertSpacedCardinal1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_cardinal2")]
  public static OS.Error* XConvertSpacedCardinal2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_cardinal2")]
  public static char ConvertSpacedCardinal2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_cardinal3")]
  public static OS.Error* XConvertSpacedCardinal3
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_cardinal3")]
  public static char ConvertSpacedCardinal3
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_cardinal4")]
  public static OS.Error* XConvertSpacedCardinal4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_cardinal4")]
  public static char ConvertSpacedCardinal4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_integer1")]
  public static OS.Error* XConvertSpacedInteger1
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_integer1")]
  public static char ConvertSpacedInteger1
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_integer2")]
  public static OS.Error* XConvertSpacedInteger2
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_integer2")]
  public static char ConvertSpacedInteger2
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_integer3")]
  public static OS.Error* XConvertSpacedInteger3
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_integer3")]
  public static char ConvertSpacedInteger3
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_spaced_integer4")]
  public static OS.Error* XConvertSpacedInteger4
    (int value,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_spaced_integer4")]
  public static char ConvertSpacedInteger4
    (int value,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_fixed_net_station")]
  public static OS.Error* XConvertFixedNetStation
    (StationNumber station_number,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_fixed_net_station")]
  public static char ConvertFixedNetStation
    (StationNumber station_number,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_net_station")]
  public static OS.Error* XConvertNetStation
    (StationNumber station_number,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_net_station")]
  public static char ConvertNetStation
    (StationNumber station_number,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_fixed_file_size")]
  public static OS.Error* XConvertFixedFileSize
    (int file_size,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_fixed_file_size")]
  public static char ConvertFixedFileSize
    (int file_size,
     string buffer,
     int size);

  [CCode (cname = "xos_convert_file_size")]
  public static OS.Error* XConvertFileSize
    (int file_size,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "os_convert_file_size")]
  public static char ConvertFileSize
    (int file_size,
     string buffer,
     int size);

  [CCode (cname = "xos_null")]
  public static OS.Error* XNull ();

  [CCode (cname = "os_null")]
  public static void Null ();

  [CCode (cname = "xos_char_to_printer")]
  public static OS.Error* XCharToPrinter ();

  [CCode (cname = "os_char_to_printer")]
  public static void CharToPrinter ();

  [CCode (cname = "xos_printer_on")]
  public static OS.Error* XPrinterOn ();

  [CCode (cname = "os_printer_on")]
  public static void PrinterOn ();

  [CCode (cname = "xos_printer_off")]
  public static OS.Error* XPrinterOff ();

  [CCode (cname = "os_printer_off")]
  public static void PrinterOff ();

  [CCode (cname = "xos_split_cursors")]
  public static OS.Error* XSplitCursors ();

  [CCode (cname = "os_split_cursors")]
  public static void SplitCursors ();

  [CCode (cname = "xos_join_cursors")]
  public static OS.Error* XJoinCursors ();

  [CCode (cname = "os_join_cursors")]
  public static void JoinCursors ();

  [CCode (cname = "xos_vdu_on")]
  public static OS.Error* XVDUOn ();

  [CCode (cname = "os_vdu_on")]
  public static void VDUOn ();

  [CCode (cname = "xos_bell")]
  public static OS.Error* XBell ();

  [CCode (cname = "os_bell")]
  public static void Bell ();

  [CCode (cname = "xos_backspace")]
  public static OS.Error* XBackspace ();

  [CCode (cname = "os_backspace")]
  public static void Backspace ();

  [CCode (cname = "xos_tab")]
  public static OS.Error* XTab ();

  [CCode (cname = "os_tab")]
  public static void Tab ();

  [CCode (cname = "xos_line_feed")]
  public static OS.Error* XLineFeed ();

  [CCode (cname = "os_line_feed")]
  public static void LineFeed ();

  [CCode (cname = "xos_vertical_tab")]
  public static OS.Error* XVerticalTab ();

  [CCode (cname = "os_vertical_tab")]
  public static void VerticalTab ();

  [CCode (cname = "xos_cls")]
  public static OS.Error* XClS ();

  [CCode (cname = "os_cls")]
  public static void ClS ();

  [CCode (cname = "xos_return")]
  public static OS.Error* XReturn ();

  [CCode (cname = "os_return")]
  public static void Return ();

  [CCode (cname = "xos_page_mode_on")]
  public static OS.Error* XPageModeOn ();

  [CCode (cname = "os_page_mode_on")]
  public static void PageModeOn ();

  [CCode (cname = "xos_page_mode_off")]
  public static OS.Error* XPageModeOff ();

  [CCode (cname = "os_page_mode_off")]
  public static void PageModeOff ();

  [CCode (cname = "xos_clg")]
  public static OS.Error* XClG ();

  [CCode (cname = "os_clg")]
  public static void ClG ();

  [CCode (cname = "xos_set_text_colour")]
  public static OS.Error* XSetTextColour ();

  [CCode (cname = "os_set_text_colour")]
  public static void SetTextColour ();

  [CCode (cname = "xos_set_gcol")]
  public static OS.Error* XSetGCOL ();

  [CCode (cname = "os_set_gcol")]
  public static void SetGCOL ();

  [CCode (cname = "xos_set_palette")]
  public static OS.Error* XSetPalette ();

  [CCode (cname = "os_set_palette")]
  public static void SetPalette ();

  [CCode (cname = "xos_reset_colours")]
  public static OS.Error* XResetColours ();

  [CCode (cname = "os_reset_colours")]
  public static void ResetColours ();

  [CCode (cname = "xos_vdu_off")]
  public static OS.Error* XVDUOff ();

  [CCode (cname = "os_vdu_off")]
  public static void VDUOff ();

  [CCode (cname = "xos_set_mode")]
  public static OS.Error* XSetMode ();

  [CCode (cname = "os_set_mode")]
  public static void SetMode ();

  [CCode (cname = "xos_misc")]
  public static OS.Error* XMisc ();

  [CCode (cname = "os_misc")]
  public static void Misc ();

  [CCode (cname = "xos_set_graphics_window")]
  public static OS.Error* XSetGraphicsWindow ();

  [CCode (cname = "os_set_graphics_window")]
  public static void SetGraphicsWindow ();

  [CCode (cname = "xos_plot_vdu")]
  public static OS.Error* XPlotVDU ();

  [CCode (cname = "os_plot_vdu")]
  public static void PlotVDU ();

  [CCode (cname = "xos_reset_windows")]
  public static OS.Error* XResetWindows ();

  [CCode (cname = "os_reset_windows")]
  public static void ResetWindows ();

  [CCode (cname = "xos_escape")]
  public static OS.Error* XEscape ();

  [CCode (cname = "os_escape")]
  public static void Escape ();

  [CCode (cname = "xos_set_text_window")]
  public static OS.Error* XSetTextWindow ();

  [CCode (cname = "os_set_text_window")]
  public static void SetTextWindow ();

  [CCode (cname = "xos_set_graphics_origin")]
  public static OS.Error* XSetGraphicsOrigin ();

  [CCode (cname = "os_set_graphics_origin")]
  public static void SetGraphicsOrigin ();

  [CCode (cname = "xos_home_text_cursor")]
  public static OS.Error* XHomeTextCursor ();

  [CCode (cname = "os_home_text_cursor")]
  public static void HomeTextCursor ();

  [CCode (cname = "xos_set_text_cursor")]
  public static OS.Error* XSetTextCursor ();

  [CCode (cname = "os_set_text_cursor")]
  public static void SetTextCursor ();

  [CCode (cname = "xos_space")]
  public static OS.Error* XSpace ();

  [CCode (cname = "os_space")]
  public static void Space ();

  [CCode (cname = "xos_delete")]
  public static OS.Error* XDelete ();

  [CCode (cname = "os_delete")]
  public static void Delete ();

  [CCode (cname = "xerrorv")]
  public static OS.Error* XErrorV
    (Error error);

  [CCode (cname = "errorv")]
  public static void ErrorV
    (Error error);

  [CCode (cname = "xirqv")]
  public static OS.Error* XIrqV ();

  [CCode (cname = "irqv")]
  public static void IrqV ();

  [CCode (cname = "xwrchv")]
  public static OS.Error* XWrchV
    (char c);

  [CCode (cname = "wrchv")]
  public static void WrchV
    (char c);

  [CCode (cname = "xrdchv")]
  public static OS.Error* XRdchV
    (out char c,
     out uint psr);

  [CCode (cname = "rdchv")]
  public static bits RdchV
    (out char c);

  [CCode (cname = "xcliv")]
  public static OS.Error* XCLIV
    (string command);

  [CCode (cname = "cliv")]
  public static void CLIV
    (string command);

  [CCode (cname = "xreadlinev")]
  public static OS.Error* XReadLineV
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used,
     out uint psr);

  [CCode (cname = "readlinev")]
  public static bits ReadLineV
    (string buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     out int used);

  [CCode (cname = "xreadline32v")]
  public static OS.Error* XReadLine32V
    (string buffer,
     int size,
     char min_char,
     char max_char,
     OSReadLine_Flags flags,
     out int used,
     out uint psr);

  [CCode (cname = "readline32v")]
  public static bits ReadLine32V
    (string buffer,
     int size,
     char min_char,
     char max_char,
     OSReadLine_Flags flags,
     out int used);

  [CCode (cname = "xkeyv_present")]
  public static OS.Error* XKeyV_Present
    (KeyV_Type keyboard_type);

  [CCode (cname = "keyv_present")]
  public static void KeyV_Present
    (KeyV_Type keyboard_type);

  [CCode (cname = "xkeyv_present_no_debounce")]
  public static OS.Error* XKeyV_PresentNoDebounce
    (KeyV_Type keyboard_type);

  [CCode (cname = "keyv_present_no_debounce")]
  public static void KeyV_PresentNoDebounce
    (KeyV_Type keyboard_type);

  [CCode (cname = "xkeyv_transition_up")]
  public static OS.Error* XKeyV_TransitionUp
    (int key);

  [CCode (cname = "keyv_transition_up")]
  public static void KeyV_TransitionUp
    (int key);

  [CCode (cname = "xkeyv_transition_down")]
  public static OS.Error* XKeyV_TransitionDown
    (int key);

  [CCode (cname = "keyv_transition_down")]
  public static void KeyV_TransitionDown
    (int key);

  [CCode (cname = "xkeyv_led_state")]
  public static OS.Error* XKeyV_LEDState
    (bits flags);

  [CCode (cname = "keyv_led_state")]
  public static void KeyV_LEDState
    (bits flags);

  [CCode (cname = "xkeyv_enable_drivers")]
  public static OS.Error* XKeyV_EnableDrivers ();

  [CCode (cname = "keyv_enable_drivers")]
  public static void KeyV_EnableDrivers ();

  [CCode (cname = "xukvdu23v")]
  public static OS.Error* XUKVDU23V
    (int n,
     byte* queue);

  [CCode (cname = "ukvdu23v")]
  public static void UKVDU23V
    (int n,
     byte* queue);

  [CCode (cname = "xukplotv")]
  public static OS.Error* XUKPLOTV
    (PlotCode plot_code);

  [CCode (cname = "ukplotv")]
  public static void UKPLOTV
    (PlotCode plot_code);

  [CCode (cname = "xmousev")]
  public static OS.Error* XMouseV
    (out int x,
     out int y,
     out bits buttons,
     out T t);

  [CCode (cname = "mousev")]
  public static void MouseV
    (out int x,
     out int y,
     out bits buttons,
     out T t);

  [CCode (cname = "xvduxv")]
  public static OS.Error* XVDUXV
    (byte b);

  [CCode (cname = "vduxv")]
  public static void VDUXV
    (byte b);

  [CCode (cname = "xtickerv")]
  public static OS.Error* XTickerV ();

  [CCode (cname = "tickerv")]
  public static void TickerV ();

  [CCode (cname = "xchangeenvironmentv")]
  public static OS.Error* XChangeEnvironmentV
    (HandlerType handler_type,
     AsmRoutine handler,
     char* handle,
     char* buffer,
     out AsmRoutine old_handler,
     out char* old_handle,
     out char* old_buffer);

  [CCode (cname = "changeenvironmentv")]
  public static AsmRoutine ChangeEnvironmentV
    (HandlerType handler_type,
     AsmRoutine handler,
     char* handle,
     char* buffer,
     out char* old_handle,
     out char* old_buffer);

  [CCode (cname = "xpalettev_read_entry")]
  public static OS.Error* XPaletteV_ReadEntry
    (ColourNumber entry,
     int colour_type,
     out Colour on,
     out Colour off,
     out bool incomplete);

  [CCode (cname = "palettev_read_entry")]
  public static bool PaletteV_ReadEntry
    (ColourNumber entry,
     int colour_type,
     out Colour on,
     out Colour off);

  [CCode (cname = "xpalettev_set_entry")]
  public static OS.Error* XPaletteV_SetEntry
    (ColourNumber entry,
     int colour_type,
     Colour on,
     Colour off,
     out bool incomplete);

  [CCode (cname = "palettev_set_entry")]
  public static bool PaletteV_SetEntry
    (ColourNumber entry,
     int colour_type,
     Colour on,
     Colour off);

  [CCode (cname = "xpalettev_set_on")]
  public static OS.Error* XPaletteV_SetOn
    (out bool incomplete);

  [CCode (cname = "palettev_set_on")]
  public static bool PaletteV_SetOn ();

  [CCode (cname = "xpalettev_set_off")]
  public static OS.Error* XPaletteV_SetOff
    (out bool incomplete);

  [CCode (cname = "palettev_set_off")]
  public static bool PaletteV_SetOff ();

  [CCode (cname = "xpalettev_default")]
  public static OS.Error* XPaletteV_Default
    (out bool incomplete);

  [CCode (cname = "palettev_default")]
  public static bool PaletteV_Default ();

  [CCode (cname = "xpalettev_blank_screen")]
  public static OS.Error* XPaletteV_BlankScreen
    (PaletteV_ScreenState state,
     out PaletteV_ScreenState state_out,
     out bool incomplete);

  [CCode (cname = "palettev_blank_screen")]
  public static bool PaletteV_BlankScreen
    (PaletteV_ScreenState state,
     out PaletteV_ScreenState state_out);

  [CCode (cname = "xpalettev_read_entries")]
  public static OS.Error* XPaletteV_ReadEntries
    (ColourNumber entry_list,
     bits colour_type_and_entry_count,
     Palette on,
     Palette off,
     out bool incomplete);

  [CCode (cname = "palettev_read_entries")]
  public static bool PaletteV_ReadEntries
    (ColourNumber entry_list,
     bits colour_type_and_entry_count,
     Palette on,
     Palette off);

  [CCode (cname = "xpalettev_write_entries")]
  public static OS.Error* XPaletteV_WriteEntries
    (ColourNumber entry_list,
     bits colour_type_and_entry_count,
     Palette palette,
     out bool incomplete);

  [CCode (cname = "palettev_write_entries")]
  public static bool PaletteV_WriteEntries
    (ColourNumber entry_list,
     bits colour_type_and_entry_count,
     Palette palette);

  [CCode (cname = "xpalettev_set_gamma_corrections")]
  public static OS.Error* XPaletteV_SetGammaCorrections
    (CorrectionTable red_table,
     CorrectionTable green_table,
     CorrectionTable blue_table,
     out bool incomplete);

  [CCode (cname = "palettev_set_gamma_corrections")]
  public static bool PaletteV_SetGammaCorrections
    (CorrectionTable red_table,
     CorrectionTable green_table,
     CorrectionTable blue_table);

  [CCode (cname = "xpalettev_disable_video_hardware")]
  public static OS.Error* XPaletteV_DisableVideoHardware
    (out bool incomplete);

  [CCode (cname = "palettev_disable_video_hardware")]
  public static bool PaletteV_DisableVideoHardware ();

  [CCode (cname = "xpalettev_enable_video_hardware")]
  public static OS.Error* XPaletteV_EnableVideoHardware
    (out bool incomplete);

  [CCode (cname = "palettev_enable_video_hardware")]
  public static bool PaletteV_EnableVideoHardware ();

  [CCode (cname = "xpointerv_status")]
  public static OS.Error* XPointerV_Status
    (PointerType pointer_type,
     out int xmove,
     out int ymove);

  [CCode (cname = "pointerv_status")]
  public static void PointerV_Status
    (PointerType pointer_type,
     out int xmove,
     out int ymove);

  [CCode (cname = "xpointerv_enumerate")]
  public static OS.Error* XPointerV_Enumerate
    (PointerV_Record record,
     out PointerV_Record record_out);

  [CCode (cname = "pointerv_enumerate")]
  public static PointerV_Record PointerV_Enumerate
    (PointerV_Record record);

  [CCode (cname = "xpointerv_selected")]
  public static OS.Error* XPointerV_Selected
    (PointerType pointer_type);

  [CCode (cname = "pointerv_selected")]
  public static void PointerV_Selected
    (PointerType pointer_type);

  [CCode (cname = "xpointerv_extended_status")]
  public static OS.Error* XPointerV_ExtendedStatus
    (PointerType pointer_type,
     out int claimed,
     out int xmove,
     out int ymove,
     out int xmove_alternate,
     out int ymove_alternate,
     out bits buttons);

  [CCode (cname = "pointerv_extended_status")]
  public static void PointerV_ExtendedStatus
    (PointerType pointer_type,
     out int claimed,
     out int xmove,
     out int ymove,
     out int xmove_alternate,
     out int ymove_alternate,
     out bits buttons);

  [CCode (cname = "xnvramv_fill_cache")]
  public static OS.Error* XNVRAMV_FillCache
    (byte* cache,
     int cache_size,
     out int nvram_size);

  [CCode (cname = "nvramv_fill_cache")]
  public static void NVRAMV_FillCache
    (byte* cache,
     int cache_size,
     out int nvram_size);

  [CCode (cname = "xnvramv_read_single_value")]
  public static OS.Error* XNVRAMV_ReadSingleValue
    (int index,
     out byte value);

  [CCode (cname = "nvramv_read_single_value")]
  public static void NVRAMV_ReadSingleValue
    (int index,
     out byte value);

  [CCode (cname = "xnvramv_write_single_value")]
  public static OS.Error* XNVRAMV_WriteSingleValue
    (int index,
     byte value);

  [CCode (cname = "nvramv_write_single_value")]
  public static void NVRAMV_WriteSingleValue
    (int index,
     byte value);

  [CCode (cname = "xservice_release_fiq")]
  public static OS.Error* XService_ReleaseFIQ ();

  [CCode (cname = "service_release_fiq")]
  public static void Service_ReleaseFIQ ();

  [CCode (cname = "xservice_claim_fiq")]
  public static OS.Error* XService_ClaimFIQ ();

  [CCode (cname = "service_claim_fiq")]
  public static void Service_ClaimFIQ ();

  [CCode (cname = "xservice_reset")]
  public static OS.Error* XService_Reset ();

  [CCode (cname = "service_reset")]
  public static void Service_Reset ();

  [CCode (cname = "xservice_key_handler")]
  public static OS.Error* XService_KeyHandler
    (int keyboard_id);

  [CCode (cname = "service_key_handler")]
  public static void Service_KeyHandler
    (int keyboard_id);

  [CCode (cname = "xservice_mode_change")]
  public static OS.Error* XService_ModeChange ();

  [CCode (cname = "service_mode_change")]
  public static void Service_ModeChange ();

  [CCode (cname = "xservice_claim_fiq_in_background")]
  public static OS.Error* XService_ClaimFIQInBackground ();

  [CCode (cname = "service_claim_fiq_in_background")]
  public static void Service_ClaimFIQInBackground ();

  [CCode (cname = "xservice_memory_moved")]
  public static OS.Error* XService_MemoryMoved ();

  [CCode (cname = "service_memory_moved")]
  public static void Service_MemoryMoved ();

  [CCode (cname = "xservice_pre_mode_change")]
  public static OS.Error* XService_PreModeChange
    (Mode mode);

  [CCode (cname = "service_pre_mode_change")]
  public static void Service_PreModeChange
    (Mode mode);

  [CCode (cname = "xservice_mode_extension")]
  public static OS.Error* XService_ModeExtension
    (Mode mode,
     int monitor_type,
     int bandwidth,
     int vram_size,
     out bool unclaimed,
     out int* vidc_list,
     out int* workspace_list);

  [CCode (cname = "service_mode_extension")]
  public static bool Service_ModeExtension
    (Mode mode,
     int monitor_type,
     int bandwidth,
     int vram_size,
     out int* vidc_list,
     out int* workspace_list);

  [CCode (cname = "xservice_mode_translation")]
  public static OS.Error* XService_ModeTranslation
    (Mode mode,
     int monitor_type);

  [CCode (cname = "service_mode_translation")]
  public static void Service_ModeTranslation
    (Mode mode,
     int monitor_type);

  [CCode (cname = "xservice_validate_address")]
  public static OS.Error* XService_ValidateAddress ();

  [CCode (cname = "service_validate_address")]
  public static void Service_ValidateAddress ();

  [CCode (cname = "xservice_monitor_lead_translation")]
  public static OS.Error* XService_MonitorLeadTranslation
    (int lead_id);

  [CCode (cname = "service_monitor_lead_translation")]
  public static void Service_MonitorLeadTranslation
    (int lead_id);

  [CCode (cname = "xservice_mode_changing")]
  public static OS.Error* XService_ModeChanging
    (Mode mode,
     int monitor_type);

  [CCode (cname = "service_mode_changing")]
  public static void Service_ModeChanging
    (Mode mode,
     int monitor_type);

  [CCode (cname = "xservice_enumerate_screen_modes")]
  public static OS.Error* XService_EnumerateScreenModes
    (int context,
     int monitor_type,
     int bandwidth,
     int vram_size,
     ModeBlock block,
     int block_size,
     out bool unclaimed,
     out int context_out,
     out char* end,
     out int free);

  [CCode (cname = "service_enumerate_screen_modes")]
  public static bool Service_EnumerateScreenModes
    (int context,
     int monitor_type,
     int bandwidth,
     int vram_size,
     ModeBlock block,
     int block_size,
     out int context_out,
     out char* end,
     out int free);

  [CCode (cname = "xservice_pages_unsafe")]
  public static OS.Error* XService_PagesUnsafe
    (PageBlock page_block,
     int request_count);

  [CCode (cname = "service_pages_unsafe")]
  public static void Service_PagesUnsafe
    (PageBlock page_block,
     int request_count);

  [CCode (cname = "xservice_pages_safe")]
  public static OS.Error* XService_PagesSafe
    (int request_count,
     PageBlock old_page_block,
     PageBlock new_page_block);

  [CCode (cname = "service_pages_safe")]
  public static void Service_PagesSafe
    (int request_count,
     PageBlock old_page_block,
     PageBlock new_page_block);

  [CCode (cname = "xservice_dynamic_area_create")]
  public static OS.Error* XService_DynamicAreaCreate
    (int area);

  [CCode (cname = "service_dynamic_area_create")]
  public static void Service_DynamicAreaCreate
    (int area);

  [CCode (cname = "xservice_dynamic_area_remove")]
  public static OS.Error* XService_DynamicAreaRemove
    (int area);

  [CCode (cname = "service_dynamic_area_remove")]
  public static void Service_DynamicAreaRemove
    (int area);

  [CCode (cname = "xservice_dynamic_area_renumber")]
  public static OS.Error* XService_DynamicAreaRenumber
    (int area);

  [CCode (cname = "service_dynamic_area_renumber")]
  public static void Service_DynamicAreaRenumber
    (int area);

  [CCode (cname = "xservice_display_changed")]
  public static OS.Error* XService_DisplayChanged
    (ServiceDisplayChanged_State state,
     int display_number);

  [CCode (cname = "service_display_changed")]
  public static void Service_DisplayChanged
    (ServiceDisplayChanged_State state,
     int display_number);

  [CCode (cname = "xservice_display_status")]
  public static OS.Error* XService_DisplayStatus
    (ServiceDisplayStatus_Reason reason,
     int display_number,
     OSDisplayDeviceDescriptor descriptor);

  [CCode (cname = "service_display_status")]
  public static void Service_DisplayStatus
    (ServiceDisplayStatus_Reason reason,
     int display_number,
     OSDisplayDeviceDescriptor descriptor);

  [CCode (cname = "xservice_device_reregister")]
  public static OS.Error* XService_DeviceReregister
    (DeviceType device);

  [CCode (cname = "service_device_reregister")]
  public static void Service_DeviceReregister
    (DeviceType device);

  [CCode (cname = "xevent_char_input")]
  public static OS.Error* XEvent_CharInput
    (int buffer,
     byte b);

  [CCode (cname = "event_char_input")]
  public static void Event_CharInput
    (int buffer,
     byte b);

  [CCode (cname = "xevent_vsync")]
  public static OS.Error* XEvent_VSync ();

  [CCode (cname = "event_vsync")]
  public static void Event_VSync ();

  [CCode (cname = "xevent_interval_timer")]
  public static OS.Error* XEvent_IntervalTimer ();

  [CCode (cname = "event_interval_timer")]
  public static void Event_IntervalTimer ();

  [CCode (cname = "xevent_escape")]
  public static OS.Error* XEvent_Escape ();

  [CCode (cname = "event_escape")]
  public static void Event_Escape ();

  [CCode (cname = "xevent_mouse_transition")]
  public static OS.Error* XEvent_MouseTransition
    (int x,
     int y,
     bits state,
     T t);

  [CCode (cname = "event_mouse_transition")]
  public static void Event_MouseTransition
    (int x,
     int y,
     bits state,
     T t);

  [CCode (cname = "xevent_key_transition")]
  public static OS.Error* XEvent_KeyTransition
    (bool down,
     int key,
     int keyboard_id);

  [CCode (cname = "event_key_transition")]
  public static void Event_KeyTransition
    (bool down,
     int key,
     int keyboard_id);

  [CCode (cname = "xevent_pointer_scroll")]
  public static OS.Error* XEvent_PointerScroll
    (int xmove,
     int ymove);

  [CCode (cname = "event_pointer_scroll")]
  public static void Event_PointerScroll
    (int xmove,
     int ymove);

  [CCode (cname = "xupcall_move_memory_request")]
  public static OS.Error* XUpCall_MoveMemoryRequest
    (int size_change,
     out bool not_moved);

  [CCode (cname = "upcall_move_memory_request")]
  public static bool UpCall_MoveMemoryRequest
    (int size_change);

}
