/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;
using OSLib.OSArgs;

[CCode (cheader_filename = "oslib/osfscontrol.h")]
namespace OSLib.OSFSControl
{
  [CCode (cname = "osfscontrol_id", has_type_id = false)]
  public struct Id : bits
  {
  }

  [CCode (cname = "osfscontrol_fs_info_block")]
  public struct FSInfoBlock
  {
    [CCode (cname = "name_offset")]
    public unowned int NameOffset;
    [CCode (cname = "banner_offset")]
    public unowned int BannerOffset;
    [CCode (cname = "open_offset")]
    public unowned int OpenOffset;
    [CCode (cname = "get_bytes")]
    public unowned int GetBytes;
    [CCode (cname = "put_bytes")]
    public unowned int PutBytes;
    [CCode (cname = "args_offset")]
    public unowned int ArgsOffset;
    [CCode (cname = "close_offset")]
    public unowned int CloseOffset;
    [CCode (cname = "file_offset")]
    public unowned int FileOffset;
    [CCode (cname = "info")]
    public unowned FileSwitch.FSInfo Info;
    [CCode (cname = "func_offset")]
    public unowned int FuncOffset;
    [CCode (cname = "gbpb_offset")]
    public unowned int GbpbOffset;
    [CCode (cname = "extra_info")]
    public unowned FileSwitch.FSExtraInfo ExtraInfo;
  }

  [CCode (cname = "osfscontrol_copy_flags")]
  public enum CopyFlags
  {
    [CCode (cname = "osfscontrol_COPY_RECURSE")]
    CopyRecurse,
    [CCode (cname = "osfscontrol_COPY_FORCE")]
    CopyForce,
    [CCode (cname = "osfscontrol_COPY_GIVEN_TIMES")]
    CopyGivenTimes,
    [CCode (cname = "osfscontrol_COPY_CONFIRM")]
    CopyConfirm,
    [CCode (cname = "osfscontrol_COPY_VERBOSE")]
    CopyVerbose,
    [CCode (cname = "osfscontrol_COPY_QUICK")]
    CopyQuick,
    [CCode (cname = "osfscontrol_COPY_PROMPT")]
    CopyPrompt,
    [CCode (cname = "osfscontrol_COPY_DELETE")]
    CopyDelete,
    [CCode (cname = "osfscontrol_COPY_ALLOW_PRINT")]
    CopyAllowPrint,
    [CCode (cname = "osfscontrol_COPY_NO_ATTRIBUTES")]
    CopyNoAttributes,
    [CCode (cname = "osfscontrol_COPY_STAMP")]
    CopyStamp,
    [CCode (cname = "osfscontrol_COPY_STRUCTURE")]
    CopyStructure,
    [CCode (cname = "osfscontrol_COPY_NEWER")]
    CopyNewer,
    [CCode (cname = "osfscontrol_COPY_GIVEN_DESCRIPTOR")]
    CopyGivenDescriptor,
    [CCode (cname = "osfscontrol_COPY_LOOK")]
    CopyLook
  }

  [CCode (cname = "osfscontrol_descriptor")]
  public struct Descriptor
  {
    [CCode (cname = "addr")]
    public unowned char* Addr;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [CCode (cname = "osfscontrol_wipe_flags")]
  public enum WipeFlags
  {
    [CCode (cname = "osfscontrol_WIPE_RECURSE")]
    WipeRecurse,
    [CCode (cname = "osfscontrol_WIPE_FORCE")]
    WipeForce,
    [CCode (cname = "osfscontrol_WIPE_GIVEN_TIMES")]
    WipeGivenTimes,
    [CCode (cname = "osfscontrol_WIPE_CONFIRM")]
    WipeConfirm,
    [CCode (cname = "osfscontrol_WIPE_VERBOSE")]
    WipeVerbose,
    [CCode (cname = "osfscontrol_WIPE_ALLOW_PRINT")]
    WipeAllowPrint
  }

  [CCode (cname = "osfscontrol_count_flags")]
  public enum CountFlags
  {
    [CCode (cname = "osfscontrol_COUNT_RECURSE")]
    CountRecurse,
    [CCode (cname = "osfscontrol_COUNT_GIVEN_TIMES")]
    CountGivenTimes,
    [CCode (cname = "osfscontrol_COUNT_CONFIRM")]
    CountConfirm,
    [CCode (cname = "osfscontrol_COUNT_VERBOSE")]
    CountVerbose,
    [CCode (cname = "osfscontrol_COUNT_ALLOW_PRINT")]
    CountAllowPrint
  }

  [CCode (cname = "osfscontrol_object_kind")]
  public enum ObjectKind
  {
    [CCode (cname = "osfscontrol_KIND_FREE")]
    KindFree,
    [CCode (cname = "osfscontrol_KIND_ALLOCATED")]
    KindAllocated,
    [CCode (cname = "osfscontrol_KIND_FOUND")]
    KindFound,
    [CCode (cname = "osfscontrol_KIND_SHARABLE")]
    KindSharable
  }

  [CCode (cname = "osfscontrol_special_dir")]
  public enum SpecialDir
  {
    [CCode (cname = "osfscontrol_DIR_CSD")]
    DirCSD,
    [CCode (cname = "osfscontrol_DIR_PSD")]
    DirPSD,
    [CCode (cname = "osfscontrol_DIR_URD")]
    DirURD,
    [CCode (cname = "osfscontrol_DIR_LIB")]
    DirLib
  }

  [CCode (cname="osfscontrol_OPT_RESTORE_DEFAULT")]
  public const int OptRestoreDefault;
  [CCode (cname="osfscontrol_OPT_MESSAGE_LEVEL")]
  public const int OptMessageLevel;
  [CCode (cname="osfscontrol_OPT_BOOT_ACTION")]
  public const int OptBootAction;

  [CCode (cname = "xosfscontrol_dir")]
  public static OS.Error* XDir
    (string dir_names);

  [CCode (cname = "osfscontrol_dir")]
  public static void Dir
    (string dir_names);

  [CCode (cname = "xosfscontrol_lib")]
  public static OS.Error* XLib
    (string dir_names);

  [CCode (cname = "osfscontrol_lib")]
  public static void Lib
    (string dir_names);

  [CCode (cname = "xosfscontrol_start_application")]
  public static OS.Error* XStartApplication
    (string tail,
     char* cao,
     string command);

  [CCode (cname = "osfscontrol_start_application")]
  public static void StartApplication
    (string tail,
     char* cao,
     string command);

  [CCode (cname = "xosfscontrol_run")]
  public static OS.Error* XRun
    (string file_names);

  [CCode (cname = "osfscontrol_run")]
  public static void Run
    (string file_names);

  [CCode (cname = "xosfscontrol_cat")]
  public static OS.Error* XCat
    (string dir_names);

  [CCode (cname = "osfscontrol_cat")]
  public static void Cat
    (string dir_names);

  [CCode (cname = "xosfscontrol_ex")]
  public static OS.Error* XEx
    (string dir_names);

  [CCode (cname = "osfscontrol_ex")]
  public static void Ex
    (string dir_names);

  [CCode (cname = "xosfscontrol_lcat")]
  public static OS.Error* XLCat
    (string dir_names);

  [CCode (cname = "osfscontrol_lcat")]
  public static void LCat
    (string dir_names);

  [CCode (cname = "xosfscontrol_lex")]
  public static OS.Error* XLEx
    (string dir_names);

  [CCode (cname = "osfscontrol_lex")]
  public static void LEx
    (string dir_names);

  [CCode (cname = "xosfscontrol_info")]
  public static OS.Error* XInfo
    (string file_names);

  [CCode (cname = "osfscontrol_info")]
  public static void Info
    (string file_names);

  [CCode (cname = "xosfscontrol_opt")]
  public static OS.Error* XOpt
    (int opt,
     int value);

  [CCode (cname = "osfscontrol_opt")]
  public static void Opt
    (int opt,
     int value);

  [CCode (cname = "xosfscontrol_set_temporary_fs")]
  public static OS.Error* XSetTemporaryFS
    (string file_name,
     out string file_name_out,
     out FileSwitch.FSNo old_fs_no,
     out string special);

  [CCode (cname = "osfscontrol_set_temporary_fs")]
  public static FileSwitch.FSNo SetTemporaryFS
    (string file_name,
     out string file_name_out,
     out string special);

  [CCode (cname = "xosfscontrol_add_fs")]
  public static OS.Error* XAddFS
    (char* module,
     int fs_info_offset,
     char* workspace);

  [CCode (cname = "osfscontrol_add_fs")]
  public static void AddFS
    (char* module,
     int fs_info_offset,
     char* workspace);

  [CCode (cname = "xosfscontrol_lookup_fs")]
  public static OS.Error* XLookupFS
    (Id fs_no_or_name,
     bool control_terminated,
     out FileSwitch.FSNo fs_no_out,
     out bool fs_found);

  [CCode (cname = "osfscontrol_lookup_fs")]
  public static bool LookupFS
    (Id fs_no_or_name,
     bool control_terminated,
     out FileSwitch.FSNo fs_no_out);

  [CCode (cname = "xosfscontrol_select_fs")]
  public static OS.Error* XSelectFS
    (Id fs_no_or_name);

  [CCode (cname = "osfscontrol_select_fs")]
  public static void SelectFS
    (Id fs_no_or_name);

  [CCode (cname = "xosfscontrol_boot_from_fs")]
  public static OS.Error* XBootFromFS ();

  [CCode (cname = "osfscontrol_boot_from_fs")]
  public static void BootFromFS ();

  [CCode (cname = "xosfscontrol_remove_fs")]
  public static OS.Error* XRemoveFS
    (string fs_name);

  [CCode (cname = "osfscontrol_remove_fs")]
  public static void RemoveFS
    (string fs_name);

  [CCode (cname = "xosfscontrol_add_secondary_fs")]
  public static OS.Error* XAddSecondaryFS
    (string fs_name,
     string secondary_name,
     char* workspace);

  [CCode (cname = "osfscontrol_add_secondary_fs")]
  public static void AddSecondaryFS
    (string fs_name,
     string secondary_name,
     char* workspace);

  [CCode (cname = "xosfscontrol_read_file_type")]
  public static OS.Error* XReadFileType
    (bits file_type,
     out bits name1,
     out bits name2);

  [CCode (cname = "osfscontrol_read_file_type")]
  public static void ReadFileType
    (bits file_type,
     out bits name1,
     out bits name2);

  [CCode (cname = "xosfscontrol_restore_current")]
  public static OS.Error* XRestoreCurrent ();

  [CCode (cname = "osfscontrol_restore_current")]
  public static void RestoreCurrent ();

  [CCode (cname = "xosfscontrol_read_module_base")]
  public static OS.Error* XReadModuleBase
    (out char* module,
     out char* workspace);

  [CCode (cname = "osfscontrol_read_module_base")]
  public static void ReadModuleBase
    (out char* module,
     out char* workspace);

  [CCode (cname = "xosfscontrol_read_fs_handle")]
  public static OS.Error* XReadFSHandle
    (OS.F file,
     out int fs_handle,
     out int info_word);

  [CCode (cname = "osfscontrol_read_fs_handle")]
  public static int ReadFSHandle
    (OS.F file,
     out int info_word);

  [CCode (cname = "xosfscontrol_read_fs_handlew")]
  public static OS.Error* XReadFSHandleW
    (OS.FW file,
     out int fs_handle,
     out int info_word);

  [CCode (cname = "osfscontrol_read_fs_handlew")]
  public static int ReadFSHandleW
    (OS.FW file,
     out int info_word);

  [CCode (cname = "xosfscontrol_shut")]
  public static OS.Error* XShut ();

  [CCode (cname = "osfscontrol_shut")]
  public static void Shut ();

  [CCode (cname = "xosfscontrol_shutdown")]
  public static OS.Error* XShutdown ();

  [CCode (cname = "osfscontrol_shutdown")]
  public static void Shutdown ();

  [CCode (cname = "xosfscontrol_access")]
  public static OS.Error* XAccess
    (string file_names,
     string access);

  [CCode (cname = "osfscontrol_access")]
  public static void Access
    (string file_names,
     string access);

  [CCode (cname = "xosfscontrol_rename")]
  public static OS.Error* XRename
    (string file_name,
     string new_file_name);

  [CCode (cname = "osfscontrol_rename")]
  public static void Rename
    (string file_name,
     string new_file_name);

  [CCode (cname = "xosfscontrol_copy")]
  public static OS.Error* XCopy
    (string from,
     string to,
     CopyFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec,
     Descriptor descriptor);

  [CCode (cname = "osfscontrol_copy")]
  public static void Copy
    (string from,
     string to,
     CopyFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec,
     Descriptor descriptor);

  [CCode (cname = "xosfscontrol_wipe")]
  public static OS.Error* XWipe
    (string file_names,
     WipeFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec);

  [CCode (cname = "osfscontrol_wipe")]
  public static void Wipe
    (string file_names,
     WipeFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec);

  [CCode (cname = "xosfscontrol_count")]
  public static OS.Error* XCount
    (string file_names,
     CountFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec,
     out int byte_count,
     out int file_count);

  [CCode (cname = "osfscontrol_count")]
  public static int Count
    (string file_names,
     CountFlags flags,
     bits start_load,
     bits start_exec,
     bits end_load,
     bits end_exec,
     out int file_count);

  [CCode (cname = "xosfscontrol_read_secondary_fs")]
  public static OS.Error* XReadSecondaryFS
    (out char* module,
     out char* workspace);

  [CCode (cname = "osfscontrol_read_secondary_fs")]
  public static void ReadSecondaryFS
    (out char* module,
     out char* workspace);

  [CCode (cname = "xosfscontrol_file_type_from_string")]
  public static OS.Error* XFileTypeFromString
    (string file_type_name,
     out bits file_type);

  [CCode (cname = "osfscontrol_file_type_from_string")]
  public static bits FileTypeFromString
    (string file_type_name);

  [CCode (cname = "xosfscontrol_file_info")]
  public static OS.Error* XFileInfo
    (string file_names);

  [CCode (cname = "osfscontrol_file_info")]
  public static void FileInfo
    (string file_names);

  [CCode (cname = "xosfscontrol_read_fs_name")]
  public static OS.Error* XReadFSName
    (FileSwitch.FSNo fs_no,
     string buffer,
     int size);

  [CCode (cname = "osfscontrol_read_fs_name")]
  public static void ReadFSName
    (FileSwitch.FSNo fs_no,
     string buffer,
     int size);

  [CCode (cname = "xosfscontrol_register_image_fs")]
  public static OS.Error* XRegisterImageFS
    (char* module,
     int fs_info_offset,
     char* workspace);

  [CCode (cname = "osfscontrol_register_image_fs")]
  public static void RegisterImageFS
    (char* module,
     int fs_info_offset,
     char* workspace);

  [CCode (cname = "xosfscontrol_deregister_image_fs")]
  public static OS.Error* XDeregisterImageFS
    (bits file_type);

  [CCode (cname = "osfscontrol_deregister_image_fs")]
  public static void DeregisterImageFS
    (bits file_type);

  [CCode (cname = "xosfscontrol_canonicalise_path")]
  public static OS.Error* XCanonicalisePath
    (string path_name,
     string buffer,
     string var,
     string path,
     int size,
     out int spare);

  [CCode (cname = "osfscontrol_canonicalise_path")]
  public static int CanonicalisePath
    (string path_name,
     string buffer,
     string var,
     string path,
     int size);

  [CCode (cname = "xosfscontrol_info_to_file_type")]
  public static OS.Error* XInfoToFileType
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int size,
     FileSwitch.Attr attr,
     FileSwitch.ObjectType obj_type,
     out bits file_type);

  [CCode (cname = "osfscontrol_info_to_file_type")]
  public static bits InfoToFileType
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int size,
     FileSwitch.Attr attr,
     FileSwitch.ObjectType obj_type);

  [CCode (cname = "xosfscontrol_urd")]
  public static OS.Error* XURD
    (string dir_name);

  [CCode (cname = "osfscontrol_urd")]
  public static void URD
    (string dir_name);

  [CCode (cname = "xosfscontrol_back")]
  public static OS.Error* XBack ();

  [CCode (cname = "osfscontrol_back")]
  public static void Back ();

  [CCode (cname = "xosfscontrol_defect_list")]
  public static OS.Error* XDefectList
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "osfscontrol_defect_list")]
  public static void DefectList
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "xosfscontrol_add_defect")]
  public static OS.Error* XAddDefect
    (string image_name,
     int offset);

  [CCode (cname = "osfscontrol_add_defect")]
  public static void AddDefect
    (string image_name,
     int offset);

  [CCode (cname = "xosfscontrol_no_dir")]
  public static OS.Error* XNoDir ();

  [CCode (cname = "osfscontrol_no_dir")]
  public static void NoDir ();

  [CCode (cname = "xosfscontrol_no_urd")]
  public static OS.Error* XNoURD ();

  [CCode (cname = "osfscontrol_no_urd")]
  public static void NoURD ();

  [CCode (cname = "xosfscontrol_no_lib")]
  public static OS.Error* XNoLib ();

  [CCode (cname = "osfscontrol_no_lib")]
  public static void NoLib ();

  [CCode (cname = "xosfscontrol_used_space_map")]
  public static OS.Error* XUsedSpaceMap
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "osfscontrol_used_space_map")]
  public static void UsedSpaceMap
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "xosfscontrol_read_boot_option")]
  public static OS.Error* XReadBootOption
    (string file_name,
     out int boot_option);

  [CCode (cname = "osfscontrol_read_boot_option")]
  public static int ReadBootOption
    (string file_name);

  [CCode (cname = "xosfscontrol_write_boot_option")]
  public static OS.Error* XWriteBootOption
    (string file_name,
     int boot_option);

  [CCode (cname = "osfscontrol_write_boot_option")]
  public static void WriteBootOption
    (string file_name,
     int boot_option);

  [CCode (cname = "xosfscontrol_free_space")]
  public static OS.Error* XFreeSpace
    (string file_name,
     out int free,
     out int max,
     out int size);

  [CCode (cname = "osfscontrol_free_space")]
  public static int FreeSpace
    (string file_name,
     out int max,
     out int size);

  [CCode (cname = "xosfscontrol_name_disc")]
  public static OS.Error* XNameDisc
    (string file_name,
     string disc_name);

  [CCode (cname = "osfscontrol_name_disc")]
  public static void NameDisc
    (string file_name,
     string disc_name);

  [CCode (cname = "xosfscontrol_stamp_image")]
  public static OS.Error* XStampImage
    (string file_name,
     bool stamp_now);

  [CCode (cname = "osfscontrol_stamp_image")]
  public static void StampImage
    (string file_name,
     bool stamp_now);

  [CCode (cname = "xosfscontrol_object_at_offset")]
  public static OS.Error* XObjectAtOffset
    (string file_name,
     int offset,
     string buffer,
     int size,
     out ObjectKind obj_kind);

  [CCode (cname = "osfscontrol_object_at_offset")]
  public static ObjectKind ObjectAtOffset
    (string file_name,
     int offset,
     string buffer,
     int size);

  [CCode (cname = "xosfscontrol_set_dir")]
  public static OS.Error* XSetDir
    (string path,
     SpecialDir dir,
     string fs_name,
     string special);

  [CCode (cname = "osfscontrol_set_dir")]
  public static void SetDir
    (string path,
     SpecialDir dir,
     string fs_name,
     string special);

  [CCode (cname = "xosfscontrol_read_dir")]
  public static OS.Error* XReadDir
    (string buffer,
     SpecialDir dir,
     string fs_name,
     int size,
     out int spare,
     out string special);

  [CCode (cname = "osfscontrol_read_dir")]
  public static int ReadDir
    (string buffer,
     SpecialDir dir,
     string fs_name,
     int size,
     out string special);

  [CCode (cname = "xosfscontrol_free_space64")]
  public static OS.Error* XFreeSpace64
    (string file_name,
     out bits free_lo,
     out int free_hi,
     out int max,
     out bits disc_size_lo,
     out bits disc_size_hi);

  [CCode (cname = "osfscontrol_free_space64")]
  public static void FreeSpace64
    (string file_name,
     out bits free_lo,
     out int free_hi,
     out int max,
     out bits disc_size_lo,
     out bits disc_size_hi);

  [CCode (cname = "xosfscontrol_defect_list64")]
  public static OS.Error* XDefectList64
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "osfscontrol_defect_list64")]
  public static void DefectList64
    (string image_name,
     char* buffer,
     int size);

  [CCode (cname = "xosfscontrol_add_defect64")]
  public static OS.Error* XAddDefect64
    (string image_name,
     bits offset_lo,
     int offset_hi);

  [CCode (cname = "osfscontrol_add_defect64")]
  public static void AddDefect64
    (string image_name,
     bits offset_lo,
     int offset_hi);

  [CCode (cname = "xosfscontrol_enum_file_handles")]
  public static OS.Error* XEnumFileHandles
    (OS.F prev_handle,
     out OSArgs.StreamStatus status,
     out OS.F next_handle,
     out FileSwitch.FSInfo info);

  [CCode (cname = "osfscontrol_enum_file_handles")]
  public static OSArgs.StreamStatus EnumFileHandles
    (OS.F prev_handle,
     out OS.F next_handle,
     out FileSwitch.FSInfo info);

  [CCode (cname = "xosfscontrol_run_absolute_file")]
  public static OS.Error* XRunAbsoluteFile
    (string filename,
     string args,
     int file_length);

  [CCode (cname = "osfscontrol_run_absolute_file")]
  public static void RunAbsoluteFile
    (string filename,
     string args,
     int file_length);

  [CCode (cname = "xosfscontrol_run_untyped_file")]
  public static OS.Error* XRunUntypedFile
    (string filename,
     string args,
     int file_length,
     bits load_addr,
     bits exec_addr);

  [CCode (cname = "osfscontrol_run_untyped_file")]
  public static void RunUntypedFile
    (string filename,
     string args,
     int file_length,
     bits load_addr,
     bits exec_addr);

  [CCode (cname = "xosfscontrol_run_transient_file")]
  public static OS.Error* XRunTransientFile
    (string filename,
     string args);

  [CCode (cname = "osfscontrol_run_transient_file")]
  public static void RunTransientFile
    (string filename,
     string args);

  [CCode (cname = "xupcall_media_not_present")]
  public static OS.Error* XUpCall_MediaNotPresent
    (FileSwitch.FSNo fs_no,
     string media_name,
     int device_no,
     int iteration_count,
     int timeout,
     string media_type,
     out int changed);

  [CCode (cname = "upcall_media_not_present")]
  public static int UpCall_MediaNotPresent
    (FileSwitch.FSNo fs_no,
     string media_name,
     int device_no,
     int iteration_count,
     int timeout,
     string media_type);

  [CCode (cname = "xupcall_media_not_known")]
  public static OS.Error* XUpCall_MediaNotKnown
    (FileSwitch.FSNo fs_no,
     string media_name,
     int device_no,
     int iteration_count,
     int timeout,
     string media_type,
     out int changed);

  [CCode (cname = "upcall_media_not_known")]
  public static int UpCall_MediaNotKnown
    (FileSwitch.FSNo fs_no,
     string media_name,
     int device_no,
     int iteration_count,
     int timeout,
     string media_type);

  [CCode (cname = "xupcallfscontrol_set_args_size")]
  public static OS.Error* XUpCallFSControl_SetArgsSize
    (OS.F f,
     int size,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfscontrol_set_args_size")]
  public static void UpCallFSControl_SetArgsSize
    (OS.F f,
     int size,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfscontrol_set_args_sizew")]
  public static OS.Error* XUpCallFSControl_SetArgsSizeW
    (OS.FW f,
     int size,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfscontrol_set_args_sizew")]
  public static void UpCallFSControl_SetArgsSizeW
    (OS.FW f,
     int size,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfscontrol_rename")]
  public static OS.Error* XUpCallFSControl_Rename
    (string file_name,
     string new_file_name,
     string special,
     string new_special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfscontrol_rename")]
  public static void UpCallFSControl_Rename
    (string file_name,
     string new_file_name,
     string special,
     string new_special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfscontrol_set_attr_string")]
  public static OS.Error* XUpCallFSControl_SetAttrString
    (string file_name,
     string attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfscontrol_set_attr_string")]
  public static void UpCallFSControl_SetAttrString
    (string file_name,
     string attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcall_media_search_end")]
  public static OS.Error* XUpCall_MediaSearchEnd ();

  [CCode (cname = "upcall_media_search_end")]
  public static void UpCall_MediaSearchEnd ();

  [CCode (cname = "xupcall_new_application")]
  public static OS.Error* XUpCall_NewApplication
    (char* cao,
     out bool started);

  [CCode (cname = "upcall_new_application")]
  public static bool UpCall_NewApplication
    (char* cao);

}
