/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/osfile.h")]
namespace OSLib.OSFile
{
  [CCode (cname = "osfile_load_flags")]
  public enum LoadFlags
  {
    [CCode (cname = "osfile_LOAD_FLAGS_IS_CODE")]
    IsCode
  }

  [CCode (cname="osfile_FILE_TYPE")]
  public const bits FileType;
  [CCode (cname="osfile_FILE_TYPE_SHIFT")]
  public const int FileTypeShift;
  [CCode (cname="osfile_TYPE_DRAW")]
  public const bits TypeDraw;
  [CCode (cname="osfile_TYPE_BBCROM")]
  public const bits TypeBBCROM;
  [CCode (cname="osfile_TYPE_CODE")]
  public const bits TypeCode;
  [CCode (cname="osfile_TYPE_RESOURCE")]
  public const bits TypeResource;
  [CCode (cname="osfile_TYPE_DOS_DISC")]
  public const bits TypeDOSDisc;
  [CCode (cname="osfile_TYPE_DEVICE")]
  public const bits TypeDevice;
  [CCode (cname="osfile_TYPE_CACHE")]
  public const bits TypeCache;
  [CCode (cname="osfile_TYPE_TASK_EXEC")]
  public const bits TypeTaskExec;
  [CCode (cname="osfile_TYPE_TASK_OBEY")]
  public const bits TypeTaskObey;
  [CCode (cname="osfile_TYPE_DOS")]
  public const bits TypeDOS;
  [CCode (cname="osfile_TYPE_DESKTOP")]
  public const bits TypeDesktop;
  [CCode (cname="osfile_TYPE_OBEY")]
  public const bits TypeObey;
  [CCode (cname="osfile_TYPE_TEMPLATE")]
  public const bits TypeTemplate;
  [CCode (cname="osfile_TYPE_PALETTE")]
  public const bits TypePalette;
  [CCode (cname="osfile_TYPE_CONFIG")]
  public const bits TypeConfig;
  [CCode (cname="osfile_TYPE_PRINTOUT")]
  public const bits TypePrintout;
  [CCode (cname="osfile_TYPE_POSTSCRIPT")]
  public const bits TypePostscript;
  [CCode (cname="osfile_TYPE_FONT")]
  public const bits TypeFont;
  [CCode (cname="osfile_TYPE_BBC_FONT")]
  public const bits TypeBBCFont;
  [CCode (cname="osfile_TYPE_ABSOLUTE")]
  public const bits TypeAbsolute;
  [CCode (cname="osfile_TYPE_SPRITE")]
  public const bits TypeSprite;
  [CCode (cname="osfile_TYPE_MODULE")]
  public const bits TypeModule;
  [CCode (cname="osfile_TYPE_BASIC")]
  public const bits TypeBASIC;
  [CCode (cname="osfile_TYPE_UTILITY")]
  public const bits TypeUtility;
  [CCode (cname="osfile_TYPE_DATA")]
  public const bits TypeData;
  [CCode (cname="osfile_TYPE_COMMAND")]
  public const bits TypeCommand;
  [CCode (cname="osfile_TYPE_TEXT")]
  public const bits TypeText;
  [CCode (cname="osfile_TYPE_DIR")]
  public const bits TypeDir;
  [CCode (cname="osfile_TYPE_APPLICATION")]
  public const bits TypeApplication;
  [CCode (cname="osfile_TYPE_UNTYPED")]
  public const bits TypeUntyped;

  [CCode (cname = "xosfile_save_stamped")]
  public static OS.Error* XSaveStamped
    (string file_name,
     bits file_type,
     char* data,
     char* end);

  [CCode (cname = "osfile_save_stamped")]
  public static void SaveStamped
    (string file_name,
     bits file_type,
     char* data,
     char* end);

  [CCode (cname = "xosfile_write")]
  public static OS.Error* XWrite
    (string file_name,
     bits load_addr,
     bits exec_addr,
     FileSwitch.Attr attr);

  [CCode (cname = "osfile_write")]
  public static void Write
    (string file_name,
     bits load_addr,
     bits exec_addr,
     FileSwitch.Attr attr);

  [CCode (cname = "xosfile_write_load")]
  public static OS.Error* XWriteLoad
    (string file_name,
     bits load_addr);

  [CCode (cname = "osfile_write_load")]
  public static void WriteLoad
    (string file_name,
     bits load_addr);

  [CCode (cname = "xosfile_write_exec")]
  public static OS.Error* XWriteExec
    (string file_name,
     bits exec_addr);

  [CCode (cname = "osfile_write_exec")]
  public static void WriteExec
    (string file_name,
     bits exec_addr);

  [CCode (cname = "xosfile_write_attr")]
  public static OS.Error* XWriteAttr
    (string file_name,
     FileSwitch.Attr attr);

  [CCode (cname = "osfile_write_attr")]
  public static void WriteAttr
    (string file_name,
     FileSwitch.Attr attr);

  [CCode (cname = "xosfile_stamp")]
  public static OS.Error* XStamp
    (string file_name);

  [CCode (cname = "osfile_stamp")]
  public static void Stamp
    (string file_name);

  [CCode (cname = "xosfile_set_type")]
  public static OS.Error* XSetType
    (string file_name,
     bits file_type);

  [CCode (cname = "osfile_set_type")]
  public static void SetType
    (string file_name,
     bits file_type);

  [CCode (cname = "xosfile_delete")]
  public static OS.Error* XDelete
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_delete")]
  public static FileSwitch.ObjectType Delete
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_create_stamped")]
  public static OS.Error* XCreateStamped
    (string file_name,
     bits file_type,
     int size);

  [CCode (cname = "osfile_create_stamped")]
  public static void CreateStamped
    (string file_name,
     bits file_type,
     int size);

  [CCode (cname = "xosfile_create_dir")]
  public static OS.Error* XCreateDir
    (string dir_name,
     int entry_count);

  [CCode (cname = "osfile_create_dir")]
  public static void CreateDir
    (string dir_name,
     int entry_count);

  [CCode (cname = "xosfile_load_stamped_path")]
  public static OS.Error* XLoadStampedPath
    (string file_name,
     char* addr,
     string path,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_path")]
  public static FileSwitch.ObjectType LoadStampedPath
    (string file_name,
     char* addr,
     string path,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_path_with_flags")]
  public static OS.Error* XLoadStampedPathWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     string path,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_path_with_flags")]
  public static FileSwitch.ObjectType LoadStampedPathWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     string path,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_path_var")]
  public static OS.Error* XLoadStampedPathVar
    (string file_name,
     char* addr,
     string var,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_path_var")]
  public static FileSwitch.ObjectType LoadStampedPathVar
    (string file_name,
     char* addr,
     string var,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_path_var_with_flags")]
  public static OS.Error* XLoadStampedPathVarWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     string var,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_path_var_with_flags")]
  public static FileSwitch.ObjectType LoadStampedPathVarWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     string var,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_no_path")]
  public static OS.Error* XLoadStampedNoPath
    (string file_name,
     char* addr,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_no_path")]
  public static FileSwitch.ObjectType LoadStampedNoPath
    (string file_name,
     char* addr,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_no_path_with_flags")]
  public static OS.Error* XLoadStampedNoPathWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_no_path_with_flags")]
  public static FileSwitch.ObjectType LoadStampedNoPathWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped")]
  public static OS.Error* XLoadStamped
    (string file_name,
     char* addr,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped")]
  public static FileSwitch.ObjectType LoadStamped
    (string file_name,
     char* addr,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_stamped_with_flags")]
  public static OS.Error* XLoadStampedWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_stamped_with_flags")]
  public static FileSwitch.ObjectType LoadStampedWithFlags
    (string file_name,
     char* addr,
     LoadFlags flags,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_make_error")]
  public static OS.Error* XMakeError
    (string file_name,
     FileSwitch.ObjectType obj_type);

  [CCode (cname = "osfile_make_error")]
  public static void MakeError
    (string file_name,
     FileSwitch.ObjectType obj_type);

  [CCode (cname = "xosfile_read_stamped")]
  public static OS.Error* XReadStamped
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "osfile_read_stamped")]
  public static FileSwitch.ObjectType ReadStamped
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "xosfile_read_stamped_path")]
  public static OS.Error* XReadStampedPath
    (string file_name,
     string path,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "osfile_read_stamped_path")]
  public static FileSwitch.ObjectType ReadStampedPath
    (string file_name,
     string path,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "xosfile_read_stamped_path_var")]
  public static OS.Error* XReadStampedPathVar
    (string file_name,
     string var,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "osfile_read_stamped_path_var")]
  public static FileSwitch.ObjectType ReadStampedPathVar
    (string file_name,
     string var,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "xosfile_read_stamped_no_path")]
  public static OS.Error* XReadStampedNoPath
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "osfile_read_stamped_no_path")]
  public static FileSwitch.ObjectType ReadStampedNoPath
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out bits file_type);

  [CCode (cname = "xosfile_read_block_size")]
  public static OS.Error* XReadBlockSize
    (string file_name,
     out int block_size);

  [CCode (cname = "osfile_read_block_size")]
  public static int ReadBlockSize
    (string file_name);

  [CCode (cname = "xosfile_save")]
  public static OS.Error* XSave
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end);

  [CCode (cname = "osfile_save")]
  public static void Save
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end);

  [CCode (cname = "xosfile_read")]
  public static OS.Error* XRead
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_read")]
  public static FileSwitch.ObjectType Read
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_read_path")]
  public static OS.Error* XReadPath
    (string file_name,
     string path,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_read_path")]
  public static FileSwitch.ObjectType ReadPath
    (string file_name,
     string path,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_read_path_var")]
  public static OS.Error* XReadPathVar
    (string file_name,
     string var,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_read_path_var")]
  public static FileSwitch.ObjectType ReadPathVar
    (string file_name,
     string var,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_read_no_path")]
  public static OS.Error* XReadNoPath
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_read_no_path")]
  public static FileSwitch.ObjectType ReadNoPath
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_create")]
  public static OS.Error* XCreate
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int size);

  [CCode (cname = "osfile_create")]
  public static void Create
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int size);

  [CCode (cname = "xosfile_load_path")]
  public static OS.Error* XLoadPath
    (string file_name,
     string path,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_path")]
  public static FileSwitch.ObjectType LoadPath
    (string file_name,
     string path,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_path_var")]
  public static OS.Error* XLoadPathVar
    (string file_name,
     string var,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_path_var")]
  public static FileSwitch.ObjectType LoadPathVar
    (string file_name,
     string var,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load_no_path")]
  public static OS.Error* XLoadNoPath
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load_no_path")]
  public static FileSwitch.ObjectType LoadNoPath
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xosfile_load")]
  public static OS.Error* XLoad
    (string file_name,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "osfile_load")]
  public static FileSwitch.ObjectType Load
    (string file_name,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xupcallfile_save")]
  public static OS.Error* XUpCallFile_Save
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_save")]
  public static void UpCallFile_Save
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_set_args")]
  public static OS.Error* XUpCallFile_SetArgs
    (string file_name,
     bits load_addr,
     bits exec_addr,
     FileSwitch.Attr attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_set_args")]
  public static void UpCallFile_SetArgs
    (string file_name,
     bits load_addr,
     bits exec_addr,
     FileSwitch.Attr attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_set_load_addr")]
  public static OS.Error* XUpCallFile_SetLoadAddr
    (string file_name,
     bits load_addr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_set_load_addr")]
  public static void UpCallFile_SetLoadAddr
    (string file_name,
     bits load_addr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_set_exec_addr")]
  public static OS.Error* XUpCallFile_SetExecAddr
    (string file_name,
     bits exec_addr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_set_exec_addr")]
  public static void UpCallFile_SetExecAddr
    (string file_name,
     bits exec_addr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_set_attr")]
  public static OS.Error* XUpCallFile_SetAttr
    (string file_name,
     FileSwitch.Attr attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_set_attr")]
  public static void UpCallFile_SetAttr
    (string file_name,
     FileSwitch.Attr attr,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_delete")]
  public static OS.Error* XUpCallFile_Delete
    (string file_name,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_delete")]
  public static void UpCallFile_Delete
    (string file_name,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_create")]
  public static OS.Error* XUpCallFile_Create
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_create")]
  public static void UpCallFile_Create
    (string file_name,
     bits load_addr,
     bits exec_addr,
     char* data,
     char* end,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfile_create_dir")]
  public static OS.Error* XUpCallFile_CreateDir
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int entry_count,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfile_create_dir")]
  public static void UpCallFile_CreateDir
    (string file_name,
     bits load_addr,
     bits exec_addr,
     int entry_count,
     string special,
     FileSwitch.FSInfo fs_info);

}
