/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osmodule.h")]
namespace OSLib.OSModule
{
  [CCode (cname = "osmodule_section_no")]
  public enum SectionNo
  {
    [CCode (cname = "osmodule_SECTION_SYSTEM")]
    SectionSystem,
    [CCode (cname = "osmodule_SECTION_CARD0")]
    SectionCard0,
    [CCode (cname = "osmodule_SECTION_CARD1")]
    SectionCard1,
    [CCode (cname = "osmodule_SECTION_CARD2")]
    SectionCard2,
    [CCode (cname = "osmodule_SECTION_CARD3")]
    SectionCard3,
    [CCode (cname = "osmodule_SECTION_ROM1")]
    SectionROM1,
    [CCode (cname = "osmodule_SECTION_ROM2")]
    SectionROM2,
    [CCode (cname = "osmodule_SECTION_ROM3")]
    SectionROM3,
    [CCode (cname = "osmodule_SECTION_ROM4")]
    SectionROM4
  }

  [CCode (cname = "osmodule_status")]
  public enum Status
  {
    [CCode (cname = "osmodule_STATUS_UNSUITABLE")]
    Unsuitable,
    [CCode (cname = "osmodule_STATUS_UNPLUGGED")]
    Unplugged,
    [CCode (cname = "osmodule_STATUS_DORMANT")]
    Dormant,
    [CCode (cname = "osmodule_STATUS_ACTIVE")]
    Active,
    [CCode (cname = "osmodule_STATUS_RUNNING")]
    Running
  }

  [CCode (cname = "osmodule_info_flags")]
  public enum InfoFlags
  {
    [CCode (cname = "osmodule_INFO_FS_COMMAND")]
    InfoFSCommand,
    [CCode (cname = "osmodule_INFO_CONFIGURATION")]
    InfoConfiguration,
    [CCode (cname = "osmodule_INFO_HELP")]
    InfoHelp,
    [CCode (cname = "osmodule_INFO_LOOKUP_STRINGS")]
    InfoLookupStrings
  }

  [CCode (cname = "osmodule_flags")]
  public enum Flags
  {
    [CCode (cname = "osmodule_32_BIT_COMPATIBLE")]
    32BitCompatible,
    [CCode (cname = "osmodule_EARLY_ROM_INITIALISATION")]
    EarlyROMInitialisation
  }

  [CCode (cname = "osmodule_state_action")]
  public enum StateAction
  {
    [CCode (cname = "osmodule_ACTION_UNPLUG")]
    ActionUnplug,
    [CCode (cname = "osmodule_ACTION_INSERT")]
    ActionInsert
  }

  [CCode (cname = "osmodule_state_event")]
  public enum StateEvent
  {
    [CCode (cname = "osmodule_EVENT_INITIALISATION")]
    EventInitialisation,
    [CCode (cname = "osmodule_EVENT_FINALISATION")]
    EventFinalisation,
    [CCode (cname = "osmodule_EVENT_CHANGED_PREFERRED_INSTANCE")]
    EventChangedPreferredInstance,
    [CCode (cname = "osmodule_EVENT_RENAMED_INSTANCE")]
    EventRenamedInstance
  }

  [CCode (cname="osmodule_ENTRY_START")]
  public const int EntryStart;
  [CCode (cname="osmodule_ENTRY_INIT")]
  public const int EntryInit;
  [CCode (cname="osmodule_ENTRY_DIE")]
  public const int EntryDie;
  [CCode (cname="osmodule_ENTRY_SERVICE")]
  public const int EntryService;
  [CCode (cname="osmodule_ENTRY_TITLE")]
  public const int EntryTitle;
  [CCode (cname="osmodule_ENTRY_HELP")]
  public const int EntryHelp;
  [CCode (cname="osmodule_ENTRY_TABLE")]
  public const int EntryTable;
  [CCode (cname="osmodule_ENTRY_SWI_CHUNK")]
  public const int EntrySWIChunk;
  [CCode (cname="osmodule_ENTRY_SWI_ENTRY")]
  public const int EntrySWIEntry;
  [CCode (cname="osmodule_ENTRY_NAME_TABLE")]
  public const int EntryNameTable;
  [CCode (cname="osmodule_ENTRY_NAME_CODE")]
  public const int EntryNameCode;
  [CCode (cname="osmodule_ENTRY_MESSAGE_FILE")]
  public const int EntryMessageFile;
  [CCode (cname="osmodule_ENTRY_FLAGS_OFFSET")]
  public const int EntryFlagsOffset;
  [CCode (cname="osmodule_SWI_CHUNK_SIZE")]
  public const int SWIChunkSize;
  [CCode (cname="osmodule_SWI_SYSTEM_BASE")]
  public const int SWISystemBase;
  [CCode (cname="osmodule_SWI_APPLICATION_BASE")]
  public const int SWIApplicationBase;
  [CCode (cname="osmodule_SWI_USER_BASE")]
  public const int SWIUserBase;
  [CCode (cname="error_MODULE_NOT_MOD")]
  public const bits Error_ModuleNotMod;
  [CCode (cname="error_MODULE_MH_NO_ROOM")]
  public const bits Error_ModuleMHNoRoom;
  [CCode (cname="error_MODULE_RM_NOT_FOUND")]
  public const bits Error_ModuleRMNotFound;
  [CCode (cname="error_MODULE_CANT_KILL")]
  public const bits Error_ModuleCantKill;
  [CCode (cname="error_MODULE_CANT_KILL2")]
  public const bits Error_ModuleCantKill2;
  [CCode (cname="error_MODULE_WIMP_CANT_KILL")]
  public const bits Error_ModuleWimpCantKill;
  [CCode (cname="error_MODULE_BAD_MODULE_REASON")]
  public const bits Error_ModuleBadModuleReason;
  [CCode (cname="error_MODULE_BAD_DYNAMIC_AREA")]
  public const bits Error_ModuleBadDynamicArea;
  [CCode (cname="error_MODULE_RM_NOT_FOUND_IN_ROM")]
  public const bits Error_ModuleRMNotFoundInROM;
  [CCode (cname="error_MODULE_MODULE_POSTFIX")]
  public const bits Error_ModuleModulePostfix;
  [CCode (cname="error_MODULE_NO_MORE_MODULES")]
  public const bits Error_ModuleNoMoreModules;
  [CCode (cname="error_MODULE_NO_MORE_INCARNATIONS")]
  public const bits Error_ModuleNoMoreIncarnations;
  [CCode (cname="error_MODULE_POSTFIX_NEEDED")]
  public const bits Error_ModulePostfixNeeded;
  [CCode (cname="error_MODULE_INCARNATION_EXISTS")]
  public const bits Error_ModuleIncarnationExists;
  [CCode (cname="error_MODULE_INCARNATION_NOT_FOUND")]
  public const bits Error_ModuleIncarnationNotFound;
  [CCode (cname="error_MODULE_CHUNK_NOT_RM")]
  public const bits Error_ModuleChunkNotRM;
  [CCode (cname="error_MODULE_BAD_RM_HEADER_FIELD")]
  public const bits Error_ModuleBadRMHeaderField;
  [CCode (cname="error_MODULE_MODULE_TOO_OLD")]
  public const bits Error_ModuleModuleTooOld;
  [CCode (cname="error_MODULE_MODULE_BAD_SWI")]
  public const bits Error_ModuleModuleBadSWI;
  [CCode (cname="error_MODULE_MODULE_IN_USE")]
  public const bits Error_ModuleModuleInUse;
  [CCode (cname="error_MODULE_BAD_FILING_SYSTEM_OPERATION")]
  public const bits Error_ModuleBadFilingSystemOperation;
  [CCode (cname="error_MODULE_FILING_SYSTEM_READ_ONLY")]
  public const bits Error_ModuleFilingSystemReadOnly;
  [CCode (cname="error_MODULE_NOT_ADIR")]
  public const bits Error_ModuleNotADir;

  [CCode (cname = "xosmodule_run")]
  public static OS.Error* XRun
    (string tail);

  [CCode (cname = "osmodule_run")]
  public static void Run
    (string tail);

  [CCode (cname = "xosmodule_load")]
  public static OS.Error* XLoad
    (string tail);

  [CCode (cname = "osmodule_load")]
  public static void Load
    (string tail);

  [CCode (cname = "xosmodule_enter")]
  public static OS.Error* XEnter
    (string module_name,
     string args);

  [CCode (cname = "osmodule_enter")]
  public static void Enter
    (string module_name,
     string args);

  [CCode (cname = "xosmodule_reinit")]
  public static OS.Error* XReinit
    (string tail);

  [CCode (cname = "osmodule_reinit")]
  public static void Reinit
    (string tail);

  [CCode (cname = "xosmodule_kill")]
  public static OS.Error* XKill
    (string module_name);

  [CCode (cname = "osmodule_kill")]
  public static void Kill
    (string module_name);

  [CCode (cname = "xosmodule_info")]
  public static OS.Error* XInfo
    (out int max,
     out int free);

  [CCode (cname = "osmodule_info")]
  public static int Info
    (out int max);

  [CCode (cname = "xosmodule_alloc")]
  public static OS.Error* XAlloc
    (int size,
     out char* blk);

  [CCode (cname = "osmodule_alloc")]
  public static char* Alloc
    (int size);

  [CCode (cname = "xosmodule_free")]
  public static OS.Error* XFree
    (char* blk);

  [CCode (cname = "osmodule_free")]
  public static void Free
    (char* blk);

  [CCode (cname = "xosmodule_tidy")]
  public static OS.Error* XTidy ();

  [CCode (cname = "osmodule_tidy")]
  public static void Tidy ();

  [CCode (cname = "xosmodule_clear")]
  public static OS.Error* XClear ();

  [CCode (cname = "osmodule_clear")]
  public static void Clear ();

  [CCode (cname = "xosmodule_add_area")]
  public static OS.Error* XAddArea
    (char* module);

  [CCode (cname = "osmodule_add_area")]
  public static void AddArea
    (char* module);

  [CCode (cname = "xosmodule_copy_area")]
  public static OS.Error* XCopyArea
    (char* module,
     int size);

  [CCode (cname = "osmodule_copy_area")]
  public static void CopyArea
    (char* module,
     int size);

  [CCode (cname = "xosmodule_enumerate")]
  public static OS.Error* XEnumerate
    (int module_no,
     int instance_no,
     out int module_no_out,
     out int instance_no_out,
     out char* module,
     out char* workspace,
     out string postfix);

  [CCode (cname = "osmodule_enumerate")]
  public static int Enumerate
    (int module_no,
     int instance_no,
     out int instance_no_out,
     out char* module,
     out char* workspace,
     out string postfix);

  [CCode (cname = "xosmodule_realloc")]
  public static OS.Error* XRealloc
    (char* blk,
     int size_increase,
     out char* blk_out);

  [CCode (cname = "osmodule_realloc")]
  public static char* Realloc
    (char* blk,
     int size_increase);

  [CCode (cname = "xosmodule_instantiate")]
  public static OS.Error* XInstantiate
    (string instance_name);

  [CCode (cname = "osmodule_instantiate")]
  public static void Instantiate
    (string instance_name);

  [CCode (cname = "xosmodule_rename")]
  public static OS.Error* XRename
    (string old_name,
     string new_postfix);

  [CCode (cname = "osmodule_rename")]
  public static void Rename
    (string old_name,
     string new_postfix);

  [CCode (cname = "xosmodule_prefer")]
  public static OS.Error* XPrefer
    (string instance_name);

  [CCode (cname = "osmodule_prefer")]
  public static void Prefer
    (string instance_name);

  [CCode (cname = "xosmodule_add_podule_area")]
  public static OS.Error* XAddPoduleArea
    (string environment,
     int chunk,
     SectionNo rom_section);

  [CCode (cname = "osmodule_add_podule_area")]
  public static void AddPoduleArea
    (string environment,
     int chunk,
     SectionNo rom_section);

  [CCode (cname = "xosmodule_lookup")]
  public static OS.Error* XLookup
    (string module_name,
     out int module_no,
     out int instance_no,
     out char* module,
     out char* workspace,
     out string postfix);

  [CCode (cname = "osmodule_lookup")]
  public static int Lookup
    (string module_name,
     out int instance_no,
     out char* module,
     out char* workspace,
     out string postfix);

  [CCode (cname = "xosmodule_enumerate_rom")]
  public static OS.Error* XEnumerateROM
    (int module_no,
     SectionNo section,
     out int module_no_out,
     out SectionNo section_out,
     out string module_name,
     out Status status,
     out int chunk_no);

  [CCode (cname = "osmodule_enumerate_rom")]
  public static int EnumerateROM
    (int module_no,
     SectionNo section,
     out SectionNo section_out,
     out string module_name,
     out Status status,
     out int chunk_no);

  [CCode (cname = "xosmodule_enumerate_rom_with_info")]
  public static OS.Error* XEnumerateROMWithInfo
    (int module_no,
     SectionNo section,
     out int module_no_out,
     out SectionNo section_out,
     out string module_name,
     out Status status,
     out int chunk_no,
     out int bcd_version);

  [CCode (cname = "osmodule_enumerate_rom_with_info")]
  public static int EnumerateROMWithInfo
    (int module_no,
     SectionNo section,
     out SectionNo section_out,
     out string module_name,
     out Status status,
     out int chunk_no,
     out int bcd_version);

  [CCode (cname = "xosmodule_enumerate_with_private_word")]
  public static OS.Error* XEnumerateWithPrivateWord
    (int module_no,
     int incarnation_number,
     out int module_no_out,
     out int incarnation_number_out,
     out char* module_code,
     out int* private_word,
     out string postfix);

  [CCode (cname = "osmodule_enumerate_with_private_word")]
  public static int EnumerateWithPrivateWord
    (int module_no,
     int incarnation_number,
     out int incarnation_number_out,
     out char* module_code,
     out int* private_word,
     out string postfix);

  [CCode (cname = "xosmodule_change_state")]
  public static OS.Error* XChangeState
    (string module_name,
     StateAction action,
     int podule_number);

  [CCode (cname = "osmodule_change_state")]
  public static void ChangeState
    (string module_name,
     StateAction action,
     int podule_number);

  [CCode (cname = "xservice_uk_command")]
  public static OS.Error* XService_UKCommand
    (string command,
     out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_uk_command")]
  public static bool Service_UKCommand
    (string command,
     out OS.Error error);

  [CCode (cname = "xservice_error")]
  public static OS.Error* XService_Error
    (OS.Error error);

  [CCode (cname = "service_error")]
  public static void Service_Error
    (OS.Error error);

  [CCode (cname = "xservice_uk_byte")]
  public static OS.Error* XService_UKByte
    (int byte,
     int arg0,
     int arg1,
     out bool unclaimed,
     out int res0,
     out int res1);

  [CCode (cname = "service_uk_byte")]
  public static bool Service_UKByte
    (int byte,
     int arg0,
     int arg1,
     out int res0,
     out int res1);

  [CCode (cname = "xservice_uk_word")]
  public static OS.Error* XService_UKWord
    (int word,
     char* block,
     out bool unclaimed);

  [CCode (cname = "service_uk_word")]
  public static bool Service_UKWord
    (int word,
     char* block);

  [CCode (cname = "xservice_help")]
  public static OS.Error* XService_Help
    (string command,
     out bool unclaimed);

  [CCode (cname = "service_help")]
  public static bool Service_Help
    (string command);

  [CCode (cname = "xservice_uk_config")]
  public static OS.Error* XService_UKConfig
    (string tail,
     out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_uk_config")]
  public static bool Service_UKConfig
    (string tail,
     out OS.Error error);

  [CCode (cname = "xservice_uk_status")]
  public static OS.Error* XService_UKStatus
    (string tail,
     out bool unclaimed);

  [CCode (cname = "service_uk_status")]
  public static bool Service_UKStatus
    (string tail);

  [CCode (cname = "xservice_new_application")]
  public static OS.Error* XService_NewApplication
    (out bool unclaimed);

  [CCode (cname = "service_new_application")]
  public static bool Service_NewApplication ();

  [CCode (cname = "xservice_lookup_file_type")]
  public static OS.Error* XService_LookupFileType
    (bits file_type,
     out bool unclaimed,
     out bits name1,
     out bits name2);

  [CCode (cname = "service_lookup_file_type")]
  public static bool Service_LookupFileType
    (bits file_type,
     out bits name1,
     out bits name2);

  [CCode (cname = "xservice_post_init")]
  public static OS.Error* XService_PostInit ();

  [CCode (cname = "service_post_init")]
  public static void Service_PostInit ();

  [CCode (cname = "xservice_syntax_error")]
  public static OS.Error* XService_SyntaxError
    (int* offset,
     char* base,
     string command,
     out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_syntax_error")]
  public static bool Service_SyntaxError
    (int* offset,
     char* base,
     string command,
     out OS.Error error);

  [CCode (cname = "xservice_module_pre_init")]
  public static OS.Error* XService_ModulePreInit
    (char* module,
     int length_plus_four);

  [CCode (cname = "service_module_pre_init")]
  public static void Service_ModulePreInit
    (char* module,
     int length_plus_four);

  [CCode (cname = "xservice_module_status")]
  public static OS.Error* XService_ModuleStatus
    (StateEvent event,
     char* module,
     int* private_word,
     string title,
     string postfix);

  [CCode (cname = "service_module_status")]
  public static void Service_ModuleStatus
    (StateEvent event,
     char* module,
     int* private_word,
     string title,
     string postfix);

  [CCode (cname = "xservice_module_status_instance_renamed")]
  public static OS.Error* XService_ModuleStatusInstanceRenamed
    (char* module,
     int* private_word,
     string title,
     string postfix,
     string new_instance_name);

  [CCode (cname = "service_module_status_instance_renamed")]
  public static void Service_ModuleStatusInstanceRenamed
    (char* module,
     int* private_word,
     string title,
     string postfix,
     string new_instance_name);

  [CCode (cname = "xservice_module_post_init")]
  public static OS.Error* XService_ModulePostInit
    (char* module,
     char* title,
     string postfix,
     int bcd_version);

  [CCode (cname = "service_module_post_init")]
  public static void Service_ModulePostInit
    (char* module,
     char* title,
     string postfix,
     int bcd_version);

  [CCode (cname = "xservice_module_post_final")]
  public static OS.Error* XService_ModulePostFinal
    (char* module,
     char* title,
     string postfix,
     int bcd_version);

  [CCode (cname = "service_module_post_final")]
  public static void Service_ModulePostFinal
    (char* module,
     char* title,
     string postfix,
     int bcd_version);

}
