/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osword.h")]
namespace OSLib.OSWord
{
  [CCode (cname = "oswordreadclock_op")]
  public enum OSWordReadClock_Op
  {
    [CCode (cname = "oswordreadclock_OP_LOCAL_STRING")]
    OpLocalString,
    [CCode (cname = "oswordreadclock_OP_LOCAL_BCD")]
    OpLocalBCD,
    [CCode (cname = "oswordreadclock_OP_CONVERT_BCD_TO_STRING")]
    OpConvertBCDToString,
    [CCode (cname = "oswordreadclock_OP_UTC")]
    OpUTC
  }

  [CCode (cname = "oswordwriteclock_op")]
  public enum OSWordWriteClock_Op
  {
    [CCode (cname = "oswordwriteclock_OP_TIME_LOCAL_STRING")]
    OpTimeLocalString,
    [CCode (cname = "oswordwriteclock_OP_DATE_LOCAL_STRING")]
    OpDateLocalString,
    [CCode (cname = "oswordwriteclock_OP_LOCAL_STRING")]
    OpLocalString
  }

  [CCode (cname = "oswordpointer_op")]
  public enum OSWordPointer_Op
  {
    [CCode (cname = "oswordpointer_OP_DEFINE")]
    OpDefine,
    [CCode (cname = "oswordpointer_OP_SET_BBOX")]
    OpSetBBox,
    [CCode (cname = "oswordpointer_OP_STEP")]
    OpStep,
    [CCode (cname = "oswordpointer_OP_SET_POSITION")]
    OpSetPosition,
    [CCode (cname = "oswordpointer_OP_READ_POSITION")]
    OpReadPosition,
    [CCode (cname = "oswordpointer_OP_SET_UNLINKED_POSITION")]
    OpSetUnlinkedPosition,
    [CCode (cname = "oswordpointer_OP_READ_UNLINKED_POSITION")]
    OpReadUnlinkedPosition,
    [CCode (cname = "oswordpointer_OP_READ_BBOX")]
    OpReadBBox
  }

  [CCode (cname = "osword_timer_block")]
  public struct TimerBlock
  {
    [CCode (cname = "b")]
    public unowned byte B[5];
  }

  [CCode (cname = "osword_char_definition_block")]
  public struct CharDefinitionBlock
  {
    [CCode (cname = "c")]
    public unowned char C;
    [CCode (cname = "definition")]
    public unowned byte Definition[8];
  }

  [CCode (cname = "osword_palette_block")]
  public struct PaletteBlock
  {
    [CCode (cname = "gcol")]
    public unowned OS.GCOL Gcol;
    [CCode (cname = "colour_number")]
    public unowned byte ColourNumber;
    [CCode (cname = "r")]
    public unowned byte R;
    [CCode (cname = "g")]
    public unowned byte G;
    [CCode (cname = "b")]
    public unowned byte B;
  }

  [CCode (cname = "osword_cursor_position_block")]
  public struct CursorPositionBlock
  {
    [CCode (cname = "xprev")]
    public unowned short Xprev;
    [CCode (cname = "yprev")]
    public unowned short Yprev;
    [CCode (cname = "x")]
    public unowned short X;
    [CCode (cname = "y")]
    public unowned short Y;
  }

  [CCode (cname = "oswordreadclock_local_string_block")]
  public struct OSWordReadClock_LocalStringBlock
  {
    [CCode (cname = "op")]
    public unowned OSWordReadClock_Op Op;
    [CCode (cname = "string.c", array_length = false)]
    public unowned char[] String_C;
  }

  [CCode (cname = "oswordreadclock_local_bcd_block")]
  public struct OSWordReadClock_LocalBCDBlock
  {
    [CCode (cname = "op")]
    public unowned OSWordReadClock_Op Op;
    [CCode (cname = "date_and_time")]
    public unowned OS.BCDDateAndTime DateAndTime;
  }

  [CCode (cname = "oswordreadclock_convert_bcd_to_string_block")]
  public struct OSWordReadClock_ConvertBCDToStringBlock
  {
    [CCode (cname = "bcd.op")]
    public unowned OSWordReadClock_Op Bcd_Op;
    [CCode (cname = "bcd.date_and_time")]
    public unowned OS.BCDDateAndTime Bcd_DateAndTime;
    [CCode (cname = "string.c", array_length = false)]
    public unowned char[] String_C;
  }

  [CCode (cname = "oswordreadclock_utc_block")]
  public struct OSWordReadClock_UTCBlock
  {
    [CCode (cname = "op")]
    public unowned OSWordReadClock_Op Op;
    [CCode (cname = "utc")]
    public unowned OS.DateAndTime Utc;
  }

  [CCode (cname = "oswordwriteclock_block_base")]
  public struct OSWordWriteClock_BlockBase
  {
    [CCode (cname = "op")]
    public unowned OSWordWriteClock_Op Op;
  }

  [Compact]
  [CCode (cname = "oswordwriteclock_block", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class OSWordWriteClock_Block
  {
    public OSWordWriteClock_Block(int numEntries);
    [CCode (cname = "op")]
    public unowned OSWordWriteClock_Op Op;
    [CCode (cname = "string", array_length = false)]
    public unowned char[] String;
  }

  [CCode (cname = "oswordpointer_define_block")]
  public struct OSWordPointer_DefineBlock
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[2];
    [CCode (cname = "op")]
    public unowned OSWordPointer_Op Op;
    [CCode (cname = "shape_no")]
    public unowned byte ShapeNo;
    [CCode (cname = "width")]
    public unowned byte Width;
    [CCode (cname = "height")]
    public unowned byte Height;
    [CCode (cname = "xactive")]
    public unowned byte Xactive;
    [CCode (cname = "yactive")]
    public unowned byte Yactive;
    [CCode (cname = "data")]
    public unowned char* Data;
  }

  [CCode (cname = "oswordpointer_bbox_block")]
  public struct OSWordPointer_BBoxBlock
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "op")]
    public unowned OSWordPointer_Op Op;
    [CCode (cname = "x0")]
    public unowned short X0;
    [CCode (cname = "y0")]
    public unowned short Y0;
    [CCode (cname = "x1")]
    public unowned short X1;
    [CCode (cname = "y1")]
    public unowned short Y1;
  }

  [CCode (cname = "oswordpointer_step_block")]
  public struct OSWordPointer_StepBlock
  {
    [CCode (cname = "op")]
    public unowned OSWordPointer_Op Op;
    [CCode (cname = "xstep")]
    public unowned byte Xstep;
    [CCode (cname = "ystep")]
    public unowned byte Ystep;
  }

  [CCode (cname = "oswordpointer_position_block")]
  public struct OSWordPointer_PositionBlock
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved;
    [CCode (cname = "op")]
    public unowned OSWordPointer_Op Op;
    [CCode (cname = "x")]
    public unowned short X;
    [CCode (cname = "y")]
    public unowned short Y;
  }

  [CCode (cname = "osword_screen_base_block")]
  public struct ScreenBaseBlock
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "type")]
    public unowned byte Type;
    [CCode (cname = "base")]
    public unowned char* Base;
  }

  [CCode (cname = "osword_block")]
  public struct Block
  {
    [CCode (cname = "timer")]
    public unowned TimerBlock Timer;
    [CCode (cname = "char_definition")]
    public unowned CharDefinitionBlock CharDefinition;
    [CCode (cname = "palette")]
    public unowned PaletteBlock Palette;
    [CCode (cname = "cursor_position")]
    public unowned CursorPositionBlock CursorPosition;
    [CCode (cname = "string")]
    public unowned OSWordReadClock_LocalStringBlock String;
    [CCode (cname = "bcd")]
    public unowned OSWordReadClock_LocalBCDBlock Bcd;
    [CCode (cname = "convert_bcd_to_string")]
    public unowned OSWordReadClock_ConvertBCDToStringBlock ConvertBcdToString;
    [CCode (cname = "utc")]
    public unowned OSWordReadClock_UTCBlock Utc;
    [CCode (cname = "write_clock")]
    public unowned OSWordWriteClock_Block WriteClock;
    [CCode (cname = "pointer_define")]
    public unowned OSWordPointer_DefineBlock PointerDefine;
    [CCode (cname = "pointer_bbox")]
    public unowned OSWordPointer_BBoxBlock PointerBbox;
    [CCode (cname = "pointer_step")]
    public unowned OSWordPointer_StepBlock PointerStep;
    [CCode (cname = "pointer_position")]
    public unowned OSWordPointer_PositionBlock PointerPosition;
    [CCode (cname = "screen_base")]
    public unowned ScreenBaseBlock ScreenBase;
    [CCode (cname = "reserved")]
    public unowned char Reserved[64];
  }

  [CCode (cname="osword_CHAR_DEFINITION_ECF1")]
  public const char CharDefinitionECF1;
  [CCode (cname="osword_CHAR_DEFINITION_ECF2")]
  public const char CharDefinitionECF2;
  [CCode (cname="osword_CHAR_DEFINITION_ECF3")]
  public const char CharDefinitionECF3;
  [CCode (cname="osword_CHAR_DEFINITION_ECF4")]
  public const char CharDefinitionECF4;
  [CCode (cname="osword_CHAR_DEFINITION_DOT_DASH_PATTERN")]
  public const char CharDefinitionDotDashPattern;

  [CCode (cname = "xosword_read_system_clock")]
  public static OS.Error* XReadSystemClock
    (TimerBlock clock);

  [CCode (cname = "osword_read_system_clock")]
  public static void ReadSystemClock
    (TimerBlock clock);

  [CCode (cname = "xosword_write_system_clock")]
  public static OS.Error* XWriteSystemClock
    (TimerBlock clock);

  [CCode (cname = "osword_write_system_clock")]
  public static void WriteSystemClock
    (TimerBlock clock);

  [CCode (cname = "xosword_read_interval_timer")]
  public static OS.Error* XReadIntervalTimer
    (TimerBlock timer);

  [CCode (cname = "osword_read_interval_timer")]
  public static void ReadIntervalTimer
    (TimerBlock timer);

  [CCode (cname = "xosword_write_interval_timer")]
  public static OS.Error* XWriteIntervalTimer
    (TimerBlock timer);

  [CCode (cname = "osword_write_interval_timer")]
  public static void WriteIntervalTimer
    (TimerBlock timer);

  [CCode (cname = "xosword_read_char_definition")]
  public static OS.Error* XReadCharDefinition
    (CharDefinitionBlock definition);

  [CCode (cname = "osword_read_char_definition")]
  public static void ReadCharDefinition
    (CharDefinitionBlock definition);

  [CCode (cname = "xosword_read_palette")]
  public static OS.Error* XReadPalette
    (PaletteBlock entry);

  [CCode (cname = "osword_read_palette")]
  public static void ReadPalette
    (PaletteBlock entry);

  [CCode (cname = "xosword_write_palette")]
  public static OS.Error* XWritePalette
    (PaletteBlock entry);

  [CCode (cname = "osword_write_palette")]
  public static void WritePalette
    (PaletteBlock entry);

  [CCode (cname = "xosword_read_cursor_position")]
  public static OS.Error* XReadCursorPosition
    (CursorPositionBlock position);

  [CCode (cname = "osword_read_cursor_position")]
  public static void ReadCursorPosition
    (CursorPositionBlock position);

  [CCode (cname = "xoswordreadclock_local_string")]
  public static OS.Error* XOSWordReadClock_LocalString
    (OSWordReadClock_LocalStringBlock string);

  [CCode (cname = "oswordreadclock_local_string")]
  public static void OSWordReadClock_LocalString
    (OSWordReadClock_LocalStringBlock string);

  [CCode (cname = "xoswordreadclock_local_bcd")]
  public static OS.Error* XOSWordReadClock_LocalBCD
    (OSWordReadClock_LocalBCDBlock bcd);

  [CCode (cname = "oswordreadclock_local_bcd")]
  public static void OSWordReadClock_LocalBCD
    (OSWordReadClock_LocalBCDBlock bcd);

  [CCode (cname = "xoswordreadclock_convert_bcd_to_string")]
  public static OS.Error* XOSWordReadClock_ConvertBCDToString
    (OSWordReadClock_ConvertBCDToStringBlock convert_bcd_to_String);

  [CCode (cname = "oswordreadclock_convert_bcd_to_string")]
  public static void OSWordReadClock_ConvertBCDToString
    (OSWordReadClock_ConvertBCDToStringBlock convert_bcd_to_String);

  [CCode (cname = "xoswordreadclock_utc")]
  public static OS.Error* XOSWordReadClock_UTC
    (OSWordReadClock_UTCBlock utc);

  [CCode (cname = "oswordreadclock_utc")]
  public static void OSWordReadClock_UTC
    (OSWordReadClock_UTCBlock utc);

  [CCode (cname = "xoswordwriteclock_time_local_string")]
  public static OS.Error* XOSWordWriteClock_TimeLocalString
    (OSWordWriteClock_Block time);

  [CCode (cname = "oswordwriteclock_time_local_string")]
  public static void OSWordWriteClock_TimeLocalString
    (OSWordWriteClock_Block time);

  [CCode (cname = "xoswordwriteclock_date_local_string")]
  public static OS.Error* XOSWordWriteClock_DateLocalString
    (OSWordWriteClock_Block date);

  [CCode (cname = "oswordwriteclock_date_local_string")]
  public static void OSWordWriteClock_DateLocalString
    (OSWordWriteClock_Block date);

  [CCode (cname = "xoswordwriteclock_date_and_time_local_string")]
  public static OS.Error* XOSWordWriteClock_DateAndTimeLocalString
    (OSWordWriteClock_Block date_and_time);

  [CCode (cname = "oswordwriteclock_date_and_time_local_string")]
  public static void OSWordWriteClock_DateAndTimeLocalString
    (OSWordWriteClock_Block date_and_time);

  [CCode (cname = "xoswordpointer_define")]
  public static OS.Error* XOSWordPointer_Define
    (OSWordPointer_DefineBlock define);

  [CCode (cname = "oswordpointer_define")]
  public static void OSWordPointer_Define
    (OSWordPointer_DefineBlock define);

  [CCode (cname = "xoswordpointer_set_bbox")]
  public static OS.Error* XOSWordPointer_SetBBox
    (OSWordPointer_BBoxBlock bbox);

  [CCode (cname = "oswordpointer_set_bbox")]
  public static void OSWordPointer_SetBBox
    (OSWordPointer_BBoxBlock bbox);

  [CCode (cname = "xoswordpointer_step")]
  public static OS.Error* XOSWordPointer_Step
    (OSWordPointer_StepBlock step);

  [CCode (cname = "oswordpointer_step")]
  public static void OSWordPointer_Step
    (OSWordPointer_StepBlock step);

  [CCode (cname = "xoswordpointer_set_position")]
  public static OS.Error* XOSWordPointer_SetPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "oswordpointer_set_position")]
  public static void OSWordPointer_SetPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "xoswordpointer_read_position")]
  public static OS.Error* XOSWordPointer_ReadPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "oswordpointer_read_position")]
  public static void OSWordPointer_ReadPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "xoswordpointer_set_unlinked_position")]
  public static OS.Error* XOSWordPointer_SetUnlinkedPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "oswordpointer_set_unlinked_position")]
  public static void OSWordPointer_SetUnlinkedPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "xoswordpointer_read_unlinked_position")]
  public static OS.Error* XOSWordPointer_ReadUnlinkedPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "oswordpointer_read_unlinked_position")]
  public static void OSWordPointer_ReadUnlinkedPosition
    (OSWordPointer_PositionBlock pos);

  [CCode (cname = "xoswordpointer_read_bbox")]
  public static OS.Error* XOSWordPointer_ReadBBox
    (OSWordPointer_BBoxBlock bbox);

  [CCode (cname = "oswordpointer_read_bbox")]
  public static void OSWordPointer_ReadBBox
    (OSWordPointer_BBoxBlock bbox);

  [CCode (cname = "xosword_write_screen_base")]
  public static OS.Error* XWriteScreenBase
    (ScreenBaseBlock screen_base);

  [CCode (cname = "osword_write_screen_base")]
  public static void WriteScreenBase
    (ScreenBaseBlock screen_base);

}
