/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/podule.h")]
namespace OSLib.Podule
{
  [CCode (cname = "podule_section")]
  public enum Section
  {
    [CCode (cname = "podule_SECTION_SYSTEM_ROM")]
    SystemROM,
    [CCode (cname = "podule_SECTION_EXPANSION_CARD0")]
    ExpansionCard0,
    [CCode (cname = "podule_SECTION_EXPANSION_CARD1")]
    ExpansionCard1,
    [CCode (cname = "podule_SECTION_EXPANSION_CARD2")]
    ExpansionCard2,
    [CCode (cname = "podule_SECTION_EXPANSION_CARD3")]
    ExpansionCard3,
    [CCode (cname = "podule_SECTION_EXTENSION_ROM1")]
    ExtensionROM1
  }

  [CCode (cname = "podule_header")]
  public struct Header
  {
    [CCode (cname = "b")]
    public unowned byte B[16];
  }

  [CCode (cname="error_PODULE_BAD_POD")]
  public const bits Error_PoduleBadPod;
  [CCode (cname="error_PODULE_BAD_SPEED")]
  public const bits Error_PoduleBadSpeed;
  [CCode (cname="error_PODULE_NO_POD")]
  public const bits Error_PoduleNoPod;
  [CCode (cname="error_PODULE_NOT_EXT")]
  public const bits Error_PoduleNotExt;
  [CCode (cname="error_PODULE_NOT_ACRN")]
  public const bits Error_PoduleNotAcrn;
  [CCode (cname="error_PODULE_NO_LDR")]
  public const bits Error_PoduleNoLdr;
  [CCode (cname="error_PODULE_IN_LDR")]
  public const bits Error_PoduleInLdr;
  [CCode (cname="error_PODULE_BAD_CHNK")]
  public const bits Error_PoduleBadChnk;
  [CCode (cname="error_PODULE_BAD_DEVICE_NUMBER")]
  public const bits Error_PoduleBadDeviceNumber;
  [CCode (cname="error_PODULE_BAD_DEVICE_TYPE")]
  public const bits Error_PoduleBadDeviceType;
  [CCode (cname="error_PODULE_RAM_CONFIG_SYNTAX")]
  public const bits Error_PoduleRAMConfigSyntax;
  [CCode (cname="error_PODULE_WRONG_PODULE_TYPE")]
  public const bits Error_PoduleWrongPoduleType;
  [CCode (cname="error_PODULE_ROM_BOARD_SYNTAX")]
  public const bits Error_PoduleROMBoardSyntax;
  [CCode (cname="error_PODULE_BAD_READ")]
  public const bits Error_PoduleBadRead;
  [CCode (cname="error_PODULE_BD_SPEED")]
  public const bits Error_PoduleBdSpeed;
  [CCode (cname="error_PODULE_NOT_EASI")]
  public const bits Error_PoduleNotEASI;
  [CCode (cname="error_PODULE_SPEED_NO")]
  public const bits Error_PoduleSpeedNo;
  [CCode (cname="error_PODULE_EC_NO_SPD")]
  public const bits Error_PoduleECNoSpd;
  [CCode (cname="error_PODULE_EC_NO_NET")]
  public const bits Error_PoduleECNoNet;
  [CCode (cname="error_PODULE_NDALLAS")]
  public const bits Error_PoduleNDallas;
  [CCode (cname="error_PODULE_NOT_WRITABLE")]
  public const bits Error_PoduleNotWritable;
  [CCode (cname="error_PODULE_ADDRESS_RANGE")]
  public const bits Error_PoduleAddressRange;
  [CCode (cname="error_PODULE_DEVICE_NOT_WRITABLE")]
  public const bits Error_PoduleDeviceNotWritable;
  [CCode (cname="error_PODULE_PODULE_READ_ONLY")]
  public const bits Error_PodulePoduleReadOnly;
  [CCode (cname="error_PODULE_ADDRESS_TOO_BIG")]
  public const bits Error_PoduleAddressTooBig;

  [CCode (cname = "xpodule_read_id")]
  public static OS.Error* XReadID
    (Section section,
     out int ec_id);

  [CCode (cname = "podule_read_id")]
  public static int ReadID
    (Section section);

  [CCode (cname = "xpodule_read_header")]
  public static OS.Error* XReadHeader
    (Header header,
     Section section);

  [CCode (cname = "podule_read_header")]
  public static void ReadHeader
    (Header header,
     Section section);

  [CCode (cname = "xpodule_enumerate_chunks")]
  public static OS.Error* XEnumerateChunks
    (int context,
     Section section,
     out int context_out,
     out int size,
     out byte os_id,
     out string module_name);

  [CCode (cname = "podule_enumerate_chunks")]
  public static int EnumerateChunks
    (int context,
     Section section,
     out int size,
     out byte os_id,
     out string module_name);

  [CCode (cname = "xpodule_read_chunk")]
  public static OS.Error* XReadChunk
    (int chunk_no,
     char* buffer,
     Section section);

  [CCode (cname = "podule_read_chunk")]
  public static void ReadChunk
    (int chunk_no,
     char* buffer,
     Section section);

  [CCode (cname = "xpodule_read_bytes")]
  public static OS.Error* XReadBytes
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "podule_read_bytes")]
  public static void ReadBytes
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "xpodule_write_bytes")]
  public static OS.Error* XWriteBytes
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "podule_write_bytes")]
  public static void WriteBytes
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "xpodule_call_loader")]
  public static OS.Error* XCallLoader
    (int arg0,
     int arg1,
     int arg2,
     int slot_no,
     out int arg0_out,
     out int arg1_out,
     out int arg2_out);

  [CCode (cname = "podule_call_loader")]
  public static void CallLoader
    (int arg0,
     int arg1,
     int arg2,
     int slot_no,
     out int arg0_out,
     out int arg1_out,
     out int arg2_out);

  [CCode (cname = "xpodule_raw_read")]
  public static OS.Error* XRawRead
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "podule_raw_read")]
  public static void RawRead
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "xpodule_raw_write")]
  public static OS.Error* XRawWrite
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "podule_raw_write")]
  public static void RawWrite
    (int start_offset,
     int size,
     char* buffer,
     int slot_no);

  [CCode (cname = "xpodule_hardware_address")]
  public static OS.Error* XHardwareAddress
    (Section section,
     out int* combined_addr);

  [CCode (cname = "podule_hardware_address")]
  public static int* HardwareAddress
    (Section section);

  [CCode (cname = "xpodule_enumerate_chunks_with_info")]
  public static OS.Error* XEnumerateChunksWithInfo
    (int context,
     Section section,
     out int context_out,
     out int size,
     out byte os_id,
     out string module_name,
     out string help_text,
     out int* module_base);

  [CCode (cname = "podule_enumerate_chunks_with_info")]
  public static int EnumerateChunksWithInfo
    (int context,
     Section section,
     out int size,
     out byte os_id,
     out string module_name,
     out string help_text,
     out int* module_base);

  [CCode (cname = "xpodule_hardware_addresses")]
  public static OS.Error* XHardwareAddresses
    (Section section,
     out int* base_addr,
     out int* combined_addr);

  [CCode (cname = "podule_hardware_addresses")]
  public static void HardwareAddresses
    (Section section,
     out int* base_addr,
     out int* combined_addr);

  [CCode (cname = "xpodule_return_number")]
  public static OS.Error* XReturnNumber
    (out int expansion_card_count,
     out int extension_rom_count);

  [CCode (cname = "podule_return_number")]
  public static void ReturnNumber
    (out int expansion_card_count,
     out int extension_rom_count);

  [CCode (cname = "xpodule_read_info")]
  public static OS.Error* XReadInfo
    (bits mask,
     char* buffer,
     int size,
     Section section,
     out int used);

  [CCode (cname = "podule_read_info")]
  public static int ReadInfo
    (bits mask,
     char* buffer,
     int size,
     Section section);

  [CCode (cname = "xpodule_set_speed")]
  public static OS.Error* XSetSpeed
    (int speed,
     Section section,
     out int old_speed);

  [CCode (cname = "podule_set_speed")]
  public static void SetSpeed
    (int speed,
     Section section,
     out int old_speed);

  [CCode (cname = "xservice_pre_reset")]
  public static OS.Error* XService_PreReset ();

  [CCode (cname = "service_pre_reset")]
  public static void Service_PreReset ();

  [CCode (cname = "xservice_adfs_podule")]
  public static OS.Error* XService_ADFSPodule
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     out bool unclaimed,
     out char* controller_out,
     out int* status_location_out,
     out bits status_bits_out,
     out int* irq_location_out,
     out bits irq_bits_out);

  [CCode (cname = "service_adfs_podule")]
  public static bool Service_ADFSPodule
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     out char* controller_out,
     out int* status_location_out,
     out bits status_bits_out,
     out int* irq_location_out,
     out bits irq_bits_out);

  [CCode (cname = "xservice_adfs_podule_ide")]
  public static OS.Error* XService_ADFSPoduleIDE
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     AsmRoutine read_code,
     AsmRoutine write_code,
     out bool unclaimed,
     out char* controller_out,
     out int* status_location_out,
     out bits status_bits_out,
     out int* irq_location_out,
     out bits irq_bits_out,
     out AsmRoutine read_code_out,
     out AsmRoutine write_code_out);

  [CCode (cname = "service_adfs_podule_ide")]
  public static bool Service_ADFSPoduleIDE
    (char* controller,
     int* status_location,
     bits status_bits,
     int* irq_location,
     bits irq_bits,
     AsmRoutine read_code,
     AsmRoutine write_code,
     out char* controller_out,
     out int* status_location_out,
     out bits status_bits_out,
     out int* irq_location_out,
     out bits irq_bits_out,
     out AsmRoutine read_code_out,
     out AsmRoutine write_code_out);

  [CCode (cname = "xservice_adfs_podule_ide_dying")]
  public static OS.Error* XService_ADFSPoduleIDEDying ();

  [CCode (cname = "service_adfs_podule_ide_dying")]
  public static void Service_ADFSPoduleIDEDying ();

}
