/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;

[CCode (cheader_filename = "oslib/quit.h")]
namespace OSLib.Quit
{
  [CCode (cname = "quit_flags")]
  public enum Flags
  {
    [CCode (cname = "quit_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "quit_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted
  }

  [CCode (cname = "quit_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "message")]
    public unowned Toolbox.MsgReference Message;
    [CCode (cname = "message_limit")]
    public unowned int MessageLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "quit_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "message")]
    public unowned Toolbox.MsgReference Message;
    [CCode (cname = "message_limit")]
    public unowned int MessageLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "quit_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "quit_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "quit_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "quit_action_quit_block")]
  public enum ActionQuitBlock
  {
    Default = 0
  }

  [CCode (cname = "quit_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "quit_action_cancel_block")]
  public enum ActionCancelBlock
  {
    Default = 0
  }

  [CCode (cname="class_QUIT")]
  public const Toolbox.Class Class_Quit;
  [CCode (cname="action_QUIT_ABOUT_TO_BE_SHOWN")]
  public const bits Action_QuitAboutToBeShown;
  [CCode (cname="action_QUIT_QUIT")]
  public const bits Action_QuitQuit;
  [CCode (cname="action_QUIT_DIALOGUE_COMPLETED")]
  public const bits Action_QuitDialogueCompleted;
  [CCode (cname="action_QUIT_CANCEL")]
  public const bits Action_QuitCancel;

  [CCode (cname = "xquit_get_window_id")]
  public static OS.Error* XGetWindowID
    (bits flags,
     Toolbox.O quit,
     out Toolbox.O window);

  [CCode (cname = "quit_get_window_id")]
  public static Toolbox.O GetWindowID
    (bits flags,
     Toolbox.O quit);

  [CCode (cname = "xquit_set_message")]
  public static OS.Error* XSetMessage
    (bits flags,
     Toolbox.O quit,
     string message);

  [CCode (cname = "quit_set_message")]
  public static void SetMessage
    (bits flags,
     Toolbox.O quit,
     string message);

  [CCode (cname = "xquit_get_message")]
  public static OS.Error* XGetMessage
    (bits flags,
     Toolbox.O quit,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "quit_get_message")]
  public static int GetMessage
    (bits flags,
     Toolbox.O quit,
     string buffer,
     int size);

  [CCode (cname = "xquit_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O quit,
     string title);

  [CCode (cname = "quit_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O quit,
     string title);

  [CCode (cname = "xquit_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O quit,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "quit_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O quit,
     string buffer,
     int size);

}
