/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/resourcefs.h")]
namespace OSLib.ResourceFS
{
  [CCode (cname = "resourcefs_file_header_base")]
  public struct FileHeaderBase
  {
    [CCode (cname = "data_size")]
    public unowned int DataSize;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
  }

  [Compact]
  [CCode (cname = "resourcefs_file_header", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class FileHeader
  {
    public FileHeader(int numEntries);
    [CCode (cname = "data_size")]
    public unowned int DataSize;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "name", array_length = false)]
    public unowned char[] Name;
  }

  [CCode (cname = "resourcefs_file_data_base")]
  public struct FileDataBase
  {
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "resourcefs_file_data", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class FileData
  {
    public FileData(int numEntries);
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "data", array_length = false)]
    public unowned byte[] Data;
  }

  [CCode (cname = "resourcefs_file")]
  public struct File
  {
    [CCode (cname = "header")]
    public unowned FileHeader Header;
    [CCode (cname = "data")]
    public unowned FileData Data;
  }

  [CCode (cname = "resourcefs_file_list")]
  public struct FileList
  {
    [CCode (cname = "file", array_length = false)]
    public unowned File[] File;
  }


  [CCode (cname = "xresourcefs_register_files")]
  public static OS.Error* XRegisterFiles
    (FileList file_list);

  [CCode (cname = "resourcefs_register_files")]
  public static void RegisterFiles
    (FileList file_list);

  [CCode (cname = "xresourcefs_deregister_files")]
  public static OS.Error* XDeregisterFiles
    (FileList file_list);

  [CCode (cname = "resourcefs_deregister_files")]
  public static void DeregisterFiles
    (FileList file_list);

  [CCode (cname = "xservice_resource_fs_started")]
  public static OS.Error* XService_ResourceFSStarted ();

  [CCode (cname = "service_resource_fs_started")]
  public static void Service_ResourceFSStarted ();

  [CCode (cname = "xservice_resource_fs_dying")]
  public static OS.Error* XService_ResourceFSDying ();

  [CCode (cname = "service_resource_fs_dying")]
  public static void Service_ResourceFSDying ();

  [CCode (cname = "xservice_resource_fs_starting")]
  public static OS.Error* XService_ResourceFSStarting
    (AsmRoutine register_files,
     char* workspace);

  [CCode (cname = "service_resource_fs_starting")]
  public static void Service_ResourceFSStarting
    (AsmRoutine register_files,
     char* workspace);

}
