/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileCore;

[CCode (cheader_filename = "oslib/scsifs.h")]
namespace OSLib.SCSIFS
{
  [CCode (cname = "scsifs_drive_status")]
  public enum DriveStatus
  {
    [CCode (cname = "scsifs_DRIVE_ABSENT")]
    DriveAbsent,
    [CCode (cname = "scsifs_DRIVE_UNREADY")]
    DriveUnready,
    [CCode (cname = "scsifs_DRIVE_READY")]
    DriveReady
  }


  [CCode (cname = "xscsifsdiscop_verify")]
  public static OS.Error* XSCSIFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "scsifsdiscop_verify")]
  public static int SCSIFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr);

  [CCode (cname = "xscsifsdiscop_read_sectors")]
  public static OS.Error* XSCSIFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "scsifsdiscop_read_sectors")]
  public static int SCSIFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xscsifsdiscop_write_sectors")]
  public static OS.Error* XSCSIFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "scsifsdiscop_write_sectors")]
  public static int SCSIFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xscsifsdiscop_read_track")]
  public static OS.Error* XSCSIFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "scsifsdiscop_read_track")]
  public static void SCSIFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xscsifsdiscop_read_id")]
  public static OS.Error* XSCSIFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "scsifsdiscop_read_id")]
  public static void SCSIFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xscsifsdiscop_write_track")]
  public static OS.Error* XSCSIFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "scsifsdiscop_write_track")]
  public static void SCSIFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xscsifsdiscop_format_track")]
  public static OS.Error* XSCSIFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "scsifsdiscop_format_track")]
  public static void SCSIFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat format);

  [CCode (cname = "xscsifsdiscop_seek")]
  public static OS.Error* XSCSIFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "scsifsdiscop_seek")]
  public static void SCSIFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xscsifsdiscop_restore")]
  public static OS.Error* XSCSIFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "scsifsdiscop_restore")]
  public static void SCSIFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xscsifsdiscop_read_sectors_via_cache")]
  public static OS.Error* XSCSIFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "scsifsdiscop_read_sectors_via_cache")]
  public static int SCSIFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xscsifsdiscop_specify")]
  public static OS.Error* XSCSIFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "scsifsdiscop_specify")]
  public static void SCSIFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xscsifs_drives")]
  public static OS.Error* XDrives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "scsifs_drives")]
  public static void Drives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "xscsifs_free_space")]
  public static OS.Error* XFreeSpace
    (string disc_name,
     out int free,
     out int max);

  [CCode (cname = "scsifs_free_space")]
  public static int FreeSpace
    (string disc_name,
     out int max);

  [CCode (cname = "xscsifs_describe_disc")]
  public static OS.Error* XDescribeDisc
    (string disc_name,
     FileCore.Disc disc);

  [CCode (cname = "scsifs_describe_disc")]
  public static void DescribeDisc
    (string disc_name,
     FileCore.Disc disc);

  [CCode (cname = "xscsifs_test_ready")]
  public static OS.Error* XTestReady
    (int drive_no,
     out DriveStatus status);

  [CCode (cname = "scsifs_test_ready")]
  public static DriveStatus TestReady
    (int drive_no);

  [CCode (cname = "xscsifs_test_mounted")]
  public static OS.Error* XTestMounted
    (out bits flags);

  [CCode (cname = "scsifs_test_mounted")]
  public static bits TestMounted ();

  [CCode (cname = "xscsifs_free_space64")]
  public static OS.Error* XFreeSpace64
    (string disc_name,
     out bits free_lo,
     out int free_hi,
     out int max);

  [CCode (cname = "scsifs_free_space64")]
  public static void FreeSpace64
    (string disc_name,
     out bits free_lo,
     out int free_hi,
     out int max);

}
