/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/scrolllist.h")]
namespace OSLib.ScrollList
{
  [CCode (cname = "scrolllist_gadget")]
  public struct Gadget
  {
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "foreground")]
    public unowned OS.Colour Foreground;
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
  }

  [CCode (cname = "scrolllist_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "foreground")]
    public unowned OS.Colour Foreground;
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
  }

  [CCode (cname = "scrolllist_selection")]
  public struct Selection
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "item")]
    public unowned int Item;
  }

  [CCode (cname = "scrolllist_action_selection")]
  public struct ActionSelection
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "item")]
    public unowned int Item;
  }

  [CCode (cname = "scrolllist_action_selection_block")]
  public struct ActionSelectionBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "objectflags")]
    public unowned bits Objectflags;
    [CCode (cname = "item")]
    public unowned int Item;
  }

  [CCode (cname = "scrolllist_state")]
  public enum State
  {
    [CCode (cname = "scrolllist_STATE_ALLOW_MULTIPLE_SELECTIONS")]
    AllowMultipleSelections
  }

  [CCode (cname = "scrolllist_add_item_flags")]
  public enum AddItemFlags
  {
    [CCode (cname = "scrolllist_ADD_ITEM_SPRITE_INFO")]
    AddItemSpriteInfo,
    [CCode (cname = "scrolllist_ADD_ITEM_PLOT_SPRITE_HALF_SIZE")]
    AddItemPlotSpriteHalfSize
  }

  [CCode (cname = "scrolllist_select_item_flags")]
  public enum SelectItemFlags
  {
    [CCode (cname = "scrolllist_SELECT_ITEM_UNIQUE")]
    SelectItemUnique
  }

  [CCode (cname = "scrolllist_set_colour_flags")]
  public enum SetColourFlags
  {
    [CCode (cname = "scrolllist_SET_COLOUR_DESKTOP")]
    SetColourDesktop
  }

  [CCode (cname = "scrolllist_colour")]
  public struct Colour
  {
    [CCode (cname = "real")]
    public unowned OS.Colour Real;
    [CCode (cname = "desktop")]
    public unowned Wimp.Colour Desktop;
  }

  [CCode (cname="class_SCROLL_LIST")]
  public const Toolbox.Class Class_ScrollList;
  [CCode (cname="action_SCROLL_LIST_SELECTION")]
  public const bits Action_ScrollListSelection;
  [CCode (cname="scrolllist_ADD_ITEM_SPRITE_INFO_SHIFT")]
  public const int AddItemSpriteInfoShift;
  [CCode (cname="scrolllist_ADD_ITEM_SPRITE_INFO_SYSTEM_SPRITE_NAME")]
  public const int AddItemSpriteInfoSystemSpriteName;
  [CCode (cname="scrolllist_ADD_ITEM_SPRITE_INFO_USER_SPRITE_NAME")]
  public const int AddItemSpriteInfoUserSpriteName;
  [CCode (cname="scrolllist_ADD_ITEM_SPRITE_INFO_USER_SPRITE_POINTER")]
  public const int AddItemSpriteInfoUserSpritePointer;

  [CCode (cname = "xscrolllist_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     out State state);

  [CCode (cname = "scrolllist_get_state")]
  public static State GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list);

  [CCode (cname = "xscrolllist_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     State state);

  [CCode (cname = "scrolllist_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     State state);

  [CCode (cname = "xscrolllist_add_item")]
  public static OS.Error* XAddItem
    (AddItemFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string text,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     int index);

  [CCode (cname = "scrolllist_add_item")]
  public static void AddItem
    (AddItemFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string text,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     int index);

  [CCode (cname = "xscrolllist_delete_items")]
  public static OS.Error* XDeleteItems
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int start_index,
     int end_index);

  [CCode (cname = "scrolllist_delete_items")]
  public static void DeleteItems
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int start_index,
     int end_index);

  [CCode (cname = "xscrolllist_select_item")]
  public static OS.Error* XSelectItem
    (SelectItemFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "scrolllist_select_item")]
  public static void SelectItem
    (SelectItemFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "xscrolllist_deselect_item")]
  public static OS.Error* XDeselectItem
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "scrolllist_deselect_item")]
  public static void DeselectItem
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "xscrolllist_get_selected")]
  public static OS.Error* XGetSelected
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int offset,
     out int new_offset);

  [CCode (cname = "scrolllist_get_selected")]
  public static int GetSelected
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int offset);

  [CCode (cname = "xscrolllist_make_visible")]
  public static OS.Error* XMakeVisible
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "scrolllist_make_visible")]
  public static void MakeVisible
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     int index);

  [CCode (cname = "xscrolllist_set_colour")]
  public static OS.Error* XSetColour
    (SetColourFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     Colour foreground,
     Colour background);

  [CCode (cname = "scrolllist_set_colour")]
  public static void SetColour
    (SetColourFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     Colour foreground,
     Colour background);

  [CCode (cname = "xscrolllist_get_colour")]
  public static OS.Error* XGetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     out OS.Colour foreground,
     out OS.Colour background);

  [CCode (cname = "scrolllist_get_colour")]
  public static void GetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     out OS.Colour foreground,
     out OS.Colour background);

  [CCode (cname = "xscrolllist_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string font_name,
     int width,
     int height);

  [CCode (cname = "scrolllist_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string font_name,
     int width,
     int height);

  [CCode (cname = "xscrolllist_get_item_text")]
  public static OS.Error* XGetItemText
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string buffer,
     int buffer_size,
     int index,
     out int size);

  [CCode (cname = "scrolllist_get_item_text")]
  public static int GetItemText
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string buffer,
     int buffer_size,
     int index);

  [CCode (cname = "xscrolllist_count_items")]
  public static OS.Error* XCountItems
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     out int items);

  [CCode (cname = "scrolllist_count_items")]
  public static int CountItems
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list);

  [CCode (cname = "xscrolllist_set_item_text")]
  public static OS.Error* XSetItemText
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string text,
     int index);

  [CCode (cname = "scrolllist_set_item_text")]
  public static void SetItemText
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_list,
     string text,
     int index);

}
