/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Territory;

[CCode (cheader_filename = "oslib/serviceinternational.h")]
namespace OSLib.ServiceInternational
{
  [CCode (cname="error_UNKNOWN_ALPHABET")]
  public const bits Error_UnknownAlphabet;
  [CCode (cname="error_UNKNOWN_COUNTRY")]
  public const bits Error_UnknownCountry;
  [CCode (cname="error_UNKNOWN_KEYBOARD")]
  public const bits Error_UnknownKeyboard;

  [CCode (cname = "xserviceinternational_country_name_to_country_number")]
  public static OS.Error* XCountryNameToCountryNumber
    (string country_name,
     out bool unclaimed,
     out Territory.T country_no);

  [CCode (cname = "serviceinternational_country_name_to_country_number")]
  public static bool CountryNameToCountryNumber
    (string country_name,
     out Territory.T country_no);

  [CCode (cname = "xserviceinternational_alphabet_name_to_alphabet_number")]
  public static OS.Error* XAlphabetNameToAlphabetNumber
    (string alphabet_name,
     out bool unclaimed,
     out Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "serviceinternational_alphabet_name_to_alphabet_number")]
  public static bool AlphabetNameToAlphabetNumber
    (string alphabet_name,
     out Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "xserviceinternational_country_number_to_country_name")]
  public static OS.Error* XCountryNumberToCountryName
    (Territory.T country_no,
     string buffer,
     int size,
     out bool unclaimed,
     out int used);

  [CCode (cname = "serviceinternational_country_number_to_country_name")]
  public static bool CountryNumberToCountryName
    (Territory.T country_no,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "xserviceinternational_alphabet_number_to_alphabet_name")]
  public static OS.Error* XAlphabetNumberToAlphabetName
    (Territory.AlphabetNumber alphabet_no,
     string buffer,
     int size,
     out bool unclaimed,
     out int used);

  [CCode (cname = "serviceinternational_alphabet_number_to_alphabet_name")]
  public static bool AlphabetNumberToAlphabetName
    (Territory.AlphabetNumber alphabet_no,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "xserviceinternational_country_number_to_alphabet_number")]
  public static OS.Error* XCountryNumberToAlphabetNumber
    (Territory.T country_no,
     out bool unclaimed,
     out Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "serviceinternational_country_number_to_alphabet_number")]
  public static bool CountryNumberToAlphabetNumber
    (Territory.T country_no,
     out Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "xserviceinternational_define_chars")]
  public static OS.Error* XDefineChars
    (Territory.AlphabetNumber alphabet_no,
     byte first,
     byte last,
     out bool unclaimed);

  [CCode (cname = "serviceinternational_define_chars")]
  public static bool DefineChars
    (Territory.AlphabetNumber alphabet_no,
     byte first,
     byte last);

  [CCode (cname = "xserviceinternational_new_keyboard")]
  public static OS.Error* XNewKeyboard
    (int keyboard_no,
     Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "serviceinternational_new_keyboard")]
  public static void NewKeyboard
    (int keyboard_no,
     Territory.AlphabetNumber alphabet_no);

  [CCode (cname = "xserviceinternational_define_ucs_character")]
  public static OS.Error* XDefineUCSCharacter
    (int char_index,
     int ucs_charcode,
     out bool unclaimed);

  [CCode (cname = "serviceinternational_define_ucs_character")]
  public static bool DefineUCSCharacter
    (int char_index,
     int ucs_charcode);

  [CCode (cname = "xserviceinternational_get_ucs_conversion_table")]
  public static OS.Error* XGetUCSConversionTable
    (int alphabet_no,
     out bool unclaimed,
     out char* char_table);

  [CCode (cname = "serviceinternational_get_ucs_conversion_table")]
  public static bool GetUCSConversionTable
    (int alphabet_no,
     out char* char_table);

}
