/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Freeway;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/sharefs.h")]
namespace OSLib.ShareFS
{
  [CCode (cname = "sharefs_attr")]
  public enum Attr
  {
    [CCode (cname = "sharefs_ATTR_DISC")]
    Disc,
    [CCode (cname = "sharefs_ATTR_DISCP")]
    DiscP,
    [CCode (cname = "sharefs_ATTR_DISC_RO")]
    DiscRO,
    [CCode (cname = "sharefs_ATTR_HIDDEN")]
    Hidden,
    [CCode (cname = "sharefs_ATTR_SUBDIR_AUTH")]
    SubdirAuth,
    [CCode (cname = "sharefs_ATTR_CDROM")]
    CDROM,
    [CCode (cname = "sharefs_ATTR_CDROM_AUTH")]
    CDROMAuth
  }

  [CCode (cname = "sharefs_share_flags")]
  public enum ShareFlags
  {
    [CCode (cname = "sharefs_SHARE_DISC")]
    ShareDisc,
    [CCode (cname = "sharefs_SHARE_DISCP")]
    ShareDiscP,
    [CCode (cname = "sharefs_SHARE_DISC_RO")]
    ShareDiscRO,
    [CCode (cname = "sharefs_SHARE_HIDDEN")]
    ShareHidden,
    [CCode (cname = "sharefs_SHARE_SUBDIR_AUTH")]
    ShareSubdirAuth,
    [CCode (cname = "sharefs_SHARE_CDROM")]
    ShareCDROM,
    [CCode (cname = "sharefs_SHARE_CDROM_AUTH")]
    ShareCDROMAuth
  }

  [CCode (cname = "sharefs_stop_flags")]
  public enum StopFlags
  {
    Default = 0
  }

  [CCode (cname = "sharefs_enumerate_flags")]
  public enum EnumerateFlags
  {
    [CCode (cname = "sharefs_ENUMERATE_PROTECTED")]
    EnumerateProtected,
    [CCode (cname = "sharefs_ENUMERATE_READ_ONLY")]
    EnumerateReadOnly,
    [CCode (cname = "sharefs_ENUMERATE_HIDDEN")]
    EnumerateHidden,
    [CCode (cname = "sharefs_ENUMERATE_SUBDIR")]
    EnumerateSubdir,
    [CCode (cname = "sharefs_ENUMERATE_CDROM")]
    EnumerateCDROM,
    [CCode (cname = "sharefs_ENUMERATE_AUTHENTICATED")]
    EnumerateAuthenticated
  }

  [CCode (cname = "sharefs_identify_share_flags")]
  public enum IdentifyShareFlags
  {
    [CCode (cname = "sharefs_IDENTIFY_SHARE_SHARE_NAME")]
    IdentifyShareShareName
  }

  [CCode (cname = "sharefsmessagefilesharedir_flags")]
  public enum ShareFSMessageFileShareDir_Flags
  {
    [CCode (cname = "sharefsmessagefilesharedirflags_PROTECTED")]
    Protected,
    [CCode (cname = "sharefsmessagefilesharedirflags_READ_ONLY")]
    ReadOnly,
    [CCode (cname = "sharefsmessagefilesharedirflags_HIDDEN")]
    Hidden,
    [CCode (cname = "sharefsmessagefilesharedirflags_AUTHENTICATED")]
    Authenticated,
    [CCode (cname = "sharefsmessagefilesharedirflags_CDROM")]
    CDROM,
    [CCode (cname = "sharefsmessagefilesharedirflags_WINDOW_POS_GIVEN")]
    WindowPosGiven
  }

  [CCode (cname = "sharefs_message_file_share_dir")]
  public struct MessageFileShareDir
  {
    [CCode (cname = "flags")]
    public unowned ShareFSMessageFileShareDir_Flags Flags;
    [CCode (cname = "xpos")]
    public unowned int Xpos;
    [CCode (cname = "ypos")]
    public unowned int Ypos;
    [CCode (cname = "dir_name")]
    public unowned char DirName[224];
  }

  [CCode (cname = "sharefs_starting_flags")]
  public enum StartingFlags
  {
    Default = 0
  }

  [CCode (cname = "sharefs_terminating_flags")]
  public enum TerminatingFlags
  {
    Default = 0
  }

  [CCode (cname="sharefs_FILE_TYPE_DISCP")]
  public const bits FileTypeDiscP;
  [CCode (cname="sharefs_FILE_TYPE_DISC")]
  public const bits FileTypeDisc;
  [CCode (cname="sharefs_FILE_TYPE_SUBDIR")]
  public const bits FileTypeSubdir;
  [CCode (cname="sharefs_FILE_TYPE_CDROM")]
  public const bits FileTypeCDROM;
  [CCode (cname="sharefs_FILE_TYPE_DISCR")]
  public const bits FileTypeDiscR;
  [CCode (cname="sharefs_FILE_TYPE_NO_DISC")]
  public const bits FileTypeNoDisc;
  [CCode (cname="sharefs_NO_MORE")]
  public const int NoMore;
  [CCode (cname="message_FILE_SHARE_DIR")]
  public const bits Message_FileShareDir;

  [CCode (cname = "xsharefs_create_share")]
  public static OS.Error* XCreateShare
    (ShareFlags flags,
     string obj_name,
     string dir_path,
     int authentication);

  [CCode (cname = "sharefs_create_share")]
  public static void CreateShare
    (ShareFlags flags,
     string obj_name,
     string dir_path,
     int authentication);

  [CCode (cname = "xsharefs_stop_share")]
  public static OS.Error* XStopShare
    (StopFlags flags,
     string obj_name);

  [CCode (cname = "sharefs_stop_share")]
  public static void StopShare
    (StopFlags flags,
     string obj_name);

  [CCode (cname = "xsharefs_enumerate_shares")]
  public static OS.Error* XEnumerateShares
    (EnumerateFlags flags,
     int context,
     int authentication,
     out string obj_name,
     out string dir_path,
     out Attr attr,
     out int context_out);

  [CCode (cname = "sharefs_enumerate_shares")]
  public static int EnumerateShares
    (EnumerateFlags flags,
     int context,
     int authentication,
     out string obj_name,
     out string dir_path,
     out Attr attr);

  [CCode (cname = "xsharefs_identify_share")]
  public static OS.Error* XIdentifyShare
    (IdentifyShareFlags flags,
     string obj_name_or_dir_path,
     char* buf,
     int buf_size,
     out Attr attr,
     out int buf_size_out);

  [CCode (cname = "sharefs_identify_share")]
  public static Attr IdentifyShare
    (IdentifyShareFlags flags,
     string obj_name_or_dir_path,
     char* buf,
     int buf_size,
     out int buf_size_out);

  [CCode (cname = "xservice_share_fs_starting")]
  public static OS.Error* XService_ShareFSStarting
    (StartingFlags flags);

  [CCode (cname = "service_share_fs_starting")]
  public static void Service_ShareFSStarting
    (StartingFlags flags);

  [CCode (cname = "xservice_share_fs_terminating")]
  public static OS.Error* XService_ShareFSTerminating
    (TerminatingFlags flags);

  [CCode (cname = "service_share_fs_terminating")]
  public static void Service_ShareFSTerminating
    (TerminatingFlags flags);

  [CCode (cname = "xservice_sharing")]
  public static OS.Error* XService_Sharing
    (string fsname,
     int state,
     string dir_name,
     string obj_name,
     int priv_data);

  [CCode (cname = "service_sharing")]
  public static void Service_Sharing
    (string fsname,
     int state,
     string dir_name,
     string obj_name,
     int priv_data);

}
