/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/sound.h")]
namespace OSLib.Sound
{
  [CCode (cname = "sound_log_table")]
  public struct LogTable
  {
    [CCode (cname = "log")]
    public unowned byte Log[8192];
  }

  [CCode (cname = "sound_log_scale_table")]
  public struct LogScaleTable
  {
    [CCode (cname = "log_scale")]
    public unowned byte LogScale[256];
  }

  [CCode (cname = "sound_channel_handler")]
  public struct ChannelHandler
  {
    [CCode (cname = "fill_code")]
    public unowned AsmRoutine FillCode;
    [CCode (cname = "overrun_fixup_code")]
    public unowned AsmRoutine OverrunFixupCode;
    [CCode (cname = "log_table")]
    public unowned LogTable* LogTable;
    [CCode (cname = "log_scale_table")]
    public unowned LogScaleTable* LogScaleTable;
  }

  [CCode (cname = "sound_sccb")]
  public struct SCCB
  {
    [CCode (cname = "amplitude")]
    public unowned byte Amplitude;
    [CCode (cname = "voice_no")]
    public unowned byte VoiceNo;
    [CCode (cname = "voice_instance")]
    public unowned byte VoiceInstance;
    [CCode (cname = "flags")]
    public unowned byte Flags;
    [CCode (cname = "pitch")]
    public unowned int Pitch;
    [CCode (cname = "timbre")]
    public unowned int Timbre;
    [CCode (cname = "buffer_fill_count")]
    public unowned int BufferFillCount;
    [CCode (cname = "r4")]
    public unowned int R4;
    [CCode (cname = "r5")]
    public unowned int R5;
    [CCode (cname = "r6")]
    public unowned int R6;
    [CCode (cname = "r7")]
    public unowned int R7;
    [CCode (cname = "r8")]
    public unowned int R8;
    [CCode (cname = "reserved")]
    public unowned int Reserved[7];
    [CCode (cname = "available")]
    public unowned int Available[48];
  }

  [CCode (cname = "sound_scheduler")]
  public struct Scheduler
  {
    [CCode (cname = "scheduler")]
    public unowned AsmRoutine Scheduler;
  }

  [CCode (cname = "sound_voice_generator")]
  public struct VoiceGenerator
  {
    [CCode (cname = "fill_code")]
    public unowned int FillCode;
    [CCode (cname = "update_code")]
    public unowned int UpdateCode;
    [CCode (cname = "gate_on_code")]
    public unowned int GateOnCode;
    [CCode (cname = "gate_off_code")]
    public unowned int GateOffCode;
    [CCode (cname = "instantiate_code")]
    public unowned int InstantiateCode;
    [CCode (cname = "free_code")]
    public unowned int FreeCode;
    [CCode (cname = "install_code")]
    public unowned int InstallCode;
    [CCode (cname = "voice_name_offset")]
    public unowned int VoiceNameOffset;
  }

  [CCode (cname = "sound_state")]
  public enum State
  {
    [CCode (cname = "sound_STATE_READ")]
    Read,
    [CCode (cname = "sound_STATE_OFF")]
    Off,
    [CCode (cname = "sound_STATE_ON")]
    On
  }

  [CCode (cname = "sound_configuration")]
  public enum Configuration
  {
    [CCode (cname = "sound_CONFIGURATION_FORMAT")]
    Format,
    [CCode (cname = "sound_CONFIGURATION_FORMAT16_BIT_ANY")]
    Format16BitAny,
    [CCode (cname = "sound_CONFIGURATION_FORMAT16_BIT_MU_LAW")]
    Format16BitMuLaw,
    [CCode (cname = "sound_CONFIGURATION_OVERSAMPLING_ENABLED")]
    OversamplingEnabled
  }

  [CCode (cname = "sound_linear_handler_flags")]
  public enum LinearHandlerFlags
  {
    [CCode (cname = "sound_DATA_FLAGS")]
    DataFlags,
    [CCode (cname = "sound_DATA_INVALID")]
    DataInvalid,
    [CCode (cname = "sound_DATA_NON_NOUGHT")]
    DataNonNought,
    [CCode (cname = "sound_DATA_NOUGHT")]
    DataNought,
    [CCode (cname = "sound_RESERVED_FLAGS")]
    ReservedFlags
  }

  [CCode (cname="sound_INSTALL_CODE")]
  public const int InstallCode;
  [CCode (cname="sound_HZ")]
  public const int Hz;

  [CCode (cname = "xsound_configure")]
  public static OS.Error* XConfigure
    (int channel_count,
     int sample_size,
     int sample_period,
     ChannelHandler channel_handler,
     Scheduler scheduler,
     out int channel_count_out,
     out int sample_size_out,
     out int sample_period_out,
     out ChannelHandler channel_handler_out,
     out Scheduler scheduler_out);

  [CCode (cname = "sound_configure")]
  public static void Configure
    (int channel_count,
     int sample_size,
     int sample_period,
     ChannelHandler channel_handler,
     Scheduler scheduler,
     out int channel_count_out,
     out int sample_size_out,
     out int sample_period_out,
     out ChannelHandler channel_handler_out,
     out Scheduler scheduler_out);

  [CCode (cname = "xsound_enable")]
  public static OS.Error* XEnable
    (State state,
     out State state_out);

  [CCode (cname = "sound_enable")]
  public static State Enable
    (State state);

  [CCode (cname = "xsound_stereo")]
  public static OS.Error* XStereo
    (int channel_no,
     int position,
     out int position_out);

  [CCode (cname = "sound_stereo")]
  public static int Stereo
    (int channel_no,
     int position);

  [CCode (cname = "xsound_speaker")]
  public static OS.Error* XSpeaker
    (State state,
     out State state_out);

  [CCode (cname = "sound_speaker")]
  public static State Speaker
    (State state);

  [CCode (cname = "xsoundmode_read_configuration")]
  public static OS.Error* XSoundMode_ReadConfiguration
    (out bool has_16bit,
     out Configuration configuration);

  [CCode (cname = "soundmode_read_configuration")]
  public static bool SoundMode_ReadConfiguration
    (out Configuration configuration);

  [CCode (cname = "xsoundmode_set_oversampling")]
  public static OS.Error* XSoundMode_SetOversampling
    (bool oversample,
     out bool old_oversample);

  [CCode (cname = "soundmode_set_oversampling")]
  public static bool SoundMode_SetOversampling
    (bool oversample);

  [CCode (cname = "xsound_linear_handler")]
  public static OS.Error* XLinearHandler
    (bool install,
     AsmRoutine new_handler_code,
     char* new_workspace,
     out AsmRoutine old_handler_code,
     out char* old_workspace);

  [CCode (cname = "sound_linear_handler")]
  public static void LinearHandler
    (bool install,
     AsmRoutine new_handler_code,
     char* new_workspace,
     out AsmRoutine old_handler_code,
     out char* old_workspace);

  [CCode (cname = "xsoundsamplerate_read_count")]
  public static OS.Error* XSoundSampleRate_ReadCount
    (out int rate_count);

  [CCode (cname = "soundsamplerate_read_count")]
  public static int SoundSampleRate_ReadCount ();

  [CCode (cname = "xsoundsamplerate_read_current")]
  public static OS.Error* XSoundSampleRate_ReadCurrent
    (out int rate_index,
     out int rate);

  [CCode (cname = "soundsamplerate_read_current")]
  public static int SoundSampleRate_ReadCurrent
    (out int rate);

  [CCode (cname = "xsoundsamplerate_lookup")]
  public static OS.Error* XSoundSampleRate_Lookup
    (int rate_index,
     out int rate);

  [CCode (cname = "soundsamplerate_lookup")]
  public static int SoundSampleRate_Lookup
    (int rate_index);

  [CCode (cname = "xsoundsamplerate_select")]
  public static OS.Error* XSoundSampleRate_Select
    (int rate_index,
     out int old_rate_index,
     out int old_rate);

  [CCode (cname = "soundsamplerate_select")]
  public static int SoundSampleRate_Select
    (int rate_index,
     out int old_rate);

  [CCode (cname = "xsound_volume")]
  public static OS.Error* XVolume
    (int volume,
     out int volume_out);

  [CCode (cname = "sound_volume")]
  public static int Volume
    (int volume);

  [CCode (cname = "xsound_sound_log")]
  public static OS.Error* XSoundLog
    (int linear,
     out int log);

  [CCode (cname = "sound_sound_log")]
  public static int SoundLog
    (int linear);

  [CCode (cname = "xsound_log_scale")]
  public static OS.Error* XLogScale
    (int log,
     out int linear);

  [CCode (cname = "sound_log_scale")]
  public static int LogScale
    (int log);

  [CCode (cname = "xsound_install_voice")]
  public static OS.Error* XInstallVoice
    (VoiceGenerator voice_generator,
     int voice_no,
     out string voice_name_out,
     out int voice_no_out);

  [CCode (cname = "sound_install_voice")]
  public static int InstallVoice
    (VoiceGenerator voice_generator,
     int voice_no,
     out string voice_name_out);

  [CCode (cname = "xsoundinstallvoice_read_name")]
  public static OS.Error* XSoundInstallVoice_ReadName
    (int voice_no,
     out string voice_name);

  [CCode (cname = "soundinstallvoice_read_name")]
  public static string SoundInstallVoice_ReadName
    (int voice_no);

  [CCode (cname = "xsoundinstallvoice_add_named_voice")]
  public static OS.Error* XSoundInstallVoice_AddNamedVoice
    (int voice_no,
     VoiceGenerator voice_generator,
     string voice_name,
     out int voice_no_out,
     out string voice_name_out);

  [CCode (cname = "soundinstallvoice_add_named_voice")]
  public static int SoundInstallVoice_AddNamedVoice
    (int voice_no,
     VoiceGenerator voice_generator,
     string voice_name,
     out string voice_name_out);

  [CCode (cname = "xsoundinstallvoice_read_local_name")]
  public static OS.Error* XSoundInstallVoice_ReadLocalName
    (int voice_no,
     out string voice_name_out,
     out string voice_local_name_out);

  [CCode (cname = "soundinstallvoice_read_local_name")]
  public static string SoundInstallVoice_ReadLocalName
    (int voice_no,
     out string voice_local_name_out);

  [CCode (cname = "xsoundinstallvoice_change_local_name")]
  public static OS.Error* XSoundInstallVoice_ChangeLocalName
    (int voice_no,
     string voice_local_name);

  [CCode (cname = "soundinstallvoice_change_local_name")]
  public static void SoundInstallVoice_ChangeLocalName
    (int voice_no,
     string voice_local_name);

  [CCode (cname = "xsound_remove_voice")]
  public static OS.Error* XRemoveVoice
    (int voice_no,
     out string voice_name_out,
     out int voice_no_out);

  [CCode (cname = "sound_remove_voice")]
  public static int RemoveVoice
    (int voice_no,
     out string voice_name_out);

  [CCode (cname = "xsound_attach_voice")]
  public static OS.Error* XAttachVoice
    (int channel_no,
     int voice_no,
     out int channel_no_out,
     out int voice_no_out);

  [CCode (cname = "sound_attach_voice")]
  public static void AttachVoice
    (int channel_no,
     int voice_no,
     out int channel_no_out,
     out int voice_no_out);

  [CCode (cname = "xsound_control_packed")]
  public static OS.Error* XControlPacked
    (bits amp_and_channel,
     bits duration_and_pitch);

  [CCode (cname = "sound_control_packed")]
  public static void ControlPacked
    (bits amp_and_channel,
     bits duration_and_pitch);

  [CCode (cname = "xsound_tuning")]
  public static OS.Error* XTuning
    (int tuning,
     out int tuning_out);

  [CCode (cname = "sound_tuning")]
  public static int Tuning
    (int tuning);

  [CCode (cname = "xsound_pitch")]
  public static OS.Error* XPitch
    (int pitch,
     out int accumulator);

  [CCode (cname = "sound_pitch")]
  public static int Pitch
    (int pitch);

  [CCode (cname = "xsound_control")]
  public static OS.Error* XControl
    (int channel_no,
     int amplitude,
     int pitch,
     int duration);

  [CCode (cname = "sound_control")]
  public static void Control
    (int channel_no,
     int amplitude,
     int pitch,
     int duration);

  [CCode (cname = "xsound_attach_named_voice")]
  public static OS.Error* XAttachNamedVoice
    (int channel_no,
     string voice_name);

  [CCode (cname = "sound_attach_named_voice")]
  public static void AttachNamedVoice
    (int channel_no,
     string voice_name);

  [CCode (cname = "xsound_read_control_block")]
  public static OS.Error* XReadControlBlock
    (int channel_no,
     int offset,
     out int channel_no_out,
     out int word);

  [CCode (cname = "sound_read_control_block")]
  public static void ReadControlBlock
    (int channel_no,
     int offset,
     out int channel_no_out,
     out int word);

  [CCode (cname = "xsound_write_control_block")]
  public static OS.Error* XWriteControlBlock
    (int channel_no,
     int offset,
     int word,
     out int channel_no_out,
     out int word_out);

  [CCode (cname = "sound_write_control_block")]
  public static void WriteControlBlock
    (int channel_no,
     int offset,
     int word,
     out int channel_no_out,
     out int word_out);

  [CCode (cname = "xsound_qinit")]
  public static OS.Error* XQInit ();

  [CCode (cname = "sound_qinit")]
  public static void QInit ();

  [CCode (cname = "xsound_qschedule")]
  public static OS.Error* XQSchedule
    (int period,
     int swi_no,
     int swi_r0,
     int swi_r1,
     out bool failed);

  [CCode (cname = "sound_qschedule")]
  public static bool QSchedule
    (int period,
     int swi_no,
     int swi_r0,
     int swi_r1);

  [CCode (cname = "xsound_qremove")]
  public static OS.Error* XQRemove ();

  [CCode (cname = "sound_qremove")]
  public static void QRemove ();

  [CCode (cname = "xsound_qfree")]
  public static OS.Error* XQFree
    (out int free_count);

  [CCode (cname = "sound_qfree")]
  public static int QFree ();

  [CCode (cname = "xsound_qtempo")]
  public static OS.Error* XQTempo
    (int tempo,
     out int tempo_out);

  [CCode (cname = "sound_qtempo")]
  public static int QTempo
    (int tempo);

  [CCode (cname = "xsound_qbeat")]
  public static OS.Error* XQBeat
    (int bar_length,
     out int bar_length_out);

  [CCode (cname = "sound_qbeat")]
  public static int QBeat
    (int bar_length);

  [CCode (cname = "xsoundqschedule32_call_control_packed")]
  public static OS.Error* XSoundQSchedule32_CallControlPacked
    (int period,
     int data1,
     int data2,
     out int error,
     out int api_check);

  [CCode (cname = "soundqschedule32_call_control_packed")]
  public static int SoundQSchedule32_CallControlPacked
    (int period,
     int data1,
     int data2,
     out int api_check);

  [CCode (cname = "xsoundqschedule32_call_routine")]
  public static OS.Error* XSoundQSchedule32_CallRoutine
    (int period,
     AsmRoutine routine,
     int data1,
     int data2,
     out int error,
     out int api_check);

  [CCode (cname = "soundqschedule32_call_routine")]
  public static int SoundQSchedule32_CallRoutine
    (int period,
     AsmRoutine routine,
     int data1,
     int data2,
     out int api_check);

  [CCode (cname = "xsoundqschedule32_call_swi")]
  public static OS.Error* XSoundQSchedule32_CallSWI
    (int period,
     int swi_number_shift_by_8,
     int data1,
     int data2,
     out int error,
     out int api_check);

  [CCode (cname = "soundqschedule32_call_swi")]
  public static int SoundQSchedule32_CallSWI
    (int period,
     int swi_number_shift_by_8,
     int data1,
     int data2,
     out int api_check);

  [CCode (cname = "xservice_sound")]
  public static OS.Error* XService_Sound
    (int part);

  [CCode (cname = "service_sound")]
  public static void Service_Sound
    (int part);

  [CCode (cname = "xevent_start_of_bar")]
  public static OS.Error* XEvent_StartOfBar ();

  [CCode (cname = "event_start_of_bar")]
  public static void Event_StartOfBar ();

}
