/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/territory.h")]
namespace OSLib.Territory
{
  [CCode (cname = "territory_t")]
  public enum T
  {
    [CCode (cname = "territory_CURRENT")]
    Current,
    [CCode (cname = "territory_UK")]
    UK,
    [CCode (cname = "territory_MASTER")]
    Master,
    [CCode (cname = "territory_COMPACT")]
    Compact,
    [CCode (cname = "territory_ITALY")]
    Italy,
    [CCode (cname = "territory_SPAIN")]
    Spain,
    [CCode (cname = "territory_FRANCE")]
    France,
    [CCode (cname = "territory_GERMANY")]
    Germany,
    [CCode (cname = "territory_PORTUGAL")]
    Portugal,
    [CCode (cname = "territory_ESPERANTO")]
    Esperanto,
    [CCode (cname = "territory_GREECE")]
    Greece,
    [CCode (cname = "territory_SWEDEN")]
    Sweden,
    [CCode (cname = "territory_FINLAND")]
    Finland,
    [CCode (cname = "territory_DENMARK")]
    Denmark,
    [CCode (cname = "territory_NORWAY")]
    Norway,
    [CCode (cname = "territory_ICELAND")]
    Iceland,
    [CCode (cname = "territory_CANADA1")]
    Canada1,
    [CCode (cname = "territory_CANADA2")]
    Canada2,
    [CCode (cname = "territory_CANADA")]
    Canada,
    [CCode (cname = "territory_TURKEY")]
    Turkey,
    [CCode (cname = "territory_ARABIC")]
    Arabic,
    [CCode (cname = "territory_IRELAND")]
    Ireland,
    [CCode (cname = "territory_HONG_KONG")]
    HongKong,
    [CCode (cname = "territory_RUSSIA")]
    Russia,
    [CCode (cname = "territory_RUSSIA2")]
    Russia2,
    [CCode (cname = "territory_ISRAEL")]
    Israel,
    [CCode (cname = "territory_MEXICO")]
    Mexico,
    [CCode (cname = "territory_LATIM_AM")]
    LatimAm,
    [CCode (cname = "territory_USA")]
    USA
  }

  [CCode (cname = "territory_collate_flags")]
  public enum CollateFlags
  {
    [CCode (cname = "territory_IGNORE_CASE")]
    IgnoreCase,
    [CCode (cname = "territory_IGNORE_ACCENT")]
    IgnoreAccent
  }

  [CCode (cname = "territory_alphabet_number")]
  public enum AlphabetNumber
  {
    [CCode (cname = "territory_ALPHABET_BFONT")]
    AlphabetBFont,
    [CCode (cname = "territory_ALPHABET_LATIN1")]
    AlphabetLatin1,
    [CCode (cname = "territory_ALPHABET_LATIN2")]
    AlphabetLatin2,
    [CCode (cname = "territory_ALPHABET_LATIN3")]
    AlphabetLatin3,
    [CCode (cname = "territory_ALPHABET_LATIN4")]
    AlphabetLatin4,
    [CCode (cname = "territory_ALPHABET_CYRILLIC")]
    AlphabetCyrillic,
    [CCode (cname = "territory_ALPHABET_ARABIC")]
    AlphabetArabic,
    [CCode (cname = "territory_ALPHABET_GREEK")]
    AlphabetGreek,
    [CCode (cname = "territory_ALPHABET_HEBREW")]
    AlphabetHebrew,
    [CCode (cname = "territory_ALPHABET_LATIN5")]
    AlphabetLatin5,
    [CCode (cname = "territory_ALPHABET_WELSH")]
    AlphabetWelsh,
    [CCode (cname = "territory_ALPHABET_UTF8")]
    AlphabetUTF8,
    [CCode (cname = "territory_ALPHABET_LATIN9")]
    AlphabetLatin9,
    [CCode (cname = "territory_ALPHABET_LATIN6")]
    AlphabetLatin6,
    [CCode (cname = "territory_ALPHABET_CYRILLIC2")]
    AlphabetCyrillic2
  }

  [CCode (cname = "territory_data")]
  public struct Data
  {
    [CCode (cname = "e")]
    public unowned AsmRoutine E[43];
  }

  [CCode (cname = "territory_ordinals")]
  public struct Ordinals
  {
    [CCode (cname = "centisecond")]
    public unowned int Centisecond;
    [CCode (cname = "second")]
    public unowned int Second;
    [CCode (cname = "minute")]
    public unowned int Minute;
    [CCode (cname = "hour")]
    public unowned int Hour;
    [CCode (cname = "date")]
    public unowned int Date;
    [CCode (cname = "month")]
    public unowned int Month;
    [CCode (cname = "year")]
    public unowned int Year;
    [CCode (cname = "weekday")]
    public unowned int Weekday;
    [CCode (cname = "yearday")]
    public unowned int Yearday;
  }

  [CCode (cname = "territory_property_table")]
  public struct PropertyTable
  {
    [CCode (cname = "p")]
    public unowned bits P[8];
  }

  [CCode (cname = "territory_calendar")]
  public struct Calendar
  {
    [CCode (cname = "first_working_day")]
    public unowned int FirstWorkingDay;
    [CCode (cname = "last_working_day")]
    public unowned int LastWorkingDay;
    [CCode (cname = "month_count")]
    public unowned int MonthCount;
    [CCode (cname = "day_count")]
    public unowned int DayCount;
    [CCode (cname = "pm_limit")]
    public unowned int PmLimit;
    [CCode (cname = "we_limit")]
    public unowned int WeLimit;
    [CCode (cname = "w3_limit")]
    public unowned int W3Limit;
    [CCode (cname = "dy_limit")]
    public unowned int DyLimit;
    [CCode (cname = "st_limit")]
    public unowned int StLimit;
    [CCode (cname = "mo_limit")]
    public unowned int MoLimit;
    [CCode (cname = "m3_limit")]
    public unowned int M3Limit;
    [CCode (cname = "tz_limit")]
    public unowned int TzLimit;
  }

  [CCode (cname = "territory_time_string_flags")]
  public enum TimeStringFlags
  {
    [CCode (cname = "territory_GIVEN_TIME")]
    GivenTime,
    [CCode (cname = "territory_GIVEN_DATE")]
    GivenDate,
    [CCode (cname = "territory_GIVEN_DATE_AND_TIME")]
    GivenDateAndTime
  }

  [CCode (cname = "territory_write_flags")]
  public enum WriteFlags
  {
    [CCode (cname = "territory_WRITE_RIGHT_TO_LEFT")]
    WriteRightToLeft,
    [CCode (cname = "territory_WRITE_BOTTOM_TO_TOP")]
    WriteBottomToTop,
    [CCode (cname = "territory_WRITE_VERTICAL")]
    WriteVertical
  }

  [CCode (cname = "territory_character_property")]
  public enum CharacterProperty
  {
    [CCode (cname = "territory_PROPERTY_CONTROL")]
    PropertyControl,
    [CCode (cname = "territory_PROPERTY_UPPERCASE")]
    PropertyUppercase,
    [CCode (cname = "territory_PROPERTY_LOWERCASE")]
    PropertyLowercase,
    [CCode (cname = "territory_PROPERTY_ALPHABETIC")]
    PropertyAlphabetic,
    [CCode (cname = "territory_PROPERTY_PUNCTUATION")]
    PropertyPunctuation,
    [CCode (cname = "territory_PROPERTY_WHITESPACE")]
    PropertyWhitespace,
    [CCode (cname = "territory_PROPERTY_DIGIT")]
    PropertyDigit,
    [CCode (cname = "territory_PROPERTY_HEX_DIGIT")]
    PropertyHexDigit,
    [CCode (cname = "territory_PROPERTY_ACCENTED")]
    PropertyAccented,
    [CCode (cname = "territory_PROPERTY_FORWARD")]
    PropertyForward,
    [CCode (cname = "territory_PROPERTY_REVERSE")]
    PropertyReverse
  }

  [CCode (cname = "territory_symbol_no")]
  public enum SymbolNo
  {
    [CCode (cname = "territory_SYMBOL_DECIMAL_POINT")]
    SymbolDecimalPoint,
    [CCode (cname = "territory_SYMBOL_GROUP_SEPARATOR")]
    SymbolGroupSeparator,
    [CCode (cname = "territory_SYMBOL_GROUPING_LIST")]
    SymbolGroupingList,
    [CCode (cname = "territory_SYMBOL_INTERNATIONAL_CURRENCY_SYMBOL")]
    SymbolInternationalCurrencySymbol,
    [CCode (cname = "territory_SYMBOL_CURRENCY_SYMBOL")]
    SymbolCurrencySymbol,
    [CCode (cname = "territory_SYMBOL_CURRENCY_POINT")]
    SymbolCurrencyPoint,
    [CCode (cname = "territory_SYMBOL_CURRENCY_GROUP_SEPARATOR")]
    SymbolCurrencyGroupSeparator,
    [CCode (cname = "territory_SYMBOL_CURRENCY_GROUP_LIST")]
    SymbolCurrencyGroupList,
    [CCode (cname = "territory_SYMBOL_CURRENCY_PLUS_SIGN")]
    SymbolCurrencyPlusSign,
    [CCode (cname = "territory_SYMBOL_CURRENCY_MINUS_SIGN")]
    SymbolCurrencyMinusSign,
    [CCode (cname = "territory_SYMBOL_INTERNATIONAL_CURRENCY_PRECISION")]
    SymbolInternationalCurrencyPrecision,
    [CCode (cname = "territory_SYMBOL_CURRENCY_PRECISION")]
    SymbolCurrencyPrecision,
    [CCode (cname = "territory_SYMBOL_CURRENCY_SYMBOL_PRECEDES_NON_NEGATIVE_VALUE")]
    SymbolCurrencySymbolPrecedesNonNegativeValue,
    [CCode (cname = "territory_SYMBOL_CURRENCY_SYMBOL_SEPARATED_FROM_NON_NEGATIVE_VALUE")]
    SymbolCurrencySymbolSeparatedFromNonNegativeValue,
    [CCode (cname = "territory_SYMBOL_CURRENCY_SYMBOL_PRECEDES_NEGATIVE_VALUE")]
    SymbolCurrencySymbolPrecedesNegativeValue,
    [CCode (cname = "territory_SYMBOL_CURRENCY_SYMBOL_SEPARATED_FROM_NEGATIVE_VALUE")]
    SymbolCurrencySymbolSeparatedFromNegativeValue,
    [CCode (cname = "territory_SYMBOL_CURRENCY_NON_NEGATIVE_FORMAT")]
    SymbolCurrencyNonNegativeFormat,
    [CCode (cname = "territory_SYMBOL_CURRENCY_NEGATIVE_FORMAT")]
    SymbolCurrencyNegativeFormat,
    [CCode (cname = "territory_SYMBOL_LIST_SEPARATOR")]
    SymbolListSeparator
  }

  [CCode (cname = "territory_symbol_currency_no")]
  public enum SymbolCurrencyNo
  {
    [CCode (cname = "territory_SYMBOL_PARENTHESISED")]
    SymbolParenthesised,
    [CCode (cname = "territory_SYMBOL_SIGN_PRECEDES")]
    SymbolSignPrecedes,
    [CCode (cname = "territory_SYMBOL_SIGN_SUCCEEDS")]
    SymbolSignSucceeds,
    [CCode (cname = "territory_SYMBOL_SIGN_IMMEDIATELY_PRECEDES")]
    SymbolSignImmediatelyPrecedes,
    [CCode (cname = "territory_SYMBOL_SIGN_IMMEDIATELY_SUCCEEDS")]
    SymbolSignImmediatelySucceeds
  }

  [CCode (cname="error_TERRITORY_BAD_SWI")]
  public const bits Error_TerritoryBadSWI;
  [CCode (cname="error_TERRITORY_UNKNOWN")]
  public const bits Error_TerritoryUnknown;
  [CCode (cname="error_TERRITORY_UNKNOWN_ALPHABET")]
  public const bits Error_TerritoryUnknownAlphabet;
  [CCode (cname="error_NO_TERRITORY")]
  public const bits Error_NoTerritory;

  [CCode (cname = "xterritory_number")]
  public static OS.Error* XNumber
    (out T territory);

  [CCode (cname = "territory_number")]
  public static T Number ();

  [CCode (cname = "xterritory_register")]
  public static OS.Error* XRegister
    (T territory,
     Data entries,
     char* workspace);

  [CCode (cname = "territory_register")]
  public static void Register
    (T territory,
     Data entries,
     char* workspace);

  [CCode (cname = "xterritory_deregister")]
  public static OS.Error* XDeregister
    (T territory);

  [CCode (cname = "territory_deregister")]
  public static void Deregister
    (T territory);

  [CCode (cname = "xterritory_number_to_name")]
  public static OS.Error* XNumberToName
    (T territory,
     string name,
     int size);

  [CCode (cname = "territory_number_to_name")]
  public static void NumberToName
    (T territory,
     string name,
     int size);

  [CCode (cname = "xterritory_exists")]
  public static OS.Error* XExists
    (T territory,
     out uint psr);

  [CCode (cname = "territory_exists")]
  public static bits Exists
    (T territory);

  [CCode (cname = "xterritory_alphabet_number_to_name")]
  public static OS.Error* XAlphabetNumberToName
    (AlphabetNumber alphabet,
     string name,
     int size);

  [CCode (cname = "territory_alphabet_number_to_name")]
  public static void AlphabetNumberToName
    (AlphabetNumber alphabet,
     string name,
     int size);

  [CCode (cname = "xterritory_select_alphabet")]
  public static OS.Error* XSelectAlphabet
    (T territory);

  [CCode (cname = "territory_select_alphabet")]
  public static void SelectAlphabet
    (T territory);

  [CCode (cname = "xterritory_set_time")]
  public static OS.Error* XSetTime
    (OS.DateAndTime date_and_time);

  [CCode (cname = "territory_set_time")]
  public static void SetTime
    (OS.DateAndTime date_and_time);

  [CCode (cname = "xterritory_read_current_time_zone")]
  public static OS.Error* XReadCurrentTimeZone
    (out string timezone,
     out int offset);

  [CCode (cname = "territory_read_current_time_zone")]
  public static void ReadCurrentTimeZone
    (out string timezone,
     out int offset);

  [CCode (cname = "xterritory_convert_time_to_utc_ordinals")]
  public static OS.Error* XConvertTimeToUTCOrdinals
    (OS.DateAndTime date_and_time,
     Ordinals ordinals);

  [CCode (cname = "territory_convert_time_to_utc_ordinals")]
  public static void ConvertTimeToUTCOrdinals
    (OS.DateAndTime date_and_time,
     Ordinals ordinals);

  [CCode (cname = "xterritory_read_time_zones")]
  public static OS.Error* XReadTimeZones
    (T territory,
     out string timezone,
     out string dst_timezone,
     out int offset,
     out int dst_offset);

  [CCode (cname = "territory_read_time_zones")]
  public static void ReadTimeZones
    (T territory,
     out string timezone,
     out string dst_timezone,
     out int offset,
     out int dst_offset);

  [CCode (cname = "xterritory_convert_date_and_time")]
  public static OS.Error* XConvertDateAndTime
    (T territory,
     OS.DateAndTime date_and_time,
     string buffer,
     int size,
     string format,
     out char* end);

  [CCode (cname = "territory_convert_date_and_time")]
  public static char ConvertDateAndTime
    (T territory,
     OS.DateAndTime date_and_time,
     string buffer,
     int size,
     string format);

  [CCode (cname = "xterritory_convert_standard_date_and_time")]
  public static OS.Error* XConvertStandardDateAndTime
    (T territory,
     OS.DateAndTime date_and_time,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "territory_convert_standard_date_and_time")]
  public static char ConvertStandardDateAndTime
    (T territory,
     OS.DateAndTime date_and_time,
     string buffer,
     int size);

  [CCode (cname = "xterritory_convert_standard_date")]
  public static OS.Error* XConvertStandardDate
    (T territory,
     OS.DateAndTime date,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "territory_convert_standard_date")]
  public static char ConvertStandardDate
    (T territory,
     OS.DateAndTime date,
     string buffer,
     int size);

  [CCode (cname = "xterritory_convert_standard_time")]
  public static OS.Error* XConvertStandardTime
    (T territory,
     OS.DateAndTime time,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "territory_convert_standard_time")]
  public static char ConvertStandardTime
    (T territory,
     OS.DateAndTime time,
     string buffer,
     int size);

  [CCode (cname = "xterritory_convert_time_to_ordinals")]
  public static OS.Error* XConvertTimeToOrdinals
    (T territory,
     OS.DateAndTime date_and_time,
     Ordinals ordinals);

  [CCode (cname = "territory_convert_time_to_ordinals")]
  public static void ConvertTimeToOrdinals
    (T territory,
     OS.DateAndTime date_and_time,
     Ordinals ordinals);

  [CCode (cname = "xterritory_convert_time_string_to_ordinals")]
  public static OS.Error* XConvertTimeStringToOrdinals
    (T territory,
     TimeStringFlags format,
     string t,
     Ordinals ordinals);

  [CCode (cname = "territory_convert_time_string_to_ordinals")]
  public static void ConvertTimeStringToOrdinals
    (T territory,
     TimeStringFlags format,
     string t,
     Ordinals ordinals);

  [CCode (cname = "xterritory_convert_ordinals_to_time")]
  public static OS.Error* XConvertOrdinalsToTime
    (T territory,
     OS.DateAndTime buffer,
     Ordinals ordinals);

  [CCode (cname = "territory_convert_ordinals_to_time")]
  public static void ConvertOrdinalsToTime
    (T territory,
     OS.DateAndTime buffer,
     Ordinals ordinals);

  [CCode (cname = "xterritory_alphabet")]
  public static OS.Error* XAlphabet
    (T territory,
     out AlphabetNumber alphabet);

  [CCode (cname = "territory_alphabet")]
  public static AlphabetNumber Alphabet
    (T territory);

  [CCode (cname = "xterritory_alphabet_identifier")]
  public static OS.Error* XAlphabetIdentifier
    (T territory,
     out string alphabet_name);

  [CCode (cname = "territory_alphabet_identifier")]
  public static string AlphabetIdentifier
    (T territory);

  [CCode (cname = "xterritory_select_keyboard_handler")]
  public static OS.Error* XSelectKeyboardHandler
    (T territory);

  [CCode (cname = "territory_select_keyboard_handler")]
  public static void SelectKeyboardHandler
    (T territory);

  [CCode (cname = "xterritory_write_direction")]
  public static OS.Error* XWriteDirection
    (T territory,
     out WriteFlags flags);

  [CCode (cname = "territory_write_direction")]
  public static WriteFlags WriteDirection
    (T territory);

  [CCode (cname = "xterritory_character_property_table")]
  public static OS.Error* XCharacterPropertyTable
    (T territory,
     CharacterProperty property,
     out PropertyTable table);

  [CCode (cname = "territory_character_property_table")]
  public static PropertyTable CharacterPropertyTable
    (T territory,
     CharacterProperty property);

  [CCode (cname = "xterritory_lower_case_table")]
  public static OS.Error* XLowerCaseTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_lower_case_table")]
  public static char LowerCaseTable
    (T territory);

  [CCode (cname = "xterritory_upper_case_table")]
  public static OS.Error* XUpperCaseTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_upper_case_table")]
  public static char UpperCaseTable
    (T territory);

  [CCode (cname = "xterritory_control_table")]
  public static OS.Error* XControlTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_control_table")]
  public static char ControlTable
    (T territory);

  [CCode (cname = "xterritory_plain_table")]
  public static OS.Error* XPlainTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_plain_table")]
  public static char PlainTable
    (T territory);

  [CCode (cname = "xterritory_value_table")]
  public static OS.Error* XValueTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_value_table")]
  public static char ValueTable
    (T territory);

  [CCode (cname = "xterritory_representation_table")]
  public static OS.Error* XRepresentationTable
    (T territory,
     out char* table);

  [CCode (cname = "territory_representation_table")]
  public static char RepresentationTable
    (T territory);

  [CCode (cname = "xterritory_collate")]
  public static OS.Error* XCollate
    (T territory,
     string s1,
     string s2,
     CollateFlags flags,
     out int collate);

  [CCode (cname = "territory_collate")]
  public static int Collate
    (T territory,
     string s1,
     string s2,
     CollateFlags flags);

  [CCode (cname = "xterritory_read_boolean_symbols")]
  public static OS.Error* XReadBooleanSymbols
    (T territory,
     SymbolNo symbol_no,
     out bool symbol);

  [CCode (cname = "territory_read_boolean_symbols")]
  public static bool ReadBooleanSymbols
    (T territory,
     SymbolNo symbol_no);

  [CCode (cname = "xterritory_read_integer_symbols")]
  public static OS.Error* XReadIntegerSymbols
    (T territory,
     SymbolNo symbol_no,
     out int symbol);

  [CCode (cname = "territory_read_integer_symbols")]
  public static int ReadIntegerSymbols
    (T territory,
     SymbolNo symbol_no);

  [CCode (cname = "xterritory_read_string_symbols")]
  public static OS.Error* XReadStringSymbols
    (T territory,
     SymbolNo symbol_no,
     out string symbol);

  [CCode (cname = "territory_read_string_symbols")]
  public static string ReadStringSymbols
    (T territory,
     SymbolNo symbol_no);

  [CCode (cname = "xterritory_read_calendar_information")]
  public static OS.Error* XReadCalendarInformation
    (T territory,
     OS.DateAndTime date_and_time,
     Calendar calendar);

  [CCode (cname = "territory_read_calendar_information")]
  public static void ReadCalendarInformation
    (T territory,
     OS.DateAndTime date_and_time,
     Calendar calendar);

  [CCode (cname = "xterritory_name_to_number")]
  public static OS.Error* XNameToNumber
    (T territory,
     string name,
     out T territory_out);

  [CCode (cname = "territory_name_to_number")]
  public static T NameToNumber
    (T territory,
     string name);

  [CCode (cname = "xterritory_transform_string")]
  public static OS.Error* XTransformString
    (T territory,
     string buffer,
     string s,
     int size,
     out int used);

  [CCode (cname = "territory_transform_string")]
  public static int TransformString
    (T territory,
     string buffer,
     string s,
     int size);

  [CCode (cname = "xservice_territory_manager_loaded")]
  public static OS.Error* XService_TerritoryManagerLoaded ();

  [CCode (cname = "service_territory_manager_loaded")]
  public static void Service_TerritoryManagerLoaded ();

  [CCode (cname = "xservice_territory_started")]
  public static OS.Error* XService_TerritoryStarted ();

  [CCode (cname = "service_territory_started")]
  public static void Service_TerritoryStarted ();

}
