/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/textarea.h")]
namespace OSLib.TextArea
{
  [CCode (cname = "textarea_gadget")]
  public struct Gadget
  {
    [CCode (cname = "type")]
    public unowned int Type;
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "foreground")]
    public unowned OS.Colour Foreground;
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
  }

  [CCode (cname = "textarea_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "type")]
    public unowned int Type;
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "foreground")]
    public unowned OS.Colour Foreground;
    [CCode (cname = "background")]
    public unowned OS.Colour Background;
  }

  [CCode (cname = "textarea_state")]
  public enum State
  {
    [CCode (cname = "textarea_STATE_HAS_VERTICAL_SCROLLBAR")]
    HasVerticalScrollbar,
    [CCode (cname = "textarea_STATE_HAS_HORIZONTAL_SCROLLBAR")]
    HasHorizontalScrollbar,
    [CCode (cname = "textarea_STATE_WORDWRAPPING_ENABLED")]
    WordwrappingEnabled,
    [CCode (cname = "textarea_ALLOW_SELECTION")]
    AllowSelection,
    [CCode (cname = "textarea_ALLOW_SAVE")]
    AllowSave,
    [CCode (cname = "textarea_ALLOW_LOAD")]
    AllowLoad,
    [CCode (cname = "textarea_CLIPBOARD")]
    Clipboard,
    [CCode (cname = "textarea_AUTO_SCROLL")]
    AutoScroll
  }

  [CCode (cname = "textarea_get_selection_flags")]
  public enum GetSelectionFlags
  {
    [CCode (cname = "textarea_GET_SELECTION_RETURN_TEXT")]
    GetSelectionReturnText
  }

  [CCode (cname = "textarea_set_colour_flags")]
  public enum SetColourFlags
  {
    [CCode (cname = "textarea_SET_COLOUR_DESKTOP")]
    SetColourDesktop
  }

  [CCode (cname = "textarea_colour")]
  public struct Colour
  {
    [CCode (cname = "real")]
    public unowned OS.Colour Real;
    [CCode (cname = "desktop")]
    public unowned Wimp.Colour Desktop;
  }

  [CCode (cname="class_TEXT_AREA")]
  public const Toolbox.Class Class_TextArea;

  [CCode (cname = "xtextarea_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out State state);

  [CCode (cname = "textarea_get_state")]
  public static State GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area);

  [CCode (cname = "xtextarea_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     State state);

  [CCode (cname = "textarea_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     State state);

  [CCode (cname = "xtextarea_set_text")]
  public static OS.Error* XSetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string text);

  [CCode (cname = "textarea_set_text")]
  public static void SetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string text);

  [CCode (cname = "xtextarea_get_text_size")]
  public static OS.Error* XGetTextSize
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out int buffer_size);

  [CCode (cname = "textarea_get_text_size")]
  public static int GetTextSize
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area);

  [CCode (cname = "xtextarea_get_text")]
  public static OS.Error* XGetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string buffer,
     int buffer_size,
     out int length);

  [CCode (cname = "textarea_get_text")]
  public static int GetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string buffer,
     int buffer_size);

  [CCode (cname = "xtextarea_insert_text")]
  public static OS.Error* XInsertText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int insert_at,
     string text);

  [CCode (cname = "textarea_insert_text")]
  public static void InsertText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int insert_at,
     string text);

  [CCode (cname = "xtextarea_replace_text")]
  public static OS.Error* XReplaceText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int start,
     int end,
     string text);

  [CCode (cname = "textarea_replace_text")]
  public static void ReplaceText
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int start,
     int end,
     string text);

  [CCode (cname = "xtextarea_get_selection")]
  public static OS.Error* XGetSelection
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out int start,
     out int end);

  [CCode (cname = "textarea_get_selection")]
  public static void GetSelection
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out int start,
     out int end);

  [CCode (cname = "xtextarea_get_selection_text_size")]
  public static OS.Error* XGetSelectionTextSize
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out int buffer_size);

  [CCode (cname = "textarea_get_selection_text_size")]
  public static int GetSelectionTextSize
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area);

  [CCode (cname = "xtextarea_get_selection_text")]
  public static OS.Error* XGetSelectionText
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string buffer,
     int buffer_size,
     out int length);

  [CCode (cname = "textarea_get_selection_text")]
  public static int GetSelectionText
    (GetSelectionFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string buffer,
     int buffer_size);

  [CCode (cname = "xtextarea_set_selection")]
  public static OS.Error* XSetSelection
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int start,
     int end);

  [CCode (cname = "textarea_set_selection")]
  public static void SetSelection
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int start,
     int end);

  [CCode (cname = "xtextarea_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string font_name,
     int width,
     int height);

  [CCode (cname = "textarea_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     string font_name,
     int width,
     int height);

  [CCode (cname = "xtextarea_set_colour")]
  public static OS.Error* XSetColour
    (SetColourFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     Colour foreground,
     Colour background);

  [CCode (cname = "textarea_set_colour")]
  public static void SetColour
    (SetColourFlags flags,
     Toolbox.O window,
     Toolbox.C text_area,
     Colour foreground,
     Colour background);

  [CCode (cname = "xtextarea_get_colour")]
  public static OS.Error* XGetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out OS.Colour foreground,
     out OS.Colour background);

  [CCode (cname = "textarea_get_colour")]
  public static void GetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     out OS.Colour foreground,
     out OS.Colour background);

  [CCode (cname = "xtextarea_set_text_border")]
  public static OS.Error* XSetTextBorder
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int border_size);

  [CCode (cname = "textarea_set_text_border")]
  public static void SetTextBorder
    (bits flags,
     Toolbox.O window,
     Toolbox.C text_area,
     int border_size);

}
