/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/timermanager.h")]
namespace OSLib.TimerManager
{
  [CCode (cname = "timermanagerreturnnumber_flags")]
  public enum TimerManagerReturnNumber_Flags
  {
    Default = 0
  }

  [CCode (cname = "timermanager_measurement_format")]
  public enum MeasurementFormat
  {
    [CCode (cname = "timermanagermeasurementformat_SCALE_EMIN9")]
    ScaleEMin9,
    [CCode (cname = "timermanagermeasurementformat_SCALE_EMIN6")]
    ScaleEMin6,
    [CCode (cname = "timermanagermeasurementformat_SCALE_EMIN3")]
    ScaleEMin3,
    [CCode (cname = "timermanagermeasurementformat_SCALE_E0")]
    ScaleE0,
    [CCode (cname = "timermanagermeasurementformat_SCALE_EPOS3")]
    ScaleEPos3,
    [CCode (cname = "timermanagermeasurementformat_SCALE_EPOS6")]
    ScaleEPos6,
    [CCode (cname = "timermanagermeasurementformat_SCALE_EPOS9")]
    ScaleEPos9,
    [CCode (cname = "timermanagermeasurementformat_TYPE_NATIVE_TICKS")]
    TypeNativeTicks,
    [CCode (cname = "timermanagermeasurementformat_TYPE_FREQUENCY")]
    TypeFrequency,
    [CCode (cname = "timermanagermeasurementformat_TYPE_PERIOD")]
    TypePeriod
  }

  [CCode (cname = "timermanagerrelease_flags")]
  public enum TimerManagerRelease_Flags
  {
    Default = 0
  }

  [CCode (cname="timermanagermeasurementformat_IN_UNIT_SCALER_SHIFT")]
  public const int TimerManagerMeasurementFormat_InUnitScalerShift;
  [CCode (cname="timermanagermeasurementformat_IN_UNIT_SCALER_MASK")]
  public const int TimerManagerMeasurementFormat_InUnitScalerMask;
  [CCode (cname="timermanagermeasurementformat_IN_TYPE_SHIFT")]
  public const int TimerManagerMeasurementFormat_InTypeShift;
  [CCode (cname="timermanagermeasurementformat_IN_TYPE_MASK")]
  public const int TimerManagerMeasurementFormat_InTypeMask;
  [CCode (cname="timermanagermeasurementformat_OUT_UNIT_SCALER_SHIFT")]
  public const int TimerManagerMeasurementFormat_OutUnitScalerShift;
  [CCode (cname="timermanagermeasurementformat_OUT_UNIT_SCALER_MASK")]
  public const int TimerManagerMeasurementFormat_OutUnitScalerMask;
  [CCode (cname="timermanagermeasurementformat_OUT_TYPE_SHIFT")]
  public const int TimerManagerMeasurementFormat_OutTypeShift;
  [CCode (cname="timermanagermeasurementformat_OUT_TYPE_MASK")]
  public const int TimerManagerMeasurementFormat_OutTypeMask;

  [CCode (cname = "xtimermanager_return_number")]
  public static OS.Error* XReturnNumber
    (TimerManagerReturnNumber_Flags flags,
     out int num_timers);

  [CCode (cname = "timermanager_return_number")]
  public static int ReturnNumber
    (TimerManagerReturnNumber_Flags flags);

  [CCode (cname = "xtimermanager_claim")]
  public static OS.Error* XClaim
    (MeasurementFormat flags,
     int timer_number,
     int timer_value,
     AsmRoutine interrupt_call,
     int ws_ptr,
     out MeasurementFormat timer_rate);

  [CCode (cname = "timermanager_claim")]
  public static MeasurementFormat Claim
    (MeasurementFormat flags,
     int timer_number,
     int timer_value,
     AsmRoutine interrupt_call,
     int ws_ptr);

  [CCode (cname = "xtimermanager_release")]
  public static OS.Error* XRelease
    (TimerManagerRelease_Flags flags,
     int timer_number);

  [CCode (cname = "timermanager_release")]
  public static void Release
    (TimerManagerRelease_Flags flags,
     int timer_number);

  [CCode (cname = "xtimermanager_set_rate")]
  public static OS.Error* XSetRate
    (MeasurementFormat flags,
     int timer_number,
     int timer_value,
     out MeasurementFormat timer_rate);

  [CCode (cname = "timermanager_set_rate")]
  public static MeasurementFormat SetRate
    (MeasurementFormat flags,
     int timer_number,
     int timer_value);

  [CCode (cname = "xtimermanager_convert")]
  public static OS.Error* XConvert
    (MeasurementFormat flags,
     int timer_number,
     int timer_value,
     out MeasurementFormat timer_rate);

  [CCode (cname = "timermanager_convert")]
  public static MeasurementFormat Convert
    (MeasurementFormat flags,
     int timer_number,
     int timer_value);

}
