/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/toolaction.h")]
namespace OSLib.ToolAction
{
  [CCode (cname = "toolaction_gadget")]
  public struct Gadget
  {
    [CCode (cname = "off_text")]
    public unowned Toolbox.StringReference OffText;
    [CCode (cname = "off_text_limit")]
    public unowned int OffTextLimit;
    [CCode (cname = "on_text")]
    public unowned Toolbox.StringReference OnText;
    [CCode (cname = "on_text_limit")]
    public unowned int OnTextLimit;
    [CCode (cname = "click_action")]
    public unowned bits ClickAction;
    [CCode (cname = "select_show")]
    public unowned Toolbox.StringReference SelectShow;
    [CCode (cname = "adjust_action")]
    public unowned bits AdjustAction;
    [CCode (cname = "adjust_show")]
    public unowned Toolbox.StringReference AdjustShow;
    [CCode (cname = "fade_text")]
    public unowned Toolbox.StringReference FadeText;
    [CCode (cname = "fade_text_limit")]
    public unowned int FadeTextLimit;
  }

  [CCode (cname = "toolaction_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "off_text")]
    public unowned Toolbox.StringReference OffText;
    [CCode (cname = "off_text_limit")]
    public unowned int OffTextLimit;
    [CCode (cname = "on_text")]
    public unowned Toolbox.StringReference OnText;
    [CCode (cname = "on_text_limit")]
    public unowned int OnTextLimit;
    [CCode (cname = "click_action")]
    public unowned bits ClickAction;
    [CCode (cname = "select_show")]
    public unowned Toolbox.StringReference SelectShow;
    [CCode (cname = "adjust_action")]
    public unowned bits AdjustAction;
    [CCode (cname = "adjust_show")]
    public unowned Toolbox.StringReference AdjustShow;
    [CCode (cname = "fade_text")]
    public unowned Toolbox.StringReference FadeText;
    [CCode (cname = "fade_text_limit")]
    public unowned int FadeTextLimit;
  }

  [CCode (cname = "toolaction_ident_flags")]
  public enum IdentFlags
  {
    [CCode (cname = "toolaction_IDENT_MASK")]
    IdentMask,
    [CCode (cname = "toolaction_IDENT_OFF")]
    IdentOff,
    [CCode (cname = "toolaction_IDENT_ON")]
    IdentOn,
    [CCode (cname = "toolaction_IDENT_FADE")]
    IdentFade
  }

  [CCode (cname="class_TOOL_ACTION")]
  public const Toolbox.Class Class_ToolAction;
  [CCode (cname="error_TOOL_ACTION_OUT_OF_MEMORY")]
  public const bits Error_ToolActionOutOfMemory;
  [CCode (cname="error_TOOL_ACTION_CANT_CREATE_ICON")]
  public const bits Error_ToolActionCantCreateIcon;
  [CCode (cname="error_TOOL_ACTION_CANT_CREATE_OBJECT")]
  public const bits Error_ToolActionCantCreateObject;
  [CCode (cname="toolaction_GENERATE_SELECT")]
  public const OSLib.Gadget.Flags GenerateSelect;
  [CCode (cname="toolaction_IS_TEXT")]
  public const OSLib.Gadget.Flags IsText;
  [CCode (cname="toolaction_ON")]
  public const OSLib.Gadget.Flags On;
  [CCode (cname="toolaction_AUTO_TOGGLE")]
  public const OSLib.Gadget.Flags AutoToggle;
  [CCode (cname="toolaction_NO_PRESSED_SPRITE")]
  public const OSLib.Gadget.Flags NoPressedSprite;
  [CCode (cname="toolaction_AUTO_REPEAT")]
  public const OSLib.Gadget.Flags AutoRepeat;
  [CCode (cname="toolaction_SHOW_TRANSIENT")]
  public const OSLib.Gadget.Flags ShowTransient;
  [CCode (cname="toolaction_SHOW_AS_POP_UP")]
  public const OSLib.Gadget.Flags ShowAsPopUp;
  [CCode (cname="toolaction_HAS_FADE_SPRITE")]
  public const OSLib.Gadget.Flags HasFadeSprite;
  [CCode (cname="action_TOOL_ACTION_SELECTED")]
  public const bits Action_ToolActionSelected;
  [CCode (cname="toolaction_SELECTED_ADJUST")]
  public const bits SelectedAdjust;
  [CCode (cname="toolaction_SELECTED_SELECT")]
  public const bits SelectedSelect;

  [CCode (cname = "xtoolaction_set_ident")]
  public static OS.Error* XSetIdent
    (IdentFlags flags,
     Toolbox.O window,
     Toolbox.C toolaction);

  [CCode (cname = "toolaction_set_ident")]
  public static void SetIdent
    (IdentFlags flags,
     Toolbox.O window,
     Toolbox.C toolaction);

  [CCode (cname = "xtoolaction_get_ident")]
  public static OS.Error* XGetIdent
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out IdentFlags ident);

  [CCode (cname = "toolaction_get_ident")]
  public static IdentFlags GetIdent
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction);

  [CCode (cname = "xtoolaction_set_action")]
  public static OS.Error* XSetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     bits select_action,
     bits adjust_action);

  [CCode (cname = "toolaction_set_action")]
  public static void SetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     bits select_action,
     bits adjust_action);

  [CCode (cname = "xtoolaction_get_action")]
  public static OS.Error* XGetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out bits select_action,
     out bits adjust_action);

  [CCode (cname = "toolaction_get_action")]
  public static void GetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out bits select_action,
     out bits adjust_action);

  [CCode (cname = "xtoolaction_set_click_show")]
  public static OS.Error* XSetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     Toolbox.O select_show_object,
     Toolbox.O adjust_show_object);

  [CCode (cname = "toolaction_set_click_show")]
  public static void SetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     Toolbox.O select_show_object,
     Toolbox.O adjust_show_object);

  [CCode (cname = "xtoolaction_get_click_show")]
  public static OS.Error* XGetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out Toolbox.O select_show_object,
     out Toolbox.O adjust_show_object);

  [CCode (cname = "toolaction_get_click_show")]
  public static void GetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out Toolbox.O select_show_object,
     out Toolbox.O adjust_show_object);

  [CCode (cname = "xtoolaction_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     bool on);

  [CCode (cname = "toolaction_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     bool on);

  [CCode (cname = "xtoolaction_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction,
     out bool on);

  [CCode (cname = "toolaction_get_state")]
  public static bool GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C toolaction);

}
