/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/wimpspriteop.h")]
namespace OSLib.WimpSpriteOp
{
  [CCode (cname="wimpspriteop_AREA")]
  public static OSSpriteOp.Area* Area;

  [CCode (cname = "xwimpspriteop_merge_sprite_file")]
  public static OS.Error* XMergeSpriteFile
    (string file_name);

  [CCode (cname = "wimpspriteop_merge_sprite_file")]
  public static void MergeSpriteFile
    (string file_name);

  [CCode (cname = "xwimpspriteop_get_sprite")]
  public static OS.Error* XGetSprite
    (string sprite_name,
     bool get_palette,
     out OSSpriteOp.Header header);

  [CCode (cname = "wimpspriteop_get_sprite")]
  public static OSSpriteOp.Header GetSprite
    (string sprite_name,
     bool get_palette);

  [CCode (cname = "xwimpspriteop_select_sprite")]
  public static OS.Error* XSelectSprite
    (string sprite_name,
     out OSSpriteOp.Header header);

  [CCode (cname = "wimpspriteop_select_sprite")]
  public static OSSpriteOp.Header SelectSprite
    (string sprite_name);

  [CCode (cname = "xwimpspriteop_put_sprite")]
  public static OS.Error* XPutSprite
    (string sprite_name,
     OS.Action action);

  [CCode (cname = "wimpspriteop_put_sprite")]
  public static void PutSprite
    (string sprite_name,
     OS.Action action);

  [CCode (cname = "xwimpspriteop_put_sprite_user_coords")]
  public static OS.Error* XPutSpriteUserCoords
    (string sprite_name,
     int x,
     int y,
     OS.Action action);

  [CCode (cname = "wimpspriteop_put_sprite_user_coords")]
  public static void PutSpriteUserCoords
    (string sprite_name,
     int x,
     int y,
     OS.Action action);

  [CCode (cname = "xwimpspriteop_set_pointer_shape")]
  public static OS.Error* XSetPointerShape
    (string sprite_name,
     bits flags,
     int xactive,
     int yactive,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "wimpspriteop_set_pointer_shape")]
  public static void SetPointerShape
    (string sprite_name,
     bits flags,
     int xactive,
     int yactive,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "xwimpspriteop_read_palette_size")]
  public static OS.Error* XReadPaletteSize
    (string sprite_name,
     out int size,
     out OS.SpritePalette palette,
     out OS.Mode mode);

  [CCode (cname = "wimpspriteop_read_palette_size")]
  public static void ReadPaletteSize
    (string sprite_name,
     out int size,
     out OS.SpritePalette palette,
     out OS.Mode mode);

  [CCode (cname = "xwimpspriteop_read_sprite_size")]
  public static OS.Error* XReadSpriteSize
    (string sprite_name,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "wimpspriteop_read_sprite_size")]
  public static void ReadSpriteSize
    (string sprite_name,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "xwimpspriteop_read_sprite_info")]
  public static OS.Error* XReadSpriteInfo
    (string sprite_name,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "wimpspriteop_read_sprite_info")]
  public static void ReadSpriteInfo
    (string sprite_name,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "xwimpspriteop_read_pixel_colour")]
  public static OS.Error* XReadPixelColour
    (string sprite_name,
     int x,
     int y,
     out OS.GCOL gcol,
     out OS.Tint tint);

  [CCode (cname = "wimpspriteop_read_pixel_colour")]
  public static void ReadPixelColour
    (string sprite_name,
     int x,
     int y,
     out OS.GCOL gcol,
     out OS.Tint tint);

  [CCode (cname = "xwimpspriteop_read_pixel_mask")]
  public static OS.Error* XReadPixelMask
    (string sprite_name,
     int x,
     int y,
     out bool solid);

  [CCode (cname = "wimpspriteop_read_pixel_mask")]
  public static bool ReadPixelMask
    (string sprite_name,
     int x,
     int y);

  [CCode (cname = "xwimpspriteop_plot_mask")]
  public static OS.Error* XPlotMask
    (string sprite_name);

  [CCode (cname = "wimpspriteop_plot_mask")]
  public static void PlotMask
    (string sprite_name);

  [CCode (cname = "xwimpspriteop_plot_mask_user_coords")]
  public static OS.Error* XPlotMaskUserCoords
    (string sprite_name,
     int x,
     int y);

  [CCode (cname = "wimpspriteop_plot_mask_user_coords")]
  public static void PlotMaskUserCoords
    (string sprite_name,
     int x,
     int y);

  [CCode (cname = "xwimpspriteop_plot_mask_scaled")]
  public static OS.Error* XPlotMaskScaled
    (string sprite_name,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "wimpspriteop_plot_mask_scaled")]
  public static void PlotMaskScaled
    (string sprite_name,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "xwimpspriteop_put_sprite_scaled")]
  public static OS.Error* XPutSpriteScaled
    (string sprite_name,
     int x,
     int y,
     OS.Action action,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "wimpspriteop_put_sprite_scaled")]
  public static void PutSpriteScaled
    (string sprite_name,
     int x,
     int y,
     OS.Action action,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "xwimpspriteop_put_sprite_grey_scaled")]
  public static OS.Error* XPutSpriteGreyScaled
    (string sprite_name,
     int x,
     int y,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "wimpspriteop_put_sprite_grey_scaled")]
  public static void PutSpriteGreyScaled
    (string sprite_name,
     int x,
     int y,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "xwimpspriteop_plot_mask_trfm")]
  public static OS.Error* XPlotMaskTrfm
    (string sprite_name,
     bits flags,
     OS.Box source_rect,
     OS.Trfm trfm);

  [CCode (cname = "wimpspriteop_plot_mask_trfm")]
  public static void PlotMaskTrfm
    (string sprite_name,
     bits flags,
     OS.Box source_rect,
     OS.Trfm trfm);

  [CCode (cname = "xwimpspriteop_put_sprite_trfm")]
  public static OS.Error* XPutSpriteTrfm
    (string sprite_name,
     bits flags,
     OS.Box source_rect,
     OS.Action action,
     OS.Trfm trfm,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "wimpspriteop_put_sprite_trfm")]
  public static void PutSpriteTrfm
    (string sprite_name,
     bits flags,
     OS.Box source_rect,
     OS.Action action,
     OS.Trfm trfm,
     OSSpriteOp.TransTab trans_tab);

}
