/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.KeyboardShortcut;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/window.h")]
namespace OSLib.Window
{
  [CCode (cname = "window_flags")]
  public enum Flags
  {
    [CCode (cname = "window_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "window_AUTO_OPEN")]
    AutoOpen,
    [CCode (cname = "window_AUTO_CLOSE")]
    AutoClose,
    [CCode (cname = "window_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "window_IS_TOOL_BAR")]
    IsToolBar
  }

  [CCode (cname = "window_gadget_flags")]
  public enum GadgetFlags
  {
    Default = 0
  }

  [CCode (cname = "window_icon_data")]
  public struct IconData
  {
    [CCode (cname = "indirected_text.text")]
    public unowned Toolbox.MsgReference IndirectedText_Text;
    [CCode (cname = "indirected_text.validation")]
    public unowned Toolbox.StringReference IndirectedText_Validation;
    [CCode (cname = "indirected_text.size")]
    public unowned int IndirectedText_Size;
  }

  [CCode (cname = "window_window")]
  public struct Window
  {
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned Wimp.W Next;
    [CCode (cname = "flags")]
    public unowned Wimp.WindowFlags Flags;
    [CCode (cname = "title_fg")]
    public unowned Wimp.Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Wimp.Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Wimp.Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Wimp.Colour WorkBg;
    [CCode (cname = "scroll_outer")]
    public unowned Wimp.Colour ScrollOuter;
    [CCode (cname = "scroll_inner")]
    public unowned Wimp.Colour ScrollInner;
    [CCode (cname = "highlight_bg")]
    public unowned Wimp.Colour HighlightBg;
    [CCode (cname = "extra_flags")]
    public unowned Wimp.ExtraWindowFlags ExtraFlags;
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "title_flags")]
    public unowned Wimp.IconFlags TitleFlags;
    [CCode (cname = "work_flags")]
    public unowned Wimp.IconFlags WorkFlags;
    [CCode (cname = "sprite_area")]
    public unowned Toolbox.SpriteAreaReference SpriteArea;
    [CCode (cname = "xmin")]
    public unowned short Xmin;
    [CCode (cname = "ymin")]
    public unowned short Ymin;
    [CCode (cname = "title_data")]
    public unowned IconData TitleData;
    [CCode (cname = "icon_count")]
    public unowned int IconCount;
  }

  [CCode (cname = "window_object_base")]
  public struct ObjectBase
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "pointer_limit")]
    public unowned int PointerLimit;
    [CCode (cname = "hotspot")]
    public unowned OS.Coord Hotspot;
    [CCode (cname = "menu_name")]
    public unowned Toolbox.StringReference MenuName;
    [CCode (cname = "shortcut_count")]
    public unowned int ShortcutCount;
    [CCode (cname = "shortcuts")]
    public unowned Toolbox.ObjectOffset Shortcuts;
    [CCode (cname = "gadget_count")]
    public unowned int GadgetCount;
    [CCode (cname = "gadgets")]
    public unowned Toolbox.ObjectOffset Gadgets;
    [CCode (cname = "default_focus")]
    public unowned Toolbox.C DefaultFocus;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "toolbar_ibl")]
    public unowned Toolbox.StringReference ToolbarIbl;
    [CCode (cname = "toolbar_itl")]
    public unowned Toolbox.StringReference ToolbarItl;
    [CCode (cname = "toolbar_ebl")]
    public unowned Toolbox.StringReference ToolbarEbl;
    [CCode (cname = "toolbar_etl")]
    public unowned Toolbox.StringReference ToolbarEtl;
    [CCode (cname = "window")]
    public unowned Window Window;
  }

  [Compact]
  [CCode (cname = "window_object", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Object
  {
    public Object(int numEntries);
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "pointer_limit")]
    public unowned int PointerLimit;
    [CCode (cname = "hotspot")]
    public unowned OS.Coord Hotspot;
    [CCode (cname = "menu_name")]
    public unowned Toolbox.StringReference MenuName;
    [CCode (cname = "shortcut_count")]
    public unowned int ShortcutCount;
    [CCode (cname = "shortcuts")]
    public unowned Toolbox.ObjectOffset Shortcuts;
    [CCode (cname = "gadget_count")]
    public unowned int GadgetCount;
    [CCode (cname = "gadgets")]
    public unowned Toolbox.ObjectOffset Gadgets;
    [CCode (cname = "default_focus")]
    public unowned Toolbox.C DefaultFocus;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "toolbar_ibl")]
    public unowned Toolbox.StringReference ToolbarIbl;
    [CCode (cname = "toolbar_itl")]
    public unowned Toolbox.StringReference ToolbarItl;
    [CCode (cname = "toolbar_ebl")]
    public unowned Toolbox.StringReference ToolbarEbl;
    [CCode (cname = "toolbar_etl")]
    public unowned Toolbox.StringReference ToolbarEtl;
    [CCode (cname = "window")]
    public unowned Window Window;
    [CCode (cname = "data", array_length = false)]
    public unowned int[] Data;
  }

  [CCode (cname = "window_object_template_base")]
  public struct ObjectTemplateBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "pointer_limit")]
    public unowned int PointerLimit;
    [CCode (cname = "hotspot")]
    public unowned OS.Coord Hotspot;
    [CCode (cname = "menu_name")]
    public unowned Toolbox.StringReference MenuName;
    [CCode (cname = "shortcut_count")]
    public unowned int ShortcutCount;
    [CCode (cname = "shortcuts")]
    public unowned Toolbox.ObjectOffset Shortcuts;
    [CCode (cname = "gadget_count")]
    public unowned int GadgetCount;
    [CCode (cname = "gadgets")]
    public unowned Toolbox.ObjectOffset Gadgets;
    [CCode (cname = "default_focus")]
    public unowned Toolbox.C DefaultFocus;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "toolbar_ibl")]
    public unowned Toolbox.StringReference ToolbarIbl;
    [CCode (cname = "toolbar_itl")]
    public unowned Toolbox.StringReference ToolbarItl;
    [CCode (cname = "toolbar_ebl")]
    public unowned Toolbox.StringReference ToolbarEbl;
    [CCode (cname = "toolbar_etl")]
    public unowned Toolbox.StringReference ToolbarEtl;
    [CCode (cname = "window")]
    public unowned Window Window;
  }

  [CCode (cname = "window_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "pointer_limit")]
    public unowned int PointerLimit;
    [CCode (cname = "hotspot")]
    public unowned OS.Coord Hotspot;
    [CCode (cname = "menu_name")]
    public unowned Toolbox.StringReference MenuName;
    [CCode (cname = "shortcut_count")]
    public unowned int ShortcutCount;
    [CCode (cname = "shortcuts")]
    public unowned Toolbox.ObjectOffset Shortcuts;
    [CCode (cname = "gadget_count")]
    public unowned int GadgetCount;
    [CCode (cname = "gadgets")]
    public unowned Toolbox.ObjectOffset Gadgets;
    [CCode (cname = "default_focus")]
    public unowned Toolbox.C DefaultFocus;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "toolbar_ibl")]
    public unowned Toolbox.StringReference ToolbarIbl;
    [CCode (cname = "toolbar_itl")]
    public unowned Toolbox.StringReference ToolbarItl;
    [CCode (cname = "toolbar_ebl")]
    public unowned Toolbox.StringReference ToolbarEbl;
    [CCode (cname = "toolbar_etl")]
    public unowned Toolbox.StringReference ToolbarEtl;
    [CCode (cname = "window")]
    public unowned Window Window;
    [CCode (cname = "data", array_length = false)]
    public unowned int[] Data;
  }

  [CCode (cname = "window_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "window_action_show_object")]
  public struct ActionShowObject
  {
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned Wimp.W Next;
  }

  [CCode (cname = "window_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "window_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "window_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "window_tool_bar_flags")]
  public enum ToolBarFlags
  {
    [CCode (cname = "window_TOOL_BAR_IBL")]
    ToolBarIBL,
    [CCode (cname = "window_TOOL_BAR_ITL")]
    ToolBarITL,
    [CCode (cname = "window_TOOL_BAR_EBL")]
    ToolBarEBL,
    [CCode (cname = "window_TOOL_BAR_ETL")]
    ToolBarETL
  }

  [CCode (cname = "window_mouse_state")]
  public enum MouseState
  {
    [CCode (cname = "window_CLICK_ADJUST")]
    ClickAdjust,
    [CCode (cname = "window_CLICK_MENU")]
    ClickMenu,
    [CCode (cname = "window_CLICK_SELECT")]
    ClickSelect,
    [CCode (cname = "window_CLICK_NOT_TOOLBOX")]
    ClickNotToolbox
  }

  [CCode (cname="class_WINDOW")]
  public const Toolbox.Class Class_Window;
  [CCode (cname="error_WINDOW_ALLOC_FAILED")]
  public const bits Error_WindowAllocFailed;
  [CCode (cname="error_WINDOW_SHORT_BUFFER")]
  public const bits Error_WindowShortBuffer;
  [CCode (cname="error_WINDOW_BAD_VERSION")]
  public const bits Error_WindowBadVersion;
  [CCode (cname="error_WINDOW_INVALID_FLAGS")]
  public const bits Error_WindowInvalidFlags;
  [CCode (cname="error_WINDOW_TASKS_ACTIVE")]
  public const bits Error_WindowTasksActive;
  [CCode (cname="error_WINDOW_NO_SUCH_TASK")]
  public const bits Error_WindowNoSuchTask;
  [CCode (cname="error_WINDOW_NO_SUCH_METHOD")]
  public const bits Error_WindowNoSuchMethod;
  [CCode (cname="error_WINDOW_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_WindowNoSuchMiscOpMethod;
  [CCode (cname="error_WINDOW_INVALID_COMPONENT_ID")]
  public const bits Error_WindowInvalidComponentID;
  [CCode (cname="error_WINDOW_DUPLICATE_COMPONENT_ID")]
  public const bits Error_WindowDuplicateComponentID;
  [CCode (cname="error_WINDOW_INVALID_GADGET_TYPE")]
  public const bits Error_WindowInvalidGadgetType;
  [CCode (cname="window_NO_FOCUS")]
  public const int NoFocus;
  [CCode (cname="window_SET_FOCUS_TO_WINDOW")]
  public const int SetFocusToWindow;
  [CCode (cname="action_WINDOW_ABOUT_TO_BE_SHOWN")]
  public const bits Action_WindowAboutToBeShown;
  [CCode (cname="action_WINDOW_DIALOGUE_COMPLETED")]
  public const bits Action_WindowDialogueCompleted;

  [CCode (cname = "xwindow_get_pointer_info")]
  public static OS.Error* XGetPointerInfo
    (bits flags,
     out int x,
     out int y,
     out MouseState state,
     out Toolbox.O obj,
     out Toolbox.C cmp);

  [CCode (cname = "window_get_pointer_info")]
  public static void GetPointerInfo
    (bits flags,
     out int x,
     out int y,
     out MouseState state,
     out Toolbox.O obj,
     out Toolbox.C cmp);

  [CCode (cname = "xwindow_wimp_to_toolbox")]
  public static OS.Error* XWimpToToolbox
    (bits flags,
     Wimp.W w,
     Wimp.I i,
     out Toolbox.O obj,
     out Toolbox.C cmp);

  [CCode (cname = "window_wimp_to_toolbox")]
  public static void WimpToToolbox
    (bits flags,
     Wimp.W w,
     Wimp.I i,
     out Toolbox.O obj,
     out Toolbox.C cmp);

  [CCode (cname = "xwindow_register_external")]
  public static OS.Error* XRegisterExternal
    (bits flags,
     Gadget.ExtensionList types,
     int handler_swi_number);

  [CCode (cname = "window_register_external")]
  public static void RegisterExternal
    (bits flags,
     Gadget.ExtensionList types,
     int handler_swi_number);

  [CCode (cname = "xwindow_deregister_external")]
  public static OS.Error* XDeregisterExternal
    (bits flags,
     int type,
     int handler_swi_number);

  [CCode (cname = "window_deregister_external")]
  public static void DeregisterExternal
    (bits flags,
     int type,
     int handler_swi_number);

  [CCode (cname = "xwindow_extract_gadget_info")]
  public static OS.Error* XExtractGadgetInfo
    (bits flags,
     Toolbox.ResourceFileObject object,
     Toolbox.C cmp,
     out OSLib.Gadget.Object gadget,
     out int size);

  [CCode (cname = "window_extract_gadget_info")]
  public static OSLib.Gadget.Object ExtractGadgetInfo
    (bits flags,
     Toolbox.ResourceFileObject object,
     Toolbox.C cmp,
     out int size);

  [CCode (cname = "xwindow_get_wimp_handle")]
  public static OS.Error* XGetWimpHandle
    (bits flags,
     Toolbox.O window,
     out Wimp.W w);

  [CCode (cname = "window_get_wimp_handle")]
  public static Wimp.W GetWimpHandle
    (bits flags,
     Toolbox.O window);

  [CCode (cname = "xwindow_add_gadget")]
  public static OS.Error* XAddGadget
    (bits flags,
     Toolbox.O window,
     OSLib.Gadget.Object object,
     out Toolbox.C gadget);

  [CCode (cname = "window_add_gadget")]
  public static Toolbox.C AddGadget
    (bits flags,
     Toolbox.O window,
     OSLib.Gadget.Object object);

  [CCode (cname = "xwindow_remove_gadget")]
  public static OS.Error* XRemoveGadget
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "window_remove_gadget")]
  public static void RemoveGadget
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "xwindow_set_menu")]
  public static OS.Error* XSetMenu
    (bits flags,
     Toolbox.O window,
     Toolbox.O menu);

  [CCode (cname = "window_set_menu")]
  public static void SetMenu
    (bits flags,
     Toolbox.O window,
     Toolbox.O menu);

  [CCode (cname = "xwindow_get_menu")]
  public static OS.Error* XGetMenu
    (bits flags,
     Toolbox.O window,
     out Toolbox.O menu);

  [CCode (cname = "window_get_menu")]
  public static Toolbox.O GetMenu
    (bits flags,
     Toolbox.O window);

  [CCode (cname = "xwindow_set_pointer")]
  public static OS.Error* XSetPointer
    (bits flags,
     Toolbox.O window,
     string sprite_name,
     int xhotspot,
     int yhotspot);

  [CCode (cname = "window_set_pointer")]
  public static void SetPointer
    (bits flags,
     Toolbox.O window,
     string sprite_name,
     int xhotspot,
     int yhotspot);

  [CCode (cname = "xwindow_get_pointer")]
  public static OS.Error* XGetPointer
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size,
     out int used,
     out int xhotspot,
     out int yhotspot);

  [CCode (cname = "window_get_pointer")]
  public static int GetPointer
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size,
     out int xhotspot,
     out int yhotspot);

  [CCode (cname = "xwindow_set_help_message")]
  public static OS.Error* XSetHelpMessage
    (bits flags,
     Toolbox.O window,
     string help);

  [CCode (cname = "window_set_help_message")]
  public static void SetHelpMessage
    (bits flags,
     Toolbox.O window,
     string help);

  [CCode (cname = "xwindow_get_help_message")]
  public static OS.Error* XGetHelpMessage
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "window_get_help_message")]
  public static int GetHelpMessage
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size);

  [CCode (cname = "xwindow_add_keyboard_shortcuts")]
  public static OS.Error* XAddKeyboardShortcuts
    (bits flags,
     Toolbox.O window,
     int add_count,
     KeyboardShortcut.Object shortcuts);

  [CCode (cname = "window_add_keyboard_shortcuts")]
  public static void AddKeyboardShortcuts
    (bits flags,
     Toolbox.O window,
     int add_count,
     KeyboardShortcut.Object shortcuts);

  [CCode (cname = "xwindow_remove_keyboard_shortcuts")]
  public static OS.Error* XRemoveKeyboardShortcuts
    (bits flags,
     Toolbox.O window,
     int remove_count,
     KeyboardShortcut.Object shortcuts);

  [CCode (cname = "window_remove_keyboard_shortcuts")]
  public static void RemoveKeyboardShortcuts
    (bits flags,
     Toolbox.O window,
     int remove_count,
     KeyboardShortcut.Object shortcuts);

  [CCode (cname = "xwindow_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O window,
     string title);

  [CCode (cname = "window_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O window,
     string title);

  [CCode (cname = "xwindow_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "window_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O window,
     string buffer,
     int size);

  [CCode (cname = "xwindow_set_default_focus")]
  public static OS.Error* XSetDefaultFocus
    (bits flags,
     Toolbox.O window,
     Toolbox.C focus);

  [CCode (cname = "window_set_default_focus")]
  public static void SetDefaultFocus
    (bits flags,
     Toolbox.O window,
     Toolbox.C focus);

  [CCode (cname = "xwindow_get_default_focus")]
  public static OS.Error* XGetDefaultFocus
    (bits flags,
     Toolbox.O window,
     out Toolbox.C focus);

  [CCode (cname = "window_get_default_focus")]
  public static Toolbox.C GetDefaultFocus
    (bits flags,
     Toolbox.O window);

  [CCode (cname = "xwindow_set_extent")]
  public static OS.Error* XSetExtent
    (bits flags,
     Toolbox.O window,
     OS.Box extent);

  [CCode (cname = "window_set_extent")]
  public static void SetExtent
    (bits flags,
     Toolbox.O window,
     OS.Box extent);

  [CCode (cname = "xwindow_get_extent")]
  public static OS.Error* XGetExtent
    (bits flags,
     Toolbox.O window,
     OS.Box extent);

  [CCode (cname = "window_get_extent")]
  public static void GetExtent
    (bits flags,
     Toolbox.O window,
     OS.Box extent);

  [CCode (cname = "xwindow_force_redraw")]
  public static OS.Error* XForceRedraw
    (bits flags,
     Toolbox.O window,
     OS.Box redraw);

  [CCode (cname = "window_force_redraw")]
  public static void ForceRedraw
    (bits flags,
     Toolbox.O window,
     OS.Box redraw);

  [CCode (cname = "xwindow_set_tool_bars")]
  public static OS.Error* XSetToolBars
    (ToolBarFlags flags,
     Toolbox.O window,
     Toolbox.O ibl,
     Toolbox.O itl,
     Toolbox.O ebl,
     Toolbox.O etl);

  [CCode (cname = "window_set_tool_bars")]
  public static void SetToolBars
    (ToolBarFlags flags,
     Toolbox.O window,
     Toolbox.O ibl,
     Toolbox.O itl,
     Toolbox.O ebl,
     Toolbox.O etl);

  [CCode (cname = "xwindow_get_tool_bars")]
  public static OS.Error* XGetToolBars
    (ToolBarFlags flags,
     Toolbox.O window,
     out Toolbox.O ibl,
     out Toolbox.O itl,
     out Toolbox.O ebl,
     out Toolbox.O etl);

  [CCode (cname = "window_get_tool_bars")]
  public static void GetToolBars
    (ToolBarFlags flags,
     Toolbox.O window,
     out Toolbox.O ibl,
     out Toolbox.O itl,
     out Toolbox.O ebl,
     out Toolbox.O etl);

  [CCode (cname = "xservice_window_module_starting")]
  public static OS.Error* XService_WindowModuleStarting ();

  [CCode (cname = "service_window_module_starting")]
  public static void Service_WindowModuleStarting ();

  [CCode (cname = "xservice_window_module_dying")]
  public static OS.Error* XService_WindowModuleDying ();

  [CCode (cname = "service_window_module_dying")]
  public static void Service_WindowModuleDying ();

  [CCode (cname = "xservice_gadget_registered")]
  public static OS.Error* XService_GadgetRegistered
    (int type,
     int handler_swi_number,
     bits feature_mask);

  [CCode (cname = "service_gadget_registered")]
  public static void Service_GadgetRegistered
    (int type,
     int handler_swi_number,
     bits feature_mask);

  [CCode (cname = "xservice_gadget_deregistered")]
  public static OS.Error* XService_GadgetDeregistered
    (int type,
     int handler_swi_number);

  [CCode (cname = "service_gadget_deregistered")]
  public static void Service_GadgetDeregistered
    (int type,
     int handler_swi_number);

}
