/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Menu;
using OSLib.OS;

[CCode (cheader_filename = "oslib/zapfontmenu.h")]
namespace OSLib.ZapFontMenu
{
  [CCode (cname = "zapfontmenu_flags")]
  public enum Flags
  {
    [CCode (cname = "zapfontmenu_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "zapfontmenu_GENERATE_HAS_BEEN_HIDDEN")]
    GenerateHasBeenHidden,
    [CCode (cname = "zapfontmenu_INCLUDE_SYSTEM_FONT")]
    IncludeSystemFont,
    [CCode (cname = "zapfontmenu_ENCODING_MENU")]
    EncodingMenu
  }

  [CCode (cname = "zapfontmenu_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "ticked_font")]
    public unowned Toolbox.StringReference TickedFont;
  }

  [CCode (cname = "zapfontmenu_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "ticked_font")]
    public unowned Toolbox.StringReference TickedFont;
  }

  [CCode (cname = "zapfontmenu_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "zapfontmenu_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "zapfontmenu_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "zapfontmenu_action_has_been_hidden_block")]
  public enum ActionHasBeenHiddenBlock
  {
    Default = 0
  }

  [CCode (cname = "zapfontmenu_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "zapfontmenu_action_selection")]
  public struct ActionSelection
  {
    [CCode (cname = "identifier")]
    public unowned char Identifier[212];
  }

  [CCode (cname = "zapfontmenu_action_selection_block")]
  public struct ActionSelectionBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "identifier")]
    public unowned char Identifier[212];
  }

  [CCode (cname="class_ZAP_FONT_MENU")]
  public const Toolbox.Class Class_ZapFontMenu;
  [CCode (cname="zapfontmenu_ENCODING_SELECTION")]
  public const bits EncodingSelection;
  [CCode (cname="action_ZAP_FONT_MENU_ABOUT_TO_BE_SHOWN")]
  public const bits Action_ZapFontMenuAboutToBeShown;
  [CCode (cname="action_ZAP_FONT_MENU_HAS_BEEN_HIDDEN")]
  public const bits Action_ZapFontMenuHasBeenHidden;
  [CCode (cname="action_ZAP_FONT_MENU_SELECTION")]
  public const bits Action_ZapFontMenuSelection;
  [CCode (cname="error_ZAP_FONT_MENU_TASKS_ACTIVE")]
  public const bits Error_ZapFontMenuTasksActive;
  [CCode (cname="error_ZAP_FONT_MENU_ALLOC_FAILED")]
  public const bits Error_ZapFontMenuAllocFailed;
  [CCode (cname="error_ZAP_FONT_MENU_SHORT_BUFFER")]
  public const bits Error_ZapFontMenuShortBuffer;
  [CCode (cname="error_ZAP_FONT_MENU_NO_SUCH_TASK")]
  public const bits Error_ZapFontMenuNoSuchTask;
  [CCode (cname="error_ZAP_FONT_MENU_NO_SUCH_METHOD")]
  public const bits Error_ZapFontMenuNoSuchMethod;
  [CCode (cname="error_ZAP_FONT_MENU_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_ZapFontMenuNoSuchMiscOpMethod;

  [CCode (cname = "xzapfontmenu_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O font_menu,
     string font_identifier);

  [CCode (cname = "zapfontmenu_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O font_menu,
     string font_identifier);

  [CCode (cname = "xzapfontmenu_get_font")]
  public static OS.Error* XGetFont
    (bits flags,
     Toolbox.O font_menu,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "zapfontmenu_get_font")]
  public static int GetFont
    (bits flags,
     Toolbox.O font_menu,
     string buffer,
     int size);

  [CCode (cname = "xzapfontmenu_set_system_available")]
  public static OS.Error* XSetSystemAvailable
    (bits flags,
     Toolbox.O font_menu,
     bool available);

  [CCode (cname = "zapfontmenu_set_system_available")]
  public static void SetSystemAvailable
    (bits flags,
     Toolbox.O font_menu,
     bool available);

  [CCode (cname = "xzapfontmenu_get_system_available")]
  public static OS.Error* XGetSystemAvailable
    (bits flags,
     Toolbox.O font_menu,
     out bool available);

  [CCode (cname = "zapfontmenu_get_system_available")]
  public static bool GetSystemAvailable
    (bits flags,
     Toolbox.O font_menu);

}
