/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Font;

[CCode (cheader_filename = "oslib/zapredraw.h")]
namespace OSLib.ZapRedraw
{
  [CCode (cname = "zapredraw_flags")]
  public enum Flags
  {
    [CCode (cname = "zapredraw_DSA")]
    DSA,
    [CCode (cname = "zapredraw_VDU")]
    VDU,
    [CCode (cname = "zapredraw_DOUBLE_HEIGHT")]
    DoubleHeight,
    [CCode (cname = "zapredraw_GIVEN_EXTENSION_ROUTINE")]
    GivenExtensionRoutine,
    [CCode (cname = "zapredraw_SUBSTYLES")]
    Substyles,
    [CCode (cname = "zapredraw_FAST_RECTANGLES")]
    FastRectangles,
    [CCode (cname = "zapredraw_AUTO_SCALE")]
    AutoScale,
    [CCode (cname = "zapredraw_EXTEND")]
    Extend,
    [CCode (cname = "zapredraw_RTOL")]
    RtoL,
    [CCode (cname = "zapredraw_TRANSPARENT")]
    Transparent,
    [CCode (cname = "zapredraw_AA_SUPPRESS")]
    AASuppress,
    [CCode (cname = "zapredraw_16_BIT")]
    16Bit,
    [CCode (cname = "zapredraw_32_BIT")]
    32Bit,
    [CCode (cname = "zapredraw_SMALL_COLOURS")]
    SmallColours
  }

  [CCode (cname = "zapredraw_f", has_type_id = false)]
  public struct F : bits
  {
  }

  [CCode (cname = "zapredraw_e", has_type_id = false)]
  public struct E : bits
  {
  }

  [CCode (cname = "zapredraw_char_cache")]
  public enum CharCache
  {
    Default = 0
  }

  [CCode (cname = "zapredraw_colour")]
  public enum Colour
  {
    Default = 0
  }

  [CCode (cname = "zapredraw_substyle")]
  public enum Substyle
  {
    [CCode (cname = "zapredraw_NORMAL")]
    Normal,
    [CCode (cname = "zapredraw_STRIKETHROUGH")]
    Strikethrough,
    [CCode (cname = "zapredraw_INVERSE")]
    Inverse,
    [CCode (cname = "zapredraw_BOLD")]
    Bold,
    [CCode (cname = "zapredraw_ITALIC")]
    Italic,
    [CCode (cname = "zapredraw_UNDERLINED")]
    Underlined
  }

  [CCode (cname = "zapredraw_encoding_table")]
  public enum EncodingTable
  {
    [CCode (cname = "zapredrawecoding_UTF8")]
    UTF8,
    [CCode (cname = "zapredrawecoding_UTF16")]
    UTF16
  }

  [CCode (cname = "zapredraw_dsa_block")]
  public struct DSABlock
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "screen")]
    public unowned byte* Screen;
    [CCode (cname = "bpl")]
    public unowned int Bpl;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "charw")]
    public unowned int Charw;
    [CCode (cname = "charh")]
    public unowned int Charh;
    [CCode (cname = "caddr")]
    public unowned CharCache* Caddr;
    [CCode (cname = "cbpl")]
    public unowned int Cbpl;
    [CCode (cname = "cbpc")]
    public unowned int Cbpc;
    [CCode (cname = "linesp")]
    public unowned int Linesp;
    [CCode (cname = "data")]
    public unowned char** Data;
    [CCode (cname = "scrollx")]
    public unowned int Scrollx;
    [CCode (cname = "scrolly")]
    public unowned int Scrolly;
    [CCode (cname = "palette")]
    public unowned Palette* Palette;
    [CCode (cname = "fg")]
    public unowned int Fg;
    [CCode (cname = "bg")]
    public unowned int Bg;
    [CCode (cname = "workarea")]
    public unowned char** Workarea;
    [CCode (cname = "magx")]
    public unowned int Magx;
    [CCode (cname = "magy")]
    public unowned int Magy;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "mode")]
    public unowned OS.Mode Mode;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "lmargin")]
    public unowned int Lmargin;
    [CCode (cname = "tmargin")]
    public unowned int Tmargin;
    [CCode (cname = "encoding")]
    public unowned E Encoding;
  }

  [CCode (cname = "zapredraw_vdu_block")]
  public struct VDUBlock
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "reserved0")]
    public unowned byte* Reserved0;
    [CCode (cname = "reserved1")]
    public unowned int Reserved1;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "charw")]
    public unowned int Charw;
    [CCode (cname = "charh")]
    public unowned int Charh;
    [CCode (cname = "caddr")]
    public unowned char* Caddr;
    [CCode (cname = "offsetx")]
    public unowned int Offsetx;
    [CCode (cname = "offsety")]
    public unowned int Offsety;
    [CCode (cname = "linesp")]
    public unowned int Linesp;
    [CCode (cname = "data")]
    public unowned char** Data;
    [CCode (cname = "scrollx")]
    public unowned int Scrollx;
    [CCode (cname = "scrolly")]
    public unowned int Scrolly;
    [CCode (cname = "palette")]
    public unowned Palette* Palette;
    [CCode (cname = "fg")]
    public unowned int Fg;
    [CCode (cname = "bg")]
    public unowned int Bg;
    [CCode (cname = "workarea")]
    public unowned char** Workarea;
    [CCode (cname = "magx")]
    public unowned int Magx;
    [CCode (cname = "magy")]
    public unowned int Magy;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "mode")]
    public unowned OS.Mode Mode;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "lmargin")]
    public unowned int Lmargin;
    [CCode (cname = "tmargin")]
    public unowned int Tmargin;
    [CCode (cname = "encoding")]
    public unowned E Encoding;
  }

  [CCode (cname = "zapredraw_font_block")]
  public struct FontBlock
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "screen")]
    public unowned byte* Screen;
    [CCode (cname = "bpl")]
    public unowned int Bpl;
    [CCode (cname = "log2_bpp")]
    public unowned int Log2Bpp;
    [CCode (cname = "charw")]
    public unowned int Charw;
    [CCode (cname = "charh")]
    public unowned int Charh;
    [CCode (cname = "font")]
    public unowned F Font;
    [CCode (cname = "cbpl")]
    public unowned int Cbpl;
    [CCode (cname = "cbpc")]
    public unowned int Cbpc;
    [CCode (cname = "linesp")]
    public unowned int Linesp;
    [CCode (cname = "data")]
    public unowned char** Data;
    [CCode (cname = "scrollx")]
    public unowned int Scrollx;
    [CCode (cname = "scrolly")]
    public unowned int Scrolly;
    [CCode (cname = "palette")]
    public unowned Palette* Palette;
    [CCode (cname = "fg")]
    public unowned int Fg;
    [CCode (cname = "bg")]
    public unowned int Bg;
    [CCode (cname = "workarea")]
    public unowned char** Workarea;
    [CCode (cname = "magx")]
    public unowned int Magx;
    [CCode (cname = "magy")]
    public unowned int Magy;
    [CCode (cname = "xsize")]
    public unowned int Xsize;
    [CCode (cname = "ysize")]
    public unowned int Ysize;
    [CCode (cname = "mode")]
    public unowned OS.Mode Mode;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "lmargin")]
    public unowned int Lmargin;
    [CCode (cname = "tmargin")]
    public unowned int Tmargin;
    [CCode (cname = "encoding")]
    public unowned E Encoding;
  }

  [CCode (cname = "zapredraw_block")]
  public struct Block
  {
    [CCode (cname = "vdu")]
    public unowned VDUBlock Vdu;
    [CCode (cname = "dsa")]
    public unowned DSABlock Dsa;
    [CCode (cname = "font")]
    public unowned FontBlock Font;
  }

  [CCode (cname = "zapredraw_dsa_palette")]
  public struct DSAPalette
  {
    [CCode (cname = "dup_colour", array_length = false)]
    public unowned bits[] DupColour;
  }

  [CCode (cname = "zapredraw_vdu_palette")]
  public struct VDUPalette
  {
    [CCode (cname = "colour", array_length = false)]
    public unowned OS.Colour[] Colour;
  }

  [CCode (cname = "zapredraw_palette")]
  public struct Palette
  {
    [CCode (cname = "vdu")]
    public unowned VDUPalette Vdu;
    [CCode (cname = "dsa")]
    public unowned DSAPalette Dsa;
  }

  [CCode (cname = "zapredraw_reason")]
  public enum Reason
  {
    [CCode (cname = "zapredraw_NEW_RASTER")]
    NewRaster,
    [CCode (cname = "zapredraw_SET_COLOURS")]
    SetColours,
    [CCode (cname = "zapredraw_MERGE")]
    Merge,
    [CCode (cname = "zapredraw_SET_FG")]
    SetFG,
    [CCode (cname = "zapredraw_SET_BG")]
    SetBG,
    [CCode (cname = "zapredraw_CURSOR_MERGE")]
    CursorMerge,
    [CCode (cname = "zapredraw_SET_STYLE")]
    SetStyle,
    [CCode (cname = "zapredraw_SET_STYLE_COLOURS")]
    SetStyleColours,
    [CCode (cname = "zapredraw_SET_CHUNK")]
    SetChunk,
    [CCode (cname = "zapredraw_SET_LARGE_CHUNK")]
    SetLargeChunk
  }

  [CCode (cname = "zapredrawcreatepalette_flags")]
  public enum ZapRedrawCreatePalette_Flags
  {
    [CCode (cname = "zapredrawcreatepalette_WIMP_TO_VDU")]
    WimpToVDU,
    [CCode (cname = "zapredrawcreatepalette_VDU_TO_DSA")]
    VDUToDSA
  }

  [CCode (cname = "zapredrawspritesize_reason")]
  public enum ZapRedrawSpriteSize_Reason
  {
    [CCode (cname = "zapredrawspritesizereason_PASS_ONE")]
    PassOne,
    [CCode (cname = "zapredrawspritesizereason_PASS_TWO")]
    PassTwo
  }

  [CCode (cname = "zapredraw_add_cursor_match")]
  public enum AddCursorMatch
  {
    [CCode (cname = "zapredraw_MATCH_FOR")]
    MatchFor,
    [CCode (cname = "zapredraw_MATCH_BAC")]
    MatchBac
  }

  [CCode (cname = "zapredraw_add_cursor_cursor")]
  public enum AddCursorCursor
  {
    [CCode (cname = "zapredraw_CURSOR_MERGE_CHARACTER")]
    CursorMergeCharacter,
    [CCode (cname = "zapredraw_CURSOR_MERGE_CURSOR")]
    CursorMergeCursor
  }

  [CCode (cname = "zapredrawfindfont_flags")]
  public enum ZapRedrawFindFont_Flags
  {
    [CCode (cname = "zapredrawfindfont_DSA")]
    DSA,
    [CCode (cname = "zapredrawfindfont_VDU")]
    VDU,
    [CCode (cname = "zapredrawfindfont_REDUCTION_BLOCK")]
    ReductionBlock
  }

  [CCode (cname = "zapredraw_reduction_block")]
  public enum ReductionBlock
  {
    Default = 0
  }

  [CCode (cname = "zapredrawreadcharsize_flags")]
  public enum ZapRedrawReadCharSize_Flags
  {
    [CCode (cname = "zapredrawreadcharsize_PIXELS")]
    Pixels,
    [CCode (cname = "zapredrawreadcharsize_OS")]
    OS,
    [CCode (cname = "zapredrawreadcharsize_LINESP")]
    Linesp,
    [CCode (cname = "zapredrawreadcharsize_SCALE")]
    Scale
  }

  [CCode (cname="zapredraw_COMMAND")]
  public const char Command;
  [CCode (cname="zapredraw_COMMAND_NULL")]
  public const char CommandNull;
  [CCode (cname="zapredraw_COMMAND_SET_COLOURS")]
  public const char CommandSetColours;
  [CCode (cname="zapredraw_COMMAND_EOL")]
  public const char CommandEOL;
  [CCode (cname="zapredraw_COMMAND16BIT_CHAR")]
  public const char Command16bitChar;
  [CCode (cname="zapredraw_COMMAND_MERGE")]
  public const char CommandMerge;
  [CCode (cname="zapredraw_COMMAND_RESERVED")]
  public const char CommandReserved;
  [CCode (cname="zapredraw_COMMAND_SET_FG")]
  public const char CommandSetFG;
  [CCode (cname="zapredraw_COMMAND_SET_BG")]
  public const char CommandSetBG;
  [CCode (cname="zapredraw_COMMAND_CURSOR_MERGE")]
  public const char CommandCursorMerge;
  [CCode (cname="zapredraw_COMMAND_SET_STYLE")]
  public const char CommandSetStyle;
  [CCode (cname="zapredraw_COMMAND_SET_STYLE_COL")]
  public const char CommandSetStyleCol;
  [CCode (cname="zapredraw_COMMAND_SET_CHUNK")]
  public const char CommandSetChunk;
  [CCode (cname="zapredraw_COMMAND_SET_CHUNK_LONG")]
  public const char CommandSetChunkLong;
  [CCode (cname="zapredraw_COMMAND_DELETE")]
  public const char CommandDelete;
  [CCode (cname="zapredraw_MATCH_FOR_COLOUR_SHIFT")]
  public const int MatchForColourShift;
  [CCode (cname="zapredraw_CURSOR_HIGH_BYTE_SHIFT")]
  public const int CursorHighByteShift;
  [CCode (cname="zapredraw_CURSOR_FOR_SHIFT")]
  public const int CursorForShift;

  [CCode (cname = "xzapredraw_redraw_area")]
  public static OS.Error* XRedrawArea
    (Block redraw_block,
     AsmRoutine extension_fn,
     char* extension_handle);

  [CCode (cname = "zapredraw_redraw_area")]
  public static void RedrawArea
    (Block redraw_block,
     AsmRoutine extension_fn,
     char* extension_handle);

  [CCode (cname = "xzapredraw_get_palette_entry")]
  public static OS.Error* XGetPaletteEntry
    (OS.ColourNumber colour_number,
     int bpp,
     out int dup_colour);

  [CCode (cname = "zapredraw_get_palette_entry")]
  public static int GetPaletteEntry
    (OS.ColourNumber colour_number,
     int bpp);

  [CCode (cname = "xzapredraw_convert_bitmap")]
  public static OS.Error* XConvertBitmap
    (Block redraw_block,
     int first,
     int last,
     CharCache bitmaps);

  [CCode (cname = "zapredraw_convert_bitmap")]
  public static void ConvertBitmap
    (Block redraw_block,
     int first,
     int last,
     CharCache bitmaps);

  [CCode (cname = "xzapredraw_prepare_data_line")]
  public static OS.Error* XPrepareDataLine
    (Colour ctrl_colour,
     Block redraw_block,
     string input,
     Colour fg,
     Colour bg,
     int length,
     string output,
     Substyle substyles,
     out int highest,
     out int length_out);

  [CCode (cname = "zapredraw_prepare_data_line")]
  public static int PrepareDataLine
    (Colour ctrl_colour,
     Block redraw_block,
     string input,
     Colour fg,
     Colour bg,
     int length,
     string output,
     Substyle substyles,
     out int highest);

  [CCode (cname = "xzapredraw_find_character")]
  public static OS.Error* XFindCharacter
    (int count,
     string input,
     Colour fg,
     Colour bg,
     Substyle style,
     out string something,
     out string next_char,
     out Colour fg_out,
     out Colour bg_out,
     out Substyle style_out);

  [CCode (cname = "zapredraw_find_character")]
  public static string FindCharacter
    (int count,
     string input,
     Colour fg,
     Colour bg,
     Substyle style,
     out string something,
     out Colour fg_out,
     out Colour bg_out,
     out Substyle style_out);

  [CCode (cname = "xzapredraw_move_bytes")]
  public static OS.Error* XMoveBytes
    (byte* source,
     byte* dest,
     int length);

  [CCode (cname = "zapredraw_move_bytes")]
  public static void MoveBytes
    (byte* source,
     byte* dest,
     int length);

  [CCode (cname = "xzapredraw_cached_char_size")]
  public static OS.Error* XCachedCharSize
    (int bpp,
     int charw,
     int charh,
     out int cbpl,
     out int cbpc);

  [CCode (cname = "zapredraw_cached_char_size")]
  public static void CachedCharSize
    (int bpp,
     int charw,
     int charh,
     out int cbpl,
     out int cbpc);

  [CCode (cname = "xzapredraw_conv_bitmap_char")]
  public static OS.Error* XConvBitmapChar
    (Block redraw_block,
     int source_bpl,
     int dest_bpl,
     CharCache source_bitmap,
     CharCache dest_bitmap,
     bits bit_mask,
     int bpp,
     out CharCache next_source,
     out CharCache next_dest);

  [CCode (cname = "zapredraw_conv_bitmap_char")]
  public static CharCache ConvBitmapChar
    (Block redraw_block,
     int source_bpl,
     int dest_bpl,
     CharCache source_bitmap,
     CharCache dest_bitmap,
     bits bit_mask,
     int bpp,
     out CharCache next_source);

  [CCode (cname = "xzapredraw_create_palette")]
  public static OS.Error* XCreatePalette
    (ZapRedrawCreatePalette_Flags flags,
     Block redraw_block,
     Palette in,
     Palette out,
     int number);

  [CCode (cname = "zapredraw_create_palette")]
  public static void CreatePalette
    (ZapRedrawCreatePalette_Flags flags,
     Block redraw_block,
     Palette in,
     Palette out,
     int number);

  [CCode (cname = "xzapredraw_insert_char")]
  public static OS.Error* XInsertChar
    (int c,
     string address,
     out string address_out);

  [CCode (cname = "zapredraw_insert_char")]
  public static string InsertChar
    (int c,
     string address);

  [CCode (cname = "xzapredraw_read_system_chars")]
  public static OS.Error* XReadSystemChars
    (CharCache buffer,
     Block redraw_block);

  [CCode (cname = "zapredraw_read_system_chars")]
  public static void ReadSystemChars
    (CharCache buffer,
     Block redraw_block);

  [CCode (cname = "xzapredraw_reverse_bitmaps")]
  public static OS.Error* XReverseBitmaps
    (CharCache source,
     CharCache dest,
     int number);

  [CCode (cname = "zapredraw_reverse_bitmaps")]
  public static void ReverseBitmaps
    (CharCache source,
     CharCache dest,
     int number);

  [CCode (cname = "xzapredraw_read_vdu_vars")]
  public static OS.Error* XReadVduVars
    (Block redraw_block);

  [CCode (cname = "zapredraw_read_vdu_vars")]
  public static void ReadVduVars
    (Block redraw_block);

  [CCode (cname = "xzapredraw_get_rectangle")]
  public static OS.Error* XGetRectangle
    (Wimp.Draw wimp_block,
     Block redraw_block);

  [CCode (cname = "zapredraw_get_rectangle")]
  public static void GetRectangle
    (Wimp.Draw wimp_block,
     Block redraw_block);

  [CCode (cname = "xzapredraw_add_vdu_bitmaps")]
  public static OS.Error* XAddVduBitmaps
    (Block redraw_block,
     int first,
     int last,
     CharCache address);

  [CCode (cname = "zapredraw_add_vdu_bitmaps")]
  public static void AddVduBitmaps
    (Block redraw_block,
     int first,
     int last,
     CharCache address);

  [CCode (cname = "xzapredrawcachefontchars_current_mode")]
  public static OS.Error* XZapRedrawCacheFontChars_CurrentMode
    (Block redraw_block,
     string font_name,
     int size,
     int xoff,
     int yoff,
     int first,
     int last);

  [CCode (cname = "zapredrawcachefontchars_current_mode")]
  public static void ZapRedrawCacheFontChars_CurrentMode
    (Block redraw_block,
     string font_name,
     int size,
     int xoff,
     int yoff,
     int first,
     int last);

  [CCode (cname = "xzapredrawcachefontchars_1bpp")]
  public static OS.Error* XZapRedrawCacheFontChars_1bpp
    (Block redraw_block,
     string font_name,
     int size,
     int xoff,
     int yoff,
     int first,
     int last);

  [CCode (cname = "zapredrawcachefontchars_1bpp")]
  public static void ZapRedrawCacheFontChars_1bpp
    (Block redraw_block,
     string font_name,
     int size,
     int xoff,
     int yoff,
     int first,
     int last);

  [CCode (cname = "xzapredrawspritesize_current_mode")]
  public static OS.Error* XZapRedrawSpriteSize_CurrentMode
    (ZapRedrawSpriteSize_Reason pass,
     Block redraw_block,
     out int size);

  [CCode (cname = "zapredrawspritesize_current_mode")]
  public static int ZapRedrawSpriteSize_CurrentMode
    (ZapRedrawSpriteSize_Reason pass,
     Block redraw_block);

  [CCode (cname = "xzapredrawspritesize_1bpp")]
  public static OS.Error* XZapRedrawSpriteSize_1bpp
    (ZapRedrawSpriteSize_Reason pass,
     Block redraw_block,
     out int size);

  [CCode (cname = "zapredrawspritesize_1bpp")]
  public static int ZapRedrawSpriteSize_1bpp
    (ZapRedrawSpriteSize_Reason pass,
     Block redraw_block);

  [CCode (cname = "xzapredraw_redraw_window")]
  public static OS.Error* XRedrawWindow
    (Wimp.Draw wimp_block,
     Block redraw_block);

  [CCode (cname = "zapredraw_redraw_window")]
  public static void RedrawWindow
    (Wimp.Draw wimp_block,
     Block redraw_block);

  [CCode (cname = "xzapredraw_divide")]
  public static OS.Error* XDivide
    (bits numerator,
     bits denominator,
     out bits div,
     out bits mod);

  [CCode (cname = "zapredraw_divide")]
  public static bits Divide
    (bits numerator,
     bits denominator,
     out bits mod);

  [CCode (cname = "xzapredraw_plot_rectangle")]
  public static OS.Error* XPlotRectangle
    (int x0,
     int y0,
     int x1,
     int y1,
     OS.ColourNumber colour);

  [CCode (cname = "zapredraw_plot_rectangle")]
  public static void PlotRectangle
    (int x0,
     int y0,
     int x1,
     int y1,
     OS.ColourNumber colour);

  [CCode (cname = "xzapredraw_add_cursor_smart")]
  public static OS.Error* XAddCursorSmart
    (string dest,
     Block redraw_block,
     int offset,
     int length,
     string cursors,
     string source,
     AddCursorMatch match,
     AddCursorCursor cursor,
     out string end);

  [CCode (cname = "zapredraw_add_cursor_smart")]
  public static void AddCursorSmart
    (string dest,
     Block redraw_block,
     int offset,
     int length,
     string cursors,
     string source,
     AddCursorMatch match,
     AddCursorCursor cursor,
     out string end);

  [CCode (cname = "xzapredraw_find_font")]
  public static OS.Error* XFindFont
    (string name,
     ZapRedrawFindFont_Flags flags,
     int size,
     int aspect,
     ReductionBlock block,
     out F handle);

  [CCode (cname = "zapredraw_find_font")]
  public static F FindFont
    (string name,
     ZapRedrawFindFont_Flags flags,
     int size,
     int aspect,
     ReductionBlock block);

  [CCode (cname = "xzapredraw_lose_font")]
  public static OS.Error* XLoseFont
    (F handle);

  [CCode (cname = "zapredraw_lose_font")]
  public static void LoseFont
    (F handle);

  [CCode (cname = "xzapredraw_read_char_size")]
  public static OS.Error* XReadCharSize
    (F handle,
     Block redraw_block,
     ZapRedrawReadCharSize_Flags flags,
     out int charw,
     out int charh);

  [CCode (cname = "zapredraw_read_char_size")]
  public static void ReadCharSize
    (F handle,
     Block redraw_block,
     ZapRedrawReadCharSize_Flags flags,
     out int charw,
     out int charh);

  [CCode (cname = "xzapredraw_version")]
  public static OS.Error* XVersion
    (out int Version);

  [CCode (cname = "zapredraw_version")]
  public static int Version ();

  [CCode (cname = "xzapredraw_list_fonts")]
  public static OS.Error* XListFonts
    (char* buffer1,
     Font.ListContext context,
     int size1,
     char* buffer2,
     int size2,
     string tick_font,
     out Font.ListContext context_out,
     out int used1,
     out int used2);

  [CCode (cname = "zapredraw_list_fonts")]
  public static Font.ListContext ListFonts
    (char* buffer1,
     Font.ListContext context,
     int size1,
     char* buffer2,
     int size2,
     string tick_font,
     out int used1,
     out int used2);

  [CCode (cname = "xzapredraw_update_window")]
  public static OS.Error* XUpdateWindow
    (Wimp.Draw update,
     Block redraw_block);

  [CCode (cname = "zapredraw_update_window")]
  public static void UpdateWindow
    (Wimp.Draw update,
     Block redraw_block);

  [CCode (cname = "xzapredraw_read_font_name")]
  public static OS.Error* XReadFontName
    (F handle,
     out string name);

  [CCode (cname = "zapredraw_read_font_name")]
  public static string ReadFontName
    (F handle);

  [CCode (cname = "xzapredraw_read_mode_bpp")]
  public static OS.Error* XReadModeBPP
    (out int bpp);

  [CCode (cname = "zapredraw_read_mode_bpp")]
  public static int ReadModeBPP ();

  [CCode (cname = "xzapredraw_decode_menu")]
  public static OS.Error* XDecodeMenu
    (Font.DecodeFlags flags,
     char* menu,
     char* block,
     char* buffer,
     int size,
     out int* next_selection,
     out int used);

  [CCode (cname = "zapredraw_decode_menu")]
  public static void DecodeMenu
    (Font.DecodeFlags flags,
     char* menu,
     char* block,
     char* buffer,
     int size,
     out int* next_selection,
     out int used);

  [CCode (cname = "xzapredrawencoding_find")]
  public static OS.Error* XZapRedrawEncoding_Find
    (string name,
     out E handle);

  [CCode (cname = "zapredrawencoding_find")]
  public static E ZapRedrawEncoding_Find
    (string name);

  [CCode (cname = "xzapredrawencoding_lose")]
  public static OS.Error* XZapRedrawEncoding_Lose
    (E handle);

  [CCode (cname = "zapredrawencoding_lose")]
  public static void ZapRedrawEncoding_Lose
    (E handle);

  [CCode (cname = "xzapredrawencoding_table")]
  public static OS.Error* XZapRedrawEncoding_Table
    (E handle,
     out EncodingTable table);

  [CCode (cname = "zapredrawencoding_table")]
  public static EncodingTable ZapRedrawEncoding_Table
    (E handle);

}
